package ch.ige.adr.client;

import java.awt.Component;
import java.awt.Container;
import java.awt.ContainerOrderFocusTraversalPolicy;

/**
 * In der Klasse FocusTraversalDefinition wird die Reihenfolge des Fokus festgelegt.
 * <p>
 * Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum
 * @author    Marc Bouquet
 * @version   $Id: FocusTraversalDefinition.java,v 1.3 2006/12/11 12:42:37 nydegger Exp $
 */
public class FocusTraversalDefinition extends ContainerOrderFocusTraversalPolicy
{
  // Array, der alle fokusierbaren Komponenten in der vorgesehenen Reihenfolge beinhaltet
  private Component[] componentsInOrder = null;

  // Position der fokusierten Komponente
  private int position = 0;

  /**
   * Komponente in vorgesehener Reihenfolge werden übergeben.
   * @param componentsInOrder Component[]
   */
  public FocusTraversalDefinition( Component[] componentsInOrder )
  {
    this.setOrder( componentsInOrder );
  }

  /**
   * Reihenfolge wird gesetzt.
   * @param componentsInOrder Component[]
   */
  public void setOrder( Component[] componentsInOrder )
  {
    this.componentsInOrder = componentsInOrder;
  }

  /**
   * Nächste Komponente gemäss der Fokus-Reihenfolge wird geliefert.
   * @param focusCycleRoot Container
   * @param aComponent Component
   * @return Component
   */
  public Component getComponentAfter( Container focusCycleRoot, Component aComponent )
  {
    if ( ( position + 1 ) == componentsInOrder.length )
    {
      position = 0;
    }
    else
    {
      position++;
    }
    return componentsInOrder[ position ];
  }

  /**
   * Vorherige Komponente gemäss der Fokus-Reihenfolge wird geliefert.
   * @param focusCycleRoot Container
   * @param aComponent Component
   * @return Component
   */
  public Component getComponentBefore( Container focusCycleRoot, Component aComponent )
  {
    if ( position == 0 )
    {
      position = ( componentsInOrder.length ) - 1;
    }
    else
    {
      position--;
    }
    return componentsInOrder[ position ];
  }

  /**
   * Erste Komponente, die im Fokus steht soll, wird geliefert.
   * @param focusCycleRoot Container
   * @param aComponent Component
   * @return Component
   */
  public Component getFirstComponent( Container focusCycleRoot, Component aComponent )
  {
    return componentsInOrder[ 0 ];
  }
}
