package ch.ige.adr.server;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;

import ch.ige.util.vo.AdrRegadrNrVO;
import ch.ige.util.vo.RegadrVO;

/**
 * SessionFacade für Adressen ab EP-Band
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Ueli Ryser
 * @version $Id: AdresseEpFacadeBean.java,v 1.24 2007/07/03 15:20:54 nydegger Exp $
 * <!-- begin-user-doc --> A generated session bean <!-- end-user-doc --> * 
 * <!-- begin-xdoclet-definition --> 
 * @ejb.bean 
 *  name="AdresseEpFacade"  
 *  description="An EJB named AdresseEpFacade"
 *  display-name="AdresseEpFacade"
 *  view-type="remote"
 *  jndi-name="AdresseEpFacade"
 *  local-jndi-name="AdresseEpFacadeHome
 *  type="Stateless"
 *  transaction-type="Container" 
 *            
 * @ejb.interface
 *  remote-class="ch.ige.adr.server.AdresseEpFacade"
 * @ejb.home
 *  remote-class="ch.ige.adr.server.AdresseEpFacadeHome"
 * @ejb.transaction
 *   type="Required"
 * @ejb.ejb-ref
 *  ejb-name="AdresseXxIdentNumber"
 *  view-type="local" 
 * <!-- end-xdoclet-definition --> 
 */
public class AdresseEpFacadeBean implements SessionBean
{
  private AdresseXxIdentNumberLocalHome xxIdentNumberLocalHome = null;
  private AdresseXxIdentNumberLocal xxIdentNumberLocal = null;
  protected SessionContext sessionContext;



  /**
   * @throws CreateException
   */
  public void ejbCreate() throws CreateException
  {
    try
    {
      // Lokale Referenz
      Context ctx = new InitialContext();
      Object ref = ctx.lookup(AdresseXxIdentNumberLocalHome.COMP_NAME);
      xxIdentNumberLocalHome = (AdresseXxIdentNumberLocalHome)ref;
      xxIdentNumberLocal = xxIdentNumberLocalHome.create();
    }
    catch(Exception ex)
    {
      throw new CreateException();
    }
  }
  public void ejbRemove()
  {
    /** ejbRemove */
  }
  public void ejbActivate()
  {
    /** ejbActivate */
  }
  public void ejbPassivate()
  {
    /** ejbPassivate */
  }
  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }

  /**
   * Kreiert eine Registeradresse.
   * Falls eine Registeradresse mit exakt gleicher regName, RegStrasse,
   * regOrt und blandCd existiert, wird keine neue Registeradresse kreiert,
   * sondern die regadrNr der existierenden Registeradresse zurück gegeben.
   * @param regadrVO Value Object der Registeradresse
   * @return Registeradressnummer
   * <!-- begin-xdoclet-definition --> 
   * @ejb.interface-method
   *   view-type="remote"
   * <!-- end-xdoclet-definition -->
   */
  public int createRegadrEp(RegadrVO regadrVO)
  {
      return new RegadrEpBO(xxIdentNumberLocal).createRegadrEp(regadrVO);
  }

  /**
   * Kreiert eine Register- und eine Versandadresse.
   * Falls eine Registeradresse mit exakt gleicher regName, RegStrasse,
   * regOrt und blandCd existiert, wird keine neue Registeradresse kreiert,
   * sondern die regadrNr der existierenden Registeradresse zurück gegeben.
   * Die Versandadresse wird ab der Registeradresse kreiert. Falls eine ähnliche
   * Versandadresse existiert, wird keine neue Versandadresse kreiert, sondern
   * die adrNr der existierenden Registeradresse zurückgegeben.
   * @param regadrVO Value Object der Registeradresse
   * @param sprachCd Sprachcode für Versandadresse
   * @return AdrRegadrNrVO
   * <!-- begin-xdoclet-definition --> 
   * @ejb.interface-method
   *   view-type="remote"
   * <!-- end-xdoclet-definition -->
   */
  public AdrRegadrNrVO createRegadrVersadrEp(RegadrVO regadrVO, String sprachCd)
  {
      return new RegadrEpBO(xxIdentNumberLocal).createRegadrVersadrEp(regadrVO, sprachCd);
  }
}
