package ch.ige.adr.server;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;

import ch.ige.adr.server.dao.AdresseDAO;
import ch.ige.adr.server.dao.BlandDAO;
import ch.ige.adr.server.dao.LandDAO;
import ch.ige.adr.server.dao.PostOrtDAO;
import ch.ige.adr.server.dao.RegadrDAO;
import ch.ige.adr.vo.AdresseSelKritVO;
import ch.ige.adr.vo.PostOrtVO;
import ch.ige.adr.vo.RegadrExistsVO;
import ch.ige.adr.vo.RegadrSelKritVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;
import ch.ige.util.vo.RegadrVO;
import ch.ige.util.vo.VersadrVO;

/**
 * SessionFacade für Adressen
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Ueli Ryser
 * @version $Id: AdresseFacadeBean.java,v 1.56 2007/07/03 15:20:54 nydegger Exp $
 * <!-- begin-user-doc --> A generated session bean <!-- end-user-doc --> * 
 * <!-- begin-xdoclet-definition --> 
 * @ejb.bean 
 *  name="AdresseFacade"  
 *  description="An EJB named AdresseFacade"
 *  display-name="AdresseFacade"
 *  view-type="remote"
 *  jndi-name="AdresseFacade"
 *  local-jndi-name="AdresseFacadeHome
 *  type="Stateless"
 *  transaction-type="Container"
 *            
 * @ejb.interface
 *  remote-class="ch.ige.adr.server.AdresseFacade"
 * @ejb.home
 *  remote-class="ch.ige.adr.server.AdresseFacadeHome"
 * @ejb.transaction
 *   type="Required"
 * @ejb.ejb-ref
 *  ejb-name="AdresseXxIdentNumber"
 *  view-type="local" 
 * <!-- end-xdoclet-definition --> 
 */
public class AdresseFacadeBean implements SessionBean
{
//  private SessionContext sessionContext;
  private AdresseXxIdentNumberLocalHome xxIdentNumberLocalHome = null;
  private AdresseXxIdentNumberLocal xxIdentNumberLocal = null;


  /**
   * ejbCreate
   */
  public void ejbCreate()
  {
    try
    {
      // Lokale Referenz
      Context ctx = new InitialContext();
      Object ref = ctx.lookup(AdresseXxIdentNumberLocalHome.COMP_NAME);
      xxIdentNumberLocalHome = (AdresseXxIdentNumberLocalHome)ref;
      xxIdentNumberLocal = xxIdentNumberLocalHome.create();
    }
    catch (Exception ex)
    {
      LogLevelHelper.log(ex, AdresseFacadeBean.class);
      throw new EJBException(ex);
    }
  }

  public void ejbRemove()
  {
    /**
     * ejbRemove 
     */
  }
  public void ejbActivate()
  {
    /**
     * ejbActivate
     */
  }
  public void ejbPassivate()
  {
    /**
     * ejbPassivate
     */
  }
  public void setSessionContext(SessionContext sessionContext)
  {
//    this.sessionContext = sessionContext;
  }

  /**
   * Gibt die URL der Bagis-Datenbank zurück.
   * @return URL der Bagis-Datenbank
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
   */
  public String getBagisDbURL()
  {
    try
    {
      Connection con  = DBHelper.getConnection(DBHelper.BAGIS);
      String url = con.getMetaData().getURL();
      con.close();
      return formatDbURL(url);
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseFacadeBean.class);
      throw new EJBException(ex);
    }
  }

  private String formatDbURL(String url)
 {
    try
    {
      String fUrl = "";
      int posPort = url.indexOf("PORT");
      String port = url.substring(posPort);
      int posPortEnd = port.indexOf(")");
      port = port.substring(0, posPortEnd);

      int posHost = url.indexOf("HOST");
      String host = url.substring(posHost);
      int posHostEnd = host.indexOf(")");
      host = host.substring(0, posHostEnd);

      int posSID = url.indexOf("SID");
      String sid = url.substring(posSID);
      int posSIDEnd = sid.indexOf(")");
      sid = sid.substring(0, posSIDEnd);
      fUrl = host + "\n" + port + "\n" + sid;
      return fUrl;
    }
    catch (Exception ex)
    {
      return url;
    }
 }


  /**
   * Kreiert eine Versandadresse.
   * @param versadrVO Value Object der Versanddresse
   * @return Adressnummer
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public int createVersadr(VersadrVO versadrVO)
  {
    return new AdresseBO(xxIdentNumberLocal).createVersadr(versadrVO);
  }

  /**
   * Gibt das Value Object der Versanddresse zurück.
   * @param adrNr Adressnummer
   * @return Value Object der Versanddresse
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public VersadrVO getVersadr(int adrNr)
  {
    return new AdresseBO(xxIdentNumberLocal).getVersadr(adrNr);
  }

  /**
   * Setzt die Versanddresse auf den Wert des Value Objects.
   * @param versadrVO Versanddresse
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public void setVersadr(VersadrVO versadrVO)
  {
    new AdresseBO(xxIdentNumberLocal).setVersadr(versadrVO);
  }

  /**
   * Löscht die Versandadresse
   * @param adrNr Adressnummer
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public void removeVersadr(int adrNr)
  {
    new AdresseBO(xxIdentNumberLocal).removeVersadr(adrNr);
  }

  /**
   * Sperrt die Versandadresse
   * @param adrNr Adressnummer
   * @param gesperrt true - sperren, false - Sperrung aufheben
   * @param geaendBen Kürzel des Benuzter
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public void blockVersadr(int adrNr, boolean gesperrt, String geaendBen)
  {
    new AdresseBO(xxIdentNumberLocal).blockVersadr(adrNr, gesperrt, geaendBen);
  }

  /**
   * Findet alle Adressen, auf welche die Selektionskriterien zutreffen.
   * @param adresseSelKritVO Value Object mit den Selektionskriterien
   * @return ArrayList mit AdresseAVO-Objekten mit den gefundenen Adressen
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public ArrayList findVersadr(AdresseSelKritVO adresseSelKritVO)
  {
    return new AdresseBO(xxIdentNumberLocal).findVersadr(adresseSelKritVO);
  }

  /**
   * Liest alle Adr-Nummern aus der Tabelle Adresse
   * @param startAdrNr Anfangs-Adressnummer
   * @return ArrayList mit Integer-Objekten mit den Adressnummern
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public ArrayList getAdrNr(int startAdrNr)
  {
    return new AdresseBO(xxIdentNumberLocal).getAdrNr(startAdrNr);
  }

  /**
   * Prüft die Existenz einer Adresse.
   * @param adrNr Adressnummer
   * @return true - LäAdresse existiert.
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public boolean existsVersadr(int adrNr)
  {
    try
    {
      return new AdresseDAO().exists(adrNr);
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseFacadeBean.class);
      throw new EJBException(ex);
    }
  }

  /**
   * Kreiert eine Registeradresse.
   * @param regadrVO Value Object der Registeradresse
   * @return Registeradressnummer
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public int createRegadr(RegadrVO regadrVO)
  {
    return new RegadrBO(xxIdentNumberLocal).createRegadr(regadrVO);
  }

  /**
   * Kreiert eine Registeradresse und eine Versandadresse.
   * @param regadrVO Value Object der Registeradresse
   * @param versadrVO Value Object der Versandadresse
   * @return Registeradressnummer
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public int createRegadr(RegadrVO regadrVO, VersadrVO versadrVO)
  {
    return new RegadrBO(xxIdentNumberLocal).createRegadr(regadrVO, versadrVO);
  }

  /**
   * Gibt das Value Object der Registeradresse zurück.
   * @param regadrNr Registeradressnummer
   * @return Value Object der Registeradresse
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public RegadrVO getRegadr(int regadrNr)
  {
    return new RegadrBO(xxIdentNumberLocal).getRegadr(regadrNr);
  }

  /**
   * Gibt die Registeradressnummer zurück.
   * @param regadrExistsVO Registeradresse
   * @return Registeradressnummer
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public int getRegadr(RegadrExistsVO regadrExistsVO)
  {
    try
    {
      return new RegadrDAO().selectRegadrNr(regadrExistsVO);
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseFacadeBean.class);
      throw new EJBException(ex);
    }
  }

  /**
   * Setzt die Registeradresse auf den Wert des Value Objects.
   * @param regadrVO Registeradresse
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public void setRegadr(RegadrVO regadrVO)
  {
    new RegadrBO(xxIdentNumberLocal).setRegadr(regadrVO);
  }

  /**
   * Erstellt eine Versandadresse und setzt die Adressnummer in der
   * Registeradresse ein und setzt die Registeradresse auf den Wert
   * des Value Objects.
   * @param regadrVO Registeradresse (wird mutiert!)
   * @param versadrVO Versandadresse (wird erstellt!)
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public void setRegadr(RegadrVO regadrVO, VersadrVO versadrVO)
  {
    new RegadrBO(xxIdentNumberLocal).setRegadr(regadrVO, versadrVO);
  }

  /**
   * Löscht die Registeradresse
   * @param regadrNr Registeradressnummer
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public void removeRegadr(int regadrNr)
  {
    new RegadrBO(xxIdentNumberLocal).removeRegadr(regadrNr);
  }

  /**
   * Findet alle Registeradresse, auf welche die Selektionskriterien zutreffen.
   * @param regadrSelKritVO Value Object mit den Selektionskriterien
   * @return ArrayList mit RegadrAVO-Objekten mit den gefundenen Registeradresse
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public ArrayList findRegadr(RegadrSelKritVO regadrSelKritVO)
  {
    return new RegadrBO(xxIdentNumberLocal).findRegadr(regadrSelKritVO);
  }

  /**
   * Findet alle Länder, auf welche die Selektionskriterien zutreffen.
   * @param landCd Ländercode als Selektionskriterium
   * @param landBez Länderbezeichnungs als Selektionskriterium
   * @return ArrayList mit LandAVO-Objekten mit den gefundenen Ländern
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public ArrayList findLaender(String landCd, String landBez)
  {
    try
    {
       return new LandDAO().select(landCd, landBez);
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseFacadeBean.class);
      throw new EJBException(ex);
    }
  }

  /**
   * Prüft die Existenz eines Ländercodes.
   * @param landCd Ländercode, z.B. CH für Schweiz, F für Frankreich etc.
   * @return true - Ländercode existiert.
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public boolean existsLand(String landCd)
  {
    try
    {
      return new LandDAO().exists(landCd);
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseFacadeBean.class);
      throw new EJBException(ex);
    }
  }

  /**
   * Findet alle BLänder, auf welche die Selektionskriterien zutreffen.
   * @param blandCd Ländercode als Selektionskriterium
   * @param landBezD Länderbezeichnungs als Selektionskriterium
   * @return ArrayList mit BlandAVO-Objekten mit den gefundenen Bländern
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public ArrayList findBland(String blandCd, String landBezD)
  {
    try
    {
       return new BlandDAO().select(blandCd, landBezD);
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseFacadeBean.class);
      throw new EJBException(ex);
    }
  }

  /**
   * Prüft die Existenz eines BLändercodes.
   * @param blandCd BLändercode, z.B. CH für Schweiz, FR für Frankreich etc.
   * @return true - BLändercode existiert.
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public boolean existsBland(String blandCd)
  {
    try
    {
      return new BlandDAO().exists(blandCd);
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseFacadeBean.class);
      throw new EJBException(ex);
    }
  }

  /**
   * Gibt den Landcode aufgrund des BLandcodes zurück.
   * @param blandCd BLandcode, z.B. CH für Schweiz, FR für Frankreich etc.
   * @return Landcode, z.B. CH für Schweiz, F für Frankreich etc.
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public String getLandCd(String blandCd)
  {
    try
    {
      return new BlandDAO().selectLandCd(blandCd);
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseFacadeBean.class);
      throw new EJBException(ex);
    }
  }


  /**
   * Findet alle Ortschaften mit plz oder ort.
   * Anfangsziffern bzw. Anfangsbuchstaben sind ausreichend.
   * @param plz Postleitzahl der Ortschaft
   * @param ort Name der Ortschaft
   * @return ArrayList mit PostOrtAVO-Objekten mit allen gefundenen Ortschaften
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public ArrayList findPostOrt(String plz, String ort)
  {
    try
    {
      if (ort != null && ort.length() > 0)
      {
        // mit normalisiertem Ort suchen
        ort = PostOrtNorm.getOrtNorm(ort);
      }
      return new PostOrtDAO().select(plz, ort);
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseFacadeBean.class);
      throw new EJBException(ex);
    }
  }

  /**
  * Speicherung der Ortschaften in der Datenbank
  *
  * @param postOrtVO Benötigte Daten für die Normalisierung
  * @author  Michael Zaugg
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public void ladeOrtsverzeichnis(PostOrtVO[] postOrtVO)
  {
    new PostOrtBO().ladeOrtsverzeichnis(postOrtVO);
  }

  /**
   * Führt den Ortsabgleich durch.
  * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
  */
  public void ortsabgleich()
  {
    new OrtsabgleichBO().startOrtsabgleich();
  }
}
