package ch.ige.adr.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;

import ch.ige.adr.server.AssemblerOracleText;
import ch.ige.adr.server.FieldParserOrAndNot;
import ch.ige.adr.vo.RegadrAVO;
import ch.ige.adr.vo.RegadrExistsVO;
import ch.ige.adr.vo.RegadrSelKritVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;
import ch.ige.util.vo.RegadrVO;

/**
 * Datenbankzugriff auf die Registeradresse
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Ueli Ryser
 * @version $Id: RegadrDAO.java,v 1.28 2007/10/02 11:33:57 nydegger Exp $
 */
public class RegadrDAO
{
  
  private AssemblerOracleText assembler = null;
  private FieldParserOrAndNot parser = null;
  private static final Logger LOG = Logger.getLogger(AdresseDAO.class);
  
  
  public RegadrDAO()
  {
    assembler = new AssemblerOracleText();
    parser = new FieldParserOrAndNot(AssemblerOracleText.OPERATOR_AND);
  }
  /**
   * Selektiert eine Registeradresse.
   * @param regadrNr Registeradressnummer
   * @return Value Object der Registeradresse
   * @exception SQLException
   */
  public RegadrVO select(int regadrNr) throws SQLException
  {
    String query = null;
    RegadrVO regadrVO = new RegadrVO();
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    try
    {
      query = "SELECT regadr_nr, bland_cd, adr_nr, reg_name, " +
              "reg_strasse, reg_ort, up_reg_ort, up_verz_name, " +
              "herkunft, herk_adr_nr, gesperrt, version " +
              "FROM regadr WHERE regadr_nr = ?";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setInt(1, regadrNr);
      rs = ps.executeQuery();
      if(!rs.next())
      {
        return null;
      }
      regadrVO.setRegadrNr(rs.getInt("regadr_nr"));
      regadrVO.setBlandCd(rs.getString("bland_cd"));
      regadrVO.setAdrNr(rs.getInt("adr_nr"));
      regadrVO.setRegName(rs.getString("reg_name"));
      regadrVO.setRegStrasse(rs.getString("reg_strasse"));
      regadrVO.setRegOrt(rs.getString("reg_ort"));
      regadrVO.setUpRegOrt(rs.getString("up_reg_ort"));
      regadrVO.setUpVerzName(rs.getString("up_verz_name"));
      regadrVO.setHerkunft(rs.getInt("herkunft"));
      regadrVO.setHerkAdrNr(rs.getInt("herk_adr_nr"));
      regadrVO.setGesperrt(rs.getInt("gesperrt"));
      regadrVO.setVersion(rs.getInt("version"));
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, RegadrDAO.class, query + " regadr_nr = " + regadrNr);
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
    return regadrVO;
  }

  /**
   * Selektiert eine Registeradresse.
   * @param regadrExistsVO Registeradresse
   * @return Registeradressnummer
   * @exception SQLException
   */
  public int selectRegadrNr(RegadrExistsVO regadrExistsVO) throws SQLException
  {
    int regadrNr = 0;
    Connection con = null;
    Statement stmt = null;
    ResultSet rs = null;
    StringBuffer sql = new StringBuffer();
    try
    {
      sql.append("SELECT regadr_nr " +
              "FROM regadr " +
              "WHERE gesperrt = 0 ");

     if (!(regadrExistsVO.blandCd == null || regadrExistsVO.blandCd.equals("")))
     {
       sql.append(" AND bland_cd = RPAD(trim('" + regadrExistsVO.blandCd + "' ), 2, ' ') ");
     }
     if (!(regadrExistsVO.regName == null || regadrExistsVO.regName.equals("")))
     {
        sql.append(" AND reg_name = " + DBHelper.getSqlString(regadrExistsVO.regName.trim()));
     }
     else
     {
        sql.append(" AND reg_name is null ");
     }
     if (!(regadrExistsVO.regStrasse == null  || regadrExistsVO.regStrasse.equals("") ))
     {
        sql.append(" AND reg_strasse = " + DBHelper.getSqlString(regadrExistsVO.regStrasse.trim()));
     }
     else
     {
         sql.append(" AND reg_strasse is null ");
     }
     if (!(regadrExistsVO.regOrt == null  || regadrExistsVO.regOrt.equals("") ))
     {
        sql.append(" AND reg_ort = " + DBHelper.getSqlString(regadrExistsVO.regOrt.trim()));
     }
     else
     {
         sql.append(" AND reg_ort is null ");
     }

     con = DBHelper.getConnection(DBHelper.BAGIS);
     stmt = con.createStatement();
     rs = stmt.executeQuery(sql.toString());

      if (rs.next())
      {
        regadrNr = rs.getInt("regadr_nr");
      }
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, RegadrDAO.class, sql + regadrExistsVO.toString());
      throw ex;
    }
    finally
    {
      DBHelper.close(stmt, con);
    }
    return regadrNr;
  }

  /**
   * Kreiert eine Registeradresse.
   * @param regadrVO Value Object der Registeradresse
   * @exception SQLException
   */
  public void insert(RegadrVO regadrVO) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    try
    {

      query = "INSERT INTO regadr(regadr_nr, bland_cd, adr_nr, reg_name, " +
              "reg_strasse, reg_ort, up_reg_ort, up_verz_name, " +
              "herkunft, herk_adr_nr, gesperrt, version )" +
              "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";

     // auf Überlängen prüfen
     String truncMsg = regadrVO.truncateFields();
     if (truncMsg != null)
     {
       LogLevelHelper.log(new Exception(""), RegadrDAO.class, truncMsg + "\n" + query + regadrVO.toString());
     }

      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setInt(1, regadrVO.getRegadrNr());
      ps.setString(2, regadrVO.getBlandCd());
      if (regadrVO.getAdrNr() == 0)
      {
        ps.setNull(3, java.sql.Types.INTEGER);
      }
      else
      {
        ps.setInt(3, regadrVO.getAdrNr());
      }
      ps.setString(4, regadrVO.getRegName());
      ps.setString(5, regadrVO.getRegStrasse());
      ps.setString(6, regadrVO.getRegOrt());
      ps.setString(7, regadrVO.getUpRegOrt());
      ps.setString(8, regadrVO.getUpVerzName());
      ps.setInt(9, regadrVO.getHerkunft());
      ps.setInt(10, regadrVO.getHerkAdrNr());
      ps.setInt(11, regadrVO.getGesperrt());
      ps.setInt(12, regadrVO.getVersion());
      ps.executeUpdate();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, RegadrDAO.class, query + regadrVO.toString());
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Setzt die Registeradresse auf den Wert des Value Objects.
   * @param regadrVO Value Object der Registeradresse
   * @return Anzahl der aktualisierten Registeradressen
   * @exception SQLException
   */
  public int update(RegadrVO regadrVO) throws SQLException
  {
    int rowCount;
    String query = null;
    Connection con = null;
    PreparedStatement ps = null;
    try
    {
      query = "UPDATE regadr SET " +
              "bland_cd = ?, " +
              "adr_nr = ?, " +
              "reg_name = ?, " +
              "reg_strasse = ?, " +
              "reg_ort = ?, " +
              "up_reg_ort = ?, " +
              "up_verz_name = ?, " +
              "herkunft = ?, " +
              "herk_adr_nr = ?, " +
              "gesperrt = ?, " +
              "version = ? " +
              "WHERE regadr_nr = ?";

          // auf Überlängen prüfen
    String truncMsg = regadrVO.truncateFields();
    if (truncMsg != null)
    {
      LogLevelHelper.log(new Exception(""), RegadrDAO.class, truncMsg + "\n" + query + regadrVO.toString());
    }

      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setString(1, regadrVO.getBlandCd());
      if (regadrVO.getAdrNr() == 0)
      {
        ps.setNull(2, java.sql.Types.INTEGER);
      }
      else
      {
        ps.setInt(2, regadrVO.getAdrNr());
      }
      ps.setString(3, regadrVO.getRegName());
      ps.setString(4, regadrVO.getRegStrasse());
      ps.setString(5, regadrVO.getRegOrt());
      ps.setString(6, regadrVO.getUpRegOrt());
      ps.setString(7, regadrVO.getUpVerzName());
      ps.setInt(8, regadrVO.getHerkunft());
      ps.setInt(9, regadrVO.getHerkAdrNr());
      ps.setInt(10, regadrVO.getGesperrt());
      ps.setInt(11, regadrVO.getVersion());
      ps.setInt(12, regadrVO.getRegadrNr());
      rowCount = ps.executeUpdate();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, RegadrDAO.class, query + " reg_adr = " + regadrVO.toString());
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
    return rowCount;
  }

  /**
   * Löscht eine Registeradresse.
   * @param regadrNr Registeradressnummer der zu löschenden Registeradressnummer
   * @return Anzahl der gelöschten Registeradressnummern
   * @exception SQLException
   */
  public int delete(int regadrNr) throws SQLException
  {
    int rowCount;
    String query = null;
    Connection con = null;
    PreparedStatement ps = null;
    try
    {
      query = "DELETE FROM regadr WHERE regadr_nr = ?";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setInt(1, regadrNr);
      rowCount = ps.executeUpdate();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, RegadrDAO.class, query + " regadr_nr = " + regadrNr);
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
    return rowCount;
  }

  /**
   * Findet alle Registeradressen, auf welche die Selektionskriterien zutreffen.
   * @param regadrSelKritVO Value Object mit den Selektionskriterien
   * @return ArrayList mit RegadrAVO-Objekten mit den gefundenen Registeradresse
   * @exception SQLException
   */
  public ArrayList select(RegadrSelKritVO regadrSelKritVO) throws SQLException
  {
    // Assertions
    // Mindestens ein Suchkriterium muss angegeben werden
    if (regadrSelKritVO.regadrNr == 0 && regadrSelKritVO.upVerzName.length() == 0 &&
        regadrSelKritVO.regName.length() == 0 && regadrSelKritVO.regStrasse.length() == 0 &&
        regadrSelKritVO.regOrt.length() == 0 && regadrSelKritVO.adrNr == 0)
    {
      if (regadrSelKritVO.blandCd.length() == 0)
      {
        throw new IllegalArgumentException("Mindestens ein Suchkriterium muss angegeben werden");
      }
      else
      {
        throw new IllegalArgumentException("Land darf nicht das einzige Suchkriterium sein");
      }
    }
    int count = 0;
    String and = "";
    ArrayList regadrAL = new ArrayList(1000);
    Connection con = null;
    Statement stmt = null;
    ResultSet rs = null;

    StringBuffer select = new StringBuffer();
    StringBuffer from = new StringBuffer();
    StringBuffer where = new StringBuffer();

    select.append("SELECT r.regadr_nr, bland_cd, adr_nr, reg_name, reg_strasse, " +
                  "reg_ort, up_verz_name ");
    from.append("FROM regadr r ");
    where.append("WHERE ");
    if (regadrSelKritVO.regadrNr == 0)
    {
      // keine Registeradress-Nr. eingegeben
      // alle anderen Fälle
      if (!regadrSelKritVO.upVerzName.equals(""))
      {
        String umlaut =ch.ige.util.StringUtil.replaceAeOeUe(regadrSelKritVO.upVerzName);
        String upVerzName = ch.ige.util.StringUtil.replaceUmlaut(regadrSelKritVO.upVerzName);
        String likeOrEquals = " = ";
        if (upVerzName.indexOf('%') > -1)
        {
          likeOrEquals = " LIKE ";
        }
        if (upVerzName.equals(umlaut))
        {
          where.append(" up_verz_name" + likeOrEquals);
          where.append(DBHelper.getSqlString(upVerzName) + " ");
        }
        else
        {
          where.append(" (up_verz_name" + likeOrEquals);
          where.append(DBHelper.getSqlString(upVerzName));
          where.append(" OR up_verz_name" + likeOrEquals);
          where.append(DBHelper.getSqlString(umlaut) + ") ");
        }
        and = " AND ";
      }
      if (!regadrSelKritVO.regName.equals(""))
      {
        if (regadrSelKritVO.suchmodusVolltext)
        {
          where.append( and );
          // Volltext-Suche
          where.append(" reg_name LIKE ");
          where.append( DBHelper.getSqlString(regadrSelKritVO.regName.trim()) + " ");
          and = " AND ";
        }
        else
        {
          // Index-Suche
          where.append( and );
          assembler.reset();
          // Index Suche
          StringTokenizer st = new StringTokenizer(regadrSelKritVO.regName, ".");
          List lst = new ArrayList();
          while(st.hasMoreTokens())
          {
            lst.add(  st.nextToken() );
          }
          try
          {
            parser.parse( lst, assembler );
          }
          catch ( Exception e )
          {
            LOG.error( "Error: " + e.getMessage(), e );
            throw new SQLException(e.getMessage());
          }        
          where.append( " contains (reg_name , '" + assembler.getValue()+"') >0 ");
          and = " AND ";
        }
      }
      if (!regadrSelKritVO.regOrt.equals(""))
      {
        where.append( and );
        if (regadrSelKritVO.suchmodusVolltext)
        {
          where.append(" reg_ort LIKE ");
          where.append(
              DBHelper.getSqlString(regadrSelKritVO.regOrt.trim()) + " ");
        }
        else
        {
          // Index-Suche
          assembler.reset();
          // Index Suche
          StringTokenizer st = new StringTokenizer(regadrSelKritVO.regOrt, ".");
          List lst = new ArrayList();
          while(st.hasMoreTokens())
          {
            lst.add(  st.nextToken() );
          }
          try
          {
            parser.parse( lst, assembler );
          }
          catch ( Exception e )
          {
            LOG.error( "Error: " + e.getMessage(), e );
            throw new SQLException(e.getMessage());
          }        
          where.append( " contains (reg_ort , '" + assembler.getValue()+"') >0 ");
        }
        and = " AND ";
      }
      if (!regadrSelKritVO.blandCd.equals(""))
      {
        where.append( and );
        where.append(" bland_cd = ");
        where.append(DBHelper.getSqlString(regadrSelKritVO.blandCd) + " ");
        and = " AND ";
      }
      if (regadrSelKritVO.adrNr != 0)
      {
        where.append( and );
        where.append(" adr_nr = ");
        where.append(regadrSelKritVO.adrNr + " ");
      }
    }
    else
    {
      // Registeradress-Nr. eingegeben
      where.append(" regadr_nr = ");
      where.append(regadrSelKritVO.regadrNr + " ");
    }
    if (regadrSelKritVO.nurMitVersAdr)
    {
      where.append("AND NOT (adr_nr IS NULL OR adr_nr = 0) ");
    }
    if (regadrSelKritVO.nurChLi)
    {
      where.append("AND bland_cd IN ('CH', 'LI') ");
    }
    if (regadrSelKritVO.suchumfang == RegadrSelKritVO.SUCHUMFANG_NUR_GUELTIGE)
    {
      where.append("AND gesperrt = 0 ");
    }
    else if (regadrSelKritVO.suchumfang == RegadrSelKritVO.SUCHUMFANG_NUR_GESPERRTE)
    {
      where.append("AND gesperrt = 1 ");
    }

    String sql = select.toString() + from.toString() + where.toString();
    // System.out.println("sql \n " + sql);
    LOG.debug("sql \n " + sql);
    try
    {
      con = DBHelper.getConnection(DBHelper.BAGIS);
      stmt = con.createStatement();
      rs = stmt.executeQuery(sql);
      while (rs.next() && count < 1000)
      {
        RegadrAVO regadrAVO = new RegadrAVO();

        regadrAVO.setRegadrNr(rs.getInt("regadr_nr"));
        regadrAVO.setBlandCd(rs.getString("bland_cd"));
        regadrAVO.setAdrNr(rs.getInt("adr_nr"));
        regadrAVO.setRegName(rs.getString("reg_name"));
        regadrAVO.setRegStrasse(rs.getString("reg_strasse"));
        regadrAVO.setRegOrt(rs.getString("reg_ort"));
        regadrAVO.setUpVerzName(rs.getString("up_verz_name"));
        regadrAL.add(regadrAVO);
        count++;
      }
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, RegadrDAO.class, sql.toString());
      throw ex;
    }
    finally
    {
      DBHelper.close(stmt, con);
    }
    regadrAL.trimToSize();
    return regadrAL;
  }

  /**
   * Selektiert mehrere gleiche Registeradresse.
   * @param regadrExistsVO Registeradresse
   * @return ArrayList mit RegadrAVO-Objekten mit den gefundenen Registeradresse
   * @exception SQLException
   */
  public ArrayList select(RegadrExistsVO regadrExistsVO) throws SQLException
  {
    int count = 0;
    String query = null;
    ArrayList regadrAL = new ArrayList(1000);
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;

    try
    {
      query = "SELECT regadr_nr, bland_cd, adr_nr, reg_name, reg_strasse, " +
              "reg_ort, up_verz_name, herkunft "  +
              "FROM regadr " +
              "WHERE gesperrt = 0 AND bland_cd = RPAD(?, 2, ' ') AND reg_name = trim(?) AND reg_strasse = trim(?) AND reg_ort = trim(?) " +
              "ORDER BY herkunft";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setString(1, regadrExistsVO.blandCd);
      ps.setString(2, regadrExistsVO.regName);
      ps.setString(3, regadrExistsVO.regStrasse);
      ps.setString(4, regadrExistsVO.regOrt);

      rs = ps.executeQuery();
      while (rs.next() && count < 1000)
      {
        RegadrAVO regadrAVO = new RegadrAVO();

        regadrAVO.setRegadrNr(rs.getInt("regadr_nr"));
        regadrAVO.setBlandCd(rs.getString("bland_cd"));
        regadrAVO.setAdrNr(rs.getInt("adr_nr"));
        regadrAVO.setRegName(rs.getString("reg_name"));
        regadrAVO.setRegStrasse(rs.getString("reg_strasse"));
        regadrAVO.setRegOrt(rs.getString("reg_ort"));
        regadrAVO.setUpVerzName(rs.getString("up_verz_name"));
        regadrAL.add(regadrAVO);
        count++;
      }
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, RegadrDAO.class, query + regadrExistsVO.toString());
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
    regadrAL.trimToSize();
    return regadrAL;
  }
}
