package ch.ige.art.client;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.print.Printable;
import java.nio.CharBuffer;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.Charset;
import java.nio.charset.CharsetEncoder;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import ch.ige.art.client.util.UserMessage;
import ch.ige.art.client.util.verifier.TextVerifier;
import ch.ige.main.client.DlgTextEditor;
import ch.ige.main.client.FrmMain;
import ch.ige.util.client.CharsetVerifier;
import ch.ige.util.client.IgeImage;


/**
 * <p>Überschrift: Text Detailansicht</p>
 * <p>Beschreibung: Dialog für die Detailansicht eines Textes</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author  Anita Rueegsegger
 * @version $Id: DlgArtikelTexte.java,v 1.31 2007/12/14 08:32:20 galli Exp $
 */

public class DlgArtikelTexte extends DlgTextEditor implements Printable
{
  // Logging
  private static final Logger LOG = Logger.getLogger(DlgArtikelTexte.class);
  private UserMessage userMessage = new UserMessage();

  private boolean verifyText = false;   // false - Textinhalt muss nicht überprüft werden

  // Menu
  private JMenuItem mnuBr = new JMenuItem();
  private JMenuItem mnuB = new JMenuItem();
  private JMenu menHelp = new JMenu();
  private JMenuItem mnuInfo = new JMenuItem();

  // ToolBar
  private JButton btnB = new JButton();
  private JButton btnBr = new JButton();

  // Images
  private ImageIcon imgCarriageReturn = IgeImage.imgCarriageReturn;

  // Charset
  private Charset charset = Charset.forName( "ISO-8859-1" );
  private CharsetEncoder encoder = charset.newEncoder();


  /**
   * Initialisiert einen modalen Texteditor-Dialog mit dem Titel "Texteditor"
   */
  public DlgArtikelTexte()
  {
    this(new FrmMain(), "Texteditor", true);
  }

  /**
   * Texteditor-Dialog
   * @param frame Aufrufendes Frame
   * @param title Titel des Dialogs
   * @param modal true-modaler Aufruf
   * @param edit  false-Ansichtsmodus, Text ist nicht bearbeitbar
   * @param verifyText false-Textinhalt muss nicht überprüft werden
   */
  public DlgArtikelTexte(Frame frame, String title, boolean modal)
  {
    super(frame, title, modal);

    try
    {
      jbInit();
    }
    catch(Exception ex)
    {
      LOG.info(ex.getClass() + " " + ex.getMessage());
      String message = userMessage.getUserMessage(ex);
      JOptionPane.showMessageDialog(null, message, "Information", JOptionPane.INFORMATION_MESSAGE);
    }
  }


  void jbInit() throws Exception
  {
    this.setSize(450, 300);


    //********** Menus *****************************************************

    // Menu Bearbeiten
    mnuBr.setText("Zeilenumbruch markieren");
    mnuBr.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        mnuBr_actionPerformed(e);
      }
    });
    mnuB.setText("Fett markieren");
    mnuB.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        mnuB_actionPerformed(e);
      }
    });

    // Menu Hilfe
    menHelp.setText("Hilfe");
    menHelp.setMnemonic('H');
    mnuInfo.setText("Info");
    mnuInfo.setMnemonic('I');
    mnuInfo.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        mnuInfo_actionPerformed(e);
      }
    });

    //********** Buttons ***************************************************

    btnB.setIcon(imgBold);
    btnB.setRequestFocusEnabled(false);
    btnB.setToolTipText("Markiert den Text als Fett");
    btnB.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        mnuB_actionPerformed(e);
      }
    });
    btnBr.setIcon(imgCarriageReturn);
    btnBr.setRequestFocusEnabled(false);
    btnBr.setToolTipText("Markiert einen Zeilenumbruch");
    btnBr.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        mnuBr_actionPerformed(e);
      }
    });

    txpText.addKeyListener(new java.awt.event.KeyAdapter()
    {
      public void keyPressed(KeyEvent e)
      {
        txpText_keyPressed(e);
      }
    });
    CharsetVerifier cv = new CharsetVerifier();
    cv.enableHighlighting( true );
    cv.showMessageDialog( true );
    txpText.setInputVerifier( cv );

    // Toolbar zusammensetzen
    toolBar.add(btnB);
    toolBar.add(btnBr);

    // Menüs zusammensetzen
    menEdit.addSeparator();
    menEdit.add(mnuBr);
    menEdit.add(mnuB);
    menHelp.add(mnuInfo);
    menuBar.add(menHelp);
  }

  /**
   * DetailAnsicht Text anzeigen.
   * @param strText Text der angezeigt werden soll
   * @param edit falls true ist Text bearbeitbar
   * @param verifyText falls true wird der Text auf Platzhalter und Escape-Sequenzen überprüft
   * @param strTextart falls "0" wird die Länge des maximalen Textes auf 40 Zeichen beschränkt
   * @return Text wird zurückgegeben
   */
  public String showDlgText(String strText)
  {
    return this.showDlgText(strText, false, false);
  }

  /**
   * DetailAnsicht Text anzeigen.
   * @param strText Text der angezeigt werden soll
   * @param edit falls true ist Text bearbeitbar
   * @param verifyText falls true wird der Text auf Platzhalter und Escape-Sequenzen überprüft
   * @param strTextart falls "0" wird die Länge des maximalen Textes auf 40 Zeichen beschränkt
   * @return Text wird zurückgegeben
   */
  public String showDlgText(String strText, boolean edit, boolean verifyText)
  {
    return this.showDlgText(strText, edit, verifyText, null);
  }

  /**
   * DetailAnsicht Text anzeigen.
   * @param strText Text der angezeigt werden soll
   * @param edit falls true ist Text bearbeitbar
   * @param verifyText falls true wird der Text auf Platzhalter und Escape-Sequenzen überprüft
   * @param maxLength Länge des maximalen Textes beschränkt
   * @return Text wird zurückgegeben
   */
  public String showDlgText(String strText, boolean edit, boolean verifyText, Integer maxLength)
  {
    this.verifyText = verifyText;
    this.setEdit(edit);
    return super.showDlgText(strText, edit, maxLength);
  }

  /**
   * Aktiviert oder Deaktiviert Menus & Buttons für "Zeilenumbruch" und "Fett markieren"
   * @param edit falls true werden Menus & Buttons für "Zeilenumbruch" und "Fett markieren" aktiviert
   */
  private void setEdit(boolean edit)
  {
    this.edit = edit;

    mnuBr.setEnabled(edit && verifyText);
    mnuB.setEnabled(edit && verifyText);
    btnB.setEnabled(edit && verifyText);
    btnBr.setEnabled(edit && verifyText);

  }

  /**
   * Falls Enter gedrückt wird, wird im Text ein Zeilenumbruch mit <br> markiert.
   * Ein Tabulator kann nicht eingegeben werden
   * @param e KeyEvent
   */
  private void txpText_keyPressed(KeyEvent e)
  {
    if (verifyText)
    {
      switch (e.getKeyCode())
      {
        case KeyEvent.VK_ENTER:
          e.consume();
          txpText.replaceSelection("<br>");
          break;
        case KeyEvent.VK_TAB:
          e.consume();
          break;
      }
    }
  }

  /**
   * OK wurde geklickt. Falls verifyText auf true,
   * wird der Text auf Platzhalter und Escape-Sequenzen überprüft
   * zusätzlich wird auf Vollständigkeit der Kombination <b>..<\b> überprüfen.
   * @param e Auslösendes Event
   */
  protected void mnuOk_actionPerformed(ActionEvent e)
  {
    if (verifyText)
    {
      boolean ok = txpText.getInputVerifier().verify( txpText );
      if (!ok) return;

      TextVerifier textVerifier = new TextVerifier();
      int pos = 0;

      try
      {
        pos = textVerifier.verifyTextMarks(txpText.getText());
        if (pos != -1)
        {
          JOptionPane.showMessageDialog
          (this.getRootPane(), textVerifier.getMessage(), "Ungültige Eingabe", JOptionPane.INFORMATION_MESSAGE);

          txpText.requestFocus();
          txpText.select(pos, txpText.getText().indexOf(">", pos)+1);
          return;
        }

        pos = textVerifier.verifyPlatzhalter(txpText.getText());
        if (pos != -1)
        {
          String message  = "Es sind nur die in den Leitlinien definierten Platzhalter erlaubt";
          JOptionPane.showMessageDialog
          (this.getRootPane(), message, "Ungültiger Platzhalter", JOptionPane.INFORMATION_MESSAGE);

          txpText.requestFocus();
          txpText.select(pos, pos + 4);
          return;
        }

        pos = textVerifier.verifyEscapeSequenzen(txpText.getText());
        if (pos != -1)
        {
          String message  = "Folgende Zeichen sind im Text nicht erlaubt:" +
                            "\n  Zeilenumbruch(Enter), Tabulator," +
                            "\n  Carriage Return, Rückschritt," +
                            "\n  Zeilenvorschub";
          JOptionPane.showMessageDialog
          (this.getRootPane(), message, "Ungültiges Eingabe", JOptionPane.INFORMATION_MESSAGE);

          txpText.requestFocus();
          txpText.setCaretPosition(pos);
          return;
        }
      }
      catch (Exception ex)
      {
        String message = "Unerwarteter Fehler ist aufgetreten, Text kann nicht übernommen werden";
        JOptionPane.showMessageDialog
          (this.getRootPane(), message, "Ungültige Eingabe", JOptionPane.INFORMATION_MESSAGE);
        return;
      }
    }
    if (edit)
    {
      text = txpText.getText();
    }

    this.setVisible(false);
    this.dispose();
  }

  /**
   * Text wird auf gueltigen Zeichensatz ueberprueft
   * @param text String
   * @return boolean
   */
  private boolean checkCharset(String text)
  {
    boolean ok = false;
    CharBuffer chars = CharBuffer.wrap( text );

    try
    {
      encoder.encode( chars );
      ok = true;
    }
    catch( CharacterCodingException ex1 )
    {
      String message  = "Text beinhaltet ungueltiges Zeichen";
      JOptionPane.showMessageDialog
      (this.getRootPane(), message, "Ungueltiges Zeichen", JOptionPane.INFORMATION_MESSAGE);

      txpText.requestFocus();
      txpText.select(chars.position(), chars.position() + 1);
    }
    return ok;
  }

  /**
   * Infofenster anzeigen
   * @param e auslösendes Ereignis
   */
  protected void mnuInfo_actionPerformed(ActionEvent e)
  {
    new CustomDlgInfo( ( JFrame )this.getOwner(), true ).setVisible( true );
  }

  /**
   * Zeilenumbruch einfügen mit dem Zeichen <br>
   * @param e auslösendes Ereignis
   */
  private void mnuBr_actionPerformed(ActionEvent e)
  {
    txpText.replaceSelection("<br>");
  }

  /**
   * Text als Fett markieren mit dem Zeichen <b>...</b>
   * @param e auslösendes Ereignis
   */
  private void mnuB_actionPerformed(ActionEvent e)
  {
    if (txpText.getSelectedText() != null)
    {
      txpText.replaceSelection("<b>" + txpText.getSelectedText() + "</b>");
    }
  }
}


