package ch.ige.art.client.art;

import java.text.SimpleDateFormat;

import javax.swing.table.AbstractTableModel;

import ch.ige.art.util.XxDomain;
import ch.ige.art.util.XxDomainFactory;
import ch.ige.art.vo.ArtpreisVO;

/**
 * <p>Überschrift: Table Model für die Preisinformationen</p>
 * <p>Beschreibung: Ist für die Datenhaltung der Preise im Anzeigemodus verantwortlich</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: AtmArtPreise.java,v 1.11 2008/07/02 11:00:20 heinzer Exp $
 */

public class AtmArtPreise extends AbstractTableModel
{
  private final String[] columnNames = new String[] { "Preisart", "Gültig ab", "Preis", "Pro" };

  private ArtpreisVO[] tableData = new ArtpreisVO[] {};

  public AtmArtPreise()
  {
  }

  /**
   * Gibt Anzahl Spalten zurück
   * @return Anzahl Spalten
   */
  public int getColumnCount()
  {
    return columnNames.length;
  }

  /**
   * Gibt Anzahl Zeilen zurück
   * @return Anzahl Zeilen
   */
  public int getRowCount()
  {
    return tableData.length;
  }

  /**
   * Gibt Spaltenüberschrift der übergebenen Spalte zurück
   * @param column Spalte deren Spaltenüberschrift angefordert wird
   * @return Spaltenüberschrift
   */
  public String getColumnName( int column )
  {
    return columnNames[column];
  }

  /**
   * Gibt den Wert einer Zelle zurück
   * @param row Zeile
   * @param col Spalte
   * @return Wert der entsprechenden Zelle
   */
  public Object getValueAt( int row, int col )
  {
    switch( col )
    {
      case 0:
        String bezArtpreisartCd = XxDomain.getBez( tableData[row].artpreisartCd, XxDomainFactory.PREISART_CD );
        return ( tableData[row].artpreisartCd + ", " + bezArtpreisartCd );
      case 1:
        return new SimpleDateFormat( "dd.MM.yyyy" ).format( tableData[row].gueltigAb );
      case 2:
        return tableData[row].preis;
      case 3:
        return tableData[row].pro;
    }
    return null;
  }

  /**
   * Setzt die Daten ins TableModel
   * @param data Alle Preise eines Artikels gekapselt in einem ArtpreisVO
   */
  public void setData( ArtpreisVO[] data )
  {
    tableData = data;
    fireTableDataChanged();
  }

  /**
   * Daten in TableModel löschen
   */
  public void clear()
  {
    fireTableDataChanged();
  }
}
