package ch.ige.art.client.art;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import ch.ige.art.client.util.ComboBoxObject;
import ch.ige.art.client.util.DtmArtikelWizard;
import ch.ige.art.util.XxDomain;
import ch.ige.art.vo.ArtextVO;

/**
 * <p>Überschrift: DefaultTableModel der Texte</p>
 * <p>Beschreibung: Ist für die Datenhaltung der ArtikelTexte verantwortlich</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: DtmArtTexte.java,v 1.15 2008/07/02 11:00:20 heinzer Exp $
 */

public class DtmArtTexte extends DtmArtikelWizard
{
  private ArrayList alDelArtextVO = new ArrayList(); // zu löschende Rows

  /**
   * xxDomainWerte werden geladen und Spaltenüberschriften werden gesetzt
   * @throws Exception falls Domainwerte nicht geladen werden können
   */
  public DtmArtTexte() throws Exception
  {
    super();
    columnIdentifiers.addElement( "Textart" );
    columnIdentifiers.addElement( "Sprache" );
    columnIdentifiers.addElement( "Text" );
    columnIdentifiers.addElement( "Status" );
    columnIdentifiers.addElement( "Version" );
    columnIdentifiers.addElement( "" ); // Editierbutton
  }

  /**
   * Editierbarkeit der persistenten Daten verhindern,
   * die Bestandteil des PrimaryKeys sind
   *
   * @param row selektierte Zeile
   * @param col selektierte Spalte
   * @return true falls Zellen editierbar sind
   */
  public boolean isCellEditable( int row, int col )
  {
    if ( ( col < 2 && row < getPersistentRows() ) || col == 2 )
    {
      return false;
    }
    else
    {
      return true;
    }
  }

  /**
   * Mutierte persistente Zeilen werden in der Spalte Status mit einem
   * "U" = Update gekennzeichnet, nachdem der mutierte Wert in die Zelle geschrieben wurde
   *
   * @param value Wert
   * @param row   Zeile
   * @param col   Spalte
   */
  public void setValueAt( Object value, int row, int col )
  {
    super.setValueAt( value, row, col );

    if ( row < getPersistentRows() )
    {
      super.setValueAt( "U", row, 3 );
    }
  }

  /**
   * Tabellendaten werden gesetzt.<br>
   * Daten werden in Form eines ValueObjects übergeben.
   * Daten des VOs werden augesplittet und in den Datenhaltungsvector(dataVector)
   * des Tabllenmodels geschrieben.
   * Anzahl Rows die in der Datenbank gespeichert sind (status "" oder "U") werden gesetzt
   *
   * @param data Alle Texte eines Artikels
   */
  public void setData( ArtextVO[] data )
  {
    dataVector.clear();
    int persitentRows = 0;

    if ( data != null )
    {
      for ( int i = 0; i < data.length; i++ )
      {
        ArtextVO artextVO = data[i];

        if ( !artextVO.status.equals( "D" ) && artextVO.isValidTextart() )
        {
          Vector artext = new Vector();

          artext.add( new ComboBoxObject( artextVO.artTextArtCd, XxDomain.getBez( "arttextart_cd", artextVO.artTextArtCd ) ) );
          artext.add( new ComboBoxObject( artextVO.sprachCd, XxDomain.getBez( "sprache", artextVO.sprachCd ) ) );
          artext.add( artextVO.text == null ? "" : artextVO.text );
          artext.add( artextVO.status );
          //artext.add(new Integer(artextVO.version));
          artext.add( artextVO.uVersion );
          artext.add( "..." );

          dataVector.add( artext );

          // Anzahl Rows ermitteln die persistent sind
          if ( artextVO.status.equals( "U" ) || artextVO.status.equals( "" ) )
          {
            persitentRows++;
          }
        }
      }

      setPersistentRows( persitentRows );
      fireTableDataChanged();
    }
  }

  /**
   * Gibt die Tabellendaten in Form einer ArrayList von ArtextVOs zurück
   *
   * @return Tabellendaten gekapselt in einer ArrayList von ArtextVOs
   */
  public ArtextVO[] getData()
  {
    List list = new ArrayList();

    for ( int j = 0; j < dataVector.size(); j++ )
    {
      Vector vec = (Vector) dataVector.get( j );
      ArtextVO vo = new ArtextVO();

      ComboBoxObject cboTx = (ComboBoxObject) vec.get( 0 );
      vo.artTextArtCd = cboTx.key == null ? "" : cboTx.key;
      ComboBoxObject cboSp = (ComboBoxObject) vec.get( 1 );
      vo.sprachCd = cboSp.key == null ? "" : cboSp.key;
      vo.text = ( (String) vec.get( 2 ) ).trim().equals( "" ) ? null : (String) vec.get( 2 );
      vo.status = (String) vec.get( 3 );
      vo.uVersion = ( (String) vec.get( 4 ) );

      list.add( vo );
    }

    // zu löschende Rows dem Array hinzufügen, sofern nicht enthalten
    // im server-teil werden alle
    for ( int i = 0; i < alDelArtextVO.size(); i++ )
    {
      ArtextVO vo = (ArtextVO) alDelArtextVO.get( i );
      list.add( vo );
    }

    if ( !list.isEmpty() )
    {
      return (ArtextVO[]) list.toArray( new ArtextVO[list.size()] );
    }
    else
    {
      return null;
    }
  }

  /**
   * Für eine neue Zeile werden die Initialwerte pro Zelle gesetzt
   * Spalte Status wird mit I=Insert initialisiert
   *
   * @return gibt eine neue Zeile mit den initialisierten Zellen zurück
   */
  public Vector getNewRow()
  {
    Vector vec = new Vector();
    vec.setSize( 5 );
    vec.add( 0, new ComboBoxObject( "REG", XxDomain.getBez( "arttextart_cd", "REG" ) ) );
    vec.add( 1, new ComboBoxObject( "D", XxDomain.getBez( "sprache", "D" ) ) );
    vec.add( 2, "" );
    vec.add( 3, "I" );
    vec.add( 4, "!" );
    vec.add( 5, "..." );
    return vec;
  }

  /**
   * Speichert die Zeile die gelöscht werden soll in einem Vector, damit
   * die Daten noch vorhanden sind wenn sie zu einem späteren Zeitpunkt
   * in der Datenbank gelöscht werden wollen
   *
   * @param iRow Index der Zeile die gelöscht werden soll
   */
  public void saveDeletedRow( int iRow )
  {
    ArtextVO artextVO = new ArtextVO();

    ComboBoxObject cboTx = (ComboBoxObject) getValueAt( iRow, 0 );
    artextVO.artTextArtCd = cboTx.key == null ? "" : cboTx.key;
    ComboBoxObject cboSp = (ComboBoxObject) getValueAt( iRow, 1 );
    artextVO.sprachCd = cboSp.key == null ? "" : cboSp.key;

    artextVO.text = (String) getValueAt( iRow, 2 );
    artextVO.status = "D";
    artextVO.uVersion = (String) getValueAt( iRow, 4 );
    alDelArtextVO.add( artextVO );
  }
}
