package ch.ige.art.client.form;

import java.util.ArrayList;
import java.util.Vector;

import ch.ige.art.client.util.ComboBoxObject;
import ch.ige.art.client.util.DtmArtikelWizard;
import ch.ige.art.util.XxDomain;
import ch.ige.art.vo.BformtxtDruckVO;

/**
 * <p>Überschrift: DefaultTableModel der Formulartexte</p>
 * <p>Beschreibung: Ist für die Datenhaltung der DtmFormtexte verantwortlich</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: DtmFormTexte.java,v 1.11 2008/07/02 11:00:20 heinzer Exp $
 */

public class DtmFormTexte extends DtmArtikelWizard
{
  private ArrayList alDelBformtxtDruckVO = new ArrayList(); // zu löschende Rows

  /**
   * xxDomainWerte werden geladen und Spaltenüberschriften werden gesetzt
   * @throws Exception falls Domainwerte nicht geladen werden können
   */
  public DtmFormTexte() throws Exception
  {
    super();
    columnIdentifiers.addElement( "Textart" );
    columnIdentifiers.addElement( "Sprache" );
    columnIdentifiers.addElement( "Text" );
    columnIdentifiers.addElement( "Status" );
    columnIdentifiers.addElement( "Version" );
    columnIdentifiers.addElement( "" ); // Editierbutton
  }

  /**
   * Editierbarkeit der persistenten Daten verhindern,
   * die Bestandteil des PrimaryKeys sind
   *
   * @param row selektierte Zeile
   * @param col selektierte Spalte
   * @return true falls Zellen editierbar sind
   */
  public boolean isCellEditable( int row, int col )
  {
    if ( ( col < 2 && row < getPersistentRows() ) || col == 2 )
    {
      return false;
    }
    else
    {
      return true;
    }
  }

  /**
   * Mutierte persistente Zeilen werden in der Spalte Status mit einem
   * "U" = Update gekennzeichnet, nachdem der mutierte Wert in die Zelle geschrieben wurde
   *
   * @param value Wert
   * @param row   Zeile
   * @param col   Spalte
   */
  public void setValueAt( Object value, int row, int col )
  {
    super.setValueAt( value, row, col );

    if ( row < getPersistentRows() )
    {
      super.setValueAt( "U", row, 3 );
    }
  }

  /**
   * Tabellendaten werden gesetzt.<br>
   * Daten werden in Form eines ValueObjects übergeben.
   * Daten des VOs werden augesplittet und in den Datenhaltungsvector(dataVector)
   * des Tabllenmodels geschrieben.
   * Anzahl Rows die in der Datenbank gespeichert sind (status "" oder "U") werden gesetzt
   *
   * @param data Alle Texte eines Artikels
   */
  public void setData( BformtxtDruckVO[] data )
  {
    dataVector.clear();
    int persitentRows = 0;

    if ( data != null )
    {
      for ( int i = 0; i < data.length; i++ )
      {
        BformtxtDruckVO bformtxtDruckVO = data[i];

        if ( !bformtxtDruckVO.status.equals( "D" ) )
        {
          Vector bformtxtDruck = new Vector();

          bformtxtDruck.add( new ComboBoxObject( bformtxtDruckVO.textart, XxDomain.getBez( new Integer( bformtxtDruckVO.textart ).intValue(), "textart" ) ) );
          bformtxtDruck.add( new ComboBoxObject( bformtxtDruckVO.sprachCd.trim(), XxDomain.getBez( "sprache", bformtxtDruckVO.sprachCd.toString().trim() ) ) );

          bformtxtDruck.add( bformtxtDruckVO.text == null ? "" : bformtxtDruckVO.text );
          bformtxtDruck.add( bformtxtDruckVO.status );
          bformtxtDruck.add( new Integer( bformtxtDruckVO.version ) );
          bformtxtDruck.add( "..." );

          dataVector.add( bformtxtDruck );
        }

        // Anzahl Rows ermitteln die persistent sind
        if ( bformtxtDruckVO.status.equals( "U" ) || bformtxtDruckVO.status.equals( "" ) )
        {
          persitentRows++;
        }
      }
      setPersistentRows( persitentRows );
      fireTableDataChanged();
    }
  }

  /**
   * Gibt die Tabellendaten in Form einer ArrayList von BformtxtDruck zurück
   *
   * @return Tabellendaten gekapselt in einer ArrayList von BformtxtDruck
   */
  public BformtxtDruckVO[] getData()
  {
    int size = dataVector.size() + alDelBformtxtDruckVO.size();

    if ( size > 0 )
    {
      BformtxtDruckVO[] arBformtxtDruckVO = new BformtxtDruckVO[size];
      int i = 0;

      for ( i = 0; i < dataVector.size(); i++ )
      {
        Vector vec = new Vector();

        arBformtxtDruckVO[i] = new BformtxtDruckVO();
        vec = (Vector) dataVector.get( i );

        ComboBoxObject cboTx = (ComboBoxObject) vec.get( 0 );
        arBformtxtDruckVO[i].textart = cboTx.key == null ? "" : cboTx.key;
        ComboBoxObject cboSp = (ComboBoxObject) vec.get( 1 );
        arBformtxtDruckVO[i].sprachCd = cboSp.key == null ? "" : cboSp.key;
        arBformtxtDruckVO[i].text = ( (String) vec.get( 2 ) ).trim().equals( "" ) ? null : (String) vec.get( 2 );
        arBformtxtDruckVO[i].status = (String) vec.get( 3 );
        arBformtxtDruckVO[i].version = ( (Integer) vec.get( 4 ) ).intValue();

      }
      // zu löschende Rows dem Array hinzufügen
      for ( int j = 0; j < alDelBformtxtDruckVO.size(); j++, i++ )
      {
        arBformtxtDruckVO[i] = (BformtxtDruckVO) alDelBformtxtDruckVO.get( j );
      }
      return arBformtxtDruckVO;
    }
    return null;
  }

  /**
   * Für eine neue Zeile werden die Initialwerte pro Zelle gesetzt
   * Spalte Status wird mit I=Insert initialisiert
   *
   * @return gibt eine neue Zeile mit den initialisierten Zellen zurück
   */
  public Vector getNewRow()
  {
    Vector vec = new Vector();
    vec.setSize( 5 );
    vec.add( 0, new ComboBoxObject( "0", XxDomain.getBez( 0, "textart" ) ) );
    vec.add( 1, new ComboBoxObject( "D", XxDomain.getBez( "sprache", "D" ) ) );
    vec.add( 2, "" );
    vec.add( 3, "I" );
    vec.add( 4, new Integer( 0 ) );
    vec.add( 5, "..." );
    return vec;
  }

  /**
   * Speichert die Zeile die gelöscht werden soll in einem Vector, damit
   * die Daten noch vorhanden sind wenn sie zu einem späteren Zeitpunkt
   * in der Datenbank gelöscht werden wollen
   *
   * @param iRow Index der Zeile die gelöscht werden soll
   */
  public void saveDeletedRow( int iRow )
  {
    BformtxtDruckVO bformtxtDruckVO = new BformtxtDruckVO();

    ComboBoxObject cboTx = (ComboBoxObject) getValueAt( iRow, 0 );
    bformtxtDruckVO.textart = cboTx.key == null ? "" : cboTx.key;
    ComboBoxObject cboSp = (ComboBoxObject) getValueAt( iRow, 1 );
    bformtxtDruckVO.sprachCd = cboSp.key == null ? "" : cboSp.key;

    bformtxtDruckVO.text = (String) getValueAt( iRow, 2 );
    bformtxtDruckVO.status = "D";
    bformtxtDruckVO.version = ( (Integer) getValueAt( iRow, 4 ) ).intValue();

    alDelBformtxtDruckVO.add( bformtxtDruckVO );
  }
}
