package ch.ige.art.client.form;

import java.awt.*;
import java.util.Observer;
import javax.swing.JOptionPane;

import ch.ige.art.client.util.*;
import ch.ige.art.server.ArtikelDelegate;
import ch.ige.main.client.*;
import ch.ige.art.vo.BformDruckVO;

import org.apache.log4j.Logger;


/**
 * <p>Überschrift: Mediator für Formular-Wizard</p>
 * <p>Beschreibung: Der FormWizardMediator steuert die Anzeige der einzelnen Panels
 * beim Erfass-/Bearbeitungsprozess eines Formulars
 * Die Attribute der Oberklasse alPanel, maxPanel und title werden gesetzt
 * und die Methoden back(), cancel(), forward und finish() überschrieben.
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: FormWizardMediator.java,v 1.11 2006/12/13 10:08:31 ruegsegger Exp $
 */

public class FormWizardMediator extends WizardMediator implements Observer
{
  private BformDruckVO bformDruckVO = new BformDruckVO();
  protected boolean finish = false;
  private String formartCd = null;

  // Logging
  private static final Logger LOG = Logger.getLogger(FormWizardMediator.class);
  private UserMessage userMessage = new UserMessage();

  private PnlFormGrunddaten pnlFormGrunddaten = new PnlFormGrunddaten();

  /**
   * Setzt Position und Grösse des Wizards<br>
   * @param parent Frame
   */
  public FormWizardMediator(Frame parent)
  {
    super(parent);
    initializePanel();
  }

  /**
   * Initialisiert Panel<br>
   */
  private void initializePanel()
  {
    alPanel.add(new WizardMediator.WizardPanel(pnlFormGrunddaten, true));
    maxPanel = alPanel.size();
    title = "Formulardaten";
  }

   /**
   * Formular erstellen
   * @return formartCd falls Fertigstellen ok
   *         null falls Erstellung abgebrochen
   */
  public String newForm()
  {
    bformDruckVO = new BformDruckVO();
    bformDruckVO.bformartVO.status = "I";

    pnlFormGrunddaten.setData(bformDruckVO);
    init();
    return finish ? formartCd : null;
  }

  /**
   * Formular duplizieren
   * Bei Duplikat formartCd löschen und Stati der Formulartexte auf "I" setzen
   * Version von Formulartexten und Formular auf 0 setzen
   * @param formartCd Formular welches dupliziert werden soll
   * @return formartCd falls Fertigstellen ok
   *         null falls Erstellung abgebrochen
   */
  public String dupliForm(String formartCd)
  {
    read(formartCd);
    bformDruckVO.bformartVO.status = "I";
    bformDruckVO.bformartVO.formartCd = null;
    bformDruckVO.bformartVO.version = 0;
    bformDruckVO.bformartVO.uVersion = "!";
    if(bformDruckVO.bformtxtDruckVO != null)
    {
      for (int i = 0; i < bformDruckVO.bformtxtDruckVO.length; i++)
      {
        bformDruckVO.bformtxtDruckVO[i].status = "I";
        bformDruckVO.bformtxtDruckVO[i].version = 0;
      }
    }

    pnlFormGrunddaten.setData(bformDruckVO);
    init();
    return finish ? this.formartCd : null;  // this.formartCd sonst wird Parameter zurückgegeben
  }

  /**
   * Datensatz bearbeiten
   * @param formartCd Formular das bearbeitet werden soll
   * @return  true falls Fertigstellen ok
   *          false wenn Mutation abgebrochen
   */
  public boolean editForm(String formartCd)
  {
    read(formartCd);
    bformDruckVO.bformartVO.status = "U";

    pnlFormGrunddaten.setData(bformDruckVO);
    init();
    return finish;
  }

  /**
   * Formular lesen
   * @param formartCd Formular das gelesen werden soll
   */
  public void read(String formartCd)
  {
    try
    {
      if (formartCd != null)
      {
         bformDruckVO = ArtikelDelegate.getInstance().getFormular(formartCd.toString());
      }
    }
    catch (Exception ex)
    {
      LOG.info(ex.getClass() + " " + ex.getMessage());
      String message = userMessage.getUserMessage(ex);
      JOptionPane.showMessageDialog(null, message, "Fehlermeldung",
                                    JOptionPane.INFORMATION_MESSAGE);
    }
  }


 /**
  * Speichert Daten des Panels in VO und in Datenbank
  * @return true  - nächstes Panel anzeigen<br>
  *         false - auf aktuellem Panel bleiben, z.B. wegen Validierungsfehler
  */
  public boolean finish()
  {
    finish = false;
    String fehler = null;

    fehler = pnlFormGrunddaten.valid();   // Daten validieren

    if(fehler != null)
    {
      if(!fehler.equals(""))
      {
        JOptionPane.showMessageDialog(null, fehler, "Ungültige Eingabe",
                                      JOptionPane.INFORMATION_MESSAGE);
      }
      return finish = false;
    }


    //Speichern des Formulars in DB
    try
    {
      bformDruckVO = pnlFormGrunddaten.getData();   // Daten aus Panel holen
      if (bformDruckVO.bformartVO.status.equals("U"))
      {
        ArtikelDelegate.getInstance().setFormular(bformDruckVO);
        finish = true;
      }
      else
      {
        ArtikelDelegate.getInstance().createFormular(bformDruckVO);
        finish = true;
      }
      super.finish();
    }
    catch (Exception ex)
    {
      LOG.info(ex.getMessage());
      String message = userMessage.getUserMessage(ex);
      JOptionPane.showMessageDialog(null, message, "Information",
                                    JOptionPane.INFORMATION_MESSAGE);
    }
    this.formartCd = bformDruckVO.bformartVO.formartCd;
    return finish;
  }
}

