package ch.ige.art.client.util.editoren;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.table.*;

import ch.ige.art.client.*;
import ch.ige.art.client.art.*;
import ch.ige.art.client.form.*;
import ch.ige.art.client.util.*;
import ch.ige.authz.client.*;
import ch.ige.art.xml.*;


/**
 * <p>Überschrift: Editor für Zellen die einen Button darstellen </p>
 * <p>Beschreibung: Stellt eine Button Komponente zur Verfügung um die Zellwerte zu ändern,
 * ist zuständig für Reaktionen die vom Benutzer kommen</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: BtnEditor.java,v 1.15 2006/12/13 10:08:31 ruegsegger Exp $
 */
public class BtnEditor extends DefaultCellEditor
{
  private JButton   button;
  private String    label;
  private boolean   isPushed;
  private JTable    table;
  private DlgArtikelTexte editor = new DlgArtikelTexte();

  /**
   * Erstellt einen Button, der als Editor Objekt verwendet werden kann.<br>
   * Dem Button wird ein ActionListener hinzugefügt.<br>
   * Überschreibt Konstruktor der Superklasse
   * @param checkBox für Aufruf der Superklasse
   */
  public BtnEditor(JCheckBox checkBox)
  {
    super(checkBox);
    button = new JButton();
    button.setOpaque(true);
    button.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        fireEditingStopped();
      }
    });
  }

  /**
   * Legt die Darstellung im Editiermodus fest je nachdem ob die Zelle
   * selektiert ist oder nicht<br>
   * Überschreibt die Methode der Superklasse
   * @param table aktuelle Tabelle
   * @param value Zellenwert
   * @param isSelected true falls Zelle selektiert ist
   * @param row aktuelle Zeile
   * @param column aktuelle Spalte
   * @return Button als Editor
   */
  public Component getTableCellEditorComponent(JTable table,
                                               Object value,
                                               boolean isSelected,
                                               int row,
                                               int column)
  {
    this.table = table;

    if (isSelected)
    {
      button.setForeground(table.getSelectionForeground());
      button.setBackground(table.getSelectionBackground());
    }
    else
    {
      button.setForeground(table.getForeground());
      button.setBackground(table.getBackground());
    }

    label = (value == null) ? "" : value.toString();
    button.setText(label);
    isPushed = true;
    return button;
  }

  /**
   * Gibt den Wert des Editors zurück.<br>
   * Falls der Button gedrückt wurde, wird ein Dialogfenster geöffnet,
   * in das der Text aus nebenstehender Zelle (Spalte Text) eingelesen wird.
   * Nach Schliessen des Dialogs wird der überarbeitete Text neu in die nebenstehende Zelle gesetzt.<br>
   * Überschreibt die Methode der Superklasse.
   * @return Wert des Editors (Label des Buttons)
   */
  public Object getCellEditorValue()
  {
    ComboBoxObject cbo = null;

    if (isPushed)
    {
      boolean edit = true;          // falls true ist text bearbeitbar
      boolean verifyText = true;    // falls true wird text auf Escape-Sequenzen und Platzhalter überprüft
      Integer maxLength = null;

      TableModel tablemodel = table.getModel();

      if (tablemodel instanceof DtmArtTexte)
      {
        maxLength = new Integer(1884);
      }
      else if (tablemodel instanceof DtmFormTexte)
      {
        cbo = (ComboBoxObject)tablemodel.getValueAt(table.getSelectedRow(), 0);

        if (cbo.key.equals("0"))
        {
          maxLength = new Integer(40);
        }
        else
        {
          maxLength = new Integer(1981);
        }
      }

      String text = (String)tablemodel.getValueAt(table.getSelectedRow(), 2);
      String rtext = editor.showDlgText(text, edit, verifyText, maxLength);
      table.setValueAt(rtext, table.getSelectedRow(), 2);
    }
    isPushed = false;
    return label;
  }

  /**
   * Aufforderung an den Editor, die Bearbeitung abzubrechen.<br>
   * Falls die Bearbeitung nicht abgebrochen werden kann, behält die Zelle den Focus.<br>
   * Überschreibt die Methode der Superklasse.
   * @return  true wenn die Bearbeitung abgebrochen werden konnte
   */
  public boolean stopCellEditing()
  {
    isPushed = false;
    return super.stopCellEditing();
  }
}

