package ch.ige.art.client.util.renderer;

import java.awt.Component;
import javax.swing.JList;
import javax.swing.plaf.basic.BasicComboBoxRenderer;

import ch.ige.art.client.util.ComboBoxObject;


/**
 * <p>Überschrift: ComboRenderer</p>
 * <p>Beschreibung: Darstellungsobjekt für Comboboxen
 *                  Wird angewendet, sofern der ComboBox-Eintrag aus zwei Einträgen besteht,
 *                  die einzeln angesprochen werden können müssen</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: ComboRenderer.java,v 1.4 2006/12/13 10:08:31 ruegsegger Exp $
 *
 */

public class ComboRenderer extends BasicComboBoxRenderer
{

  public ComboRenderer()
  {
    this.setOpaque(true);
  }

	/**
	 * Überschreibt Methode in BasicComboBoxRenderer<br>
	 * Setzt Text und Tooltip für Comboboxeinträge<br>
	 * value[0] und value[1] werden in Combobox geschrieben<br>
	 * allfälliger Wert von value[2] wird als Tooltiptext gesetzt
	 *
	 * @param list          JList
	 * @param value         Object hier String Array
	 * @param index         Index
	 * @param isSelected    boolean, ob Zelle selektiert ist
	 * @param cellHasFocus  boolean, ob Zelle Fokus hat
	 * @return              Referenz auf sich selbst
	 */
  public Component getListCellRendererComponent( JList list,
                                                  Object value,
                                                  int index,
                                                  boolean isSelected,
                                                  boolean cellHasFocus )
  {
    ComboBoxObject cbo = (ComboBoxObject)value;

    if (isSelected)
    {
      setBackground(list.getSelectionBackground());
      setForeground(list.getSelectionForeground());
      if (-1 < index)
      {
        list.setToolTipText(cbo.tooltip);
      }
    }
    else
    {
      setBackground(list.getBackground());
      setForeground(list.getForeground());
    }

    setFont(list.getFont());
    setText(cbo.toString());
    return this;
  }
}

