package ch.ige.art.client.util.renderer;

import java.awt.Component;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import javax.swing.*;
import javax.swing.table.DefaultTableCellRenderer;


/**
 * <p>Überschrift: Darstellungsobjekt für Datumswerte</p>
 * <p>Beschreibung: Darstellungsobjekt um ein Datum innerhalb einer Tabelle darzustellen<br>
 *  Datum wird folgendermassen dargestellt: DD.MM.YYYY</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: TblCellDateRenderer.java,v 1.4 2006/12/13 10:08:31 ruegsegger Exp $
 */

public class TblCellDateRenderer extends DefaultTableCellRenderer
{
  private static SimpleDateFormat dateFormatter = new SimpleDateFormat("dd.MM.yyyy");
  private static SimpleDateFormat timeFormatter = new SimpleDateFormat("HH:mm");

  public TblCellDateRenderer()
  {
    super();
    setHorizontalAlignment(RIGHT);
  }

  /**
   * Gibt eine Referenz auf sich selber zurück. Bevor die Referenz zurückgegeben
   * wird, werden Darstellungen angepasst.<br>
   * Überschreibt die Methode der Superklasse
   *
   * @param table       aktuelle Tabelle
   * @param value       Wert der Zelle
   * @param isSelected  true wenn Zelle selektiert ist
   * @param hasFocus    true wenn Zelle den Focus hat
   * @param row         aktuelle Zeile
   * @param column      aktuelle Spalte
   * @return            Referenz auf sich selbst
   */
  public Component getTableCellRendererComponent( JTable table,
                                                  Object value,
                                                  boolean isSelected,
                                                  boolean isFocused,
                                                  int row,
                                                  int column)
  {
    Component component = super.getTableCellRendererComponent
    (table, value, isSelected, isFocused, row, column);
    if ( value != null )
    {
      if ( value instanceof java.sql.Date )
      {
        ((JLabel) component).setText (dateFormatter.format ((java.sql.Date) value));
      }
      else if ( value instanceof java.sql.Time )
      {
        ((JLabel) component).setText (timeFormatter.format ((java.sql.Time) value));
      }
      else if ( value instanceof java.util.Date )
      {
        ((JLabel) component).setText (dateFormatter.format ((java.util.Date) value));
      }
      else if ( value instanceof Calendar )
      {
        ((JLabel) component).setText (dateFormatter.format ( ((Calendar)value).getTime() ));
      }
      else
      {
        ((JLabel) component).setText (value.toString());
      }
    }
    else
    {
      ((JLabel) component).setText ("");
    }
    return component;
  }
}


