package ch.ige.art.client.util.verifier;

import java.text.SimpleDateFormat;
import java.util.Date;
import javax.swing.*;


/**
 * <p>Überschrift:  DateVerifier</p>
 * <p>Beschreibung: Überprüfung den Inhalte einer Komponenten.<br>
 * Beim Versuch die Komponente zu verlassen wird ein Ereigniss ausgelöst.<br>
 * Input muss ein gültiges Datum sein<br>
 * erlaubte Eingabeformate: DD.MM.YY und DD.MM.YYYY</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: DateVerifier.java,v 1.6 2006/12/13 10:08:31 ruegsegger Exp $
 */


public class DateVerifier extends InputVerifier
{

  /**
   *  Methode wird aufgerufen, falls der Benutzer versucht die Komponente zu verlassen<br>
   *  Methode verify(input) wird augerufen um den Input zu überprüfen
   *
   *  @param  input    JComponent   Komponente deren Inhalt überprüft werden soll
   *  @return true     - Komponente kann verlassen werden<br>
   *          false    - Focus bleibt in der Komponente, an den Benutzer wird eine Meldung ausgegeben
   */
  public boolean shouldYieldFocus( JComponent input )
  {
    boolean valid = super.shouldYieldFocus(input);
    if (!valid)
    {
      String message  = "Datum ist ungültig oder entspricht nicht dem erlaubten Eingabeformat" +
                        "\nerlaubte Eingabeformate:  DDMMYY,  DD.MM.YY,  DD.MM.YYYY";
      String title    = "Ungültiges Datum";
      input.setInputVerifier(null);
      JOptionPane.showMessageDialog
        (null, message, title, JOptionPane.INFORMATION_MESSAGE);
      input.setInputVerifier(this);
    }
    return valid;
  }

  /**
   *  Überprüft ob der Inhalt der Komponente gültig ist<br>
   *  Datum muss einen gültigen Wert haben
   *  @param  comp  JComponent   Komponente deren Inhalt überprüft werden soll
   *  @return true  - Input in Ordnung<br>
   *          false - Inhalt der Komponente nicht gültig
   */
  public boolean verify( JComponent comp )
  {
    SimpleDateFormat sdfShort = new SimpleDateFormat("ddMMyy");
    SimpleDateFormat sdfMiddle = new SimpleDateFormat("dd.MM.yy");
    SimpleDateFormat sdfLong = new SimpleDateFormat("dd.MM.yyyy");
    sdfShort.setLenient(false);   //striktes Format
    sdfMiddle.setLenient(false);
    sdfLong.setLenient(false);

    Date inpDate = new Date();
		String newContent;
    JTextField textfield = (JTextField)comp;
    String content = textfield.getText();

    if (content.length() > 0)
    {
      try
      {
        // Eingabeformat ddMMyy
        if (content.length() == 6)
        {
          inpDate = sdfShort.parse(content);

          // Überprüfen ob geparstes Datum == Eingabedatum
          newContent = sdfShort.format(inpDate);
          if (newContent.equals(content))
          {
            textfield.setText(sdfLong.format(inpDate));
          }
          else
          {
            return false;
          }
        }
        // Eingabeformat dd.MM.yy
        else if (content.length() == 8)
        {
          inpDate = sdfMiddle.parse(content);

          // Überprüfen ob geparstes Datum == Eingabedatum
          newContent = sdfMiddle.format(inpDate);
          if (newContent.equals(content))
          {
            textfield.setText(sdfLong.format(inpDate));
          }
          else
          {
            return false;
          }
        }
        // Eingabeformat dd.MM.yyyy
        else if (content.length() == 10)
        {
          inpDate = sdfLong.parse(content);

          // Überprüfen ob geparstes Datum == Eingabedatum
          newContent = sdfLong.format(inpDate);
          if (!newContent.equals(content))
          {
            return false;
          }
        }
        else return false;
      }
      catch (Exception ex)
      {
        return false;
      }
    }
    return true;
  }
}


