package ch.ige.art.server;

import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;

import javax.ejb.EJBException;

import org.apache.log4j.Logger;

import ch.ige.art.server.dao.AltArtDAO;
import ch.ige.art.server.dao.ArtbezDAO;
import ch.ige.art.server.dao.ArtextDAO;
import ch.ige.art.server.dao.ArtikelDAO;
import ch.ige.art.server.dao.ArtikelSucheDAO;
import ch.ige.art.server.dao.ArtpreisDAO;
import ch.ige.art.server.dao.BArtikelDAO;
import ch.ige.art.server.dao.BformartDAO;
import ch.ige.art.util.DomainJMSSender;
import ch.ige.art.vo.AltArtVO;
import ch.ige.art.vo.ArtbezVO;
import ch.ige.art.vo.ArtextVO;
import ch.ige.art.vo.ArtikelAVO;
import ch.ige.art.vo.ArtikelGuiAVO;
import ch.ige.art.vo.ArtikelSelKritVO;
import ch.ige.art.vo.ArtikelVO;
import ch.ige.art.vo.ArtpreisVO;
import ch.ige.art.vo.BArtikelVO;
import ch.ige.util.Crescendo;

/**
 * Business Object für Artikel
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: ArtBO.java,v 1.31 2008/07/02 07:19:59 heinzer Exp $
 */
public class ArtBO
{
  private static final Logger LOG = Logger.getLogger( ArtikelFacadeBean.class );

  private ArtikelVO artikelVO = new ArtikelVO();

  private BArtikelVO bArtikelVO = new BArtikelVO();

  private ArtbezVO artbezVO = new ArtbezVO();

  private ArtikelGuiAVO artikelGuiAVO = new ArtikelGuiAVO();

  private ArtextVO[] artextVO;

  private AltArtVO[] altArtVO;

  private ArtpreisVO[] artpreisVO;

  private ArtikelDAO artikelDAO = new ArtikelDAO();

  private BArtikelDAO bArtikelDAO = new BArtikelDAO();

  private ArtbezDAO artbezDAO = new ArtbezDAO();

  private AltArtDAO altArtDAO = new AltArtDAO();

  private ArtextDAO artextDAO = new ArtextDAO();

  private ArtpreisDAO artpreisDAO = new ArtpreisDAO();

  /**
   * Gibt das Value Object des Artikels zurück.
   *
   * @param   artNr          Artikelnummer
   * @return  ArtikelGuiAVO  Value Object des Artikels
   */
  public ArtikelGuiAVO getArtikel( String artNr )
  {
    try
    {
      ArrayList<ArtextVO> alArtext = new ArrayList<ArtextVO>();
      ArrayList<AltArtVO> alAltArt = new ArrayList<AltArtVO>();
      ArrayList<ArtpreisVO> alArtpreis = new ArrayList<ArtpreisVO>();

      //holen aller Daten eines Artikels
      artikelVO = artikelDAO.select( artNr );
      bArtikelVO = bArtikelDAO.select( artNr );
      artbezVO = artbezDAO.select( artNr );
      alArtext = artextDAO.select( artNr );
      artextVO = alArtext.toArray( new ArtextVO[0] );
      alAltArt = altArtDAO.select( artNr );
      altArtVO = alAltArt.toArray( new AltArtVO[0] );
      alArtpreis = artpreisDAO.select( artNr );
      artpreisVO = alArtpreis.toArray( new ArtpreisVO[0] );

      //Zusammensetzen des ArtikelGuiAVO
      artikelGuiAVO.artNr = artikelVO.artNr;
      artikelGuiAVO.artextVO = artextVO;
      artikelGuiAVO.altArtVO = altArtVO;
      artikelGuiAVO.artpreisVO = artpreisVO;
      if ( artbezVO != null )
      {
        artikelGuiAVO.bezArt = artbezVO.bezArt;
        artikelGuiAVO.artBezVersion = artbezVO.uVersion == null ? "!" : artbezVO.uVersion;
      }
      if ( artikelVO == null )
      {
        throw new EJBException( " | Kein Artikel mit dieser ArtikelNr vorhanden" + artikelGuiAVO.artNr + "|" );
      }
      artikelGuiAVO.altNrCd = artikelVO.altNrCd;
      artikelGuiAVO.artgrpCd = artikelVO.artgrpCd;
      artikelGuiAVO.bonusCd = artikelVO.bonusCd;
      artikelGuiAVO.kondartCd = artikelVO.kondartCd;
      artikelGuiAVO.kzGratis = artikelVO.kzGratis;
      artikelGuiAVO.kzHistDesign = artikelVO.kzHistDesign;

      artikelGuiAVO.kzMetnot = artikelVO.kzMetnot;
      artikelGuiAVO.lageinh = artikelVO.lageinh;
      artikelGuiAVO.minverkMenge = artikelVO.minverkMenge;
      artikelGuiAVO.optBestMenge = artikelVO.optBestMenge;
      artikelGuiAVO.provCd = artikelVO.provCd;
      artikelGuiAVO.sperrCd = artikelVO.sperrCd;
      artikelGuiAVO.statcd1 = artikelVO.statcd1;
      artikelGuiAVO.statcd2 = artikelVO.statcd2;
      artikelGuiAVO.statcd3 = artikelVO.statcd3;
      artikelGuiAVO.suchbegrArt = artikelVO.suchbegrArt;
      artikelGuiAVO.artikelVersion = artikelVO.uVersion == null ? "!" : artikelVO.uVersion;

      artikelGuiAVO.status = artikelVO.status;
      if ( bArtikelVO != null )
      {
        artikelGuiAVO.anzKopie = bArtikelVO.anzKopie;
        artikelGuiAVO.druckModus = bArtikelVO.druckModus;
        artikelGuiAVO.einzFaktpos = bArtikelVO.einzFaktpos;
        artikelGuiAVO.formartCd = bArtikelVO.formartCd;
        artikelGuiAVO.kzEinschr = bArtikelVO.kzEinschr;
        artikelGuiAVO.kzUntschr = bArtikelVO.kzUntschr;
        artikelGuiAVO.publPflicht = bArtikelVO.publPflicht;
        artikelGuiAVO.regauszGrp = bArtikelVO.regauszGrp;
        artikelGuiAVO.fristArt = bArtikelVO.fristArt;
        artikelGuiAVO.fristEinh = bArtikelVO.fristEinh;
        artikelGuiAVO.fristDauer = bArtikelVO.fristDauer;
        artikelGuiAVO.kzBesch = bArtikelVO.kzBesch;
        artikelGuiAVO.erfassProc = bArtikelVO.erfassProc;
        artikelGuiAVO.processModus = bArtikelVO.processModus;
        artikelGuiAVO.processNr = bArtikelVO.processNr;
      }
      return artikelGuiAVO;
    }
    catch ( SQLException ex )
    {
      LOG.error( new java.util.Date() + " " + this.getClass().toString() + ": " + "method getArtikel error: " + ex.getMessage() );
      throw new EJBException( " | Artikel kann nicht gelesen werden | " );
    }
  }

  /**
   * Gibt eine ArrayList der gefundenen Artikel zurück.
   *
   * @param   artikelSelKritVO VO mit Selektionskriterien
   * @return  ArrayList        ArrayList mit Value Objects 
   */
  public ArrayList<ArtikelAVO> findArtikel( ArtikelSelKritVO artikelSelKritVO )
  {
    try
    {
      return new ArtikelSucheDAO().select( artikelSelKritVO );
    }
    catch ( SQLException ex )
    {
      LOG.error( new java.util.Date() + " " + this.getClass().toString() + ": " + "method findArtikel error: " + ex.getMessage() );
      throw new EJBException( " | Suche kann nicht durchgeführt werden | " );
    }
  }

  public void setArtikel( ArtikelGuiAVO artikelGuiAVO, String domainMessage ) throws EJBException
  {
    setArtikel( artikelGuiAVO );
    if ( domainMessage != null )
    {
      updateDomain( domainMessage );
    }
  }

  /**
   * Aktualisiert den Artikel auf die Wertes des Value Objects.
   *
   * @param      artikelGuiAVO   Value Object eines Artikels
   * @exception  EJBException
   */
  private void setArtikel( ArtikelGuiAVO artikelGuiAVO )
  {
    try
    {
      // Checken der Integritätsbedingungen
      checkIntegrity( artikelGuiAVO );
      // Aufteilen des ArtikelGuiAVO auf die einzelnen VO
      divideValueObject( artikelGuiAVO );
      artikelVO.uVersion = Crescendo.getNextUVersion( artikelVO.uVersion );
      bArtikelVO.uVersion = Crescendo.getNextUVersion( bArtikelVO.uVersion );
      new ArtikelDAO().update( artikelVO );
      new BArtikelDAO().update( bArtikelVO );
      artbezVO.uVersion = Crescendo.getNextUVersion( artbezVO.uVersion );
      new ArtbezDAO().update( artbezVO );

      if ( artikelGuiAVO.altArtVO != null )
      {
        //Datensätze mit Status "D" müssen vor dem einfügen gelöscht werden,
        //da sonst die Integrietätsbedingungen der DB verletzt werden könnten
        for ( int i = 0; i < artikelGuiAVO.altArtVO.length; i++ )
        {
          artikelGuiAVO.altArtVO[i].artNr = artikelGuiAVO.artNr;
          artikelGuiAVO.altArtVO[i].uVersion = artikelGuiAVO.altArtVO[i].uVersion == null ? "!" : Crescendo
              .getNextUVersion( artikelGuiAVO.altArtVO[i].uVersion );
          if ( artikelGuiAVO.altArtVO[i].status.equals( "D" ) )
          {
            altArtDAO.delete( artikelGuiAVO.altArtVO[i].artNr, artikelGuiAVO.altArtVO[i].altArtNr );
          }
        }
        for ( int i = 0; i < artikelGuiAVO.altArtVO.length; i++ )
        {
          if ( artikelGuiAVO.altArtVO[i].status.equals( "U" ) )
          {
            artikelGuiAVO.altArtVO[i].geaendBen = artikelGuiAVO.userName;
            artikelGuiAVO.altArtVO[i].geaendDat = new java.util.Date();
            altArtDAO.update( artikelGuiAVO.altArtVO[i] );
          }
          if ( artikelGuiAVO.altArtVO[i].status.equals( "I" ) )
          {
            artikelGuiAVO.altArtVO[i].erstBen = artikelGuiAVO.userName;
            artikelGuiAVO.altArtVO[i].erstDat = new java.util.Date();
            artikelGuiAVO.altArtVO[i].uVersion = "!";
            altArtDAO.insert( artikelGuiAVO.altArtVO[i] );
          }
        }
      }
      //Löschen der artexte mit Status "D"
      if ( artikelGuiAVO.artextVO != null )
      {
        for ( int i = 0; i < artikelGuiAVO.artextVO.length; i++ )
        {
          artikelGuiAVO.artextVO[i].uVersion = artikelGuiAVO.artextVO[i].uVersion == null ? "!" : Crescendo
              .getNextUVersion( artikelGuiAVO.artextVO[i].uVersion );
          artikelGuiAVO.artextVO[i].artNr = artikelGuiAVO.artNr;
          if ( artikelGuiAVO.artextVO[i].status.equals( "D" ) )
          {
            artextDAO.delete( artikelGuiAVO.artextVO[i].artNr, artikelGuiAVO.artextVO[i].sprachCd, artikelGuiAVO.artextVO[i].artTextArtCd );
          }
        }
        //einfügen und aktualisieren
        for ( int i = 0; i < artikelGuiAVO.artextVO.length; i++ )
        {
          if ( artikelGuiAVO.artextVO[i].status.equals( "U" ) )
          {
            artikelGuiAVO.artextVO[i].geaendBen = artikelGuiAVO.userName;
            artikelGuiAVO.artextVO[i].geaendDat = new java.util.Date();
            artextDAO.update( artikelGuiAVO.artextVO[i] );
          }
          if ( artikelGuiAVO.artextVO[i].status.equals( "I" ) )
          {
            artikelGuiAVO.artextVO[i].erstBen = artikelGuiAVO.userName;
            artikelGuiAVO.artextVO[i].erstDat = new java.util.Date();
            artikelGuiAVO.artextVO[i].geaendBen = null;
            artikelGuiAVO.artextVO[i].geaendDat = null;
            artikelGuiAVO.artextVO[i].uVersion = "!";
            artextDAO.insert( artikelGuiAVO.artextVO[i] );
          }
        }
      }
      //Löschen der Artikelpreise mit Status "D"
      if ( artikelGuiAVO.artpreisVO != null )
      {
        for ( int i = 0; i < artikelGuiAVO.artpreisVO.length; i++ )
        {
          artikelGuiAVO.artpreisVO[i].artNr = artikelGuiAVO.artNr;
          artikelGuiAVO.artpreisVO[i].mengeinh = artikelGuiAVO.lageinh; // die mengeinh entspricht immer der lageinh
          artikelGuiAVO.artpreisVO[i].uVersion = artikelGuiAVO.artpreisVO[i].uVersion == null ? "!" : Crescendo
              .getNextUVersion( artikelGuiAVO.artpreisVO[i].uVersion );
          if ( artikelGuiAVO.artpreisVO[i].status.equals( "D" ) )
          {
            artpreisDAO.delete( artikelGuiAVO.artpreisVO[i].artNr, artikelGuiAVO.artpreisVO[i].mengeinh, artikelGuiAVO.artpreisVO[i].artpreisartCd,
                artikelGuiAVO.artpreisVO[i].gueltigAb );
          }
        }
        for ( int i = 0; i < artikelGuiAVO.artpreisVO.length; i++ )
        {
          if ( artikelGuiAVO.artpreisVO[i].status.equals( "U" ) )
          {
            artikelGuiAVO.artpreisVO[i].geaendBen = artikelGuiAVO.userName;
            artikelGuiAVO.artpreisVO[i].geaendDat = new java.util.Date();
            artpreisDAO.update( artikelGuiAVO.artpreisVO[i] );
          }
          if ( artikelGuiAVO.artpreisVO[i].status.equals( "I" ) )
          {
            artikelGuiAVO.artpreisVO[i].erstBen = artikelGuiAVO.userName;
            artikelGuiAVO.artpreisVO[i].erstDat = new java.util.Date();
            artikelGuiAVO.artpreisVO[i].uVersion = "!";
            artpreisDAO.insert( artikelGuiAVO.artpreisVO[i] );
          }
        }
      }
    }
    catch ( SQLException ex )
    {
      LOG.error( new java.util.Date() + " " + this.getClass().toString() + ": " + "method setArtikel error: " + ex.getMessage() );
      throw new EJBException( " | Artikel kann nicht gespeichert werden | " );
    }
  }

  public void createArtikel( ArtikelGuiAVO artikelGuiAVO, String domainMessage ) throws EJBException
  {
    createArtikel( artikelGuiAVO );
    if ( domainMessage != null )
    {
      updateDomain( domainMessage );
    }
  }

  /**
   * Kreiert einen Artikel
   *
   * @param      artikelGuiAVO Value Object des Artikels
   * @exception  EJBException
   */
  private void createArtikel( ArtikelGuiAVO artikelGuiAVO )
  {
    try
    {
      //artikelGuiAVO darf nicht null sein
      if ( artikelGuiAVO.artNr == null || artikelGuiAVO.artNr.equals( "" ) )
      {
        throw new EJBException( " | Keine Artikelnummer eingegeben! | " );
      }

      // Prüfen ob Adressnummer bereits existiert
      if ( artikelDAO.exists( artikelGuiAVO.artNr ) )
      {
        throw new EJBException( " | Artikelnummer " + artikelGuiAVO.artNr + " existiert bereits! | " );
      }

      // Checken der Integritätsbedingungen
      checkIntegrity( artikelGuiAVO );
      // Aufteilen des ArtikelGuiAVO auf einzelne VO
      divideValueObject( artikelGuiAVO );

      // Einfügen der VO in DB
      new ArtikelDAO().insert( artikelVO );
      new BArtikelDAO().insert( bArtikelVO );
      new ArtbezDAO().insert( artbezVO );
      if ( artikelGuiAVO.altArtVO != null )
      {
        for ( int i = 0; i < artikelGuiAVO.altArtVO.length; i++ )
        {
          artikelGuiAVO.altArtVO[i].erstBen = artikelGuiAVO.userName;
          artikelGuiAVO.altArtVO[i].erstDat = new java.util.Date();
          artikelGuiAVO.altArtVO[i].artNr = artikelGuiAVO.artNr;
          altArtDAO.insert( artikelGuiAVO.altArtVO[i] );
        }
      }
      if ( artikelGuiAVO.artextVO != null )
      {
        for ( int i = 0; i < artikelGuiAVO.artextVO.length; i++ )
        {
          artikelGuiAVO.artextVO[i].erstBen = artikelGuiAVO.userName;
          artikelGuiAVO.artextVO[i].erstDat = new java.util.Date();
          artikelGuiAVO.artextVO[i].geaendBen = null;
          artikelGuiAVO.artextVO[i].geaendDat = null;
          artikelGuiAVO.artextVO[i].artNr = artikelGuiAVO.artNr;
          artextDAO.insert( artikelGuiAVO.artextVO[i] );
        }
      }
      if ( artikelGuiAVO.artpreisVO != null )
      {
        for ( int i = 0; i < artikelGuiAVO.artpreisVO.length; i++ )
        {
          artikelGuiAVO.artpreisVO[i].erstBen = artikelGuiAVO.userName;
          artikelGuiAVO.artpreisVO[i].erstDat = new java.util.Date();
          artikelGuiAVO.artpreisVO[i].artNr = artikelGuiAVO.artNr;
          artikelGuiAVO.artpreisVO[i].mengeinh = artikelGuiAVO.lageinh;
          artpreisDAO.insert( artikelGuiAVO.artpreisVO[i] );
        }
      }
    }
    catch ( SQLException ex )
    {
      LOG.error( new java.util.Date() + " " + this.getClass().toString() + ": " + "method createArtikel error: " + ex.getMessage() );
      throw new EJBException( " | Artikel kann nicht erfasst werden | " + ex );
    }
  }

  /**
   * Prüfen der Integritätsbedingungen
   *
   * @param  artikelGuiAVO    ValueObject eines Artikels
   * @throws EJBException
   */
  private void checkIntegrity( ArtikelGuiAVO artikelGuiAVO ) throws EJBException
  {
    SimpleDateFormat sdfLong = new SimpleDateFormat( "dd.MM.yyyy" );
    String strDate1 = null;
    String strDate2 = null;
    sdfLong.setLenient( false );

    try
    {
      ArtikelDAO artikelDAO = new ArtikelDAO();

      // Formular muss existieren, sofern formart_cd erfasst wurde
      if ( artikelGuiAVO.formartCd != null && !artikelGuiAVO.formartCd.equals( "" ) )
      {
        BformartDAO bformartDAO = new BformartDAO();
        if ( bformartDAO.exists( artikelGuiAVO.formartCd ) == false )
        {
          throw new EJBException( " | Gewähltes Formular existiert nicht! | " );
        }
      }
      // Folgeereigniss muss existieren sofern altart_cd erfasst wurde
      if ( artikelGuiAVO.altArtVO != null )
      {
        for ( int i = 0; i < artikelGuiAVO.altArtVO.length; i++ )
        {
          if ( ( artikelGuiAVO.altArtVO[i].altArtNr != null ) && ( !artikelGuiAVO.altArtVO[i].status.equals( "D" ) ) )
          {
            if ( artikelDAO.exists( artikelGuiAVO.altArtVO[i].altArtNr ) == false )
            {
              throw new EJBException( " | Das Folgeereignis " + artikelGuiAVO.altArtVO[i].altArtNr + " existiert in der Datenbank nicht als Artikel | " );
            }
          }
        }
      }

      // Kombination Preisart und GueltigAb darf nicht doppelt vorkommen
      if ( artikelGuiAVO.artpreisVO != null )
      {
        for ( int i = 0; i < artikelGuiAVO.artpreisVO.length; i++ )
        {
          for ( int j = 0; j < artikelGuiAVO.artpreisVO.length; j++ )
          {
            strDate1 = sdfLong.format( artikelGuiAVO.artpreisVO[i].gueltigAb );
            strDate2 = sdfLong.format( artikelGuiAVO.artpreisVO[j].gueltigAb );
            if ( ( artikelGuiAVO.artpreisVO[i].artpreisartCd.equals( artikelGuiAVO.artpreisVO[j].artpreisartCd ) ) && ( strDate1.equals( strDate2 ) )
                && ( !artikelGuiAVO.artpreisVO[i].status.equals( "D" ) ) && ( j != i ) )
            {
              throw new EJBException( " | Kombination Preisart und GueltigAb darf nicht doppelt existieren | " );
            }
          }
        }
      }
      // Folgeartikel darf nicht zweimal die gleiche Artikelnummer haben
      if ( artikelGuiAVO.altArtVO != null )
      {
        for ( int i = 0; i < artikelGuiAVO.altArtVO.length; i++ )
        {
          for ( int j = 0; j < artikelGuiAVO.altArtVO.length; j++ )
          {
            if ( ( artikelGuiAVO.altArtVO[i].altArtNr.equals( artikelGuiAVO.altArtVO[j].altArtNr ) ) && ( !artikelGuiAVO.altArtVO[i].status.equals( "D" ) )
                && ( !artikelGuiAVO.altArtVO[j].status.equals( "D" ) ) && ( j != i ) )
            {
              throw new EJBException( " | Folgeartikel darf nicht mehrmals mit gleicher Artikelnummer erfasst werden | " );
            }
          }
        }
      }
      // Folgeartikel darf nicht auf sich selber verweisen
      if ( artikelGuiAVO.altArtVO != null )
      {
        for ( int i = 0; i < artikelGuiAVO.altArtVO.length; i++ )
        {
          if ( artikelGuiAVO.artNr.equals( artikelGuiAVO.altArtVO[i].altArtNr ) )
          {
            throw new EJBException( " | Ein Artikel darf nicht die eigene Artikelnummer \nals Folgeartikel enthalten | " );
          }
        }
      }
      // Kombination Textart und Sprache darf nicht doppelt vorkommen
      if ( artikelGuiAVO.artextVO != null )
      {
        for ( int i = 0; i < artikelGuiAVO.artextVO.length; i++ )
        {
          for ( int j = 0; j < artikelGuiAVO.artextVO.length; j++ )
          {
            if ( ( artikelGuiAVO.artextVO[i].artTextArtCd.equals( artikelGuiAVO.artextVO[j].artTextArtCd ) )
                && ( artikelGuiAVO.artextVO[i].sprachCd.equals( artikelGuiAVO.artextVO[j].sprachCd ) ) && ( !artikelGuiAVO.artextVO[i].status.equals( "D" ) )
                && ( !artikelGuiAVO.artextVO[j].status.equals( "D" ) ) && ( i != j ) )
            {
              throw new EJBException( " | Kombination Textart und Sprache darf nicht doppelt existieren | " );
            }
          }
        }
      }
    }
    catch ( SQLException ex )
    {
      throw new EJBException( " | Artikel kann nicht gespeichert werden | " );
    }
  }

  /**
   * Teilt das Value Object ArtikelGuiAVO auf die einzelnen VO auf
   *
   * @param artikelGuiAVO  ValueObject eines Artikels
   */
  private void divideValueObject( ArtikelGuiAVO artikelGuiAVO )
  {
    artikelVO.artNr = artikelGuiAVO.artNr;

    artbezVO.artNr = artikelGuiAVO.artNr;
    artbezVO.bezArt = artikelGuiAVO.bezArt;
    artbezVO.uVersion = artikelGuiAVO.artBezVersion == null ? "!" : artikelGuiAVO.artBezVersion;

    artikelVO.altNrCd = artikelGuiAVO.altNrCd;
    artikelVO.artgrpCd = artikelGuiAVO.artNr; // entpricht dem Wert von artNr
    artikelVO.bonusCd = artikelGuiAVO.bonusCd;
    artikelVO.kondartCd = artikelGuiAVO.kondartCd;
    artikelVO.kzGratis = artikelGuiAVO.kzGratis;
    artikelVO.kzHistDesign = artikelGuiAVO.kzHistDesign;

    artikelVO.kzMetnot = artikelGuiAVO.kzMetnot;
    artikelVO.lageinh = artikelGuiAVO.lageinh;
    artikelVO.minverkMenge = artikelGuiAVO.minverkMenge;
    artikelVO.optBestMenge = artikelGuiAVO.optBestMenge;
    artikelVO.provCd = "1";
    artikelVO.sperrCd = artikelGuiAVO.sperrCd;
    artikelVO.statcd1 = artikelGuiAVO.statcd1;
    artikelVO.statcd2 = artikelGuiAVO.statcd2;
    artikelVO.statcd3 = artikelGuiAVO.statcd3;
    artikelVO.suchbegrArt = artikelGuiAVO.suchbegrArt;
    artikelVO.uVersion = artikelGuiAVO.artikelVersion == null ? "!" : artikelGuiAVO.artikelVersion;
    artikelVO.status = artikelGuiAVO.status;
    //Erstell / Änderungs datum und user zuweisen
    if ( artikelGuiAVO.status.equals( "U" ) )
    {
      artikelVO.geaendBen = artikelGuiAVO.userName;
      artikelVO.geaendDat = new java.util.Date();
      bArtikelVO.geaendBen = artikelGuiAVO.userName;
      bArtikelVO.geaendDat = new java.util.Date();
      artbezVO.geaendBen = artikelGuiAVO.userName;
      artbezVO.geaendDat = new java.util.Date();
    }
    if ( artikelGuiAVO.status.equals( "I" ) )
    {
      artikelVO.erstBen = artikelGuiAVO.userName;
      artikelVO.erstDat = new java.util.Date();
      bArtikelVO.erstBen = artikelGuiAVO.userName;
      bArtikelVO.erstDat = new java.util.Date();
      artbezVO.erstBen = artikelGuiAVO.userName;
      artbezVO.erstDat = new java.util.Date();
    }

    bArtikelVO.artNr = artikelGuiAVO.artNr;
    bArtikelVO.anzKopie = artikelGuiAVO.anzKopie;
    bArtikelVO.druckModus = artikelGuiAVO.druckModus;
    bArtikelVO.einzFaktpos = artikelGuiAVO.einzFaktpos;
    bArtikelVO.formartCd = artikelGuiAVO.formartCd;
    bArtikelVO.kzEinschr = artikelGuiAVO.kzEinschr;
    bArtikelVO.kzUntschr = artikelGuiAVO.kzUntschr;
    bArtikelVO.publPflicht = artikelGuiAVO.publPflicht;
    bArtikelVO.regauszGrp = artikelGuiAVO.regauszGrp;
    bArtikelVO.fristArt = artikelGuiAVO.fristArt;
    bArtikelVO.fristEinh = artikelGuiAVO.fristEinh;
    bArtikelVO.fristDauer = artikelGuiAVO.fristDauer;
    bArtikelVO.kzBesch = artikelGuiAVO.kzBesch;
    bArtikelVO.erfassProc = artikelGuiAVO.erfassProc;
    bArtikelVO.processModus = artikelGuiAVO.processModus;
    bArtikelVO.processNr = artikelGuiAVO.processNr;

    artextVO = artikelGuiAVO.artextVO;
    artpreisVO = artikelGuiAVO.artpreisVO;
    altArtVO = artikelGuiAVO.altArtVO;
  }

  /**
   * Veranlasst Nachführung der Publikation-/Registertexte auf Swissret
   * @param domainMessage XML
   */
  public void updateDomain( String domainMessage )
  {
    try
    {
      LOG.debug( "send domainMessage: " + domainMessage );
      DomainJMSSender.submit( domainMessage );
    }
    catch ( Exception ex )
    {
      LOG.error( "exception in method updateDomain : " + ex.getMessage() );
      throw new EJBException( " | Artikel kann nicht gespeichert werden\n"
          + " Bei der Aktualisierung der Domain-Werte auf Swissreg2 ist ein Problem aufgetreten | " + ex.getMessage() );
    }
  }

}
