package ch.ige.art.server;

import java.util.ArrayList;

import ch.ige.art.vo.ArtikelGuiAVO;
import ch.ige.art.vo.ArtikelSelKritVO;
import ch.ige.art.vo.BformDruckVO;
import ch.ige.art.vo.MaVO;
import ch.ige.art.vo.XxDomainSelKritVO;
import ch.ige.art.vo.XxDomainVO;
import ch.ige.ses.client.saprequest.SapArtikel;
import ch.ige.ses.exception.SapCallException;
import ch.ige.util.ServiceLocator;

/**
 * BusinessDelegate für Artikel
 * 
 * TODO move to client part
 * 
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: ArtikelDelegate.java,v 1.33 2008/07/02 11:00:20 heinzer Exp $
 */

public class ArtikelDelegate
{
  private static ArtikelDelegate artikelDelegate;

  private ArtikelFacade facade;

  private static final Class HOME_CLASS = ArtikelFacadeHome.class;

  private ArtikelDelegate() throws Exception
  {
    ArtikelFacadeHome home = (ArtikelFacadeHome) ServiceLocator.getInstance().getHome( ArtikelFacadeHome.JNDI_NAME, HOME_CLASS );
    facade = home.create();
  }

  /**
   * Gibt die einzige Instanz zurück (Singleton)
   * 
   * @return ArtikelDelegate
   * @exception Exception
   */
  public static ArtikelDelegate getInstance() throws Exception
  {
    if ( artikelDelegate == null )
    {
      artikelDelegate = new ArtikelDelegate();
    }
    return artikelDelegate;
  }

  /**
   * Gibt das Value Object des Artikels zurück
   *
   * @param  artNr          ArtikelNr
   * @return ArtikelGuiAVO  Value Object des Artikels
   * @exception Exception
   */
  public ArtikelGuiAVO getArtikel( final String artNr ) throws Exception
  {
    return facade.getArtikel( artNr );
  }

  /**
   * Kreiert einen Artikel
   *
   * @param     artikelGuiAVO     Value Object eines Artikels
   * @exception Exception
   */
  public void createArtikel( final ArtikelGuiAVO artikelGuiAVO, final String domainMessage ) throws Exception
  {
    facade.createArtikel( artikelGuiAVO, domainMessage );
  }

  /**
   * Aktualisiert einen Artikel
   *
   * @param     artikelGuiAVO     Value Object eines Artikels
   * @exception Exception
   */
  public void setArtikel( final ArtikelGuiAVO artikelGuiAVO, final String domainMessage ) throws Exception
  {
    facade.setArtikel( artikelGuiAVO, domainMessage );
  }

  /**
   * Findet alle Artikel, auf welche die Selektionskriterien zutreffen.
   *
   * @param     artikelSelKritVO Value Object mit den Selektionskriterien
   * @return    ArrayList mit ArtikelAVO-Objekten mit den gefundenen Artikel
   * @exception Exception
   */
  public ArrayList findArtikel( final ArtikelSelKritVO artikelSelKritVO ) throws Exception
  {
    return (ArrayList) facade.findArtikel( artikelSelKritVO );
  }

  /**
   * Gibt das Value Object des Formulars zurück
   *
   * @param  formartCd          Formartcode
   * @return BformDruckVO  Value Object des Artikels
   * @exception Exception
   */
  public BformDruckVO getFormular( final String formartCd ) throws Exception
  {
    return facade.getFormular( formartCd );
  }

  /**
   * Kreiert ein Formular
   *
   * @param     bformDruckVO     Value Object eines Formular
   * @exception Exception
   */
  public void createFormular( final BformDruckVO bformDruckVO ) throws Exception
  {
    facade.createFormular( bformDruckVO );
  }

  /**
   * Aktualisiert ein Formular
   *
   * @param     bformDruckVO     Value Object eines Formulars
   * @exception Exception
   */
  public void setFormular( final BformDruckVO bformDruckVO ) throws Exception
  {
    facade.setFormular( bformDruckVO );
  }

  /**
   * Findet alle Formulare, auf welche die Selektionskriterien zutreffen.
   *
   * @param     formartCd    Formartcode
   * @param     bez          Bezeichnung
   * @param     druckSystem  druckSystem
   * @return    ArrayList mit BformDruckVO-Objekten mit den gefundenen Formularen
   * @exception Exception
   */
  public ArrayList findFormular( final String formartCd, final String bez, final String druckSystem ) throws Exception
  {
    return facade.findFormular( formartCd, bez, druckSystem );
  }

  /**
   * Gibt Mitarbeiter-Informationen (Bagisuser) zurück
   *
   * @return    MaVO Mitarbeiter-Informationen
   * @exception Exception
   */
  public MaVO getMa( final String vmsUsername ) throws Exception
  {
    return facade.getMa( vmsUsername );
  }

  /**
   * Findet alle Domains der Tabelle xx_domain
   *
   * @return    ArrayList mit XxDomainVOs mit allen Domains
   * @exception Exception
   */
  public ArrayList<XxDomainVO> findXxDomain() throws Exception
  {
    return facade.findXxDomain();
  }

  /**
   * Findet alle Domains, auf welche die Selektionskriterien zutreffen.
   *
   * @param     xxDomainSelKritVO ValueObject mit den Selektionskriterien
   * @return    ArrayList mit XxDomainVOs mit den gefundenen Domains
   * @exception Exception
   */
  public ArrayList findXxDomain( final XxDomainSelKritVO xxDomainSelKritVO ) throws Exception
  {
    return facade.findXxDomain( xxDomainSelKritVO );
  }

  /**
   * Prüft via JMS auf SAP ob der mitgegebene Artikel existiert
   * @param art_nr zu prüfende Artikelnummer
   * @return status (true = Artikel vorhanden, false = Artikel nicht vorhanden
   * @throws SapCallException
   */
  public boolean existenceCheck( String art_nr ) throws SapCallException
  {
    return new SapArtikel().existenceCheck( art_nr );
  }

  /**
   * Gibt die Meta-Daten von der Bagis-DB zurück.
   * @return String URL
   * @throws Exception Unerwarteter Fehlersituation
   */
  public String getBagisDBURL() throws Exception
  {
    return facade.getBagisDBURL();
  }
}
