package ch.ige.art.server.dao;

import java.sql.*;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für bformtxt
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: BformtxtDAO.java,v 1.8 2006/12/13 10:08:31 ruegsegger Exp $
 */

public class BformtxtDAO
{

  /**
   * Prüft die Existenz eines Formart Codes
   *
   * @param     formartCd     zu prüfender FormartCode
   * @return    true          formartCode existiert.
   * @exception SQLException  Unerwarteter SQL-Fehler
   */
  public boolean exists( String formartCd ) throws SQLException
  {
    if ( formartCd == null )
    {
      throw new IllegalArgumentException( "formartCode darf nicht null sein" );
    }
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query =  "SELECT trim(formart_cd) formart_cd FROM bformtxt WHERE formart_cd = RPAD(?, 3, ' ') ";
      ps = con.prepareStatement(query);
      ps.setString( 1, formartCd );
      rs = ps.executeQuery();
      return rs.next();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BformtxtDAO.class, query + formartCd);
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}

