package ch.ige.art.server.dao.test;

import java.sql.SQLException;
import java.util.List;

import ch.ige.art.server.dao.AltArtDAO;
import ch.ige.art.server.dao.ArtikelDAO;
import ch.ige.art.vo.AltArtVO;
import ch.ige.art.vo.ArtikelVO;
import ch.ige.art.vo.test.Testdata;

/**
 * Test der Klasse AltArtDAO
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: AltArtDAOTest.java,v 1.10 2008/06/25 13:47:29 ruegsegger Exp $
 */

public class AltArtDAOTest extends DAOTestCase
{
  private ArtikelDAO artikelDAO = new ArtikelDAO();

  private ArtikelVO artikelVO = null;

  private AltArtDAO altArtDAO = new AltArtDAO();

  private AltArtVO altArtVO = null;

  public AltArtDAOTest( String s )
  {
    super( s );
  }

  @Override
  protected void setUp()
  {
    artikelVO = Testdata.createArtikelVO();
    altArtVO = Testdata.createAltArtVO();

    try
    {
      artikelDAO.insert( artikelVO );
    }
    catch ( SQLException ex )
    {
      System.out.println( "set up failed: " + ex );
    }
  }

  @Override
  protected void tearDown()
  {
    try
    {
      artikelDAO.delete( artikelVO.artNr );
    }
    catch ( SQLException ex )
    {
      System.out.println( "tear down failed: " + ex );
    }
  }

  /**
   *  Fügt einen Datensatz in die  Tabelle alt_art ein
   */
  public void test01Insert()
  {
    System.out.println( "Test INSERT" );
    AltArtVO selectVO = null;

    try
    {
      altArtDAO.insert( altArtVO );

      // testen ob Datensatz eingefügt wurde
      selectVO = altArtDAO.select( altArtVO.artNr, altArtVO.altArtNr );
      if ( selectVO == null )
      {
        fail( "Datensatz existiert nicht" );
      }

      assertEquals( altArtVO.artNr, selectVO.artNr );
      assertEquals( altArtVO.altArtNr, selectVO.altArtNr );
      assertEquals( altArtVO.priorCd, selectVO.priorCd );
      assertEquals( null, selectVO.erstDat );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Selektiert einen Datensatz aus der Tabelle alt_art
   */
  public void test02Select()
  {
    System.out.println( "Test SELECT " );
    AltArtVO selectVO = null;

    try
    {
      altArtDAO.insert( altArtVO );
      selectVO = altArtDAO.select( altArtVO.artNr, altArtVO.altArtNr );

      if ( selectVO == null )
      {
        fail( "Datensatz existiert nicht" );
      }

      assertEquals( altArtVO.artNr, selectVO.artNr );
      assertEquals( altArtVO.altArtNr, selectVO.altArtNr );
      assertEquals( altArtVO.priorCd, selectVO.priorCd );
      assertEquals( null, selectVO.erstDat );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Selektiert mehrere Datensätze aus der Tabelle alt_art
   */
  public void test03SelectAll()
  {
    System.out.println( "Test SELECT mehrere VO" );
    List<AltArtVO> list = Testdata.createListAltArtVO( ( altArtVO.artNr       ) );
    AltArtVO selectVO = null;
    List<AltArtVO> selectList = null;

    try
    {
      for ( int i = 0; i < list.size(); i++ )
      {
        selectVO = list.get( i );
        altArtDAO.insert( selectVO );
      }

      selectList = altArtDAO.select( altArtVO.artNr );
      selectVO = list.get( 0 );

      assertTrue( "Anzahl = ", selectList.size() == 2 );
      assertEquals( ( altArtVO.artNr       ), selectVO.artNr );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Aktualisiert einen Datensatz
   */
  public void test04Update()
  {
    System.out.println( "Test UPDATE " );
    AltArtVO selectVO = null;
    AltArtVO updateVO = null;

    try
    {
      // set up
      altArtDAO.insert( altArtVO );

      updateVO = altArtDAO.select( altArtVO.artNr, altArtVO.altArtNr );
      if ( updateVO == null )
      {
        fail( "UPDATE: Datensatz existiert nicht" );
      }
      else
      {
        // Datensatz ändern
        updateVO.priorCd = "2";
        updateVO.uVersion = ch.ige.util.Crescendo.getNextUVersion( updateVO.uVersion );

        int retVal = altArtDAO.update( updateVO );
        assertEquals( "Anzahl geänderter Datensätze = 1", 1, retVal );

        //Datensatz wieder selektieren und testen, ob Änderungen übernommen
        selectVO = altArtDAO.select( altArtVO.artNr, altArtVO.altArtNr );
        assertEquals( updateVO.priorCd, selectVO.priorCd );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   * Löscht einen Datensatz aus der Tabelle alt_art
   */
  public void test05Delete()
  {
    System.out.println( "Test DELETE" );
    AltArtVO selectVO = null;

    try
    {
      // set up
      altArtDAO.insert( altArtVO );

      int retVal = altArtDAO.delete( altArtVO.artNr, altArtVO.altArtNr );
      assertEquals( "Anzahl gelöschte Datensätze ", 1, retVal );

      List<AltArtVO> selectList = altArtDAO.select( ( altArtVO.artNr + 1 ) );

      for ( int i = 0; i < selectList.size(); i++ )
      {
        selectVO = selectList.get( i );
        int ret = altArtDAO.delete( selectVO.artNr, selectVO.altArtNr );
        assertEquals( "Anzahl gelöschte Datensätze ", 1, ret );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }
}
