package ch.ige.art.server.dao.test;

import java.sql.SQLException;
import java.util.List;

import ch.ige.art.server.dao.ArtextDAO;
import ch.ige.art.server.dao.ArtikelDAO;
import ch.ige.art.vo.ArtextVO;
import ch.ige.art.vo.ArtikelVO;
import ch.ige.art.vo.test.Testdata;

/**
 * Test der Klasse ArtextDAO
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: ArtextDAOTest.java,v 1.11 2008/06/25 13:47:29 ruegsegger Exp $
 */

public class ArtextDAOTest extends DAOTestCase
{
  private ArtikelDAO artikelDAO = new ArtikelDAO();

  private ArtikelVO artikelVO = null;

  private ArtextDAO artextDAO = new ArtextDAO();

  private ArtextVO artextVO = null;

  public ArtextDAOTest( String s )
  {
    super( s );
  }

  @Override
  protected void setUp()
  {
    artikelVO = Testdata.createArtikelVO();
    artextVO = Testdata.createArtextVO();

    try
    {
      artikelDAO.insert( artikelVO );
    }
    catch ( SQLException ex )
    {
      System.out.println( "set up failed: " + ex );
    }
  }

  @Override
  protected void tearDown()
  {
    try
    {
      artikelDAO.delete( artikelVO.artNr );
    }
    catch ( SQLException ex )
    {
      System.out.println( "tear down failed: " + ex );
    }
  }

  /**
   *  Fügt einen Datensatz in die  Tabelle artext ein
   */
  public void test1Insert()
  {
    System.out.println( "Test INSERT" );
    ArtextVO selectVO = null;

    try
    {
      artextDAO.insert( artextVO );

      // Testen ob eingefügt
      selectVO = artextDAO.select( artextVO.artNr, artextVO.sprachCd, artextVO.artTextArtCd );
      if ( selectVO == null )
      {
        fail( "Datensatz existiert nicht" );
      }

      assertEquals( "1. Test: art_nr: ", artextVO.artNr, selectVO.artNr );
      assertEquals( "2. Test: sprach_cd: ", artextVO.sprachCd, selectVO.sprachCd );
      assertEquals( "2. Test: sprach_cd: ", artextVO.artTextArtCd, selectVO.artTextArtCd );
      assertEquals( "3. Test: text: ", artextVO.text, selectVO.text );
      assertEquals( "4. Test erst_dat ", null, selectVO.erstDat );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Selektiert einen Datensatz aus der Tabelle artext
   */
  public void test2Select()
  {
    System.out.println( "Test SELECT " );
    ArtextVO selectVO = null;

    try
    {
      artextDAO.insert( artextVO );
      selectVO = artextDAO.select( artextVO.artNr, artextVO.sprachCd, artextVO.artTextArtCd );
      if ( selectVO == null )
      {
        fail( "Datensatz existiert nicht" );
      }

      assertEquals( "1. Test: art_nr: ", artextVO.artNr, selectVO.artNr );
      assertEquals( "2. Test: sprach_cd: ", artextVO.sprachCd, selectVO.sprachCd );
      assertEquals( "2. Test: sprach_cd: ", artextVO.artTextArtCd, selectVO.artTextArtCd );
      assertEquals( "3. Test: text: ", artextVO.text, selectVO.text );
      assertEquals( "4. Test erst_dat ", null, selectVO.erstDat );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Selektiert mehrere Datensätze aus der Tabelle artext
   */
  public void test3SelectAll()
  {
    System.out.println( "Test SELECT mehrere VO" );
    ArtextVO selectVO = null;
    List<ArtextVO> selectList = null;
    List<ArtextVO> list = Testdata.createListArtextVO( ( artextVO.artNr        ) );

    try
    {

      for ( int i = 0; i < list.size(); i++ )
      {
        selectVO = list.get( i );
        artextDAO.insert( selectVO );
      }

      selectList = artextDAO.select( artextVO.artNr );
      selectVO = list.get( 0 );

      assertTrue( "Anzahl = ", selectList.size() == 6 );
      assertEquals( ( artextVO.artNr  ), selectVO.artNr );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Aktualisiert einen Datensatz
   */
  public void test4Update()
  {
    System.out.println( "Test UPDATE " );
    ArtextVO selectVO = null;
    ArtextVO updateVO = null;

    try
    {
      artextDAO.insert( artextVO );
      updateVO = artextDAO.select( artextVO.artNr, artextVO.sprachCd, artextVO.artTextArtCd );
      if ( updateVO == null )
      {
        fail( "UPDATE: Datensatz existiert nicht" );
      }
      else
      {
        updateVO.text = "JUnit Test after Update";

        int retVal = artextDAO.update( updateVO );
        assertEquals( "Anzahl geänderter Datensätze = 1", 1, retVal );

        //prüfen ob Änderung gespeichert
        selectVO = artextDAO.select( artextVO.artNr, artextVO.sprachCd, artextVO.artTextArtCd );
        assertEquals( "1. Test: ", updateVO.text, selectVO.text );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   * Löscht einen Datensatz aus der Tabelle artext
   */
  public void test5Delete()
  {
    System.out.println( "Test DELETE" );

    try
    {
      artextDAO.insert( artextVO );
      int retVal = artextDAO.delete( artextVO.artNr, artextVO.sprachCd, artextVO.artTextArtCd );
      assertEquals( "Anzahl gelöschte Datensätze ", 1, retVal );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }
}
