package ch.ige.art.server.dao.test;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;

import ch.ige.art.server.dao.ArtikelDAO;
import ch.ige.art.vo.ArtikelVO;
import ch.ige.art.vo.test.Testdata;

/**
 * Test der Klasse TestArtikelDAO
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: ArtikelDAOTest.java,v 1.11 2008/06/25 13:47:29 ruegsegger Exp $
 */

public class ArtikelDAOTest extends DAOTestCase
{
  private ArtikelDAO artikelDAO = new ArtikelDAO();

  private ArtikelVO artikelVO = null;

  private SimpleDateFormat sdf = new SimpleDateFormat( "yyyy-MM-dd" );

  public ArtikelDAOTest( String s )
  {
    super( s );
  }

  @Override
  protected void setUp()
  {
    artikelVO = Testdata.createArtikelVO();
  }

  /**
   *  Fügt einen Datensatz in die Tabelle Artikel ein
   */
  public void test01Insert()
  {
    System.out.println( "Test INSERT " );
    ArtikelVO selectVO = null;

    try
    {
      artikelDAO.insert( artikelVO );

      // Testen ob eingefügt
      selectVO = artikelDAO.select( artikelVO.artNr );
      if ( selectVO == null )
      {
        fail( "INSERT: Datensatz existiert nicht" );
      }
      assertEquals( artikelVO.artNr, selectVO.artNr );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Selektiert einen Datensatz aus der Tabelle artikel
   */
  public void test02Select()
  {
    System.out.println( "Test SELECT " );
    ArtikelVO selectVO = null;

    try
    {
      selectVO = artikelDAO.select( artikelVO.artNr );

      if ( selectVO == null )
      {
        fail( "SELECT: Datensatz existiert nicht" );
      }
      else
      {
        assertEquals( artikelVO.artNr, selectVO.artNr );
        assertEquals( null, selectVO.abcCd );
        assertEquals( null, selectVO.abcRangnr );
        assertEquals( artikelVO.altNrCd, selectVO.altNrCd );
        assertEquals( "0", selectVO.artCd );
        assertEquals( null, selectVO.artgrp1 );
        assertEquals( null, selectVO.artgrp2 );
        assertEquals( null, selectVO.artgrp3 );
        assertEquals( null, selectVO.artgrp4 );
        assertEquals( artikelVO.artgrpCd, selectVO.artgrpCd );
        assertEquals( "0", selectVO.beschaffart );
        assertEquals( artikelVO.bonusCd, selectVO.bonusCd );
        assertEquals( null, selectVO.datDschnEp );
        assertEquals( null, selectVO.datStandpreis );
        assertEquals( null, selectVO.dschnEpSfr );
        assertEquals( null, selectVO.eanCd );
        assertEquals( "0", selectVO.etikettenCd );
        assertEquals( null, selectVO.ekeinh );
        assertEquals( null, selectVO.fakteinh );
        assertEquals( null, selectVO.gewicht );
        assertEquals( null, selectVO.gewichtNetto );
        assertEquals( null, selectVO.invpreis );
        assertEquals( "0", selectVO.kennzAuslArt );
        assertEquals( "0", selectVO.kennzCharge );
        assertEquals( "0", selectVO.kennzLagFueh );
        assertEquals( "0", selectVO.kennzSerieNr );
        assertEquals( artikelVO.kondartCd, selectVO.kondartCd );
        assertEquals( "0", selectVO.kzFerart );
        assertEquals( artikelVO.kzGratis, selectVO.kzGratis );
        assertEquals( artikelVO.kzMetnot, selectVO.kzMetnot );
        assertEquals( "0", selectVO.kzMinverk );
        assertEquals( "0", selectVO.kzNettoArt );
        assertEquals( "1", selectVO.kzSktoBer );
        assertEquals( artikelVO.lageinh, selectVO.lageinh );
        assertEquals( null, selectVO.landCd );
        assertEquals( null, selectVO.letzEpFw );
        assertEquals( null, selectVO.letzEpSfr );
        assertEquals( null, selectVO.loskosten );
        assertEquals( null, selectVO.margeMin );
        assertEquals( null, selectVO.margeSoll );
        assertEquals( null, selectVO.markenCd );
        assertEquals( artikelVO.minverkMenge, selectVO.minverkMenge );
        assertEquals( artikelVO.optBestMenge, selectVO.optBestMenge );
        assertEquals( null, selectVO.preisgruppe );
        assertEquals( artikelVO.provCd, selectVO.provCd );
        assertEquals( null, selectVO.provgrpCd );
        assertEquals( "BAGIS", selectVO.sachbCd );
        assertEquals( "0", selectVO.sammelartCd );
        assertEquals( artikelVO.sperrCd, selectVO.sperrCd );
        assertEquals( null, selectVO.spesensatz );
        assertEquals( new BigDecimal( 0 ), selectVO.standpreis );
        assertEquals( artikelVO.statcd1, selectVO.statcd1 );
        assertEquals( artikelVO.statcd2, selectVO.statcd2 );
        assertEquals( artikelVO.statcd3, selectVO.statcd3 );
        assertEquals( "G1", selectVO.steuZeilCd );
        assertEquals( "0", selectVO.stueliCd );
        assertEquals( artikelVO.suchbegrArt, selectVO.suchbegrArt );
        assertEquals( null, selectVO.umrFaktEk );
        assertEquals( null, selectVO.umrFaktVk );
        assertEquals( null, selectVO.waehrCd );
        assertEquals( null, selectVO.wbeschZeit );
        assertEquals( null, selectVO.zolltarif );
        assertEquals( "!", selectVO.uVersion );
        assertEquals( artikelVO.erstBen, selectVO.erstBen );
        assertEquals( sdf.format( artikelVO.erstDat ), sdf.format( selectVO.erstDat ) );
        assertEquals( artikelVO.geaendBen, selectVO.geaendBen );
        assertEquals( sdf.format( artikelVO.geaendDat ), sdf.format( selectVO.geaendDat ) );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Aktualisiert einen Datensatz in der Tabelle artikel
   */
  public void test03Update()
  {
    System.out.println( "Test UPDATE " );
    ArtikelVO selectVO = null;
    ArtikelVO updateVO = null;

    try
    {
      updateVO = artikelDAO.select( artikelVO.artNr );
      if ( updateVO == null )
      {
        fail( "UPDATE: Datensatz existiert nicht" );
      }
      else
      {
        updateVO.altNrCd = "0";
        updateVO.bonusCd = "1";
        updateVO.kondartCd = "PATSCHUL";
        updateVO.kzGratis = "0";
        updateVO.kzMetnot = null;
        updateVO.lageinh = "ANZ";
        updateVO.minverkMenge = new BigDecimal( 1 );
        updateVO.optBestMenge = new BigDecimal( 2 );
        updateVO.sperrCd = "0";
        updateVO.statcd1 = null;
        updateVO.statcd2 = null;
        updateVO.statcd3 = "M05";
        updateVO.suchbegrArt = "TEST AFTER UPDATE";
        updateVO.geaendBen = "j-unit";

        // Datensatz updaten
        int retVal = artikelDAO.update( updateVO );
        assertEquals( "Anzahl Datensätze ", 1, retVal );

        // Datensatz wieder sektieren und auf Änderungen testen
        selectVO = artikelDAO.select( artikelVO.artNr );

        assertEquals( artikelVO.artNr, selectVO.artNr );
        assertEquals( null, selectVO.abcCd );
        assertEquals( null, selectVO.abcRangnr );
        assertEquals( updateVO.altNrCd, selectVO.altNrCd );
        assertEquals( "0", selectVO.artCd );
        assertEquals( null, selectVO.artgrp1 );
        assertEquals( null, selectVO.artgrp2 );
        assertEquals( null, selectVO.artgrp3 );
        assertEquals( null, selectVO.artgrp4 );
        assertEquals( artikelVO.artgrpCd, selectVO.artgrpCd );
        assertEquals( "0", selectVO.beschaffart );
        assertEquals( updateVO.bonusCd, selectVO.bonusCd );
        assertEquals( null, selectVO.datDschnEp );
        assertEquals( null, selectVO.datStandpreis );
        assertEquals( null, selectVO.dschnEpSfr );
        assertEquals( null, selectVO.eanCd );
        assertEquals( "0", selectVO.etikettenCd );
        assertEquals( null, selectVO.ekeinh );
        assertEquals( null, selectVO.fakteinh );
        assertEquals( null, selectVO.gewicht );
        assertEquals( null, selectVO.gewichtNetto );
        assertEquals( null, selectVO.invpreis );
        assertEquals( "0", selectVO.kennzAuslArt );
        assertEquals( "0", selectVO.kennzCharge );
        assertEquals( "0", selectVO.kennzLagFueh );
        assertEquals( "0", selectVO.kennzSerieNr );
        assertEquals( updateVO.kondartCd, selectVO.kondartCd );
        assertEquals( "0", selectVO.kzFerart );
        assertEquals( updateVO.kzGratis, selectVO.kzGratis );
        assertEquals( updateVO.kzMetnot, selectVO.kzMetnot );
        assertEquals( "0", selectVO.kzMinverk );
        assertEquals( "0", selectVO.kzNettoArt );
        assertEquals( "1", selectVO.kzSktoBer );
        assertEquals( updateVO.lageinh, selectVO.lageinh );
        assertEquals( null, selectVO.landCd );
        assertEquals( null, selectVO.letzEpFw );
        assertEquals( null, selectVO.letzEpSfr );
        assertEquals( null, selectVO.loskosten );
        assertEquals( null, selectVO.margeMin );
        assertEquals( null, selectVO.margeSoll );
        assertEquals( null, selectVO.markenCd );
        assertEquals( updateVO.minverkMenge, selectVO.minverkMenge );
        assertEquals( updateVO.optBestMenge, selectVO.optBestMenge );
        assertEquals( null, selectVO.preisgruppe );
        assertEquals( artikelVO.provCd, selectVO.provCd );
        assertEquals( null, selectVO.provgrpCd );
        assertEquals( "BAGIS", selectVO.sachbCd );
        assertEquals( "0", selectVO.sammelartCd );
        assertEquals( updateVO.sperrCd, selectVO.sperrCd );
        assertEquals( null, selectVO.spesensatz );
        assertEquals( new BigDecimal( 0 ), selectVO.standpreis );
        assertEquals( updateVO.statcd1, selectVO.statcd1 );
        assertEquals( updateVO.statcd2, selectVO.statcd2 );
        assertEquals( updateVO.statcd3, selectVO.statcd3 );
        assertEquals( "G1", selectVO.steuZeilCd );
        assertEquals( "0", selectVO.stueliCd );
        assertEquals( updateVO.suchbegrArt, selectVO.suchbegrArt );
        assertEquals( null, selectVO.umrFaktEk );
        assertEquals( null, selectVO.umrFaktVk );
        assertEquals( null, selectVO.waehrCd );
        assertEquals( null, selectVO.wbeschZeit );
        assertEquals( null, selectVO.zolltarif );
        assertEquals( artikelVO.erstBen, selectVO.erstBen );
        assertEquals( sdf.format( artikelVO.erstDat ), sdf.format( selectVO.erstDat ) );
        assertEquals( updateVO.geaendBen, selectVO.geaendBen );
        assertEquals( sdf.format( new Date( System.currentTimeMillis() ) ), sdf.format( selectVO.geaendDat ) );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Prüft die Existenz eines Artikels in der Tabelle artikel
   */
  public void test04Exists()
  {
    System.out.println( "Test EXISTS " );

    try
    {
      boolean exist = artikelDAO.exists( artikelVO.artNr );
      assertEquals( "Datensatz existiert ", true, exist );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Löscht einen Datensatz aus der Tabelle artikel
   */
  public void test05Delete()
  {
    System.out.println( "Test DELETE " );

    try
    {
      int retVal = artikelDAO.delete( artikelVO.artNr );
      assertEquals( "Anzahl gelöschte Datensätze ", 1, retVal );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }
}
