package ch.ige.art.server.dao.test;

import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import ch.ige.art.server.dao.ArtikelDAO;
import ch.ige.art.server.dao.BArtikelDAO;
import ch.ige.art.vo.ArtikelVO;
import ch.ige.art.vo.BArtikelVO;
import ch.ige.art.vo.test.Testdata;

/**
 * Test der Klasse BArtikelDAO
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: BArtikelDAOTest.java,v 1.9 2008/06/25 13:47:29 ruegsegger Exp $
 */

public class BArtikelDAOTest extends DAOTestCase
{
  private ArtikelDAO artikelDAO = new ArtikelDAO();

  private ArtikelVO artikelVO = null;

  private BArtikelDAO bArtikelDAO = new BArtikelDAO();

  private BArtikelVO bArtikelVO = null;

  private SimpleDateFormat sdf = new SimpleDateFormat( "yyyy-MM-dd" );

  public BArtikelDAOTest( String s )
  {
    super( s );
  }

  @Override
  protected void setUp()
  {
    artikelVO = Testdata.createArtikelVO();
    bArtikelVO = Testdata.createBartikelVO();

    try
    {
      artikelDAO.insert( artikelVO );
    }
    catch ( SQLException ex )
    {
      System.out.println( "set up failed: " + ex );
    }
  }

  @Override
  protected void tearDown()
  {
    try
    {
      bArtikelDAO.delete( bArtikelVO.artNr );
      artikelDAO.delete( artikelVO.artNr );
    }
    catch ( SQLException ex )
    {
      System.out.println( "tear down failed: " + ex );
    }
  }

  /**
   *  Fügt einen Datensatz in die Tabelle bartikel ein
   */
  public void test01Insert()
  {
    System.out.println( "Test INSERT " );
    BArtikelVO selectVO = null;

    try
    {
      bArtikelDAO.insert( bArtikelVO );

      // Testen ob eingefügt
      selectVO = bArtikelDAO.select( bArtikelVO.artNr );
      if ( selectVO == null )
      {
        fail( "INSERT: Datensatz existiert nicht" );
      }
      assertEquals( bArtikelVO.artNr, selectVO.artNr );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Selektiert einen Datensatz aus der Tabelle bartikel
   */
  public void test02Select()
  {
    System.out.println( "Test SELECT " );
    BArtikelVO selectVO = null;

    try
    {
      bArtikelDAO.insert( bArtikelVO );
      selectVO = bArtikelDAO.select( bArtikelVO.artNr );
      if ( selectVO == null )
      {
        fail( "SELECT: Datensatz existiert nicht" );
      }
      else
      {
        assertEquals( bArtikelVO.artNr, selectVO.artNr );

        assertEquals( bArtikelVO.anzKopie, selectVO.anzKopie );
        assertEquals( bArtikelVO.druckModus, selectVO.druckModus );
        assertEquals( bArtikelVO.einzFaktpos, selectVO.einzFaktpos );
        assertEquals( bArtikelVO.formartCd, selectVO.formartCd );
        assertEquals( bArtikelVO.kzEinschr, selectVO.kzEinschr );
        assertEquals( bArtikelVO.kzUntschr, selectVO.kzUntschr );
        assertEquals( bArtikelVO.publPflicht, selectVO.publPflicht );
        assertEquals( bArtikelVO.regauszGrp, selectVO.regauszGrp );
        assertEquals( bArtikelVO.fristArt, selectVO.fristArt );
        assertEquals( bArtikelVO.fristEinh, selectVO.fristEinh );
        assertEquals( bArtikelVO.fristDauer, selectVO.fristDauer );
        assertEquals( bArtikelVO.kzBesch, selectVO.kzBesch );
        assertEquals( bArtikelVO.ereigPlaus, selectVO.ereigPlaus );
        assertEquals( bArtikelVO.erfassProc, selectVO.erfassProc );
        assertEquals( bArtikelVO.processModus, selectVO.processModus );
        assertEquals( bArtikelVO.processNr, selectVO.processNr );
        assertEquals( sdf.format( bArtikelVO.erstDat ), sdf.format( selectVO.erstDat ) );
        assertEquals( bArtikelVO.erstBen, selectVO.erstBen );
        assertEquals( bArtikelVO.geaendDat, selectVO.geaendDat );
        assertEquals( bArtikelVO.geaendBen, selectVO.geaendBen );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Aktualisiert einen Datensatz in der Tabelle bartikel
   */
  public void test03Update()
  {
    System.out.println( "Test UPDATE " );
    BArtikelVO selectVO = null;
    BArtikelVO updateVO = null;

    try
    {
      bArtikelDAO.insert( bArtikelVO );
      updateVO = bArtikelDAO.select( bArtikelVO.artNr );
      if ( updateVO == null )
      {
        fail( "UPDATE: Datensatz existiert nicht" );
      }
      else
      {
        updateVO.anzKopie = new Integer( 2 );
        updateVO.druckModus = "2";
        updateVO.einzFaktpos = "2";
        updateVO.formartCd = "501";
        updateVO.kzEinschr = "0";
        updateVO.kzUntschr = "3";
        updateVO.publPflicht = "3";
        updateVO.regauszGrp = 700;
        updateVO.fristArt = "3";
        updateVO.fristEinh = "2";
        updateVO.fristDauer = new Integer( 30 );
        updateVO.kzBesch = "0";
        updateVO.ereigPlaus = new Integer( 6 );
        updateVO.erfassProc = new Integer( 35 );
        updateVO.processModus = "2";
        updateVO.processNr = new Integer( 10 );
        updateVO.geaendDat = new java.util.Date();
        updateVO.geaendBen = "j-unit";

        // Datensatz updaten
        int retVal = bArtikelDAO.update( updateVO );
        assertEquals( "Anzahl Datensätze ", 1, retVal );

        // Datensatz wieder sektieren und auf Änderungen testen
        selectVO = bArtikelDAO.select( bArtikelVO.artNr );

        assertEquals( updateVO.artNr, selectVO.artNr );
        assertEquals( updateVO.anzKopie, selectVO.anzKopie );
        assertEquals( updateVO.druckModus, selectVO.druckModus );
        assertEquals( updateVO.einzFaktpos, selectVO.einzFaktpos );
        assertEquals( updateVO.formartCd, selectVO.formartCd );
        assertEquals( updateVO.kzEinschr, selectVO.kzEinschr );
        assertEquals( updateVO.kzUntschr, selectVO.kzUntschr );
        assertEquals( updateVO.publPflicht, selectVO.publPflicht );
        assertEquals( updateVO.regauszGrp, selectVO.regauszGrp );
        assertEquals( updateVO.fristArt, selectVO.fristArt );
        assertEquals( updateVO.fristEinh, selectVO.fristEinh );
        assertEquals( updateVO.fristDauer, selectVO.fristDauer );
        assertEquals( updateVO.kzBesch, selectVO.kzBesch );
        assertEquals( null, selectVO.ereigPlaus );
        assertEquals( updateVO.erfassProc, selectVO.erfassProc );
        assertEquals( updateVO.processModus, selectVO.processModus );
        assertEquals( updateVO.processNr, selectVO.processNr );
        assertEquals( sdf.format( new Date( System.currentTimeMillis() ) ), sdf.format( selectVO.geaendDat ) );
        assertEquals( updateVO.geaendBen, selectVO.geaendBen );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Löscht einen Datensatz aus der Tabelle bartikel
   */
  public void test05Delete()
  {
    System.out.println( "Test DELETE " );

    try
    {
      bArtikelDAO.insert( bArtikelVO );
      int retVal = bArtikelDAO.delete( bArtikelVO.artNr );
      assertEquals( "Anzahl gelöschte Datensätze ", 1, retVal );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  public void test06SelectArtNr()
  {
    System.out.println( "Test SELECT ARTNR " );

    try
    {
      List<?> list = bArtikelDAO.selectListArtNr( "995" );
      assertTrue( "Zum Formuar 995 sind mehr als 70 Artikel vorhanden", list.size() > 70 );

      list = bArtikelDAO.selectListArtNr( "110" );
      assertTrue( "Zum Formular 110 sind keine Artikel vorhanden", list.size() == 0 );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }

  }
}
