package ch.ige.art.xml;

import java.io.*;
import java.util.List;
import org.apache.log4j.Logger;
import ch.ige.art.util.XxDomainFactory;
import ch.ige.art.vo.*;
import java.util.ArrayList;
import ch.ige.art.server.ArtikelDelegate;
import ch.ige.art.xml.jaxb.formular.*;
import ch.ige.art.xml.jaxb.formular.impl.*;
import javax.xml.bind.*;


/**
 * <p>Überschrift: Mit dieser Klasse werden die Daten für die Erstellung des XML-Stream zusammengetragen</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author  Anita Rueegsegger
 * @version $Id: FormStammblatt.java,v 1.13 2008/01/14 09:49:15 ruegsegger Exp $
 */

public class FormStammblatt extends Stammblatt
{
  private static final Logger LOG = Logger.getLogger( FormStammblatt.class );
  private JAXBContext jc = JAXBContext.newInstance( "ch.ige.art.xml.jaxb.formular" );


  public FormStammblatt() throws Exception
  {
  }


  /**
   * Liest die Daten von der Datenbank
   */
  public List read( List listFormartCd ) throws Exception
  {
    BformDruckVO bformDruckVO;
    List listFormular = new ArrayList();

    for ( int i = 0; i < listFormartCd.size(); i++ )
    {
      String formartCd = ( String )listFormartCd.get( i );
      bformDruckVO = ArtikelDelegate.getInstance().getFormular( formartCd );
      listFormular.add( bformDruckVO );
    }

    return listFormular;
  }



  /**
   * Erstellt den String mit den XMLDaten
   * @param auszugVO ValueObject mit den Daten für eines Auszuges
   * @return String mit den XMLDaten
   * @throws XMLException
   */
  public String getXMLString( List listForm ) throws Exception
  {
    BformDruckVO bformDruckVO;
    FormularEl formularEl;

    PrintOrder printOrder = new PrintOrderImpl();
    printOrder.setDoktyp( String.valueOf( PrintDoc.DOKTYP_FORMSTAMMBLATT ) );

    printOrder.setMaNr( getMaNr() );  
    printOrder.setNtusername( getNtusername() );
    
    LOG.debug( "PrintOrder" );

    for (int i = 0; i < listForm.size(); i++)
    {
      bformDruckVO = (BformDruckVO)listForm.get(i);
      formularEl = writeFormStammblatt(bformDruckVO);
      printOrder.getFormularEl().add(formularEl);
    }

    String xmlStream=  super.marshalToString( jc, printOrder );;

    return replaceTags(xmlStream);
  }


  private FormularEl writeFormStammblatt( BformDruckVO bformDruckVO )
  {
    FormularEl formularEl = new FormularElImpl();

    //Header füllen
    Header header = writeHeader( bformDruckVO.bformartVO );
    formularEl.setHeader( header );
    LOG.debug( "Header" );

    ArtNumbers artNumbers = writeArtNumbers( bformDruckVO.listArtNr );
    formularEl.setArtNumbers( artNumbers );

    List listFormtxt = formularEl.getFormtxt();
    Formtxt formtxt = writeFormtexe(bformDruckVO.bformtxtDruckVO);
    listFormtxt.add(formtxt);
    LOG.debug( "Formulartexte" );

    return formularEl;
  }

  /**
   * Schreibt die Headerzeile ins XMLFile
   * @param BformartVO enthält alle für ein Stammblatt benötigten Werte
   * @return header Die geschriebene headerZeile
   */
  private Header writeHeader( BformartVO bformartVO )
  {
    Header header = new HeaderImpl();
    header.setHeaderTitle( "Formular-Stammblatt" );
    header.setHeaderDate( currentDate );
    header.setHeaderNr(bformartVO.formartCd );
    header.setHeaderDescription(bformartVO.bez == null ? "" :  bformartVO.bez);
    String printSystem = getLegend(bformartVO.druckSystem, XxDomainFactory.DRUCK_SYSTEM );
    header.setHeaderPrintsystem(printSystem);

    return header;
  }

  /**
   * Schreibt den Abschnitt Zugeordnete Artikel-Nummern ins XMLFile
   * Pro Formular werden alle zugehörigen Artikel angedruckt
   * @param listArtNr List
   * @return ArtNumbers
   */
  private ArtNumbers writeArtNumbers( List listArtNr )
  {
    ArtNumbers artNumbers = new ArtNumbersImpl();
    artNumbers.setArtNoTitle( "Zugeordnete Artikel-Nummern" );

    if (listArtNr != null)
    {
      for ( int i = 0; i < listArtNr.size(); i++ )
      {
        artNumbers.getArtNo().add( ( String )listArtNr.get( i ) );
      }
    }
    return artNumbers;
  }


   /**
   * Schreibt den Abschnitt Formular-Texte ins XMLFile.
   * Pro Textart wird eine Gruppe ausgedruckt
   * @param BformtxtDruckVO enthält alle für die Formulartexte benötigten Werte
   * @return Formtxt der geschriebene Abschnitt Formulartexte
   */
  public Formtxt writeFormtexe( BformtxtDruckVO[] bformtxtDruckVO  )
  {
    Formtxt formtxt = new FormtxtImpl();
    FormtxtDetail formtxtDetail = null;
    List listFormtxtDetail = null;
    FormtxtData formtxtData = null;

    String formTxtNr = null;
    String textart = null;

    formtxt.setFormtxtTitle("Formular-Texte");
    List listFormtxtData = formtxt.getFormtxtData();

    for ( int i = 0; i < bformtxtDruckVO.length;)
    {
      formtxtData = new FormtxtDataImpl();
      formTxtNr = bformtxtDruckVO[i].textart;
      formtxtData.setFormtxtNr( formTxtNr );
      textart =  getText( XxDomainFactory.TEXTART, formTxtNr );
      formtxtData.setFormtxtArt( textart );

      listFormtxtDetail = formtxtData.getFormtxtDetail();

      do
      {
        formtxtDetail = new FormtxtDetailImpl();
        formtxtDetail.setFormtxtLanguage( bformtxtDruckVO[i].sprachCd );
        formtxtDetail.setFormtxtTxt( bformtxtDruckVO[i].text == null ? "" : bformtxtDruckVO[i].text);

        listFormtxtDetail.add( formtxtDetail );
        i++;
      }
      while ( i < bformtxtDruckVO.length && bformtxtDruckVO[i].textart.equals( formTxtNr ) );

      listFormtxtData.add( formtxtData );
      LOG.debug( "Formular-Texte " + textart  );
    }

    return formtxt;
  }
}

