package ch.ige.art.xml;

import java.io.File;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.bind.JAXBContext;

import org.apache.log4j.Logger;

import ch.ige.art.xml.jaxb.listen.Header;
import ch.ige.art.xml.jaxb.listen.Kriterium;
import ch.ige.art.xml.jaxb.listen.ListeEl;
import ch.ige.art.xml.jaxb.listen.PrintOrder;
import ch.ige.art.xml.jaxb.listen.Spalten;
import ch.ige.art.xml.jaxb.listen.Zeilen;
import ch.ige.art.xml.jaxb.listen.impl.HeaderImpl;
import ch.ige.art.xml.jaxb.listen.impl.KriteriumImpl;
import ch.ige.art.xml.jaxb.listen.impl.ListeElImpl;
import ch.ige.art.xml.jaxb.listen.impl.PrintOrderImpl;

/**
 * <p>Überschrift: Mit dieser Klasse werden die Daten für die Erstellung des XML-File zusammengetragen.
 * Dient als Oberklasse für den Druck einer Trefferliste</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author  Anita Rueegsegger
 * @version $Id: Liste.java,v 1.11 2008/02/02 14:27:12 ruegsegger Exp $
 */

abstract public class Liste extends PrintDoc
{
  private static final Logger LOG = Logger.getLogger( Liste.class );

  private JAXBContext jc = JAXBContext.newInstance( "ch.ige.art.xml.jaxb.listen" );

  public Liste() throws Exception
  {
  }

  /**
   * Druckt Trefferliste
   * @param listHits List zu druckende Daten
   * @param mapKriterium Map zu druckende Kriterien
   * @throws Exception
   */
  public void save( List listHits, Map mapKriterium, File file ) throws Exception
  {
    String xmlStream = getXMLString( listHits, mapKriterium );
    super.saveToUTF8File( xmlStream, file );
  }

  /**
   * Druckt Trefferliste
   * @param listHits List zu druckende Daten
   * @param mapKriterium Map zu druckende Kriterien
   * @throws Exception
   */
  public void print( List listHits, Map mapKriterium ) throws Exception
  {
    String xmlStream = getXMLString( listHits, mapKriterium );
    sendMessage( xmlStream );
  }

  public String getXMLString( List listHits, Map mapKriterium ) throws Exception
  {
    PrintOrder printOrder = createPrintOrder( listHits, mapKriterium );
    String xmlStream = marshalToString( jc, printOrder );

    LOG.debug( "XML-Stream: \n" + xmlStream );
    return xmlStream;
  }

  public PrintOrder createPrintOrder( List listHits, Map mapKriterium )
  {
    // PrintOrder füllen
    PrintOrder printOrder = new PrintOrderImpl();
    printOrder.setDoktyp( String.valueOf( PrintDoc.DOKTYP_LISTEN ) );

    printOrder.setMaNr( getMaNr() );
    printOrder.setNtusername( getNtusername() );

    LOG.debug( "PrintOrder" );

    //Header füllen
    Header header = prepareHeader( listHits );
    List alKriterum = header.getKriterium();
    prepareKriterium( alKriterum, mapKriterium );
    LOG.debug( "Header" );

    // ListeEl füllen
    ListeEl listeEl = new ListeElImpl();
    listeEl.setHeader( header );
    LOG.debug( "Liste" );

    // Spalte füllen
    Spalten spalten = prepareSpalte(); //beinhaltet alle Spalten
    listeEl.setSpalten( spalten );
    LOG.debug( "Spalte" );

    // Zeile füllen
    Zeilen zeilen = prepareZeile( listHits );
    listeEl.setZeilen( zeilen );
    LOG.debug( "Zeile" );

    printOrder.setListeEl( listeEl );

    return printOrder;
  }

  /**
   * Bereitet die Headerzeile auf
   * @return header Die aufbereitete Header-Zeile
   */
  private Header prepareHeader( List artikelAVO )
  {
    Header header = new HeaderImpl();
    header.setTitel( getTitle() );
    header.setAnzahl( String.valueOf( artikelAVO.size() ) );
    header.setZusammenfassung( getTitle() + " (" + ( maVO == null ? "" : maVO.maKurzz ) + ")" );

    return header;
  }

  /**
   * Gibt den Titel der Trefferliste zurück
   * @return String Titel der Trefferliste
   */
  protected String getTitle()
  {
    return "Trefferliste vom " + currentDate;
  }

  /**
   * Bereitet die Kriteriumliste auf
   * @return List aufbereitete Kriteriumliste
   */
  protected void prepareKriterium( List alKriterum, Map map )
  {
    Kriterium kriterium = null;

    Iterator it = map.entrySet().iterator();
    while ( it.hasNext() )
    {
      Map.Entry entry = (Map.Entry) it.next();

      kriterium = new KriteriumImpl();
      kriterium.setKritname( (String) entry.getKey() );
      kriterium.setKritwert( (String) entry.getValue() );
      alKriterum.add( kriterium );
    }
  }

  /**
   * Bereitet die Spalte auf für den Eintrag ins XML-File
   * @return List aufbereitete Spalten
   */
  protected abstract Spalten prepareSpalte();

  /**
   * Bereitet die Zeilen für den Eintrag ins XML-File auf
   * @return List aufbereitete Spalten
   */
  protected abstract Zeilen prepareZeile( List listHits );
}
