package ch.ige.art.xml;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.Serializable;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.channels.FileChannel;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;

import org.apache.log4j.Logger;

import ch.ige.art.server.ArtikelDelegate;
import ch.ige.art.util.XxDomain;
import ch.ige.art.vo.MaVO;

/**
 * <p>Überschrift: Mit dieser Klasse werden die Daten für die Erstellung des XML-Streams zusammengetragen</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author  Anita Rueegsegger
 * @version $Id: PrintDoc.java,v 1.26 2008/07/02 11:00:20 heinzer Exp $
 */

public class PrintDoc implements Serializable
{
  public static final int DOKTYP_AUSZUG = 1;

  public static final int DOKTYP_SCHREIBEN = 2;

  public static final int DOKTYP_LISTEN = 3;

  public static final int DOKTYP_STAMMBLATT = 4;

  public static final int DOKTYP_FORMSTAMMBLATT = 6;

  protected MaVO maVO = null;

  protected String currentDate = null;

  private static final Logger LOG = Logger.getLogger( PrintDoc.class );

  private static final String HOST = "localhost";

  private static final int RETRY = 10;

  private int port = 0;

  private SimpleDateFormat sdf = new SimpleDateFormat( "dd.MM.yyyy" );

  public PrintDoc() throws Exception
  {
    port = Integer.parseInt( System.getProperty( "jippsclient.port" ) );

//    lokal port for testing purpose
//    port = 5000;

    LOG.debug( "Port: " + port );
    maVO = getMitarbeiter();
    currentDate = getCurrentDate();
  }

  /**
   * Jippsclient wird beim Starten von Bagis automatisch gestartet, läuft der Client noch nicht wird eine Exception geworfen
   * @param message String
   * @throws Exception
   */
  public void sendMessage( String message ) throws Exception
  {
    Socket socket = null;

    // Falls Jippsclient noch nicht läuft, wird die Applikation gestartet
    // (ServerSocket der an Port 500? horcht)
    try
    {
      socket = new Socket( HOST, port );
    }
    catch ( Exception ex )
    {
      LOG.debug( " Exception bei Socketverbindung : " + ex );

      this.startJippsclient();
      LOG.debug( "Jippsclient gestartet" );

      socket = tryToConnectSocket();
    }

    LOG.debug( "Socket gestartet auf Host: " + HOST + " Port: " + port );

    DataOutputStream out = new DataOutputStream( socket.getOutputStream() );//TODO: socket communication seems very slow for large data (ar)
    out.writeBytes( message );
    out.flush();
    out.close();
    socket.close();

    LOG.debug( "Nachricht versendet" );
  }

  /**
   * Startet den Jippsclient auf localhost port 5000-5003 (je nach Schiene)
   * Systemproperty "BUILD_JIPPSCLIENT_CMDLINE" wird via Ant-Skript gesetzt
   * Die Trefferliste wird an den Jippsclient gesendet (ServerSocket),
   * welcher das Dokument aufbereitet und an den Printservice zum ausdrucken schickt
   * @param command String Kommandozeile
   * @return int Returnwert des Druck-Tools
   */
  private void startJippsclient() throws Exception
  {
    String command = System.getProperty( "jippsclient.cmdline" );
    LOG.debug( "Jippsclient starten mit: " + command );

    Process p = Runtime.getRuntime().exec( command );

    // warten auf Beendigung des Prozesses
    // Returnwert des Prozesses 0 bedeutet normales Prozessende
    if ( p.waitFor() != 0 )
    {
      throw new Exception( "Ungültiger Returnvalue: " + p.exitValue() );
    }
  }

  /**
   *  Creates a stream socket and connects it to the specified port number on the named host.
   */
  private Socket tryToConnectSocket() throws Exception
  {
    Socket socket = null;

    for ( int i = 0; i < RETRY; i++ )
    {
      try
      {
        socket = new Socket( HOST, port );
        break;
      }
      catch ( Exception ex )
      {
        LOG.info( "Retry: " + i );

        // Beim letzen Versuch Exception werfen
        if ( i == ( RETRY - 1 ) )
        {
          throw ex;
        }

        // nächster Versuch nach einer Sekunde
        Thread.sleep( 1000 );
      }
    }

    return socket;
  }

  /**
   * Gibt den eingeloggten Mitarbeiter zurück
   * @throws Exception
   * @return MaVO Mitarbeiter-Informationen
   */
  private MaVO getMitarbeiter() throws Exception
  {
    ArtikelDelegate ad = ArtikelDelegate.getInstance();
    String userName = System.getProperties().getProperty( "user.name" );
    LOG.debug( "userName: " + userName );
    return ad.getMa( userName );
  }

  protected String getMaNr()
  {
    String maNr = "0";

    if ( maVO != null )
    {
      maNr = String.valueOf( maVO.maNr );
    }

    return maNr;
  }

  protected String getNtusername()
  {
    String vmsUsername = "unknown user";

    if ( maVO != null && maVO.vmsUsername != null )
    {
      vmsUsername = maVO.vmsUsername;
    }

    return vmsUsername;
  }

  /**
   * Geibt das aktuelle Datum im Format dd.MM.yyyy zurück
   * @return String aktuelles Datum im Fromat dd.MM.yyyy
   */
  private String getCurrentDate()
  {
    Date currentDate = new Date( System.currentTimeMillis() );
    return sdf.format( currentDate );
  }

  /**
   * Gibt den Wert und die Bezeichnung komma-separiert zurück
   * @param domain String Domäne
   * @param value Object Wert
   * @return String Wert mit der entsprechenden Bezeichnung
   */
  protected String getLegend( Object value, String... domain )
  {
    if ( value != null && !value.equals( "" ) )
    {
      return XxDomain.getLegend( value.toString(), domain );
    }
    else
    {
      return "";
    }
  }

  /**
   * Gibt die Definition einer Domain zurück
   * @param domain String Domain-Name
   * @param value String Wert
   * @return String Definition einer Domain
   */
  protected String getText( String domain, String value )
  {
    return XxDomain.getText( domain, value );
  }

  public String marshalToString( JAXBContext jc, Object object ) throws Exception
  {
    ByteArrayOutputStream os = new ByteArrayOutputStream();

    Marshaller m = jc.createMarshaller();
    m.setProperty( Marshaller.JAXB_ENCODING, "UTF-8" );
    m.setProperty( Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE );
    m.marshal( object, os );

    return os.toString( "UTF-8" );
  }

  public void saveToUTF8File( String xmlString, File destination ) throws Exception
  {
    //TODO ch.ige.util.IOUtil#writeUTF8File(File, CharSequence) does the same
    Charset encoding = Charset.forName( "UTF-8" );
    ByteBuffer data = encoding.encode( CharBuffer.wrap( xmlString ) );
    FileOutputStream out = new FileOutputStream( destination );
    try
    {
      FileChannel outChannel = out.getChannel();
      while ( data.hasRemaining() )
      {
        outChannel.write( data );
      }
      outChannel.close();
    }
    finally
    {
      if ( out != null )
      {
        out.close();
      }
    }
  }
}
