package ch.ige.epband.clientneu.test;

import java.util.List;

import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;

import ch.ige.epband.clientneu.ParseSGML;
import ch.ige.epband.exception.ProcessingException;
import ch.ige.epband.util.XMLDoc;
import ch.ige.epband.vo.EpbandVO;
import ch.ige.epband.vo.EreigTxtVO;

/**
 * Tests für die Beschreibung
 * 
 *
 * <p>Copyright (c) 2006, Swiss Federal Institute of Intellectual Property</p>
 * @author   wild
 * @version  $Id: TestBeschreibung.java,v 1.15 2008/07/10 10:16:48 wild Exp $
 */
public class TestBeschreibung extends TestCase
{

  /**
   * Hilfsliste für BLand
   */
  public static List<String> blandWork;

  /**
   * Hilfsliste für IPC
   */
  public static List<String> ipcWork;

  /**
   * Initialisieren der Klasse
   * @param s
   */
  public TestBeschreibung( String s )
  {
    super( s );
    BasicConfigurator.configure();

  }

  @Override
  protected void setUp()
  {
    blandWork = CreateTestpatent.createblandWork();
    ipcWork = CreateTestpatent.createipcWork();
  }

  /**
   * Pulikations Sprache en
   * -> deutsche Übersetzung in Beschreibung
   * -> französische Übersetzung in Ereignistext
   */
  public void testPublSpracheEn()
  {
    String text = CreateTestpatent.createBeschreibung1();
    EpbandVO epbandVO = new EpbandVO();

    parseWithCatchingEx( text, epbandVO );

    assertEquals( "Falscher Titel in schutit", "Titel deutsch", epbandVO.schutitVO.beschreibung );
    assertEquals( "Kein ereigTXT gefunden", 1, epbandVO.ereigTxtAL.size() );
    EreigTxtVO ereigTxtVO = new EreigTxtVO();
    ereigTxtVO = epbandVO.ereigTxtAL.get( 0 );
    assertEquals( "Falscher Titel in EreigTxT", "Titel franz", ereigTxtVO.ereigTxt );
    assertEquals( "Falscher ereigTxtCd", 1, ereigTxtVO.ereigTxtCd );
    assertEquals( "Falsches EreigIdat", 1996111305, ereigTxtVO.ereigIdat );
  }

  /**
   * Pulikations Sprache fr
   * -> französiche Übersetzung in Beschreibung
   * -> kein Ereignistext
   */
  public void testPublSpracheFr()
  {
    String text = CreateTestpatent.createBeschreibung2();
    EpbandVO epbandVO = new EpbandVO();

    parseWithCatchingEx( text, epbandVO );

    assertEquals( "Falscher Titel in schutit", "Titel franz", epbandVO.schutitVO.beschreibung );
    assertEquals( "EreigTXT gefunden", 0, epbandVO.ereigTxtAL.size() );
  }

  /**
   * Pulikations Sprache de, Titel mehr als 340 Zeichen
   * -> deutsche Übersetzung in Beschreibung, Beschreibung gekürzt auf 340 Zeichen
   * -> kein Ereignistext
   */
  public void testPublSpracheDe()
  {
    String text = CreateTestpatent.createBeschreibung3();
    EpbandVO epbandVO = new EpbandVO();

    parseWithCatchingEx( text, epbandVO );

    assertTrue( "Falscher Titel in schutit", epbandVO.schutitVO.beschreibung.indexOf( "deutsch" ) > 0 );
    assertTrue( "Beschreibung nicht auf 340 Zeichen gekürzt", epbandVO.schutitVO.beschreibung.length() == 340 );
    assertTrue( "Upp-Beschreibung nicht auf 340 Zeichen gekürzt", epbandVO.schutitVO.uppBeschreibung.length() == 340 );
    assertEquals( "EreigTXT gefunden", 0, epbandVO.ereigTxtAL.size() );
  }

  /**
   * Pulikations Sprache weder de, en, fr Schutit-Nr vorhanden
   * -> Beschreibung null
   * -> kein Ereignistext
   * -> Meldung ins Logfile
   */
  public void testPublSpracheInvalid()
  {
    String text = CreateTestpatent.createBeschreibung4();
    EpbandVO epbandVO = new EpbandVO();

    parseWithCatchingEx( text, epbandVO );

    assertNull( "Titel muss null sein", epbandVO.schutitVO.beschreibung );
    assertEquals( "EreigTXT gefunden", 0, epbandVO.ereigTxtAL.size() );
  }

  /**
   * Gesuch Pulikations Sprache de, deutsche Übersetzung leer, keine Schutit-Nr
   * -> keine Verarbeitung
   */
  public void testPublSpracheNull()
  {
    String text = CreateTestpatent.createBeschreibung5();
    EpbandVO epbandVO = new EpbandVO();

    try
    {
      parse( text, epbandVO );
      fail( "Datensatz darf nicht verarbeitet werden, da weder Titel noch Schutit-Nr vorhanden sind" );
    }
    catch ( Exception ex )
    {
      assertTrue( ex instanceof ProcessingException );
    }
  }

  /**
   * Gesuch Pulikations Sprache de, deutsche Übersetzung leer, keine Schutit-Nr
   * -> keine Verarbeitung
   */
  public void testPatentWithoutTitle()
  {
    String text = CreateTestpatent.createBeschreibung6();
    EpbandVO epbandVO = new EpbandVO();

    parseWithCatchingEx( text, epbandVO );

    assertNull( "Titel muss null sein", epbandVO.schutitVO.beschreibung );
    assertEquals( "EreigTXT gefunden", 0, epbandVO.ereigTxtAL.size() );
  }

  /**
   * Parsing des Textes mit Exceptionbehandlung
   * @param text
   * @param epbandVO
   */
  private void parseWithCatchingEx( String text, EpbandVO epbandVO )
  {
    try
    {
      this.parse( text, epbandVO );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  /**
   * Reines Parsen des Textes ohne Exceptionhandling
   * @param text
   * @param epbandVO
   * @throws Exception
   */
  private void parse( String text, EpbandVO epbandVO ) throws Exception
  {
    XMLDoc xml = new XMLDoc( text );
    ParseSGML parsesgml = new ParseSGML( blandWork );
    parsesgml.setData( epbandVO, xml );
    parsesgml.doParseSGML();
  }
}
