package ch.ige.epband.clientneu.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.Date;

import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;

import ch.ige.epband.clientneu.ListeMutationLimitation;
import ch.ige.epband.clientneu.ListeMutationRevocation;
import ch.ige.epband.clientneu.Listen;
import ch.ige.epband.util.XMLDoc;

/**
 * Testen der Listenfuktionalität
 * 
 *
 * <p>Copyright (c) 2006, Swiss Federal Institute of Intellectual Property</p>
 * @author   wild
 * @version  $Id: TestListen.java,v 1.18 2008/05/22 16:13:06 ruegsegger Exp $
 */
public class TestListen extends TestCase
{
  private String tagesDatum = null;

  private SimpleDateFormat sdfnew = new SimpleDateFormat( "dd.MM.yyyy" );

  /**
   * Initialisieren des Objekts
   * @param s
   */
  public TestListen( String s )
  {
    super( s );
    BasicConfigurator.configure();
  }

  @Override
  protected void setUp()
  {
    tagesDatum = sdfnew.format( new Date() );
  }

  /**
   * Testmethode Listen erstellen
   *
   */
  public void testDoListen()
  {
    StringBuffer sb = new StringBuffer();
    Listen listen = new Listen();

    String filePath = "C:\\jp\\projects\\epband\\testfiles\\test-listen.txt";

    try
    {
      char[] ch = new char[8192];

      FileInputStream fi = new FileInputStream( new File( filePath ) );
      InputStreamReader ir = new InputStreamReader( fi );

      int j;

      while ( ( j = ir.read( ch ) ) > 0 )
      {
        sb.append( ch, 0, j );
      }

      fi.close();

      // -1 if not found
      int start = sb.indexOf( "<?xml" );
      int end = sb.indexOf( "</ep-bulletin>" );
      String xmlString = null;

      while ( start != -1 && end != -1 )
      {
        xmlString = sb.substring( start, end + 14 );

        XMLDoc xml = new XMLDoc( xmlString );
        listen.doListen( xml );

        start = sb.indexOf( "<?xml", end );
        end = sb.indexOf( "</ep-bulletin>", start );
      }

      ir.close();

      sendListen( listen );
    }
    catch ( Exception ex )
    {
      fail( ex.getMessage() );
      System.out.println( ex );
    }
  }

  /**
   * Testmethode Für Limitation-Teil der Listen
   */
  public void testLimitationProcedure()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-liste-limitation.xml";
    XMLDoc xml;
    StringBuffer sb = new StringBuffer();
    ListeMutationLimitation listen = new ListeMutationLimitation();

    try
    {
      FileInputStream fi = new FileInputStream( filename );
      xml = new XMLDoc( fi );
      listen.genLimitationText( sb, xml );
    }
    catch ( Exception ex )
    {
      fail( ex.getMessage() );
    }
    StringBuffer result = new StringBuffer();

    result
        .append( "<inhalt><ktext>Beschränkung</ktext><text>Datum: 14.09.2008</text></inhalt><inhalt><text>Limitations kind: 01; eingereicht innerhalb Einspruchsfrist</text></inhalt>"
            + "<inhalt><text>Decision code: 01; zulässig</text></inhalt>"
            + "<inhalt><ktext>Beschränkung Zurückweisung</ktext><text>Datum: 08.08.2008</text></inhalt>"
            + "<inhalt><ktext>Beschränkung</ktext><text>Datum: 09.10.2008</text></inhalt>"
            + "<inhalt><text>Limitations kind: 02; eingereicht nach Einspruchsfrist</text></inhalt>"
            + "<inhalt><text>Decision code: 02; nicht zulässig</text></inhalt>"
            + "<inhalt><ktext>Beschränkung Gutheissung</ktext><text>Absendedatum: 23.04.2008</text></inhalt><inhalt><text>Entscheiddatum: 24.04.2008</text></inhalt>"
            + "<inhalt><ktext>Beschränkung</ktext><text>Datum: 15.05.2008</text></inhalt>"
            + "<inhalt><text>Limitations kind: 01; eingereicht innerhalb Einspruchsfrist</text></inhalt>"
            + "<inhalt><text>Decision code: 03; gilt als nicht eingereicht</text></inhalt>"
            + "<inhalt><ktext>Beschränkung Zurückweisung</ktext><text>Datum: 08.08.2009</text></inhalt>"
            + "<inhalt><ktext>Beschränkung</ktext><text>Datum: 10.10.2008</text></inhalt>"
            + "<inhalt><text>Limitations kind: 02; eingereicht nach Einspruchsfrist</text></inhalt>"
            + "<inhalt><text>Decision code: 04; gilt als nicht eingereicht; Einspruch hängig</text></inhalt>"
            + "<inhalt><ktext>Beschränkung Zurückweisung</ktext></inhalt>"
            + "<inhalt><ktext>Beschränkung Gutheissung</ktext><text>Absendedatum: 28.02.2008</text></inhalt><inhalt><text>Entscheiddatum: 06.05.2008</text></inhalt>" );

    //System.out.println( sb );
    //System.out.println( result );

    assertEquals( result.toString(), sb.toString() );

  }

  /**
   * Testmethode Für Revocation-Teil der Listen
   */
  public void testRevocationProcedure()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-liste-revocation.xml";
    XMLDoc xml;
    StringBuffer sb = new StringBuffer();
    ListeMutationRevocation listen = new ListeMutationRevocation();

    try
    {
      FileInputStream fi = new FileInputStream( filename );
      xml = new XMLDoc( fi );
      listen.genRevocationText( sb, xml );
    }
    catch ( Exception ex )
    {
      fail( ex.getMessage() );
    }
    StringBuffer result = new StringBuffer();

    result.append( "<inhalt><ktext>Widerruf Inhaber beantragt</ktext><text>Datum Antrag: 28.02.2008</text></inhalt>"
        + "<inhalt><text>Entscheid Widerruf: 01; zulässig</text></inhalt>" + "<inhalt><text>Datum Entscheid: 13.03.2008</text></inhalt>"
        + "<inhalt><ktext>Widerruf Inhaber beantragt</ktext><text>Datum Antrag: 28.02.2007</text></inhalt>"
        + "<inhalt><text>Entscheid Widerruf: 02; unzulässig</text></inhalt>" + "<inhalt><text>Datum Entscheid: 13.03.2007</text></inhalt>"
        + "<inhalt><ktext>Widerruf Inhaber beantragt</ktext><text>Datum Antrag: 28.02.2007</text></inhalt>"
        + "<inhalt><text>Entscheid Widerruf: 03; gilt als nicht eingelegt</text></inhalt><inhalt><text>Datum Entscheid: 13.03.2007</text></inhalt>" );

    //System.out.println( sb );
    //System.out.println( result );

    assertEquals( result.toString(), sb.toString() );

  }

  private void sendListen( Listen listen ) throws FileNotFoundException, IOException
  {
    listen.sendJIPPS();

    String filename = "c:\\temp\\ep_001.xml";
    String xmlString = readFile( filename );

    System.out.println( xmlString );

    String ep001 = "<?xml version = \"1.0\" encoding=\"ISO-8859-15\"?> <liste><head><titel>Gesuche nach Patent Nummern</titel><datum>"
        + tagesDatum
        + "</datum><publtxt>Veröffentlichungsdatum</publtxt><publdat>17.07.2002 2002/29</publdat></head><inhalt><patnr>00 091 500.9</patnr><gesuchnr>99400100.1</gesuchnr></inhalt><inhalt><patnr>00 091 501.7</patnr><gesuchnr>99400101.1</gesuchnr></inhalt></liste>";
    assertEquals( ep001, xmlString );

    filename = "c:\\temp\\ep_002.xml";
    xmlString = readFile( filename );

    System.out.println( xmlString );
    String ep002 = "<?xml version = \"1.0\" encoding=\"ISO-8859-15\"?> <liste><ep><head><datum>"
        + tagesDatum
        + "</datum><epnr>00 000 154.5 B1</epnr></head><inhalt><ktext>Text</ktext><text>Derzeit sind die WIPO-Publikationsdaten A3 nicht verfuegbar.</text></inhalt><inhalt><ktext>Text</ktext><text>WIPO A3 publication data is not currently available.</text></inhalt><inhalt><ktext>Text</ktext><text>L&apos;OMPI donnes de publication A3 ne sont pas actuellement disponibles.</text></inhalt></ep></liste>";

    assertEquals( ep002, xmlString );

    filename = "c:\\temp\\ep_003.xml";
    xmlString = readFile( filename );

    System.out.println( xmlString );

    filename = "c:\\temp\\ep_004.xml";
    xmlString = readFile( filename );

    System.out.println( xmlString );

    String ep004 = "<?xml version = \"1.0\" encoding=\"ISO-8859-15\"?> <liste><head><titel>Eingetragene Patente nach Nummern</titel><datum>"
        + tagesDatum
        + "</datum><publtxt>Veröffentlichungsdatum</publtxt><publdat>17.07.2002 2002/29</publdat></head><inhalt><patnr>00 000 154.5</patnr></inhalt><inhalt><patnr>00 000 943.1</patnr></inhalt><inhalt><patnr>00 001 503.2</patnr></inhalt><inhalt><patnr>00 001 530.5</patnr></inhalt><inhalt><patnr>00 001 532.1</patnr></inhalt></liste>";
    assertEquals( ep004, xmlString );
  }

  private String readFile( String filename ) throws FileNotFoundException, IOException
  {
    StringBuffer xmlString = new StringBuffer();
    String line;

    BufferedReader in = new BufferedReader( new FileReader( filename ) );
    while ( ( line = in.readLine() ) != null )
    {
      xmlString.append( line );
    }
    in.close();
    return xmlString.toString();
  }
}
