package ch.ige.epband.clientneu.test;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;

import ch.ige.epband.clientneu.ParseSGML;
import ch.ige.epband.clientneu.Pat;
import ch.ige.epband.clientneu.Util;
import ch.ige.epband.util.XMLDoc;
import ch.ige.epband.vo.EpbandVO;
import ch.ige.epband.vo.EreignisVO;
import ch.ige.util.IGEDate;

/**
 * Testklasse XML-Parsing
 * 
 *
 * <p>Copyright (c) 2006, Swiss Federal Institute of Intellectual Property</p>
 * @author   wild
 * @version  $Id: TestParseSGML.java,v 1.17 2008/05/26 08:36:55 ruegsegger Exp $
 */
public class TestParseSGML extends TestCase
{

  /**
   * Lokale Liste für blandWork
   */
  public static List<String> blandWork;

  /**
   * Lokale Liste für IPC
   */
  public static List<String> ipcWork;

  private SimpleDateFormat sdf = new SimpleDateFormat( "dd.MM.yyyy" );

  private String currentDate = null;

  /**
   * Initialisierung
   * @param s
   */
  public TestParseSGML( String s )
  {
    super( s );
    sdf.setLenient( false );
    BasicConfigurator.configure();
  }

  @Override
  protected void setUp()
  {
    blandWork = CreateTestpatent.createblandWork();
    ipcWork = CreateTestpatent.createipcWork();
    currentDate = sdf.format( new Date() );
  }

  /**
   * Parsing von WIPO-XML
   */
  public void testWIPO()
  {
    String wopat = CreateTestpatent.createwopatent();
    EpbandVO epbandVO = new EpbandVO();
    EreignisVO ereignisVO = new EreignisVO();

    try
    {
      XMLDoc xml = new XMLDoc( wopat );
      ParseSGML parsesgml = new ParseSGML( blandWork );
      parsesgml.setData( epbandVO, xml );
      parsesgml.doParseSGML();
    }
    catch ( Exception ex )
    {
      fail( ex.getMessage() );
      ex.printStackTrace();
    }

    assertTrue( "WO nicht erkannt", epbandVO.wipo );
    assertEquals( "Anzahl Ereignisse falsch", 1, epbandVO.ereigAL.size() );
    ereignisVO = epbandVO.ereigAL.get( 0 );
    assertEquals( "EreigIdat ist falsch", 2001022805, ereignisVO.ereigIdat );
    assertEquals( "maNr", 199, ereignisVO.maNr );
    assertNull( "publKey", ereignisVO.publKey );
    assertEquals( "ereigStatNr", 20, ereignisVO.ereigStatNr );
    assertNull( "imaNr", ereignisVO.iMaNr );
    assertEquals( "artNr", "30060", ereignisVO.artNr );
    assertNull( "adrNr", ereignisVO.adrNr );
    assertNull( "fristAblDat", ereignisVO.fristablDat );
    assertNull( "abschlDat", ereignisVO.abschlDat );
    assertNull( "abschmutDat", ereignisVO.abschlmutDat );
    assertEquals( "erfassDat", currentDate, sdf.format( ereignisVO.erfassDat ) );
    assertNull( "pmmblSeite", ereignisVO.pmmblSeite );
    assertNull( "pmmblJahr", ereignisVO.pmmblJahr );
    assertEquals( "kzKorrekt", 0, ereignisVO.kzKorrekt );
    assertEquals( "regSrtat", 8, ereignisVO.regStat );
    assertEquals( "schutitNr", "2000007087", epbandVO.schutitVO.schutitNr );
    assertNull( "publDat", ereignisVO.publDat );
    assertEquals( "publStat", 0, ereignisVO.publStat );
  }

  /**
   * Parsing von Zurücknahme
   * 
   * @throws ParseException
   */
  public void testchzuruecknahme() throws ParseException
  {
    String chzur = CreateTestpatent.createchzurueckgesuch();
    EpbandVO epbandVO = new EpbandVO();
    EreignisVO ereignisVO = new EreignisVO();
    Date testDate = IGEDate.stringToDate( "17.07.2002" );

    try
    {
      XMLDoc xml = new XMLDoc( chzur );
      ParseSGML parsesgml = new ParseSGML( blandWork );
      parsesgml.setData( epbandVO, xml );
      parsesgml.doParseSGML();
    }
    catch ( Exception ex )
    {
      fail();
      ex.printStackTrace();
    }

    assertTrue( "CHzuruecknahme nicht erkannt", epbandVO.chzuruecknahme );

    assertEquals( "schutitNr", "000001545", epbandVO.schutitVO.schutitNr );
    assertEquals( "gesuchNr", "781001698", epbandVO.schutitVO.gesuchNr );
    assertEquals( "Löschdatum", testDate, epbandVO.schutitVO.loeschDat );
    assertEquals( "MutMa", 199, epbandVO.schutitVO.mutMa.intValue() );
    assertNull( "Mutdatum", epbandVO.schutitVO.mutDat );
    assertEquals( "Anzahl Ereignisse falsch", 1, epbandVO.ereigAL.size() );
    ereignisVO = epbandVO.ereigAL.get( 0 );
    assertEquals( "ArtNr", "20250", ereignisVO.artNr );
    assertEquals( "Regstat", 8, ereignisVO.regStat );
    assertEquals( "EreigStatNr", 20, ereignisVO.ereigStatNr );
    assertEquals( "EreigIdat", 2002071705, ereignisVO.ereigIdat );
    assertEquals( "MaNr", 199, ereignisVO.maNr );
    assertNull( "publKey", ereignisVO.publKey );
    assertNull( "imaNr", ereignisVO.iMaNr );
    assertNull( "adrNr", ereignisVO.adrNr );
    assertNull( "fristAblDat", ereignisVO.fristablDat );
    assertNull( "abschlDat", ereignisVO.abschlDat );
    assertNull( "abschmutDat", ereignisVO.abschlmutDat );
    assertEquals( "erfassDat", currentDate, sdf.format( ereignisVO.erfassDat ) );
    assertNull( "pmmblSeite", ereignisVO.pmmblSeite );
    assertNull( "pmmblJahr", ereignisVO.pmmblJahr );
    assertEquals( "kzKorrekt", 0, ereignisVO.kzKorrekt );
    assertNull( "publDat", ereignisVO.publDat );
    assertEquals( "publStat", 0, ereignisVO.publStat );
  }

  /**
   * Kein Einspruch eingegangen
   * 
   */
  public void testkeinEinspruch()
  {
    String keinsp = CreateTestpatent.createkeinEinspruch();
    EpbandVO epbandVO = new EpbandVO();
    EreignisVO ereignisVO = new EreignisVO();

    try
    {
      XMLDoc xml = new XMLDoc( keinsp );
      ParseSGML parsesgml = new ParseSGML( blandWork );
      parsesgml.setData( epbandVO, xml );
      parsesgml.doParseSGML();
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail();
    }

    assertTrue( "Kein Einspruch nicht erkannt", epbandVO.keinEinspruch );
    assertEquals( "Anzahl Ereignisse falsch", 1, epbandVO.ereigAL.size() );

    ereignisVO = epbandVO.ereigAL.get( 0 );
    assertEquals( "ArtNr", "20450", ereignisVO.artNr );
    assertEquals( "Regstat", 9, ereignisVO.regStat );
    assertEquals( "EreigStatNr", 20, ereignisVO.ereigStatNr );
    assertEquals( "EreigIdat", 1983021505, ereignisVO.ereigIdat );
    assertEquals( "MaNr", 199, ereignisVO.maNr );
    assertNull( "publKey", ereignisVO.publKey );
    assertNull( "imaNr", ereignisVO.iMaNr );
    assertNull( "adrNr", ereignisVO.adrNr );
    assertNull( "fristAblDat", ereignisVO.fristablDat );
    assertNull( "abschlDat", ereignisVO.abschlDat );
    assertNull( "abschmutDat", ereignisVO.abschlmutDat );
    assertEquals( "erfassDat", currentDate, sdf.format( ereignisVO.erfassDat ) );
    assertNull( "pmmblSeite", ereignisVO.pmmblSeite );
    assertNull( "pmmblJahr", ereignisVO.pmmblJahr );
    assertEquals( "kzKorrekt", 0, ereignisVO.kzKorrekt );
    assertNull( "publDat", ereignisVO.publDat );
    assertEquals( "publStat", EreignisVO.PUBLSTAT_READY_PUBLICATE, ereignisVO.publStat );
  }

  /**
   * Aufbereiten der PCT Nummer
   *
   */
  public void testprepPCTNr()
  {
    String pctNr = "AB0212345";

    try
    {
      Pat pat = new Pat();
      String str = pat.prepPctNr( pctNr );

      assertEquals( "PCT AnmeldeNr falsch", "AB2002012345", str );

      pctNr = "CH03012345";
      str = pat.prepPctNr( pctNr );
      assertEquals( "PCT AnmeldeNr 2 falsch", "CH2003012345", str );
      pctNr = "GB2003012345";
      str = pat.prepPctNr( pctNr );
      assertEquals( "PCT AnmeldeNr 3 falsch", "GB2003012345", str );
    }
    catch ( Exception ex )
    {
      fail( ex.getMessage() );
    }
  }

  /**
   * WO-Nummer aufbereiten.
   *
   */
  public void testprepWONr()
  {
    String WONr = "WO0112345";

    String str = Util.prepWONr( WONr );
    assertEquals( "PCT AnmeldeNr falsch", "2001012345", str );

    WONr = "0112345";
    str = Util.prepWONr( WONr );
    assertEquals( "PCT AnmeldeNr falsch", "2001012345", str );

    WONr = "02067899";
    str = Util.prepWONr( WONr );
    assertEquals( "PCT AnmeldeNr falsch", "2002067899", str );

    WONr = "2003067899";
    str = Util.prepWONr( WONr );
    assertEquals( "PCT AnmeldeNr falsch", "2003067899", str );
  }
}
