package ch.ige.epband.server;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;

import ch.ige.epband.exception.EpbandDelException;
import ch.ige.epband.exception.ProcessingException;
import ch.ige.epband.vo.EpbandVO;
import ch.ige.epband.vo.IpcVO;
import ch.ige.util.ServiceLocator;

/**
 * BusinessDelegate für Epband
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * 
 * @author Martin Trachsel
 * @version $Id: EpbandDelegate.java,v 1.31 2009/01/12 14:21:07 blank_blt Exp $
 */

public class EpbandDelegate
{
  private static EpbandDelegate epbandDelegate;

  private EpbandFacade facade;

  @SuppressWarnings( "unchecked" )
  private static final Class HOME_CLASS = EpbandFacadeHome.class;

  // Singleton
  @SuppressWarnings( "unchecked" )
  private EpbandDelegate() throws Exception
  {
    EpbandFacadeHome home = (EpbandFacadeHome) ServiceLocator.getInstance().getHome( EpbandFacadeHome.JNDI_NAME, HOME_CLASS );
    facade = home.create();
  }

  /**
   * Gibt die einzige Instanz zurück (Singleton)
   * 
   * @return EpbandDelegate
   * @throws Exception
   */
  public static EpbandDelegate getInstance() throws Exception
  {
    if ( epbandDelegate == null )
    {
      epbandDelegate = new EpbandDelegate();
    }
    return epbandDelegate;
  }

  /**
   * Kreiert ein Patent DBTimeoutExcpetion wird als ApplicationDBTimeoutException zurückgegeben.
   * ApplicationDBTimeoutException ist eine ApplicationException somit erfolgt nicht wie bei der SystemException
   * automatisch ein Rollback der Trankaktion. Mit setRollbackOnly() wird ein Flag gesetzt, sodass ein Rollback der
   * Transaktion erfolgt. Bei ApplicationExcpetion wird BeanInstanz(AdresseEpFacade) nicht zerstört, was bei einer
   * SystemException der Fall wäre
   * 
   * @param epbandVO Value Object mit den Patentdaten
   * @throws EpbandDelException bei fehlerhatem Löschen des EP
   * @throws Exception
   */
  public void createEp( final EpbandVO epbandVO ) throws EpbandDelException, Exception
  {
    facade.createEp( epbandVO );
  }

  /**
   * Holt alle BLänder.
   * 
   * @return blandWork Liste mit allen bländer
   * @throws Exception
   */
  @SuppressWarnings( "unchecked" )
  public ArrayList<String> getAllBland() throws Exception
  {
    return facade.getAllBland();
  }

  /**
   * Holt alle Mappings uniode iso8859
   * 
   * @return Map
   * @throws Exception
   */
  @SuppressWarnings( "unchecked" )
  public HashMap<String, String> getAllMappingUnicode() throws Exception
  {
    return facade.getAllMappingUnicode();
  }

  /**
   * Holt einen IPC-Eintrag
   * 
   * @param ipcSymb String IPC-Symbol
   * @param gueltigVon
   * @throws Exception
   * @return IpcVO IPC-Eintrag
   */
  public IpcVO getIpc( final String ipcSymb, final Date gueltigVon ) throws Exception
  {
    return facade.getIpc( ipcSymb, gueltigVon );
  }

  /**
   * Holt einen IPC-Eintrag
   * 
   * @param ipcSymb String IPC-Symbol
   * @throws Exception
   * @return IpcVO IPC-Eintrag
   */
  public IpcVO getYoungestIpc( final String ipcSymb ) throws Exception
  {
    return facade.getYoungestIpc( ipcSymb );
  }

  /**
   * @param epbandVO
   * @throws ProcessingException
   * @throws Exception
   */
  public void createEreigWipo( final EpbandVO epbandVO ) throws ProcessingException, Exception
  {
    facade.createEreigWipo( epbandVO );
  }

  /**
   * @param epbandVO
   * @throws ProcessingException
   * @throws Exception
   */
  public void createCHzurueck( final EpbandVO epbandVO ) throws ProcessingException, Exception
  {
    facade.createCHzurueck( epbandVO );
  }

  /**
   * @param epbandVO
   * @throws ProcessingException
   * @throws Exception
   */
  public void createkeinEinspruch( final EpbandVO epbandVO ) throws ProcessingException, Exception
  {
    facade.createkeinEinspruch( epbandVO );
  }
}
