package ch.ige.epband.util;

import java.util.Map;

import ch.ige.epband.server.EpbandDelegate;

/**
 * <p>
 * Beschreibung: class for replacing unicode values not included inside database character set ISO8859-15
 * <p>
 * Copyright: Copyright (c) 2006
 * </p>
 * <p>
 * Organisation: Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * 
 * @author A.Rueegsegger
 * @version $Id: MappingUnicode.java,v 1.8 2008/03/10 15:57:10 wild Exp $
 */
public class MappingUnicode
{
  private static MappingUnicode mappingUnicode;

  private static Map<String, String> mapUnicode = null;

  /**
   * loading all values from table mapping_unicode into hashMap
   */
  private MappingUnicode() throws Exception
  {
    mapUnicode = EpbandDelegate.getInstance().getAllMappingUnicode();
  }

  /**
   * Gibt die einzige Instanz zurück (Singleton)
   * 
   * @return PctDelegate
   * @throws Exception
   */
  public static MappingUnicode getInstance() throws Exception
  {
    if ( mappingUnicode == null )
    {
      mappingUnicode = new MappingUnicode();
    }
    return mappingUnicode;
  }

  /**
   * replaces unicode values not included inside ISO8859-15 mapping 
   * with database table mapping_unicode
   * 
   * @param str String
   * @return returnValue mit umgewandelten Zeichen
   */
  public String replaceUnicodeValues( String str )
  {
    String key = null;
    String value = null;
    String returnValue = str;

    if ( !mapUnicode.isEmpty() )
    {

      for ( Map.Entry<String, String> entry: mapUnicode.entrySet() )
      {
        key = entry.getKey();
        value = entry.getValue();

        returnValue = returnValue.replaceAll( key, ( value == null ? "" : value ) );
      }
    }
    return returnValue;
  }
}
