package ch.ige.epband.vo;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;

import ch.ige.util.vo.RegadrVO;

/**
 * <p>Überschrift:    Europäische Patente (EP) einlesen</p>
 * <p>Beschreibung:   Value Object dass die Daten für ein EP enthält</p>
 * <p>Copyright:      Copyright (c) 2002</p>
 * <p>Organisation:   IGE</p>
 * @author            M. Trachsel
 * @version           $Id: EpbandVO.java,v 1.15 2008/07/10 10:16:48 wild Exp $
 */

public class EpbandVO implements Serializable
{
  /**
   * Defaultwert für die serialVersionUID
   */
  private static final long serialVersionUID = 1L;

  /**
   * Mitarbeiternummer, die für die DB-Einträge benutzt wird
   */
  public static final int MA_NR = 199;

  /**
   * Daten der Tabelle Schutit
   */
  public SchutitVO schutitVO;

  /**
   * Daten der Tabelle pat
   */
  public PatVO patVO;

  /**
   * Liste mit allen Ereignissen
   */
  public ArrayList<EreignisVO> ereigAL = new ArrayList<EreignisVO>();

  /**
   * Sammlung von EreigTxtVO's
   */
  public ArrayList<EreigTxtVO> ereigTxtAL = new ArrayList<EreigTxtVO>();

  /**
   * Liste aller Prioritäten
   */
  public ArrayList<PrioritaetVO> prioAL = new ArrayList<PrioritaetVO>();

  /**
   * Liste aller pat_Ipc Einträgen
   */
  public ArrayList<PatIpcVO> patIpcAL = new ArrayList<PatIpcVO>();

  /**
   * alle vorhandenen Registeradressen
   */
  public ArrayList<RegadrVO> regAdrAL = new ArrayList<RegadrVO>();

  /**
  * alle Einträge für die Tabelle Schutit_Regadr
  */
  public ArrayList<SchutitRegadrVO> schutitregAdrAL = new ArrayList<SchutitRegadrVO>();

  /**
   * es handelt sich um ein PCT
   */
  public boolean wipo = false;

  /**
   * CH zurücknahme eintragen
   */
  public boolean chzuruecknahme = false;

  /**
   * Kein Einspruch bei eingetragenen EP
   */
  public boolean keinEinspruch = false;

  /**
   * Änderungsstatus
   *     N = New
   *     R = Replacement
   *     C = Correction (not yet used)
   *     D = Deletion
   */
  public String epStatus = "";

  /**
   * Patentart:
   *    A1, A2, A3 =  EP Patentgesuche und Euro-PCT's,
   *                  welche noch in keiner der drei Sprachen publiziert wurde.
   *    B1, B2     =  EP-Patente (beim EP-Bulletin in unterschiedlichen Sektionen aufgelistet
   */
  public String epArt = "";

  /**
   * Anmeldedatum als String
   */
  public String anmeldedateString = "";

  /**
   * Anmeldedatum als Date
   */
  public Date anmeldeDate = null;

  /**
  * EreigIdat
  */
  public int anmeldeIdat = 0;

  //----------------------------------------------------------------------------
  // methods
  //----------------------------------------------------------------------------
  /**
   * Gibt an ob es sich um ein Patent handelt
   * @return true falls es sich um ein Patent handelt
   */
  public boolean isPatent()
  {
    return epArt.substring( 0, 1 ).equals( "B" );
  }

  /**
   * Gibt an ob es sich um ein Patent handelt
   * @return true falls es sich um ein Patent handelt
   */
  public boolean isGesuch()
  {
    return epArt.substring( 0, 1 ).equals( "A" );
  }

  /**
   * Gibt den Inhalt des ValueObject SchutitEmpfadrVO zurück
   * @return Inhalt von SchutitEmpfadrVO
   */
  @Override
  public String toString()
  {
    StringBuffer buf = new StringBuffer();
    StringBuffer tmp = new StringBuffer();

    buf.append( "  wipo             = <" + this.wipo + ">\n" );
    buf.append( "  chzuruecknahme   = <" + this.chzuruecknahme + ">\n" );
    buf.append( "  keinEinspruch    = <" + this.keinEinspruch + ">\n" );
    buf.append( "  epStatus         = <" + this.epStatus + ">\n" );
    buf.append( "  epArt            = <" + this.epArt + ">\n" );
    buf.append( "  anmeldedateString= <" + this.anmeldedateString + ">\n" );
    buf.append( "  anmeldeDate      = <" + this.anmeldeDate + ">\n" );
    buf.append( "  anmeldeIdat      = <" + this.anmeldeIdat + ">\n" );

    // SchutitVO
    if ( schutitVO == null )
    {
      buf.append( "  schutitVO        = <keine>\n" );
    }
    else
    {
      buf.append( "  schutitVO        \n" + schutitVO.toString() + "\n" );
    }

    // PatVO
    if ( patVO == null )
    {
      buf.append( "  pat             = <keine>\n" );
    }
    else
    {
      buf.append( "  patVO            \n" + patVO.toString() + "\n" );
    }

    // ArrayList EreignisVO
    if ( ereigAL == null || ereigAL.size() == 0 )
    {
      buf.append( "  ereigAL         = <keine>\n" );
    }
    else
    {
      tmp = new StringBuffer();

      for ( int i = 0; i < ereigAL.size(); i++ )
      {
        tmp.append( "EreignisVO " + i + "\n" + ( ereigAL.get( i ) ).toString() + "\n" );
      }

      buf.append( "  ereigAL         \n" + tmp.toString() );
    }

    // ArrayList EreiTxtVO
    if ( ereigTxtAL == null || ereigTxtAL.size() == 0 )
    {
      buf.append( "  ereigTxtAL      = <keine>\n" );
    }
    else
    {
      tmp = new StringBuffer();

      for ( int i = 0; i < ereigTxtAL.size(); i++ )
      {
        tmp.append( "EreigTxtVO " + i + "\n" + ( ereigTxtAL.get( i ) ).toString() + "\n" );
      }

      buf.append( "  ereigTxtAL      \n" + tmp.toString() );
    }

    // ArrayList PrioVO
    if ( prioAL == null || prioAL.size() == 0 )
    {
      buf.append( "  prioAL          = <keine>\n" );
    }
    else
    {
      tmp = new StringBuffer();

      for ( int i = 0; i < prioAL.size(); i++ )
      {
        tmp.append( "PrioVO " + i + "\n" + ( prioAL.get( i ) ).toString() + "\n" );
      }

      buf.append( "  prioAL      \n" + tmp.toString() );
    }

    // ArrayList PatIpcVO
    if ( patIpcAL == null || patIpcAL.size() == 0 )
    {
      buf.append( "  patIpcAL        = <keine>\n" );
    }
    else
    {
      tmp = new StringBuffer();

      for ( int i = 0; i < patIpcAL.size(); i++ )
      {
        tmp.append( "PatIpcVO " + i + "\n" + ( patIpcAL.get( i ) ).toString() + "\n" );
      }

      buf.append( "  patIpcAL      \n" + tmp.toString() );
    }

    // ArrayList RegAdr
    if ( regAdrAL == null || regAdrAL.size() == 0 )
    {
      buf.append( "  regAdrAL        = <keine>\n" );
    }
    else
    {
      tmp = new StringBuffer();

      for ( int i = 0; i < regAdrAL.size(); i++ )
      {
        tmp.append( "RegAdrVO " + i + "\n" + ( regAdrAL.get( i ) ).toString() + "\n" );
      }

      buf.append( "  regAdrAL      \n" + tmp.toString() );
    }

    // ArrayList SchutitRegAdrVO
    if ( schutitregAdrAL == null || schutitregAdrAL.size() == 0 )
    {
      buf.append( "  schutitregAdrAL = <keine>\n" );
    }
    else
    {
      tmp = new StringBuffer();

      for ( int i = 0; i < schutitregAdrAL.size(); i++ )
      {
        tmp.append( "SchutitRegAdrVO " + i + "\n" + ( schutitregAdrAL.get( i ) ).toString() + "\n" );
      }

      buf.append( "  schutitregAdrAL      \n" + tmp.toString() );
    }

    return buf.toString();
  }
}
