package ch.ige.image;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel.MapMode;

import javax.imageio.stream.ImageInputStreamImpl;

/**
 * Wraps a ByteBuffer as an ImageInputStream.
 * <p>
 * Copyright: Copyright (c) 2006
 * </p>
 * <p>
 * Organisation: IGE
 * </p>
 * 
 * @author Michael Locher
 * @version 1.0
 */
public class ByteBufferImageInputStream extends ImageInputStreamImpl
{

  private ByteBuffer imageData;

  public static ByteBufferImageInputStream mapFromFile( File imageFile ) throws FileNotFoundException, IOException
  {
    return new ByteBufferImageInputStream( IOUtil.asByteBuffer( imageFile, true, MapMode.READ_ONLY ) );
  }

  public ByteBufferImageInputStream( ByteBuffer imageData )
  {
    this.imageData = imageData.duplicate();
    this.imageData.rewind();
  }

  @Override
  public void seek( long pos ) throws IOException
  {
    this.imageData.position( (int) pos );
    super.seek( pos );
  }

  @Override
  public int read() throws IOException
  {
    checkClosed();
    bitOffset = 0;
    if ( this.imageData.hasRemaining() )
    {
      streamPos++;
      return 0xFF & imageData.get();
    }
    else
    {
      return -1;
    }
  }

  @Override
  public int read( byte[] b, int off, int len ) throws IOException
  {
    checkClosed();
    if ( off < 0 || len < 0 || off + len > b.length || off + len < 0 )
    {
      throw new IndexOutOfBoundsException( "off < 0 || len < 0 || off + len > b.length!" );
    }
    bitOffset = 0;
    if ( len == 0 )
    {
      return 0;
    }
    else
    {
      int minLen = Math.min( this.imageData.remaining(), len );
      this.imageData.get( b, off, minLen );
      streamPos = this.imageData.position();
      return minLen == 0 ? -1 : minLen;
    }
  }

  @Override
  public boolean isCached()
  {
    return true;
  }

  @Override
  public boolean isCachedMemory()
  {
    return true;
  }

  @Override
  public boolean isCachedFile()
  {
    return false;
  }

  @Override
  public void close() throws IOException
  {
    super.close();
    this.imageData = null;
  }

  @Override
  public long length()
  {
    return this.imageData.limit();
  }

  public byte[] getBytes()
  {
    ByteBuffer buf = ByteBuffer.allocate( this.imageData.limit() );
    ByteBuffer src = this.imageData.duplicate();
    src.rewind();
    buf.put( src );
    return buf.array();
  }

}
