package ch.ige.image.test;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import ch.ige.image.EncodedImage;
import ch.ige.image.Image;
import ch.ige.image.ImageException;
import ch.ige.image.ImageFactoryImpl;
import ch.ige.image.MetadataBean;
import ch.ige.image.MetadataException;
import ch.ige.image.format.Format;

/**
 * Common Infrastructure for Encoding tests.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * @author    Michael Locher
 * @version   $Id: AbstractEncodingTester.java,v 1.6 2009/03/19 13:26:21 locher Exp $
 */
public abstract class AbstractEncodingTester extends AbstractImageTester
{

  public AbstractEncodingTester( String name )
  {
    super( name );
  }

  protected abstract String getTargetType();

  protected Image load( String name ) throws Exception
  {
    assertNotNull( AbstractEncodingTester.class.getResource( name ) );
    return new ImageFactoryImpl().loadURL( AbstractEncodingTester.class.getResource( name ) );
  }

  protected File getOutputDirectory()
  {
    File dir = new File( AllTests.OUTPUT_DIRECTORY, "encoding-" + this.getTargetType() );
    if ( !dir.exists() )
    {
      if ( !dir.mkdirs() )
      {
        throw new RuntimeException( "could not create directory: " + dir.getAbsolutePath() );
      }
    }
    return dir;
  }

  protected void save( EncodedImage img, String filename ) throws IOException
  {
    OutputStream fos = new FileOutputStream( new File( this.getOutputDirectory(), filename ) );
    try
    {
      img.write( fos );
    }
    finally
    {
      fos.close();
    }
  }

  protected EncodedImage encode( Image img ) throws IOException, ImageException
  {
    return Format.fromTypeString( this.getTargetType() ).encode( img, true );
  }

  protected void postEncoding( Image img, EncodedImage encoded )
  {
    //System.out.println(img + " --> " + encoded);
    MetadataBean metaOrig = new MetadataBean( img.getMetadata() );
    MetadataBean metaEncoded = new MetadataBean( encoded.getMetadata() );
    assertTrue( "Format of encoded", metaEncoded.getFormat().isKindOf( this.getTargetType() ) );
    assertTrue( "Size", metaOrig.getSize().equals( metaEncoded.getSize() ) );
    int origResolution = -1;
    int encodedResolution = -1;
    try
    {
      origResolution = metaOrig.getDPI();
      encodedResolution = metaEncoded.getDPI();
    }
    catch ( MetadataException e1 )
    {
      // ignore
    }
    if ( origResolution != -1 )
    {
      assertTrue( "No Resolution in encoded image", encodedResolution != -1 );
      assertEquals( "Resolution", origResolution, encodedResolution );
    }
    assertTrue( "Size and Resolution", metaOrig.sameSizeAndResolution( metaEncoded ) );
    // img.show();
    // encoded.show();
  }

}
