package ch.ige.image.test;

import java.awt.color.ColorSpace;
import java.awt.color.ICC_ColorSpace;

import junit.framework.TestCase;
import ch.ige.image.EncodedImage;

/**
 * Some color space tests. 
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   Michael Locher
 * @version  $Id: ColorspaceTest.java,v 1.1 2008/05/21 14:28:36 locher Exp $
 */
public class ColorspaceTest extends TestCase
{

  public ColorspaceTest( String name )
  {
    super( name );
  }

  public void testGetSRGB() throws Exception
  {
    ColorSpace sRGB = ColorSpace.getInstance( ColorSpace.CS_sRGB );
    assertEquals( true, sRGB.isCS_sRGB() );
    assertEquals( 3, sRGB.getNumComponents() );
    assertEquals( ColorSpace.TYPE_RGB, sRGB.getType() );
  }

  public void testGetLinarRGB() throws Exception
  {
    ColorSpace linRGB = ColorSpace.getInstance( ColorSpace.CS_LINEAR_RGB );
    assertEquals( false, linRGB.isCS_sRGB() );
    assertEquals( 3, linRGB.getNumComponents() );
    assertEquals( ColorSpace.TYPE_RGB, linRGB.getType() );
  }

  public void testGetGray() throws Exception
  {
    ColorSpace gray = ColorSpace.getInstance( ColorSpace.CS_GRAY );
    assertEquals( false, gray.isCS_sRGB() );
    assertEquals( 1, gray.getNumComponents() );
    assertEquals( ColorSpace.TYPE_GRAY, gray.getType() );
  }

  public void testGetCYMK() throws Exception
  {
    EncodedImage image = SampleImages.getCYMKImageWithICCProfile();

    ColorSpace cymkaSpaceFromImage = image.asRenderedImage().getColorModel().getColorSpace();
    assertEquals( false, cymkaSpaceFromImage.isCS_sRGB() );
    assertEquals( 4, cymkaSpaceFromImage.getNumComponents() );
    assertEquals( ColorSpace.TYPE_CMYK, cymkaSpaceFromImage.getType() );

    ColorSpace cymkaSpaceViaMetadata = new ICC_ColorSpace( image.getMetadata().getEmbeddedICCProfile() );
    assertEquals( false, cymkaSpaceViaMetadata.isCS_sRGB() );
    assertEquals( 4, cymkaSpaceViaMetadata.getNumComponents() );
    assertEquals( ColorSpace.TYPE_CMYK, cymkaSpaceViaMetadata.getType() );
  }

}
