package ch.ige.image.test;

import java.io.File;
import java.util.EnumSet;
import java.util.Set;

import junit.framework.TestCase;
import ch.ige.image.Image;
import ch.ige.image.color.GrayScale;
import ch.ige.image.color.GrayScale.Algorithm;
import ch.ige.image.format.Format;

/**
 * Tests {@link GrayScale}.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   Michael Locher
 * @version  $Id: GrayScaleTest.java,v 1.4 2008/05/22 13:26:13 locher Exp $
 */
public class GrayScaleTest extends TestCase
{

  public GrayScaleTest( String name )
  {
    super( name );
  }

  public void testAnimGIF() throws Exception
  {
    checkGrayscaling( SampleImages.getAnimatedGIF() );
  }

  public void testCircleImage() throws Exception
  {
    checkGrayscaling( SampleImages.getCirclePNG() );
  }

  public void testFromRGB() throws Exception
  {
    checkGrayscaling( SampleImages.getRGBImageWithICCProfile() );
  }

  public void testFromGray() throws Exception
  {
    checkGrayscaling( SampleImages.getGrayImageWithICCProfile() );
  }

  public void testFromCMYK() throws Exception
  {
    checkGrayscaling( SampleImages.getCYMKImageWithICCProfile() );
  }

  private void checkGrayscaling( Image img ) throws Exception
  {
    Set<Algorithm> algorithms = EnumSet.allOf( Algorithm.class );
    algorithms.remove( Algorithm.SETA );
    for ( Algorithm alg: algorithms )
    {
      Image gray = img.grayscale( alg );
      Image grayAgain = gray.grayscale( alg );
      Image grayAgainAgain = grayAgain.grayscale( alg );
      String expectedColorSpaceType = Algorithm.LINEAR.equals( alg ) ? "CUSTOM" : "GRAY";
      File tmp = File.createTempFile( img.hashCode() + "_" + alg + "_", ".png" );
      tmp.deleteOnExit();
      Format.fromTypeString( "PNG" ).encode( gray, true ).write( tmp );
      assertEquals( "grayscaling with " + alg, expectedColorSpaceType, gray.getMetadata().getColorspaceType() );
      assertEquals( "grayscaling again with " + alg, expectedColorSpaceType, grayAgain.getMetadata().getColorspaceType() );
      assertEquals( "grayscaling again again with " + alg, expectedColorSpaceType, grayAgainAgain.getMetadata().getColorspaceType() );
    }
  }

}
