package ch.ige.imagehandler.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.nio.charset.Charset;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.AbstractListModel;
import javax.swing.ComboBoxModel;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.TransferHandler;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import ch.ige.image.IOUtil;
import ch.ige.imagehandler.profile.Profile;
import ch.ige.imagehandler.profile.ProfileFactory;
import ch.ige.imagehandler.rule.RuleList;

public class ProfileEditor extends JPanel
{
  /** The serialization version. */
  private static final long serialVersionUID = "$Revision: 1.10 $".hashCode();

  public static final String PROFILE_PROPERTY_NAME = "profile";

  private static final Color DIRTY_COLOR = Color.PINK;

  private static final Color NON_DIRTY_COLOR = new JButton().getBackground();

  private JTextPane editor;

  private JButton btnParse;

  private boolean dirty;

  private transient Profile profile;

  private final RuleNameListModel converterListModel;

  private final RuleNameListModel listModel;

  public ProfileEditor()
  {
    super( new BorderLayout() );
    this.initEditor();
    this.initParseButton();
    this.converterListModel = new RuleNameListModel( true );
    this.listModel = new RuleNameListModel( false );
    this.initDropArea();
    this.add( new JScrollPane( this.editor ), BorderLayout.CENTER );
    this.add( this.btnParse, BorderLayout.SOUTH );
    this.setPreferredSize( new Dimension( 800, 600 ) );
  }

  private void initDropArea()
  {
    final TransferHandler basicTransferHandler = this.editor.getTransferHandler();
    this.editor.setTransferHandler( new TransferHandler()
    {
      /** The serialization version. */
      private static final long serialVersionUID = 1L;

      @Override
      @SuppressWarnings( "unchecked" )
      public boolean importData( JComponent c, Transferable t )
      {
        if ( hasFileFlavor( t.getTransferDataFlavors() ) )
        {
          try
          {
            List<File> files = (List<File>) t.getTransferData( DataFlavor.javaFileListFlavor );
            if ( !files.isEmpty() )
            {
              FileInputStream fis = new FileInputStream( files.get( 0 ) );
              try
              {
                InputStreamReader reader = new InputStreamReader( fis, Charset.forName( "UTF-8" ) );
                setProfileXML( reader );
              }
              finally
              {
                fis.close();
              }
            }
            return true;
          }
          catch ( Exception e )
          {
            JOptionPane.showMessageDialog( null, e.getMessage(), "Exception", JOptionPane.ERROR_MESSAGE );
            return false;
          }
        }
        return basicTransferHandler.importData( c, t );
      }

      @Override
      public boolean canImport( JComponent c, DataFlavor[] flavors )
      {
        return hasFileFlavor( flavors ) || ( basicTransferHandler != null && basicTransferHandler.canImport( c, flavors ) );
      }

      private boolean hasFileFlavor( DataFlavor[] flavors )
      {
        for ( DataFlavor flavor: flavors )
        {
          if ( DataFlavor.javaFileListFlavor.equals( flavor ) )
          {
            return true;
          }
        }
        return false;
      }

      @Override
      public void exportAsDrag( JComponent comp, InputEvent e, int action )
      {
        basicTransferHandler.exportAsDrag( comp, e, action );
      }

      @Override
      public void exportToClipboard( JComponent comp, Clipboard clip, int action ) throws IllegalStateException
      {
        basicTransferHandler.exportToClipboard( comp, clip, action );
      }

      @Override
      public int getSourceActions( JComponent c )
      {
        return basicTransferHandler.getSourceActions( c );
      }

      @Override
      public Icon getVisualRepresentation( Transferable t )
      {
        return basicTransferHandler.getVisualRepresentation( t );
      }

    } );
  }

  private void initParseButton()
  {
    AbstractAction parseAction = new AbstractAction( "Parse Profile" )
    {
      /** The serialization version. */
      private static final long serialVersionUID = 1L;

      public void actionPerformed( ActionEvent e )
      {
        parse();
      }
    };
    this.btnParse = new JButton( parseAction );
  }

  protected void parse()
  {
    try
    {
      this.profile = ProfileFactory.createFromXML( this.editor.getText() );
      this.setDirty( false );
      this.firePropertyChange( PROFILE_PROPERTY_NAME, null, null );
      this.listModel.changed();
    }
    catch ( Exception e )
    {
      JOptionPane.showMessageDialog( this, e.toString(), "Parsing Error", JOptionPane.ERROR_MESSAGE );
    }
  }

  private void initEditor()
  {
    this.editor = new JTextPane();
    this.editor.getDocument().addDocumentListener( new DocumentListener()
    {
      public void insertUpdate( DocumentEvent e )
      {
        setDirty( true );

      }

      public void removeUpdate( DocumentEvent e )
      {
        setDirty( true );
      }

      public void changedUpdate( DocumentEvent e )
      {
        setDirty( true );
      }
    } );
  }

  protected void setDirty( boolean isDirty )
  {
    this.dirty = isDirty;
    this.btnParse.setBackground( isDirty ? DIRTY_COLOR : NON_DIRTY_COLOR );
  }

  // @see javax.swing.JComponent#setEnabled(boolean)
  @Override
  public void setEnabled( boolean enabled )
  {
    super.setEnabled( enabled );
    this.btnParse.setEnabled( enabled );
    this.editor.setEnabled( enabled );
  }

  public void setProfileXML( Reader xmlDescription ) throws IOException
  {
    setProfileXML( IOUtil.toString( xmlDescription ) );
  }

  public void setProfileXML( String xmlDescription )
  {
    this.editor.setText( xmlDescription );
    this.profile = null;
    this.setDirty( true );
  }

  public String getProfileXML()
  {
    getProfile();
    return this.editor.getText();
  }

  public Profile getProfile()
  {
    if ( this.profile == null || this.dirty )
    {
      this.parse();
    }
    return this.profile;
  }

  public ComboBoxModel getConverterRuleNameListModel()
  {
    return this.converterListModel;
  }

  public ComboBoxModel getRuleNameListModel()
  {
    return this.listModel;
  }

  private class RuleNameListModel extends AbstractListModel implements ComboBoxModel
  {
    /** The serialization version. */
    private static final long serialVersionUID = 1L;

    private final boolean converterOnly;

    private Object selected;

    public RuleNameListModel( boolean converterOnly )
    {
      this.converterOnly = converterOnly;
    }

    private List<RuleList> getRules()
    {
      return this.converterOnly ? profile.getAllConverterRules() : profile.getAllRules();
    }

    public void changed()
    {
      this.fireContentsChanged( this, 0, this.getSize() );
    }

    public int getSize()
    {
      return ( profile == null ) ? 0 : getRules().size();
    }

    public Object getElementAt( int index )
    {
      return getRules().get( index ).getName();
    }

    // @see javax.swing.ComboBoxModel#getSelectedItem()
    public Object getSelectedItem()
    {
      return this.selected;
    }

    // @see javax.swing.ComboBoxModel#setSelectedItem(java.lang.Object)
    public void setSelectedItem( Object anItem )
    {
      this.selected = anItem;
      this.fireContentsChanged( this, -1, -1 );
    }

  } //inner-class

}
