package ch.ige.imagehandler.rule;

import org.apache.log4j.Logger;

import ch.ige.image.BitmapImage;
import ch.ige.image.ImageException;
import ch.ige.image.filter.BrightnessDebugFilter;

/**
 * Debug Rule. Does nothing but logging. 
 * <p>
 * Copyright (c) 2009, Swiss Federal Institute of Intellectual Property
 * @author    Michael Locher
 * @version   $Id: Debug.java,v 1.1 2009/05/06 09:56:12 locher Exp $
 */
public class Debug<I extends BitmapImage> extends AbstractRule<I> implements FlowControl<I, I>
{

  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( Debug.class );

  /*
   * @see ch.ige.imagehandler.rule.AbstractRule#getRuleName()
   */
  @Override
  protected String getRuleName()
  {
    return "Debug";
  }

  // @see ch.ige.imagehandler.rule.FlowControl#isConverter()
  public boolean isConverter()
  {
    return true;
  }

  /*
   * @see ch.ige.imagehandler.rule.Checker#checkImage(ch.ige.image.Image)
   */
  public void checkImage( I image ) throws RuleException
  {
    inspect( image );
  }

  /*
   * @see ch.ige.imagehandler.rule.Converter#convert(ch.ige.image.Image)
   */
  public I convert( I source ) throws RuleException, ImageException
  {
    inspect( source );
    return source;
  }

  private void inspect( I image )
  {
    if ( LOG.isDebugEnabled() )
    {
      try
      {
        //brightness
        BrightnessDebugFilter brightness = new BrightnessDebugFilter();
        brightness.filter( image );
        float averageBrightness = brightness.getAverageBrightness();
        LOG.debug( "average brightness: " + averageBrightness );
      }
      catch ( Exception e )
      {
        LOG.error( e );
      }
    }
  }

}
