package ch.ige.imagehandler.rule.test;

import junit.framework.TestCase;
import ch.ige.image.Image;
import ch.ige.image.test.SampleImages;
import ch.ige.imagehandler.rule.Converter;
import ch.ige.imagehandler.rule.GrayScaleConverter;
import ch.ige.imagehandler.rule.RuleException;
import ch.ige.imagehandler.rule.parameter.SingleBinding;

/**
 *  Tests {@link GrayScaleConverter}. 
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   Michael Locher
 * @version  $Id: GrayScaleConverterTest.java,v 1.1 2008/06/16 06:54:24 locher Exp $
 */
public class GrayScaleConverterTest extends TestCase
{

  public GrayScaleConverterTest( String name )
  {
    super( name );
  }

  private Converter createConverter() throws RuleException
  {
    GrayScaleConverter conv = new GrayScaleConverter();
    conv.addParameter( new SingleBinding( "algorithm", "BT709" ) );
    conv.addParameter( new SingleBinding( "skipIfGray", "true" ) );
    conv.checkParameters();
    return conv;
  }

  public void testSkipSecond() throws Exception
  {
    Converter conv = createConverter();
    Image in = SampleImages.getCirclePNG();
    Image gray = conv.convert( in );
    assertTrue( in != gray );
    Image grayAgain = conv.convert( gray );
    assertSame( gray, grayAgain );
  }

  public void testSkipGray() throws Exception
  {
    Converter conv = createConverter();
    Image in = SampleImages.getGrayImageWithICCProfile();
    Image gray = conv.convert( in );
    assertSame( in, gray );
  }

}
