package ch.ige.ipcloadscheme.client.text;

import java.io.InputStream;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;
import java.util.Locale;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.apache.log4j.Logger;

import ch.ige.ipcloadscheme.client.IPCLoad.SaveMode;
import ch.ige.ipcloadscheme.client.file.LanguageFile;
import ch.ige.ipcloadscheme.client.file.TextFile;
import ch.ige.ipcloadscheme.client.file.TextZipFile;

/**
 * Steuerung Verarbeitung Texte
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: TextProcess.java,v 1.16 2008/07/01 06:27:03 lewis_lee Exp $
 */
public class TextProcess
{
  private IpcKlassenParser parser = new IpcKlassenParser();

  private TextStorage textStorage = new TextStorage();

  private Date processDate;

  private static final Logger LOG = Logger.getLogger( TextProcess.class );

  private StringBuffer logMsg = new StringBuffer();

  private boolean error = false;

  public TextProcess( Date processDate )
  {
    this.processDate = processDate;
  }

  /**
   * Gibt an ob sich ein Fehler ereignet hat
   * @return boolean true fehler aufgetreten
   */
  public boolean isErrorOccured()
  {
    return this.error;
  }

  /**
    * Gibt die geloggten Debugmeldungen zurück
    * @return String
    */
  public String getLogMessage()
  {
    return this.logMsg.toString();
  }

  /**
    * Fügt übergebenen String der Fehlermeldung hinzu
    * @param message anzufügende Fehlermeldung
    */
  private void logAndAppendMsg( String message )
  {
    LOG.error( message );
    logMsg.append( "\n" );
    logMsg.append( message );
  }

  public void process( List<Locale> languages, SaveMode saveMode )
  {
    LOG.info( "text process started" );

    for ( Locale language: languages )
    {
      try
      {
        processLanguage( language );
      }
      catch ( Exception ex )
      {
        this.error = true;
        this.logAndAppendMsg( ex.getMessage() );
      }
    }

    save( saveMode );
    cleanUp( languages );

    LOG.info( "text process done" );
  }

  private void save( SaveMode saveMode )
  {
    try
    {
      if ( saveMode.equals( SaveMode.CREATE ) )
      {
        textStorage.createNew( parser.getMap(), parser.getMapBilder() );
      }
      else
      {
        textStorage.alter( parser.getMap(), parser.getMapBilder() );
      }
    }
    catch ( Exception ex1 )
    {
      this.error = true;
      this.logAndAppendMsg( ex1.getMessage() );
    }
  }

  private void processLanguage( Locale language ) throws Exception
  {
    Enumeration entries;
    ZipFile zipFile;
    ZipEntry entry;

    TextZipFile textZipFile = new TextZipFile( processDate, language );
    TextFile textFile = new TextFile( processDate, language );

    if ( !textZipFile.exists() )
    {
      throw new Exception( "File : " + textZipFile.getPath() + " doesn't exist" );
    }

    zipFile = new ZipFile( textZipFile );
    entries = zipFile.entries();

    while ( entries.hasMoreElements() )
    {
      entry = (ZipEntry) entries.nextElement();

      if ( entry.getName().equals( textFile.getName() ) )
      {
        LOG.debug( "Extracting file: " + entry.getName() );

        InputStream is = zipFile.getInputStream( entry );
        parser.parse( is, language );
        is.close();
      }
    }

    zipFile.close();
  }

  public void cleanUp( List<Locale> languages )
  {
    TextZipFile textZipFile;
    LanguageFile langFile;

    for ( Locale language: languages )
    {
      textZipFile = new TextZipFile( processDate, language );

      if ( textZipFile.exists() )
      {
        textZipFile.moveToDoneDirectory();
        LOG.debug( textZipFile.getPath() + " moved to directory done" );
      }

      langFile = new LanguageFile( language, processDate );
      if ( langFile.exists() )
      {
        langFile.moveToDoneDirectory();
        LOG.debug( langFile.getPath() + " moved to directory done" );
      }
    }
  }
}
