package ch.ige.ipcloadscheme.client.text;


import java.util.*;
import org.apache.log4j.Logger;
import ch.ige.ipcloadscheme.server.IpcSchemeDelegate;
import ch.ige.ipcloadscheme.vo.*;


/**
 * Speichert Textinformationen in der Datenbank
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: TextStorage.java,v 1.11 2006/12/19 16:19:52 ruegsegger Exp $
 */

public class TextStorage
{
  private static final Logger LOG = Logger.getLogger(TextStorage.class);

  public TextStorage()
  {
  }


  public void createNew( Map mapKlassen, Map mapBilder ) throws Exception
  {
    LOG.info("number of classes sent to server for insert into ipc_klasse: " + mapKlassen.size());
    CountVO countVO = IpcSchemeDelegate.getInstance().insertKlasse( mapKlassen );

    LOG.debug("check insert count for ipc_klasse");
    checkInsertCount( mapKlassen, countVO.count );

    LOG.info("number of classes sent to server for insert into ipc_bild_klasse: " + mapBilder.size());
    countVO = IpcSchemeDelegate.getInstance().insertBildKlasse( mapBilder );

    LOG.debug("check insert count for ipc_bild_klasse");
    checkInsertCount( mapBilder, countVO.count );
  }


  public void alter( Map map, Map mapBilder  ) throws Exception
  {
    LOG.info("number of classes sent to server for update of ipc_klasse: " + map.size());
    CountVO countVO = IpcSchemeDelegate.getInstance().alterKlassen( map );

    LOG.debug("check update count for ipc_klasse");
    checkUpdateCount( map, countVO.count );

    LOG.info("number of classes sent to server for update of ipc_bild_klasse: " + mapBilder.size());
    countVO = IpcSchemeDelegate.getInstance().alterBildKlassen( mapBilder );
  }


  private void checkInsertCount( Map map, int[] insertCount )
  {
    IpcKlasseVO vo;
    Collection ipcVOs =  map.values();
    Iterator it = ipcVOs.iterator();

    for ( int i = 0; i < insertCount.length; i++ )
    {
      if ( insertCount[i] > 0 || insertCount[i] == java.sql.Statement.SUCCESS_NO_INFO )
      {
        it.next();
      }
      else
      {
        vo = ( IpcKlasseVO ) it.next();
        LOG.info( "insertCount is < 1 - insertion failed for : \n" + vo.toString() );
      }
    }
  }

  /**
   * If the map was implemented using a 'self ordering' data structure, the order
   * of the two iterators might well be different.
   * @param map Map
   * @param updateCounts int[]
   */
  private void checkUpdateCount( Map map, int[] updateCounts )
  {
    IpcKlasseVO vo;
    Collection ipcVOs =  map.values();
    Iterator it = ipcVOs.iterator();

    for ( int i = 0; i < updateCounts.length; i++ )
    {
      if ( updateCounts[i] > 0 || updateCounts[i] == java.sql.Statement.SUCCESS_NO_INFO)
      {
        it.next();
      }
      else
      {
        // update nicht erfolgreich try insert
        vo = ( IpcKlasseVO ) it.next();
        LOG.info( "updateCount is < 1 - try to insert for "  + vo.ipcKlasse );
        insert( vo );
      }
    }
  }


   private void insert( IpcKlasseVO vo )
  {
    try
    {
      IpcSchemeDelegate.getInstance().insert( vo );
    }
    catch ( Exception ex )
    {
      LOG.error("insertion failed for : \n" + vo.toString() + " Exception: " + ex);
    }
  }

}
