package ch.ige.ipcloadscheme.client.transfer;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;

import org.apache.log4j.Logger;

import sun.misc.BASE64Encoder;
import ch.ige.ipcloadscheme.client.SchemeProperties;

/**
 * Herstellen einer Verbindung zu einer URL, setzen der Proxy- und Authorization-Einstellungen
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: HTTPConnection.java,v 1.5 2008/07/01 06:27:02 lewis_lee Exp $
 */
public class HTTPConnection
{
  private static final Logger LOG = Logger.getLogger( HTTPConnection.class );

  private HttpURLConnection con = null;

  private String serverAuthName = null;

  private String serverAuthPassword = null;

  private BASE64Encoder encoder = null;

  /**
   * Default-Konstruktor<br>
   * Setzen der Proxy-Einstellungen
   */
  public HTTPConnection()
  {
    // Initialize System properties!
    SchemeProperties.getInstance();
    System.setProperty( "proxySet", "true" );

    serverAuthName = SchemeProperties.getInstance().getProperty( SchemeProperties.HTTP_USER );
    serverAuthPassword = SchemeProperties.getInstance().getProperty( SchemeProperties.HTTP_PW );

    encoder = new sun.misc.BASE64Encoder();
  }

  /**
   * Verbindung zu einer URL herstellen
   * @param urlname Url zu welcher eine Verbindung via Proxy hergestellt werden soll
   */
  public void connect( String urlname ) throws Exception
  {
    LOG.debug( "try to connect to : " + urlname );

    if ( con != null )
    {
      con.disconnect();
    }

    URL url = new URL( urlname );
    con = (HttpURLConnection) url.openConnection();

    String authentication = encoder.encode( ( serverAuthName + ":" + serverAuthPassword ).getBytes() );
    con.setRequestProperty( "Authorization", "Basic " + authentication );
    con.setUseCaches( false );
    con.connect();
  }

  public int getResponseCode() throws IOException
  {
    return con.getResponseCode();
  }

  public String getContentType()
  {
    return con.getContentType();
  }

  /**
   * Gibt den InputStream der HttpURLConnection zurück
   * @return InputStream der HttpURLConnection
   * @throws IOException
   */
  public InputStream getInputStream() throws IOException
  {
    return con.getInputStream();
  }

  /**
  * Trennen der HttpURLConnection
  */
  public void disconnect()
  {
    con.disconnect();
  }

  public void checkResponseCode() throws IOException, Exception
  {
    int code = con.getResponseCode();
    if ( code != HttpURLConnection.HTTP_OK )
    {
      throw new Exception( "Response Code is not OK (200): " + code );
    }
  }
}
