package ch.ige.ipcloadscheme.server.dao.test;

import java.sql.*;
import java.text.*;
import ch.ige.ipcloadscheme.server.dao.*;
import junit.framework.*;
import ch.ige.util.server.DBHelper;
import java.util.Date;

import org.apache.log4j.BasicConfigurator;


/**
 * Test der Klasse IpcDAO
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  A. Rüegsegger
 * @version $Id: IpcDAOTest.java,v 1.5 2007/01/11 18:11:57 ruegsegger Exp $
 */

public class IpcDAOTest extends DAOTestCase
{
  private IpcDAO ipcDAO = new IpcDAO();
  private SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");

  
  public IpcDAOTest( String s )
  {
    super( s );
  }

 
  public void test00SelectGueltigVon()
  {
    System.out.println( "Test SELECT GUELTIG-VON " );

    try
    {
      String ipcSymb = "Z01Z";

      Date gueltigVon = ipcDAO.selectGueltigVon( ipcSymb);
      this.assertNull( gueltigVon );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }


  public void test01SelectGueltigVon()
  {
    System.out.println( "Test SELECT GUELTIG-VON " );

    try
    {
      String ipcSymb = "Z01Z";
      Date dateVon = sdf.parse( "20060101" );
      Date dateBis = null;
      this.insert( ipcSymb, dateVon, dateBis );

      Date gueltigVon = ipcDAO.selectGueltigVon( ipcSymb);
      this.assertEquals( sdf.format(dateVon), sdf.format( gueltigVon ) );

      this.delete( ipcSymb );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  public void test02SelectGueltigVon()
  {
    System.out.println( "Test SELECT GUELTIG-VON " );

    try
    {
      String ipcSymb = "Z01Z000101";
      Date dateVon = sdf.parse( "19740101" );
      Date dateBis = sdf.parse( "20010101" );;
      this.insert( ipcSymb, dateVon, dateBis );

      Date gueltigVon = ipcDAO.selectGueltigVon( ipcSymb);
      this.assertEquals( sdf.format(dateVon), sdf.format( gueltigVon ) );

      this.delete( ipcSymb );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  public void test03SelectGueltigVon()
  {
    System.out.println( "Test SELECT GUELTIG-VON " );

    try
    {
      String ipcSymb = "Z01Z000102";
      Date dateVon = sdf.parse( "20060101" );
      Date dateBis = sdf.parse( "20060930" );
      this.insert( ipcSymb, dateVon, dateBis );

      dateVon = sdf.parse( "20061001" );
      dateBis = null;
      this.insert( ipcSymb, dateVon, dateBis );

      Date gueltigVon = ipcDAO.selectGueltigVon( ipcSymb);
      this.assertEquals( sdf.format(dateVon), sdf.format( gueltigVon ) );

      this.delete( ipcSymb );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  public void test04SelectGueltigVon()
  {
    System.out.println( "Test SELECT GUELTIG-VON " );

    try
    {
      String ipcSymb = "Z01Z000102";
      Date dateVon = sdf.parse( "20000101" );
      Date dateBis = sdf.parse( "20051231" );
      this.insert( ipcSymb, dateVon, dateBis );

      dateVon = sdf.parse( "20061001" );
      dateBis = sdf.parse( "20060930" );;
      this.insert( ipcSymb, dateVon, dateBis );

      Date gueltigVon = ipcDAO.selectGueltigVon( ipcSymb);
      this.assertEquals( sdf.format(dateVon), sdf.format( gueltigVon ) );

      this.delete( ipcSymb );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }


  /**
   * Fügt einen Datensatz in die Tabelle IPC ein
   *
   * @param     ipcVO            ValueObject IpcVO
   * @exception SQLException     Unerwarteter SQL Fehler
   */
  public int insert( String ipcSymb, Date gueltigVon, Date gueltigBis ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "INSERT INTO ipc(" +
          "ipc_symb, " +
          "zusatz, " +
          "gueltig_von, " +
          "gueltig_bis, " +
          "class_level, " +
          "core_pred, " +
          "version) " +
          "VALUES( ?,'K',?,?,'A',null,1)";

      ps = con.prepareStatement(query);
      ps.setString( 1, ipcSymb );

      if ( gueltigVon == null )
      {
        ps.setDate( 2, null );
      }
      else
      {
        ps.setDate( 2, new java.sql.Date( gueltigVon.getTime() ) );
      }

      if ( gueltigBis == null )
      {
        ps.setDate( 3, null );
      }
      else
      {
        ps.setDate( 3, new java.sql.Date( gueltigBis.getTime() ) );
      }

      return ps.executeUpdate();
    }
    catch (SQLException ex)
    {
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
  * Löscht einen Datensatz aus der Tabelle IPC ein
  * @param     ipcSymb          IPC-Symbol
  * @exception SQLException     Unerwarteter SQL Fehler
  * @return int Anzahl gelöschte Datensätze
  */
 public int delete( String ipcSymb ) throws SQLException
 {
   Connection con = null;
   PreparedStatement ps = null;
   String query = null;

   try
   {
     con = DBHelper.getConnection( DBHelper.BAGIS );
     query = "DELETE FROM ipc WHERE ipc_symb = RPAD(?, 14, ' ')";
     ps = con.prepareStatement(query);
     ps.setString( 1, ipcSymb );

     return ps.executeUpdate();
   }
   catch (SQLException ex)
   {
     throw ex;
   }
   finally
   {
     DBHelper.close( ps, con );
   }
 }

}
