package ch.ige.ipcloadvalidity.server.dao.test;

import java.text.SimpleDateFormat;

import org.apache.log4j.BasicConfigurator;

import junit.framework.TestCase;
import ch.ige.ipcloadvalidity.server.dao.IpcDAO;
import ch.ige.ipcloadvalidity.util.test.BasicTestCase;
import ch.ige.ipcloadvalidity.vo.IpcVO;
import ch.ige.ipcloadvalidity.vo.test.Testdata;
import ch.ige.util.server.DBHelper;


/**
 * Test der Klasse IpcDAO
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  A. Rüegsegger
 * @version $Id: IpcDAOTest.java,v 1.21 2007/01/09 13:11:26 ruegsegger Exp $
 */

public class IpcDAOTest extends BasicTestCase 
{
  private IpcDAO ipcDAO = new IpcDAO();
  private IpcVO ipcVO = null;
  private SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");


  public IpcDAOTest( String s )
  {
    super( s );    
  }

  protected void setUp()
  {
    ipcVO = Testdata.createIpcVO();    
  }

  /**
   *  Fügt einen Datensatz in die Tabelle ipc ein
   */
  public void test01Insert()
  {
    System.out.println( "Test INSERT " );
    IpcVO selectVO = null;

    try
    {
      ipcDAO.insert(ipcVO);

      // Testen ob eingefügt
      selectVO = ipcDAO.select( ipcVO.ipcSymb, ipcVO.gueltigVon );
      if ( selectVO == null )
      {
        fail( "INSERT: Datensatz existiert nicht" );
      }
      assertEquals( ipcVO.ipcSymb, selectVO.ipcSymb );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }


  /**
   *  Selektiert einen Datensatz aus der Tabelle ipc
   */
  public void test02Select()
  {
    System.out.println( "Test SELECT " );
    IpcVO selectVO = null;

    try
    {
      selectVO = ipcDAO.select( ipcVO.ipcSymb, ipcVO.gueltigVon);
      if ( selectVO == null )
      {
        fail( "SELECT: Datensatz existiert nicht" );
      }
      else
      {
        assertEquals( ipcVO.ipcSymb, selectVO.ipcSymb );
        assertEquals( ipcVO.zusatz, selectVO.zusatz );
        assertEquals( sdf.format(ipcVO.gueltigVon), sdf.format(selectVO.gueltigVon) );

        String ipcVOGueltigBis = ipcVO.gueltigBis != null ? sdf.format(ipcVO.gueltigBis) : null;
        String selectVOGueltigBis = selectVO.gueltigBis != null ? sdf.format(selectVO.gueltigBis) : null;

        assertEquals( ipcVOGueltigBis, selectVOGueltigBis);
        assertEquals( ipcVO.classLevel, selectVO.classLevel );
        assertEquals( ipcVO.corePred, selectVO.corePred );
        assertEquals( ipcVO.version, selectVO.version );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }


  /**
    *  Aktualisiert einen Datensatz in der Tabelle ipc
    */
   public void test04UpdateValidTo()
   {
     System.out.println( "Test UPDATE VALID-TO" );
     IpcVO selectVO = null;
     IpcVO updateVO = null;

     try
     {
       updateVO = ipcDAO.select( ipcVO.ipcSymb, ipcVO.gueltigVon );
       if ( updateVO == null )
       {
         fail( "UPDATE: Datensatz existiert nicht" );
       }
       else
       {
         updateVO.gueltigBis = sdf.parse( "20051231" );
         updateVO.version++;

         // Datensatz updaten
         int retVal = ipcDAO.updateValidTo( updateVO, updateVO.gueltigBis );
         assertEquals( "Anzahl Datensätze ", 1, retVal );

         // Datensatz wieder sektieren und auf Änderungen testen
         selectVO = ipcDAO.select( ipcVO.ipcSymb, ipcVO.gueltigVon );

         assertEquals( updateVO.ipcSymb, selectVO.ipcSymb );
         String ipcVOGueltigBis = updateVO.gueltigBis != null ? sdf.format(updateVO.gueltigBis) : null;
         String selectVOGueltigBis = selectVO.gueltigBis != null ? sdf.format(selectVO.gueltigBis) : null;
         assertEquals( ipcVOGueltigBis, selectVOGueltigBis);
         assertEquals( updateVO.version, selectVO.version );


         updateVO.gueltigBis = sdf.parse( "20051231" );
         // Datensatz updaten
         retVal = ipcDAO.updateValidTo( updateVO, updateVO.gueltigBis );
         assertEquals( "Anzahl Datensätze ", 1, retVal );


         updateVO.gueltigBis = sdf.parse( "20051031" );

         // Datensatz updaten
         retVal = ipcDAO.updateValidTo( updateVO, updateVO.gueltigBis );
         assertEquals( "Anzahl Datensätze ", 0, retVal );

       }
     }
     catch ( Exception e )
     {
       e.printStackTrace();
       fail( e.toString() );
     }
   }


  /**
   *  Prüft die Existenz eines Artikels in der Tabelle artikel
   */
  public void test05Exists()
  {
    System.out.println( "Test EXISTS " );

    try
    {
      boolean exist = ipcDAO.exists( ipcVO.ipcSymb, ipcVO.gueltigVon );
      assertEquals( "Datensatz existiert ", true, exist );

      exist = ipcDAO.exists( "z01", null );
      assertEquals( "Datensatz existiert ", false, exist );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }


   /**
    *  Löscht einen Datensatz aus der Tabelle ipc
    */
   public void test06Delete()
   {
     System.out.println( "Test DELETE " );

     try
     {
       int retVal = ipcDAO.delete( ipcVO.ipcSymb, ipcVO.gueltigVon );
       assertEquals( "Anzahl gelöschte Datensätze ", 1, retVal );
     }
     catch ( Exception e )
     {
       e.printStackTrace();
       fail( e.toString() );
     }
   }

}
