
package ch.ige.ipcloadvalidity.xml.generate;

import ch.ige.ipcloadvalidity.xml.generate.IpcrSymbol;
import ch.ige.ipcloadvalidity.xml.generate.SubclassDescription;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.xml.bind.Dispatcher;
import javax.xml.bind.Element;
import javax.xml.bind.InvalidAttributeException;
import javax.xml.bind.InvalidContentObjectException;
import javax.xml.bind.LocalValidationException;
import javax.xml.bind.MarshallableObject;
import javax.xml.bind.Marshaller;
import javax.xml.bind.MissingContentException;
import javax.xml.bind.PredicatedLists;
import javax.xml.bind.PredicatedLists.Predicate;
import javax.xml.bind.StructureValidationException;
import javax.xml.bind.UnmarshalException;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.ValidatableObject;
import javax.xml.bind.Validator;
import javax.xml.marshal.XMLScanner;
import javax.xml.marshal.XMLWriter;


public class SubclassList
    extends MarshallableObject
    implements Element
{

    private SubclassDescription _SubclassDescription;
    private List _IpcrSymbol = PredicatedLists.createInvalidating(this, new IpcrSymbolPredicate(), new ArrayList());
    private PredicatedLists.Predicate pred_IpcrSymbol = new IpcrSymbolPredicate();

    public SubclassDescription getSubclassDescription() {
        return _SubclassDescription;
    }

    public void setSubclassDescription(SubclassDescription _SubclassDescription) {
        this._SubclassDescription = _SubclassDescription;
        if (_SubclassDescription == null) {
            invalidate();
        }
    }

    public List getIpcrSymbol() {
        return _IpcrSymbol;
    }

    public void deleteIpcrSymbol() {
        _IpcrSymbol = null;
        invalidate();
    }

    public void emptyIpcrSymbol() {
        _IpcrSymbol = PredicatedLists.createInvalidating(this, pred_IpcrSymbol, new ArrayList());
    }

    public void validateThis()
        throws LocalValidationException
    {
        if (_SubclassDescription == null) {
            throw new MissingContentException("subclass-description");
        }
        if (_IpcrSymbol == null) {
            throw new MissingContentException("ipcr-symbol");
        }
    }

    public void validate(Validator v)
        throws StructureValidationException
    {
        v.validate(_SubclassDescription);
        for (Iterator i = _IpcrSymbol.iterator(); i.hasNext(); ) {
            v.validate(((ValidatableObject) i.next()));
        }
    }

    public void marshal(Marshaller m)
        throws IOException
    {
        XMLWriter w = m.writer();
        w.start("subclass-list");
        m.marshal(_SubclassDescription);
        for (Iterator i = _IpcrSymbol.iterator(); i.hasNext(); ) {
            m.marshal(((MarshallableObject) i.next()));
        }
        w.end("subclass-list");
    }

    public void unmarshal(Unmarshaller u)
        throws UnmarshalException
    {
        XMLScanner xs = u.scanner();
        Validator v = u.validator();
        xs.takeStart("subclass-list");
        while (xs.atAttribute()) {
            String an = xs.takeAttributeName();
            throw new InvalidAttributeException(an);
        }
        _SubclassDescription = ((SubclassDescription) u.unmarshal());
        {
            List l = PredicatedLists.create(this, pred_IpcrSymbol, new ArrayList());
            while (xs.atStart("ipcr-symbol")) {
                l.add(((IpcrSymbol) u.unmarshal()));
            }
            _IpcrSymbol = PredicatedLists.createInvalidating(this, pred_IpcrSymbol, l);
        }
        xs.takeEnd("subclass-list");
    }

    public static SubclassList unmarshal(InputStream in)
        throws UnmarshalException
    {
        return unmarshal(XMLScanner.open(in));
    }

    public static SubclassList unmarshal(XMLScanner xs)
        throws UnmarshalException
    {
        return unmarshal(xs, newDispatcher());
    }

    public static SubclassList unmarshal(XMLScanner xs, Dispatcher d)
        throws UnmarshalException
    {
        return ((SubclassList) d.unmarshal(xs, (SubclassList.class)));
    }

    public boolean equals(Object ob) {
        if (this == ob) {
            return true;
        }
        if (!(ob instanceof SubclassList)) {
            return false;
        }
        SubclassList tob = ((SubclassList) ob);
        if (_SubclassDescription!= null) {
            if (tob._SubclassDescription == null) {
                return false;
            }
            if (!_SubclassDescription.equals(tob._SubclassDescription)) {
                return false;
            }
        } else {
            if (tob._SubclassDescription!= null) {
                return false;
            }
        }
        if (_IpcrSymbol!= null) {
            if (tob._IpcrSymbol == null) {
                return false;
            }
            if (!_IpcrSymbol.equals(tob._IpcrSymbol)) {
                return false;
            }
        } else {
            if (tob._IpcrSymbol!= null) {
                return false;
            }
        }
        return true;
    }

    public int hashCode() {
        int h = 0;
        h = ((127 *h)+((_SubclassDescription!= null)?_SubclassDescription.hashCode(): 0));
        h = ((127 *h)+((_IpcrSymbol!= null)?_IpcrSymbol.hashCode(): 0));
        return h;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer("<<subclass-list");
        if (_SubclassDescription!= null) {
            sb.append(" subclass-description=");
            sb.append(_SubclassDescription.toString());
        }
        if (_IpcrSymbol!= null) {
            sb.append(" ipcr-symbol=");
            sb.append(_IpcrSymbol.toString());
        }
        sb.append(">>");
        return sb.toString();
    }

    public static Dispatcher newDispatcher() {
        return IpcrSymbol.newDispatcher();
    }


    private static class IpcrSymbolPredicate
        implements PredicatedLists.Predicate
    {


        public void check(Object ob) {
            if (!(ob instanceof IpcrSymbol)) {
                throw new InvalidContentObjectException(ob, (IpcrSymbol.class));
            }
        }

    }

}
