package ch.ige.jipps;

import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import org.apache.log4j.Logger;

import ch.ige.jipps.constants.URLConstants;
import ch.ige.jipps.constants.XslConstants;
import ch.ige.jipps.vo.ByteArrayVO;

/**
 * DirectControl: Steuerungsklasse für das Erstellen der Listen:<br> - Europäischen Patente
 * <p>
 * Copyright (c) 2002-2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author Marc Bouquet
 * @version $Id: DirectControl.java,v 1.42 2007/12/19 17:52:00 lewis_lee Exp $
 */
public class DirectControl extends ReportControl
{
  private static final Logger LOG = Logger.getLogger( DirectControl.class );

  private HttpURLConnection con = null;

  private static DirectControl me = null; // stat. Feld für Singleton

  private ByteArrayVO byteArrayVO;

  /**
   * The constructor
   */
  private DirectControl()
  {
    // empty constructor
  }

  /**
   * statische Methode, die eine Referenz auf den DirectControl zurück. Hier ist das Singleton-Pattern implementiert.
   * @return Referenz auf DirectControl (= me)
   */
  public static DirectControl getInstance()
  {
    if ( me == null )
    {
      me = new DirectControl();
    }
    return me;
  }

  /**
   * Methode makePrint Öffentliche Methode die das erstellen des Drucks steuert
   * @param xmlStream XmlStream mit den zu druckenden Daten
   * @param listenTyp Typ der Liste
   * @param ladeDatum Datum des Laden der EP-Listen.
   */
  public synchronized void makePrint( String xmlStream, String listenTyp, String ladeDatum )
  {
    byte[] pdfData = makePdf( xmlStream, listenTyp, ladeDatum );
    callFileWriter( pdfData );
  }

  /**
   * Creates a PDF from the given XML document and list type.
   * @param xmlDocument The XML document containing the data
   * @param listType The type of list
   * @param loadDate The date when the EP lists were loaded
   * @return The rendered PDF data.
   */
  public synchronized byte[] makePdf( String xmlDocument, String listType, String loadDate )
  {
    // TODO Document the type of list

    byteArrayVO = new ByteArrayVO();
    byteArrayVO.ladeDatum = loadDate;

    if ( getListenTyp( listType ) != 0 )
    {
      // SWITCH der aufgrund des ListenTyps das jeweilige XML und Filename ins VO schreibt.
      switch( getListenTyp( listType ) )
      {
        case 1:
        {
          // ListenTyp = Namen der XSL-Datei
          byteArrayVO.listenTyp = XslConstants.EP_001;
          // Dateinamen des jeweiligen ListenTyps
          byteArrayVO.fileName = "ep_pat_nr_neu";
          LOG.debug( "*** Case 1: " + byteArrayVO.listenTyp + ", " + byteArrayVO.fileName );
        }
          break;

        case 2:
        {
          // ListenTyp = Namen der XSL-Datei
          byteArrayVO.listenTyp = XslConstants.EP_002;
          // Dateinamen des jeweiligen ListenTyps
          byteArrayVO.fileName = "ep_spezial";
          LOG.debug( "*** Case 2: " + byteArrayVO.listenTyp + ", " + byteArrayVO.fileName );
        }
          break;

        case 3:
        {
          // ListenTyp = Namen der XSL-Datei
          byteArrayVO.listenTyp = XslConstants.EP_003;
          // Dateinamen des jeweiligen ListenTyps
          byteArrayVO.fileName = "ep_patent_mut";
          LOG.debug( "*** Case 3: " + byteArrayVO.listenTyp + ", " + byteArrayVO.fileName );
        }
          break;

        case 4:
        {
          // ListenTyp = Namen der XSL-Datei
          // Das DTD ist das selbe wie beim Case 1, da das gleiche XSL verwendet wird.
          byteArrayVO.listenTyp = XslConstants.EP_001;
          // Dateinamen des jeweiligen ListenTyps
          byteArrayVO.fileName = "ep_pat_nr_eingetragen";
          LOG.debug( "*** Case 4: " + byteArrayVO.listenTyp + ", " + byteArrayVO.fileName );
        }
          break;
      }
      return render( xmlDocument, byteArrayVO.listenTyp );
    }
    else
    {
      LOG.error( "Es wurde kein gültiger ListenTyp übergeben." );
      return null;
    }
  }

  /**
   * Die Liste wird via Servlet gerendert. Der DirectControl erhält vom PdfRenderServlet ein byte[] zurück und
   * verarbeitet diesen weiter.
   * @param reportXml XmlStream
   * @param xslName Name des XSL, welches gedruckt werden möchte
   * @return byte[] gerendertes PDF
   */
  protected byte[] render( String reportXml, String xslName )
  {
    LOG.debug( "*** Grösse des XML-Streams: " + reportXml.length() );
    try
    {
      return createPdf( reportXml, xslName );
    }
    catch ( Exception ex )
    {
      LOG.error( ex.getMessage(), ex );
      return null;
    }
  }

  /**
   * Diese Methode liest den ListenTyp aus dem Value-Object und setzt dazu einen einsprechenden int-Wert, damit im
   * Switch die entsprechenden XML-Datei übergeben werden kann.
   * @param listenTyp String der im XML-Stream übergeben wird.
   * @return listenTypInt dem String entsprechender int-Wert (internes Mapping)
   */
  private int getListenTyp( String listenTyp )
  {
    // TODO Use constants which are not related to XslConstants
    int listenTypInt;
    // Mapping für EP-Liste: Gesuche Patente nach Nummern
    if ( listenTyp.equals( "ep_001" ) )
    {
      listenTypInt = 1;
      LOG.debug( "*** Listentyp: " + listenTypInt );
    }
    // Mapping für EP-Liste: Eingetragene Patente nach Nummern
    else if ( listenTyp.equals( "ep_002" ) )
    {
      listenTypInt = 2;
      LOG.debug( "*** Listentyp: " + listenTypInt );
    }
    // Mapping für EP-Liste: Übernahmeprotokoll Mutation EP-Patent
    else if ( listenTyp.equals( "ep_003" ) )
    {
      listenTypInt = 3;
      LOG.debug( "*** Listentyp: " + listenTypInt );
    }
    // Mapping für EP-Liste: Manuelle Mutation
    else if ( listenTyp.equals( "ep_004" ) )
    {
      listenTypInt = 4;
      LOG.debug( "*** Listentyp: " + listenTypInt );
    }
    // Mapping wenn kein Listentyp übergeben wird (Fehlerfall)
    else
    {
      listenTypInt = 0;
      LOG.debug( "*** Listentyp: " + listenTypInt );
    }

    return listenTypInt;
  }

  /**
   * Erstellt die Verbindung zum FileServlet und empfängt deren Response. Die Methode schreibt den erhaltenen Byte[] in
   * ein definiertes Verzeichnis.
   * @param fileArray byte[] Datei welche ins File-System gespeichert werden soll.
   */
  private void callFileWriter( byte[] fileArray )
  {
    try
    {
      byteArrayVO.file = fileArray;
      url = new URL( URLConstants.FILE_SERVLET_URL );
      con = (HttpURLConnection) url.openConnection();
      con.setUseCaches( false );
      con.setDoOutput( true );

      ByteArrayOutputStream buffer = new ByteArrayOutputStream();
      ObjectOutputStream out = new ObjectOutputStream( buffer );

      out.writeObject( byteArrayVO );
      out.flush();
      out.close();

      buffer.writeTo( con.getOutputStream() );

      // TODO Findbugs: equals() used to compare array and nonarray
      if ( getResponse( con ).equals( "0" ) )
      {
        LOG.error( "Pdf konnte nicht gespeichert werden." );
      }
      else
      {
        LOG.info( "Pdf konnte gespeichert werden." );
      }
    }
    catch ( MalformedURLException mex )
    {
      LOG.fatal( "DirectControl: URL falsch", mex );
    }
    catch ( IOException ex )
    {
      LOG.error( "DirectControl: " + ex.getMessage(), ex );
    }
  }

  /**
   * Get the http-response from a HttpURLConnection
   * @param con HttpURLConnection to get the response from
   * @return inbuffer ByteArray the http-response as a byte-Array
   */
  protected byte[] getResponse( HttpURLConnection con )
  {
    DataInputStream inStream = null;
    byte[] inbuffer = null;
    int length = 0;
    try
    {
      inStream = new DataInputStream( con.getInputStream() );
      length = con.getContentLength();
      if ( length == -1 )
      {
        // fallback!
        length = inStream.available(); // WARNING!! dangerous
      }
    }
    catch ( IOException iex )
    {
      LOG.warn( "InputStream der ServletConnection kann nicht erstellt werden: " + iex.getMessage(), iex );
    }

    inbuffer = new byte[length];

    try
    {
      inStream.readFully( inbuffer );
      inStream.close();
    }
    catch ( EOFException e )
    {
      LOG.error( "Unerwartetes EOF beim Lesen der Servlet-Response", e );
    }
    catch ( IOException e )
    {
      LOG.error( "IOException beim Lesen der Servlet-Response", e );
    }
    finally
    {
      url = null;
      con.disconnect();
    }
    return inbuffer;
  }

}
