package ch.ige.jipps;

import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;

import org.apache.log4j.Logger;

/**
 * DirectMDBean: Direkter Einstieg in die Komponente JIPPS.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author    Johannes Lang, Marc Bouquet
 * @version   $Id: DirectMDBean.java,v 1.31 2007/01/05 10:19:04 lang Exp $
 * 
 * <!-- begin-xdoclet-definition -->
 * @ejb.bean name="DirectMDBean" 
 *     destination-type="javax.jms.Queue"
 *     transaction-type="Container"
 *     destination-jndi-name="jms/JIPPS.xmlQueue"
 *     connection-factory-jndi-name="jms/IGEQueueConnectionFactory" 
 * @jboss.destination-jndi-name name="jms/JIPPS.xmlQueue"
 * <!-- end-xdoclet-definition -->
 **/
public class DirectMDBean implements MessageDrivenBean, MessageListener
{
  MessageDrivenContext messageDrivenContext;
  private DirectControl directControl;
  private LoeschungControl loeschungControl;
  private static final Logger LOG = Logger.getLogger( DirectMDBean.class );

  public void ejbCreate()
  {}

  public void ejbRemove()
  {}

  /**
   * can currently process Patent-Löschungen (Schutzablauf und Nichtbezahlen) and EP-Bänder.
   * @param msg JMS-Message.
   */
  public void onMessage( Message msg )
  {
    if ( msg instanceof javax.jms.TextMessage )
    {
      try
      {
        String str = msg.getStringProperty( "Doktyp" );
        LOG.debug( "*** Doktyp ist: (" + str + ")" );

        if ( ( str.equalsIgnoreCase( "loeschung" ) ) || ( str.equalsIgnoreCase( "schutzablauf" ) ) )
        {
          loeschungControl = new LoeschungControl();
          loeschungControl.doAufbereitung( str );
        }
        else
        {
          directControl = DirectControl.getInstance();
          directControl.makePrint( ( ( javax.jms.TextMessage ) msg ).getText(),
                                   msg.getStringProperty( "Doktyp" ), msg.getStringProperty( "LadeDatum" ) );
        }
      }
      catch ( JMSException ex )
      {
        LOG.error( "Fehler beim Lesen der JMS-Nachricht: " + ex.getMessage(), ex );
      }
      catch ( NullPointerException nex )
      {
        LOG.error( "Null-Pointer-Exception beim Lesen der JMS-Nachricht: " + nex.getMessage(), nex );
      }
    }
    else
    {
      LOG.info( "Unbekannten Message-Typ erhalten. javax.jms.TextMessage erwartet." );
    }
  }

  /**
   * Setzt den Context des MDB zurück.
   * @param messageDrivenContext Objekt welches dem Bean übergeben wird.
   */
  public void setMessageDrivenContext( MessageDrivenContext messageDrivenContext )
  {
    this.messageDrivenContext = messageDrivenContext;
  }

}

