package ch.ige.jipps;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.PrintWriter;
import java.net.SocketException;
import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPReply;
import org.apache.log4j.Logger;

import ch.ige.jipps.constants.PropertyConstants;
import ch.ige.jipps.vo.ByteArrayVO;

/**
 * FileServlet: Schreibt die gerenterte Pdf-Datei in ein File-System
 * <p>
 * Copyright (c) 2002-2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author Marc Bouquet
 * @version $Id: FileServlet.java,v 1.46 2007/12/20 08:24:19 lewis_lee Exp $
 */
public class FileServlet extends HttpServlet
{
  private static final Logger LOG = Logger.getLogger( FileServlet.class );

  private String hostName = null;

  private boolean isTempPath = false;

  private JIPPSProperties jippsProperties = null;

  private static final String CONTENT_TYPE = "text/html";

  public void init()
  {
    // nothing to do
  }

  /**
   * Leitet die Anfrage an doGet() weiter
   * @param request HttpServletRequest
   * @param response HttpServletResponse
   * @throws IOException
   */
  public void doPost( HttpServletRequest request, HttpServletResponse response ) throws IOException
  {
    doGet( request, response );
  }

  // Die HTTP-Anforderung Get bearbeiten
  public void doGet( HttpServletRequest request, HttpServletResponse response ) throws IOException
  {
    PrintWriter out = response.getWriter();
    ObjectInputStream ois = new ObjectInputStream( request.getInputStream() );
    ByteArrayVO byteArrayVO = null;
    isTempPath = false;

    try
    {
      byteArrayVO = (ByteArrayVO) ois.readObject();

      try
      {
        ftpUpload( byteArrayVO );
      }
      catch ( Exception e )
      {
        isTempPath = true;
        LOG.error( "Error in FileServlet", e );
        FileOutputStream fileOut = new FileOutputStream( getTempPath( byteArrayVO ) + "/" + byteArrayVO.fileName + ".pdf" );
        fileOut.write( byteArrayVO.file );
        fileOut.flush();
        fileOut.close();
      }

      // This is only relevant for EP
      if ( byteArrayVO != null && !byteArrayVO.containsFullPath )
      {
        createMail( byteArrayVO );
      }
      response.setContentType( CONTENT_TYPE );
      out.print( "1" );
    }
    catch ( ClassNotFoundException e )
    {
      LOG.error( "Error in FileServlet", e );
      response.setContentType( CONTENT_TYPE );
      out.print( "0" );
    }
    finally
    {
      out.flush();
      out.close();
    }
  }

  /**
   * @param byteArrayVO
   * @throws Exception
   */
  private void ftpUpload( ByteArrayVO byteArrayVO ) throws Exception
  {
    String ftpServername = getJIPPSProperties().getProperty( "ep_ftp_hostname" );
    String ftpUsername = getJIPPSProperties().getProperty( "ep_ftp_username" );
    String ftpPassword = getJIPPSProperties().getProperty( "ep_ftp_password" );
    String ftpDirectory = getJIPPSProperties().getProperty( "ep_ftp_directory" ) + byteArrayVO.ladeDatum;
    ftpUpload( byteArrayVO.file, ftpServername, ftpUsername, ftpPassword, ftpDirectory, byteArrayVO.fileName + ".pdf" );
  }

  /**
   * get the Windows-path for the email.
   */
  private String getWindowsPath( ByteArrayVO byteArrayVO )
  {
    String path = null;
    try
    {
      path = getJIPPSProperties().getDirectoryPath( "ep" ) + byteArrayVO.ladeDatum;
    }
    catch ( RemoteException e )
    {
      LOG.error( "Error in FileServlet", e );
    }
    catch ( NamingException e )
    {
      LOG.error( "Error in FileServlet", e );
    }
    catch ( CreateException e )
    {
      LOG.error( "Error in FileServlet", e );
    }
    catch ( Exception e )
    {
      LOG.error( "Error in FileServlet", e );
    }

    if ( path != null )
    {
      return path;
    }
    else
    {
      return "";
    }
  }

  /**
   * Erstellt im temporären Verzeichnis ein Verzeichnis mit dem Ladedatum.
   * @return tempPath gibt den Pfad des erstellten Verzeichnis zurück.
   */
  private String getTempPath( ByteArrayVO byteArrayVO )
  {
    String path = System.getProperty( PropertyConstants.JAVA_IO_TMPDIR ) + "/epband/" + byteArrayVO.ladeDatum;
    LOG.warn( "EP-Band Data will be stored in: " + path );
    File file = new File( path );
    if ( !file.isDirectory() )
    {
      file.mkdirs();
    }
    return path;
  }

  private void ftpUpload( byte[] data, String ftpServername, String user, String password, String directory, String filename ) throws SocketException,
      IOException, Exception
  {
    FTPClient ftp = new FTPClient();
    try
    {
      ftp.connect( ftpServername );
      ftp.login( user, password );
      if ( FTPReply.isPositiveCompletion( ftp.getReplyCode() ) )
      {
        LOG.debug( "Connected to " + ftpServername + "." );
        LOG.debug( ftp.getReplyString() );
        ftp.makeDirectory( directory );
        ftp.changeWorkingDirectory( directory );
        ftp.setFileType( FTP.BINARY_FILE_TYPE );
        ByteArrayInputStream is = new ByteArrayInputStream( data );
        ftp.storeFile( filename, is );
        is.close();
        ftp.logout();
      }
      else
      {
        throw new Exception( "FTP-Connection cannot properly be established." );
      }
    }
    finally
    {
      if ( ftp.isConnected() )
      {
        try
        {
          ftp.disconnect();
        }
        catch ( IOException ioe )
        {
          LOG.warn( "FTP connection could not be disconnected.", ioe );
        }
      }
    }

  }

  /**
   * Mit dieser Methode wird ein Mail generiert, welches dem Empfänger einen Statusbericht der Aufbereitung der
   * löschungsanzeige zustellt.
   * @param byteArrayVO Value-Object
   * @throws RemoteException Unerwarteter Remote-Fehler
   */
  private void createMail( ByteArrayVO byteArrayVO ) throws RemoteException
  {
    String betreff = "EP-Liste (" + byteArrayVO.fileName + ")";

    StringBuffer bufMsg = new StringBuffer();
    bufMsg.append( "Statusbericht der EP-Listen:\n" );
    bufMsg.append( "Lade Datum:\t\t\t\t\t" );
    bufMsg.append( byteArrayVO.ladeDatum );
    bufMsg.append( "\n" );
    bufMsg.append( "Bezeichnung:\t\t\t\t" );
    bufMsg.append( byteArrayVO.fileName );
    bufMsg.append( "\n" );
    bufMsg.append( "Ablage-Verzeichnis:\t\t\t" );

    if ( !isTempPath )
    {
      bufMsg.append( getWindowsPath( byteArrayVO ) );
    }
    else
    {
      bufMsg.append( getHostname() );
      bufMsg.append( getTempPath( byteArrayVO ) );
      bufMsg.append( "; (Application Server) - bei IS-O die PDF-Datei verlangen!" );
    }

    bufMsg.append( "\n" );

    bufMsg.append( "Verarbeitungs-Servername:\t\t" );
    bufMsg.append( getHostname() );

    jippsProperties.doMail( betreff, bufMsg.toString(), "ep" );
    LOG.debug( "*** Meldung wurde an JIPPSProperties gesendet: " + bufMsg.toString() );
  }

  /**
   * Ermittelt via DOS-Command den Hostnamen des aktuellen Servers
   * @return String hostName
   */
  private String getHostname()
  {
    if ( hostName == null )
    {
      try
      {
        Process proc = Runtime.getRuntime().exec( "hostname" );
        BufferedReader in = new BufferedReader( new InputStreamReader( proc.getInputStream() ) );
        String computerName = in.readLine();
        in.close();
        hostName = computerName;
      }
      catch ( IOException ex )
      {
        LOG.error( "Computername konnte nicht ermittelt werden", ex );
        hostName = "unknown";
      }
    }
    return hostName;
  }

  /**
   * Erstellt den Lookup auf das JIPPSPropertiesBean
   * @return JIPPSProperties
   * @throws NamingException
   * @throws RemoteException
   * @throws CreateException
   */
  public JIPPSProperties getJIPPSProperties() throws NamingException, RemoteException, CreateException
  {
    if ( jippsProperties == null )
    {
      Context ctx = new InitialContext();
      JIPPSPropertiesHome jippsPropertiesHome = (JIPPSPropertiesHome) ctx.lookup( JIPPSPropertiesHome.JNDI_NAME );
      jippsProperties = jippsPropertiesHome.create();
    }
    return jippsProperties;
  }

  public void destroy()
  {
    // nothing to do
  }

}
