package ch.ige.jipps.dao;

import java.sql.SQLException;
import java.text.MessageFormat;
import java.util.Collections;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import ch.ige.dbutils.dao.AbstractBagisDAO;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.jipps.vo.BfakTxtVO;
import ch.ige.util.constants.TextConstants;
import ch.ige.util.validation.Check;

/**
 * Accesses the optional form texts in the table <code>BFAKTXT</code>.
 * 
 * <p>Copyright (c) 2002-2006, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author Marc Bouquet
 * @author Eric Lewis
 * @version $Id: BfakTxtDAO.java,v 1.26 2008/05/06 07:37:43 lewis_lee Exp $
 */
public class BfakTxtDAO extends AbstractBagisDAO
{
  /**
   * The type of value object to return
   */
  private static final Class<BfakTxtVO> VO_TYPE = BfakTxtVO.class;

  /**
   * Selects a bunch of texts
   */
  private static final SQLStatement<BfakTxtVO> SELECT = SQL.select( "SELECT text_nr, " + "DECODE(text, ''" + TextConstants.TEXT_IS_TOO_LONG
      + "'', text_long, to_clob(text)) AS text " + "FROM bfaktxt " + "WHERE text_nr IN ( {0} ) ORDER BY text_nr" );

  /**
   * Selects all texts having the given text numbers.
   * @param textNumbers The text numbers to search for
   * @return The corresponding texts
   * @throws SQLException if the query fails
   */
  public List<BfakTxtVO> select( List<Integer> textNumbers ) throws SQLException
  {
    Check.notNull( textNumbers, "text numbers" );
    if ( textNumbers.isEmpty() )
    {
      return Collections.emptyList();
    }

    String textNumbersList = StringUtils.join( textNumbers.iterator(), ", " );
    String sqlStatement = MessageFormat.format( SELECT.toString(), textNumbersList );
    return selectVOList( textNumbers.size(), VO_TYPE, SQL.<BfakTxtVO>select( sqlStatement ) );
  }
}
