package ch.ige.jipps.dao;

import java.sql.Connection;
import java.sql.SQLException;

import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.AbstractBagisDAO;
import ch.ige.dbutils.dao.AbstractJippsDAO;

/**
 * Accesses the metadata for the database connections. Copyright (c) 2003-2006, Eidgenössisches Institut für Geistiges
 * Eigentum
 * @author Marc Bouquet
 * @author Johannes Lang
 * @version $Id: InfoDAO.java,v 1.22 2007/12/19 17:53:35 lewis_lee Exp $
 */
public class InfoDAO
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( InfoDAO.class );

  /**
   * The URL for JIPPS
   */
  private static String jippsUrl = null;

  /**
   * The URL for BAGIS
   */
  private static String bagisUrl = null;

  /**
   * Part of the error message
   */
  private static String RESOLVE_ERROR_MESSAGE = "konnte nicht ermittelt werden";

  /**
   * Hidden constructor
   */
  private InfoDAO()
  {
    // This should only be used with static method calls
  }

  /**
   * Gets the URL of the JIPPS database.
   * @return The URL of the JIPPS database
   */
  public static String getJippsDbURL()
  {
    if ( jippsUrl == null )
    {
      try
      {
        jippsUrl = new JippsUrlDAO().getJippsUrl();
      }
      catch ( SQLException ex )
      {
        LOG.error( "Auf die JIPPS-DB konnte keine Verbindung erstellt werden", ex );
        jippsUrl = RESOLVE_ERROR_MESSAGE;
      }
    }
    return jippsUrl;
  }

  /**
   * Gets the JIPPS database URL.
   */
  private static class JippsUrlDAO extends AbstractJippsDAO
  {
    /**
     * Gets the JIPPS database URL.
     * @return The JIPPS database URL
     * @throws SQLException if the URL can't be fetched
     */
    public String getJippsUrl() throws SQLException
    {
      Connection connection = null;
      try
      {
        connection = getConnection();
        return connection.getMetaData().getURL();
      }
      finally
      {
        if ( connection != null )
        {
          connection.close();
        }
      }
    }
  }

  /**
   * Holt die Meta-Daten der Bagis-Datenbank.
   * @return url URL der Bagis-Datenbank
   */
  public static String getBagisDbURL()
  {
    if ( bagisUrl == null )
    {
      try
      {
        bagisUrl = new BagisUrlDAO().getBagisUrl();
      }
      catch ( SQLException ex )
      {
        LOG.error( "Auf die BAGIS-DB konnte keine Verbindung erstellt werden", ex );
        bagisUrl = RESOLVE_ERROR_MESSAGE;
      }
    }
    return bagisUrl;
  }

  /**
   * Gets the BAGIS database URL.
   */
  private static class BagisUrlDAO extends AbstractBagisDAO
  {
    /**
     * Gets the BAGIS database URL.
     * @return The BAGIS database URL
     * @throws SQLException if the URL can't be fetched
     */
    public String getBagisUrl() throws SQLException
    {
      Connection connection = null;
      try
      {
        connection = getConnection();
        return connection.getMetaData().getURL();
      }
      finally
      {
        if ( connection != null )
        {
          connection.close();
        }
      }
    }
  }
}
