package ch.ige.jipps.ftp.web.test;

import java.io.File;
import java.io.IOException;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

import junit.framework.TestCase;

import org.apache.commons.io.FilenameUtils;

import ch.ige.jipps.ftp.web.AbstractService;
import ch.ige.jipps.ftp.web.FTPServiceException;
import ch.ige.jipps.ftp.web.ResultBean;
import ch.ige.jipps.ftp.web.util.FTPServletConfiguration;

public class ServiceTest extends TestCase
{
  
  private static final String TMP_PATH_DOWNLOAD = FilenameUtils.concat( System.getProperty( "java.io.tmpdir" ) , "ftpservlet/downlaod" );
  
  private static final String TMP_PATH_UPLOAD = FilenameUtils.concat( System.getProperty( "java.io.tmpdir" ) , "ftpservlet/uplaod" );
  
  
  private static final String FOLDER_FOR_DELETION_TEST = "testFailureOfDelete";
  
  
  @Override
  protected void setUp() throws Exception
  {
    super.setUp();
    FTPServletConfiguration.addConfigurationEntry( FTPServletConfiguration.TMP_DIR_DOWNLOAD, TMP_PATH_DOWNLOAD);
    FTPServletConfiguration.addConfigurationEntry( FTPServletConfiguration.TMP_DIR_UPLOAD, TMP_PATH_UPLOAD);
  }
  
  public ServiceTest( String name )
  {
    super( name );
  }

  public void testServiceTmpDirDeleted() throws Exception
  {
    AbstractService testService = new TestService(null)
    {

      public String execute( HttpServletRequest request, ResultBean bean ) throws FTPServiceException
      {
        try
        {
          this.testFolderDeleted();
          return "Success";
        }
        catch ( IOException e )
        {
          throw new FTPServiceException(e);
        }
      }
   };
       
    testService.execute( null, null );
    
    
  }
  
  public void testServiceTmpDirNotDeleted() throws Exception
  {
    AbstractService testService = new TestService(null)
    {

      public String execute( HttpServletRequest request, ResultBean bean ) throws FTPServiceException
      {
        try
        {
          this.testFolderNotDeleted();
          return "Success";
        }
        catch ( IOException e )
        {
          throw new FTPServiceException(e);
        }
      }
   };
       
    testService.execute( null, null );   
    
  }
  
  
  public void testServiceTmpDirNotDeletedBecauseErroneousConfig() throws Exception
  {
    AbstractService testService = new TestService(null)
    {

      public String execute( HttpServletRequest request, ResultBean bean ) throws FTPServiceException
      {
        try
        {
          this.testDeleteNotSucessfulBecauseOferroneousConfiguration();
          return "Success";
        }
        catch ( IOException e )
        {
          throw new FTPServiceException(e);
        }
      }
   };
       
    testService.execute( null, null );
    
    
  }
  
  public void testServiceTmpDirDeletionWithException() throws Exception
  {
    AbstractService testService = new TestService(null)
    {

      public String execute( HttpServletRequest request, ResultBean bean ) throws FTPServiceException
      {
        try
        {
          this.testDeleteNotSucessful();
          return "Success";
        }
        catch ( IOException e )
        {
          throw new FTPServiceException(e);
        }
      }
   };
       
    testService.execute( null, null );
    
    
  }
  
  private static abstract class TestService extends AbstractService
  {


    public TestService( ServletContext context )
    {
      super( context );
    }



    protected void testFolderNotDeleted() throws IOException
    {
      testDeleteFolder(new File(System.getProperty( "java.io.tmpdir" ), FOLDER_FOR_DELETION_TEST ), false);
    }
    
    protected void testFolderDeleted() throws IOException
    {
    
      testDeleteFolder( new File( TMP_PATH_DOWNLOAD, FOLDER_FOR_DELETION_TEST), true );
      testDeleteFolder( new File( TMP_PATH_UPLOAD, FOLDER_FOR_DELETION_TEST), true );
      
    }
    
    protected void testDeleteNotSucessful() throws IOException
    {
      // missing configuration
      FTPServletConfiguration.addConfigurationEntry( FTPServletConfiguration.TMP_DIR_DOWNLOAD, null);
      try
      {
        testDeleteFolder(new File( TMP_PATH_DOWNLOAD, FOLDER_FOR_DELETION_TEST ), false);
      }
      catch (Exception e)
      {
        assertTrue ( e instanceof IllegalArgumentException);
      }
      
      // missing configuration
      FTPServletConfiguration.addConfigurationEntry( FTPServletConfiguration.TMP_DIR_UPLOAD, null);
      
      try
      {
        testDeleteFolder(new File( TMP_PATH_UPLOAD, FOLDER_FOR_DELETION_TEST), false);
      }
      catch (Exception e)
      {
        assertTrue ( e instanceof IllegalArgumentException);
      }

    }
    
    protected void testDeleteNotSucessfulBecauseOferroneousConfiguration() throws IOException
    {
      // missing "ftpservlet" in configuration
      FTPServletConfiguration.addConfigurationEntry( FTPServletConfiguration.TMP_DIR_DOWNLOAD, "download");
      
      testDeleteFolder(new File( TMP_PATH_DOWNLOAD, FOLDER_FOR_DELETION_TEST ), false);
     
      
      // missing "ftpservlet" in configuration
      FTPServletConfiguration.addConfigurationEntry( FTPServletConfiguration.TMP_DIR_UPLOAD, "upload");

      testDeleteFolder(new File( TMP_PATH_UPLOAD, FOLDER_FOR_DELETION_TEST), false);


    }
    
    private void testDeleteFolder(File file, boolean success ) throws IOException
    {
      
      file.mkdirs();
      assertTrue(file.exists());
      this.deleteTmpDirectory( file );
      if ( success )
      {
        assertTrue(!file.exists());
      }
      else
      {
        assertTrue(file.exists());
      }
      
    }
  }

}
