package ch.ige.jipps.header;

import java.sql.Date;

import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.common.history.HistoryItem;
import ch.ige.ipobjects.design.Design;
import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.util.constants.NumericLiterals;
import ch.ige.util.date.LanguageDateFormatter;

/**
 * Builds the special cases for the IP header in the report for designs.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: DesignHeaderBuilder.java,v 1.8 2007/12/18 12:32:24 lewis_lee Exp $
 */
class DesignHeaderBuilder extends BaseIpHeaderBuilder
{
  // See interface
  @Override
  public String getDescription( GenericIP ip, Date reportDate )
  {
    Design design = (Design) ip;
    HistoryItem<String> titleHistory = design.getTitle().getItemContaining( EreigIDat.max( reportDate ) );
    if ( titleHistory != null )
    {
      return titleHistory.getValue();
    }
    else
    {
      return EMPTY;
    }
  }

  // See interface
  @Override
  public PublicationVO getPublication( GenericIP ip, LanguageDateFormatter dateFormatter, Date reportDate, boolean isFirstPublication )
  {
    // There's no last publication for designs
    if ( !isFirstPublication )
    {
      return super.getDefaultPublicationResults( ip, reportDate, isFirstPublication );
    }

    Event publicationEvent = getPublicationEvent( ip, reportDate, isFirstPublication );
    PublicationVO publication = getDefaultPublicationResults( ip, reportDate, isFirstPublication );
    if ( publicationEvent != null && publicationEvent.getDate() != null )
    {
      // Designs show the full publication date, not only the publication year.
      publication.formattedPublicationDate = Header.formatDate( publicationEvent.getPubl_dat(), dateFormatter );
      publication.publicationYear = NumericLiterals.ZERO;
      publication.publicationNumber = publicationEvent.getPmmbl_seite();
      publication.publishPublicationEvent = false; // Irrelevant for designs
    }
    return publication;
  }
}
