package ch.ige.jipps.header;

import java.util.Locale;

import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.util.enums.IPType;
import ch.ige.util.enums.RegistrationState;
import ch.ige.util.enums.report.ReportType;

/**
 * Contains the data for the header in the report concerning the IP's base data.
 * <p>
 * Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * 
 * @author Eric Lewis
 * @version $Id: IpHeaderVO.java,v 1.18 2008/09/22 10:05:14 lewis_lee Exp $
 */
public class IpHeaderVO extends AbstractVO implements IpHeader
{
  private ReportType reportType;

  private String reportTitle;

  private String reportDate;

  private Locale reportLanguage;

  private Integer ipKey;

  private String ipNumber;

  private String formattedIpNumber;

  private String formattedBaseIpNumber;

  private IPType ipType;

  private String description;

  private String imageFilename;

  private String protectionEnd;

  private String baseProtectionStart;

  private String applicationDate;

  private String applicationPublicationDate;

  private String formattedApplicationNumber;

  private String cancellation;

  private RegistrationState registrationState;

  private String registrationDate;

  private boolean isPublishedElectronically;

  private String firstPublicationDate;

  private Integer firstPublicationYear;

  private Integer firstPublicationNumber;

  private Boolean eventOfFirstPublicationPublished;

  private String lastPublicationDate;

  private Integer lastPublicationYear;

  private Integer lastPublicationNumber;

  private Boolean eventOfLastPublicationPublished;

  private String oppositionDeadline;

  /**
   * The empty constructor.
   */
  public IpHeaderVO()
  {
    // Nothing to do
  }

  // See interface
  public ReportType getReportType()
  {
    return this.reportType;
  }

  /**
   * Sets the report type.
   * 
   * @param reportType The report type to set
   */
  public void setReportType( ReportType reportType )
  {
    this.reportType = reportType;
  }

  // See interface
  public String getReportTitle()
  {
    return this.reportTitle;
  }

  /**
   * Sets the report title.
   * 
   * @param reportTitle The report title to set
   */
  public void setReportTitle( String reportTitle )
  {
    this.reportTitle = reportTitle;
  }

  // See interface
  public String getReportDate()
  {
    return this.reportDate;
  }

  /**
   * Sets the formatted date of the report.
   * 
   * @param reportDate The formatted date of the report
   */
  public void setReportDate( String reportDate )
  {
    this.reportDate = reportDate;
  }

  // See interface
  public Locale getReportLanguage()
  {
    return this.reportLanguage;
  }

  /**
   * Sets the report's language.
   * 
   * @param reportLanguage The report's language
   */
  public void setReportLanguage( Locale reportLanguage )
  {
    this.reportLanguage = reportLanguage;
  }

  // See interface
  public Integer getIpKey()
  {
    return this.ipKey;
  }

  /**
   * Sets the unique key identifying the IP.
   * 
   * @param ipKey The unique key identifying the IP
   */
  public void setIpKey( Integer ipKey )
  {
    this.ipKey = ipKey;
  }

  // See interface
  public String getIpNumber()
  {
    return this.ipNumber;
  }

  /**
   * Sets the IP number, which is unique per IP type.
   * 
   * @param ipNumber The IP number, which is unique per IP type in the source format (like in the database)
   */
  public void setIpNumber( String ipNumber )
  {
    this.ipNumber = ipNumber;
  }

  // See interface
  public String getFormattedIpNumber()
  {
    return this.formattedIpNumber;
  }

  /**
   * The formatted version of the IP number, which loses the checksum.
   * 
   * @param formattedIpNumber The formatted version of the IP number
   */
  public void setFormattedIpNumber( String formattedIpNumber )
  {
    this.formattedIpNumber = formattedIpNumber;
  }

  // See interface
  public IPType getIpType()
  {
    return this.ipType;
  }

  /**
   * Sets the IP type.
   * 
   * @param ipType The IP type
   */
  public void setIpType( IPType ipType )
  {
    this.ipType = ipType;
  }

  // See interface
  public String getDescription()
  {
    return this.description;
  }

  /**
   * Sets the IP's description.
   * 
   * @param description The IP's description
   */
  public void setDescription( String description )
  {
    this.description = description;
  }

  // See interface
  public String getImageFilename()
  {
    return this.imageFilename;
  }

  /**
   * Sets the image's filename.
   * 
   * @param imageFilename The image's filename
   */
  public void setImageFilename( String imageFilename )
  {
    this.imageFilename = imageFilename;
  }

  // See interface
  public String getFormattedBaseIpNumber()
  {
    return this.formattedBaseIpNumber;
  }

  /**
   * Sets the formatted IP number for an SPC's base patent.
   * 
   * @param formattedBaseIpNumber The formatted IP number for an SPC's base patent
   */
  public void setFormattedBaseIpNumber( String formattedBaseIpNumber )
  {
    this.formattedBaseIpNumber = formattedBaseIpNumber;
  }

  // See interface
  public String getProtectionEnd()
  {
    return this.protectionEnd;
  }

  /**
   * Sets the formatted end of protection for an IP.
   * 
   * @param protectionEnd The formatted end of protection for an IP
   */
  public void setProtectionEnd( String protectionEnd )
  {
    this.protectionEnd = protectionEnd;
  }

  // See interface
  public String getBaseProtectionStart()
  {
    return this.baseProtectionStart;
  }

  /**
   * For an SPC, this sets the formatted date when the original patent was registered.
   * 
   * @param baseProtectionStart The formatted start of protection for an SPC's base patent
   */
  public void setBaseProtectionStart( String baseProtectionStart )
  {
    this.baseProtectionStart = baseProtectionStart;
  }

  // See interface
  public String getApplicationDate()
  {
    return this.applicationDate;
  }

  /**
   * Sets the formatted application date.
   * 
   * @param applicationDate The formatted formatted application date
   */
  public void setApplicationDate( String applicationDate )
  {
    this.applicationDate = applicationDate;
  }

  // See interface
  public String getApplicationPublicationDate()
  {
    return this.applicationPublicationDate;
  }

  /**
   * Sets the formatted date showing when the application was published.
   * @param applicationPublicationDate The formatted date showing when the application was published
   */
  public void setApplicationPublicationDate( String applicationPublicationDate )
  {
    this.applicationPublicationDate = applicationPublicationDate;
  }

  // See interface
  public String getFormattedApplicationNumber()
  {
    return this.formattedApplicationNumber;
  }

  /**
   * Gets the formatted application number.
   * 
   * @param formattedApplicationNumber The formatted application number
   */
  public void setFormattedApplicationNumber( String formattedApplicationNumber )
  {
    this.formattedApplicationNumber = formattedApplicationNumber;
  }

  // See interface
  public String getCancellation()
  {
    return this.cancellation;
  }

  /**
   * Sets the formatted date of cancellation.
   * 
   * @param cancellation The formatted date of cancellation
   */
  public void setCancellation( String cancellation )
  {
    this.cancellation = cancellation;
  }

  // See interface
  public RegistrationState getRegistrationState()
  {
    return this.registrationState;
  }

  /**
   * Sets the registration state.
   * 
   * @param registrationState The registration state to set
   */
  public void setRegistrationState( RegistrationState registrationState )
  {
    this.registrationState = registrationState;
  }

  // See interface
  public String getRegistrationDate()
  {
    return this.registrationDate;
  }

  /**
   * Sets the formatted date when this IP was registered.
   * 
   * @param registrationDate the formatted date when this IP was registered
   */
  public void setRegistrationDate( String registrationDate )
  {
    this.registrationDate = registrationDate;
  }

  // See interface
  public boolean isPublishedElectronically()
  {
    return this.isPublishedElectronically;
  }

  /**
   * Sets the flag that shows whether this is published electronically.
   * 
   * @param isPublishedElectronically <code>true</code> if this is published electronically, otherwise <code>false</code>
   */
  public void setPublishedElectronically( boolean isPublishedElectronically )
  {
    this.isPublishedElectronically = isPublishedElectronically;
  }

  // See interface
  public String getFirstPublicationDate()
  {
    return this.firstPublicationDate;
  }

  /**
   * Sets the date of first publication.
   * 
   * @param firstPublicationDate The date of first publication
   */
  public void setFirstPublicationDate( String firstPublicationDate )
  {
    this.firstPublicationDate = firstPublicationDate;
  }

  // See interface
  public Integer getFirstPublicationYear()
  {
    return this.firstPublicationYear;
  }

  /**
   * Sets the four-digit year of first publication (taken from the date of publication).
   * 
   * @param firstPublicationYear The four-digit year of first publication (taken from the date of publication)
   */
  public void setFirstPublicationYear( Integer firstPublicationYear )
  {
    this.firstPublicationYear = firstPublicationYear;
  }

  // See interface
  public Integer getFirstPublicationNumber()
  {
    return this.firstPublicationNumber;
  }

  /**
   * Sets the publication number given when the IP was published.
   * 
   * @param firstPublicationNumber The publication number given when the IP was published
   */
  public void setFirstPublicationNumber( Integer firstPublicationNumber )
  {
    this.firstPublicationNumber = firstPublicationNumber;
  }

  // See interface
  public Boolean isEventOfFirstPublicationPublished()
  {
    return this.eventOfFirstPublicationPublished;
  }

  /**
   * Sets the indication whether the event containing the date of publication must be published.
   * 
   * @param eventOfFirstPublicationPublished <code>true</code> if the event containing the date of publication must be
   *          published, otherwise <code>false</code>
   */
  public void setEventOfFirstPublicationPublished( Boolean eventOfFirstPublicationPublished )
  {
    this.eventOfFirstPublicationPublished = eventOfFirstPublicationPublished;
  }

  // See interface
  public String getLastPublicationDate()
  {
    return this.lastPublicationDate;
  }

  /**
   * Sets the date of last publication.
   * 
   * @param lastPublicationDate The date of last publication
   */
  public void setLastPublicationDate( String lastPublicationDate )
  {
    this.lastPublicationDate = lastPublicationDate;
  }

  // See interface
  public Integer getLastPublicationYear()
  {
    return this.lastPublicationYear;
  }

  /**
   * Sets the four-digit year of last publication (taken from the date of publication).
   * 
   * @param lastPublicationYear The four-digit year of last publication (taken from the date of publication)
   */
  public void setLastPublicationYear( Integer lastPublicationYear )
  {
    this.lastPublicationYear = lastPublicationYear;
  }

  // See interface
  public Integer getLastPublicationNumber()
  {
    return this.lastPublicationNumber;
  }

  /**
   * Sets the publication number given when the IP was published.
   * 
   * @param lastPublicationNumber The publication number given when the IP was published
   */
  public void setLastPublicationNumber( Integer lastPublicationNumber )
  {
    this.lastPublicationNumber = lastPublicationNumber;
  }

  // See interface
  public Boolean isEventOfLastPublicationPublished()
  {
    return this.eventOfLastPublicationPublished;
  }

  /**
   * Sets the indication whether the event containing the date of publication must be published.
   * 
   * @param eventOfLastPublicationPublished <code>true</code> if the event containing the date of publication must be
   *          published, otherwise <code>false</code>
   */
  public void setEventOfLastPublicationPublished( Boolean eventOfLastPublicationPublished )
  {
    this.eventOfLastPublicationPublished = eventOfLastPublicationPublished;
  }

  // See interface
  public String getOppositionDeadline()
  {
    return this.oppositionDeadline;
  }

  /**
   * Sets the formatted last date for opposition to a trademark.
   * 
   * @param oppositionDeadline The formatted last date for opposition to a trademark
   */
  public void setOppositionDeadline( String oppositionDeadline )
  {
    this.oppositionDeadline = oppositionDeadline;
  }
}
