package ch.ige.jipps.header;

import java.sql.Date;

import org.apache.commons.lang.StringUtils;

import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.spc.SPC;
import ch.ige.util.constants.NumericLiterals;
import ch.ige.util.date.LanguageDateFormatter;
import ch.ige.util.format.ApplicationNumberFormatter;
import ch.ige.util.format.ReportIpNumberFormatter;

/**
 * Builds the special cases for the IP header in the report for SPCs.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: SpcHeaderBuilder.java,v 1.15 2007/12/18 12:32:48 lewis_lee Exp $
 */
class SpcHeaderBuilder extends BaseIpHeaderBuilder
{
  // See interface
  @Override
  public String getFormattedIpNumber( GenericIP ip )
  {
    SPC spc = (SPC) ip;
    return ReportIpNumberFormatter.formatSpcWithDefault( spc.getIPNumber(), spc.getBAGISType(), spc.getBasePatent().getBAGISType() );
  }

  // See interface
  @Override
  public String getFormattedBaseIpNumber( GenericIP ip )
  {
    SPC spc = (SPC) ip;
    return ReportIpNumberFormatter.formatWithDefault( spc.getBasePatent().getIPNumber(), spc.getBasePatent().getBAGISType() );
  }

  // See interface
  @Override
  public String getFormattedApplicationNumber( GenericIP ip )
  {
    SPC spc = (SPC) ip;
    if ( StringUtils.isNotEmpty( spc.getApplicationNumber() ) )
    {
      // The SPC application number contains slashes
      String trueNumber = spc.getApplicationNumber().replaceAll( "/", "" );
      return ApplicationNumberFormatter.formatSpc( trueNumber, spc.getBAGISType(), spc.getBasePatent().getBAGISType() );
    }
    else
    {
      return super.getFormattedApplicationNumber( ip );
    }
  }

  // See interface
  @Override
  public String getBaseProtectionStart( GenericIP ip, LanguageDateFormatter dateFormatter )
  {
    SPC spc = (SPC) ip;
    return Header.formatDate( spc.getBasePatent().getProtectionStart(), dateFormatter );
  }

  // See interface
  @Override
  public PublicationVO getPublication( GenericIP ip, LanguageDateFormatter dateFormatter, Date reportDate, boolean isFirstPublication )
  {
    // There's no last publication for SPCs
    if ( !isFirstPublication )
    {
      return super.getDefaultPublicationResults( ip, reportDate, isFirstPublication );
    }

    PublicationVO publication = getDefaultPublicationResults( ip, reportDate, isFirstPublication );
    Event publicationEvent = getPublicationEvent( ip, reportDate, isFirstPublication );
    if ( publicationEvent != null && publicationEvent.getDate() != null )
    {
      // SPCs show the full publication date, not only the publication year.
      publication.formattedPublicationDate = Header.formatDate( publicationEvent.getPubl_dat(), dateFormatter );
      publication.publicationYear = NumericLiterals.ZERO;
      publication.publicationNumber = publicationEvent.getPmmbl_seite();
      publication.publishPublicationEvent = false; // Irrelevant for SPCs
    }
    return publication;
  }
}
