package ch.ige.jipps.history.description;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.time.PointInTime;
import ch.ige.ipobjects.time.TimePeriod;
import ch.ige.ipobjects.trademark.Trademark;
import ch.ige.jipps.history.detail.BaseHistoryItemDetail;
import ch.ige.jipps.history.detail.trademark.ColorClaimDetail;
import ch.ige.jipps.history.detail.trademark.GoodsAndServicesDetail;
import ch.ige.jipps.history.detail.trademark.ImageDetail;
import ch.ige.jipps.history.detail.trademark.PublicationRemarkDetail;
import ch.ige.jipps.history.detail.trademark.TrademarkKindDetail;
import ch.ige.jipps.history.detail.trademark.TrademarkPrioritiesDetail;
import ch.ige.util.enums.report.ReportType;

/**
 * Configures how the history part of the report is built for a trademark.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: TrademarkHistoryDescription.java,v 1.18 2007/12/20 08:24:54 lewis_lee Exp $
 */
public class TrademarkHistoryDescription extends BaseHistoryDescription
{
  /**
   * Creates this with all required data.
   * @param ip The patent
   * @param currentState The history is built for this date
   * @param language The language of the report
   * @param includeWaitListEvents If this is <code>true</code>, events that are on the wait list are also shown
   * @param startFromApplication If this is <code>true</code>, events starting from the application are shown,
   * otherwise from the registration
   * @param allowedReportGroups The report groups, a <code>Map</code> where the keys show whether an event must be
   * included on a report. The values can be ignored.
   * @param reportType The report type
   */
  public TrademarkHistoryDescription( GenericIP ip, PointInTime currentState, Locale language, boolean includeWaitListEvents, boolean startFromApplication,
      Map<Integer, String> allowedReportGroups, ReportType reportType )
  {
    super( ip, currentState, language, includeWaitListEvents, startFromApplication, allowedReportGroups, reportType );
  }

  // See interface
  @SuppressWarnings("unchecked")
  @Override
  public List<BaseHistoryItemDetail<? extends TimePeriod>> getIpSpecificDetailDescriptions()
  {
    Trademark trademark = (Trademark) this.ip;
    List<BaseHistoryItemDetail<? extends TimePeriod>> descriptions = new ArrayList<BaseHistoryItemDetail<? extends TimePeriod>>();
    descriptions.add( new ImageDetail( trademark.getImages() ) );
    descriptions.add( new TrademarkPrioritiesDetail( trademark.getGroupedPriorites().getTimePeriods(), trademark.getType(), this.ip ) );
    descriptions.add( new GoodsAndServicesDetail( trademark.getGoodsAndServices().getTimePeriods(), trademark.getType(), this.language ) );
    descriptions.add( new TrademarkKindDetail( trademark.getGroupedTrademarkKinds().getTimePeriods(), this.language ) );
    descriptions.add( new ColorClaimDetail( trademark.getColorClaim() ) );
    descriptions.add( new PublicationRemarkDetail( trademark.getPublishedRemark() ) );
    return descriptions;
  }
}
