package ch.ige.jipps.history.description.test;

import java.text.ParseException;
import java.util.Locale;

import ch.ige.ipobjects.Types;
import ch.ige.ipobjects.common.model.GenericType;
import ch.ige.ipobjects.design.Design;
import ch.ige.ipobjects.design.model.DesignEvent;
import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.jipps.history.description.PublicationText;
import ch.ige.util.date.DateConverter;
import ch.ige.util.enums.PublicationTarget;
import ch.ige.util.enums.report.ReportType;

/**
 * Tests the creation of the publication text for online publication.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: OnlinePublicationTextTest.java,v 1.6 2008/05/21 13:49:57 lewis_lee Exp $
 */
public class OnlinePublicationTextTest extends BasePublicationTextTestTemplate
{
  /**
   * The only constructor
   * @param name The test's name
   */
  public OnlinePublicationTextTest( String name )
  {
    super( name );
  }

  /**
   * Since the handling is much easier for online publication, all tests just call this.
   * @throws Exception if the test fails
   */
  private void genericTest() throws Exception
  {
    // Normal publication on Swissreg
    Design design = createDesign( true, true, "03.02.2008" );
    Event event = design.getEvents().get( 0 );

    String publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2008040199 ), event, Locale.GERMAN,
        ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking published event", "Veröffentlicht in Swissreg am 03. Februar 2008", publicationText );

    // Doesn't need to be published
    design = createDesign( true, false, null );
    event = design.getEvents().get( 0 );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2008040199 ), event, Locale.GERMAN,
        ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking event that doesn't need to be published", null, publicationText );

    // Doesn't need to be published, but has a publication date
    try
    {
      design = createDesign( true, false, "03.02.2008" );
      event = design.getEvents().get( 0 );

      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2008040199 ), event, Locale.GERMAN,
          ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
      fail( "Event without publication but with publication date must fail" );
    }
    catch ( IllegalStateException e )
    {
      // This is expected
    }

    // Must be published, but hasn't been published yet
    design = createDesign( true, true, null );
    event = design.getEvents().get( 0 );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2008040199 ), event, Locale.GERMAN,
        ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking event that must be published, but hasn't been published yet", "Noch nicht publiziert.", publicationText );
  }

  // @see ch.ige.jipps.history.description.test.BasePublicationTextTestTemplate#testDefinitiveAndNonRequiredPublication()
  @Override
  public void testDefinitiveAndNonRequiredPublication() throws Exception
  {
    genericTest();
  }

  // @see ch.ige.jipps.history.description.test.BasePublicationTextTestTemplate#testDefinitiveAndRequiredPublication()
  @Override
  public void testDefinitiveAndRequiredPublication() throws Exception
  {
    genericTest();
  }

  // @see ch.ige.jipps.history.description.test.BasePublicationTextTestTemplate#testPreliminaryAndNonRequiredPublication()
  @Override
  public void testPreliminaryAndNonRequiredPublication() throws Exception
  {
    genericTest();
  }

  // @see ch.ige.jipps.history.description.test.BasePublicationTextTestTemplate#testPreliminaryAndRequiredPublication()
  @Override
  public void testPreliminaryAndRequiredPublication() throws Exception
  {
    genericTest();
  }

  // @see ch.ige.jipps.history.description.test.BasePublicationTextTestTemplate#testPublicationChange()
  @Override
  public void testPublicationChange() throws Exception
  {
    genericTest();
  }

  /**
   * Creates a design (it doesn't matter what IP is taken, they are all handled the same way).
   * @param isPublicationDefinitive Shows whether the publication of the event is definitive
   * @param isPublicationRequired Shows whether the publication of the event is required
   * @param publicationDate The publication date in the format <code>dd.MM.yyyy</code>
   * @return The whole design with one event containing the given parameters
   * @throws ParseException if the publication date can't be parsed
   */
  private Design createDesign( boolean isPublicationDefinitive, boolean isPublicationRequired, String publicationDate ) throws ParseException
  {
    Design design = (Design) Types.DESIGN.createFacade( Types.DESIGN.newInstance() );
    design.set( GenericType.PROP_IPKEY, Integer.valueOf( 99999 ) );
    design.set( GenericType.PROP_TYPE, Types.DESIGN.getBAGISType() );
    Event event = DesignEvent.wrap( new MockEvent( isPublicationRequired, isPublicationDefinitive, DateConverter.stringToSqlDate( publicationDate ), null,
        null, PublicationTarget.SWISSREG2 ) );
    setEvent( design, event );
    return design;
  }
}
