package ch.ige.jipps.history.detail;

import java.util.List;
import java.util.Locale;

import ch.ige.ipobjects.model.IPType;
import ch.ige.ipobjects.report.history.impl.AbstractHistoryItemDetailDescription;
import ch.ige.ipobjects.time.TimePeriod;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.jipps.vo.HistoryItemVO;

/**
 * Builds the detail description for an event in the history part of the report.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: BaseHistoryItemDetail.java,v 1.9 2007/12/19 08:57:02 lewis_lee Exp $
 * @param <T> The time period type
 */
public abstract class BaseHistoryItemDetail<T extends TimePeriod> extends AbstractHistoryItemDetailDescription<T, List<HistoryItemVO>, HistoryItemVO, Detail>
{
  /**
   * The report's language
   */
  protected Locale language;

  /**
   * The IP's type
   */
  protected IPType ipType;

  /**
   * Gets the language.
   * @return The language
   */
  public Locale getLanguage()
  {
    return this.language;
  }

  /**
   * Gets the IP type.
   * @return The IP type
   */
  public IPType getIpType()
  {
    return this.ipType;
  }

  /**
   * Creates this with its detail data.
   * @param detailData The detail data to format
   * @param ipType The IP type
   * @param language The report language
   */
  public BaseHistoryItemDetail( TimePeriodCollection<T> detailData, IPType ipType, Locale language )
  {
    super( detailData );
    this.ipType = ipType;
    this.language = language;
  }

  // See interface
  @Override
  public final void attachDetailToItem( HistoryItemVO historyItem, Detail detail )
  {
    if ( detail.reportTextNumber != null )
    {
      historyItem.setReportTextNumber( detail.reportTextNumber );
    }
    if ( detail.endDate != null )
    {
      historyItem.setEndDate( detail.endDate );
    }

    historyItem.setTextLines( detail.textLines );
    historyItem.setCreator( detail.getCreator() );
    historyItem.setImage( detail.image );
  }

  /**
   * For certain details, the end dates should be grouped, which means that in a series of equal dates, only the last
   * must be shown.
   * @return <code>true</code> if the end dates should be grouped, otherwise <code>false</code>
   */
  public boolean shouldGroupEndDates()
  {
    return false;
  }
}
