package ch.ige.jipps.history.detail.design;

import java.util.ArrayList;
import java.util.List;

import ch.ige.ipobjects.design.model.LocarnoClassificationHistory.LocarnoClassificationItem;
import ch.ige.ipobjects.design.vo.ModKlasseVO;
import ch.ige.ipobjects.model.IPType;
import ch.ige.ipobjects.report.history.HistoryItemDetailBuilder;
import ch.ige.ipobjects.report.history.HistoryItemDetailList;
import ch.ige.ipobjects.report.history.HistoryItemDetailList.EmptyHistoryItemDetailList;
import ch.ige.ipobjects.report.history.impl.AbstractHistoryItemDetailBuilder;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.jipps.history.detail.BaseHistoryItemDetail;
import ch.ige.jipps.history.detail.Detail;
import ch.ige.util.constants.ReportTextNumberHandler;
import ch.ige.util.format.LocarnoClassFormatter;

/**
 * Creates the detail lines for a design's classification.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: DesignClassificationDetail.java,v 1.9 2007/12/19 08:57:08 lewis_lee Exp $
 */
public class DesignClassificationDetail extends BaseHistoryItemDetail<LocarnoClassificationItem>
{
  /**
   * Creates this with its data.
   * @param designClasses The design classes
   * @param ipType The IP type
   */
  public DesignClassificationDetail( TimePeriodCollection<LocarnoClassificationItem> designClasses, IPType ipType )
  {
    super( designClasses, ipType, null );
  }

  // See abstract method
  @Override
  protected HistoryItemDetailBuilder<LocarnoClassificationItem, Detail> createBuilder()
  {
    return new DesignClassificationDetailBuilder();
  }

  // See superclass
  @Override
  public boolean shouldGroupEndDates()
  {
    return true;
  }

  /**
   * The class that actually formats the design classes.
   */
  public class DesignClassificationDetailBuilder extends AbstractHistoryItemDetailBuilder<LocarnoClassificationItem, Detail>
  {
    /**
     * Creates the builder.
     */
    public DesignClassificationDetailBuilder()
    {
      super();
    }

    // @see
    // ch.ige.ipobjects.report.history.HistoryItemDetailBuilder#buildDetails(ch.ige.ipobjects.time.TimePeriodCollection)
    public HistoryItemDetailList<Detail> buildDetails( TimePeriodCollection<LocarnoClassificationItem> detailData )
    {
      final LocarnoClassificationItem classItem = detailData.get( 0 );
      if ( !classItem.isEmpty() )
      {
        Detail detail = new Detail( classItem, DesignClassificationDetail.this );
        detail.reportTextNumber = ReportTextNumberHandler.getReportTextNumberForClassification( DesignClassificationDetail.this.getIpType().getBAGISType() );
        detail.textLines = new ArrayList<String>();

        List<String> classNumbers = new ArrayList<String>();
        for ( ModKlasseVO classVO: classItem.getClassifications() )
        {
          classNumbers.add( classVO.getModkl_nr() );
        }
        detail.textLines.add( LocarnoClassFormatter.format( classNumbers, true ) );

        return createListManyDataItemsSingleRepresentation( detailData, detail );
      }
      else
      {
        return EmptyHistoryItemDetailList.getInstance();
      }
    }
  }
}
