package ch.ige.jipps.history.detail.design;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import ch.ige.ipobjects.design.model.DesignItemsHistory.DesignItemsItem;
import ch.ige.ipobjects.design.vo.ModOrdnrVO;
import ch.ige.ipobjects.report.history.HistoryItemDetailBuilder;
import ch.ige.ipobjects.report.history.HistoryItemDetailList;
import ch.ige.ipobjects.report.history.HistoryItemDetailList.EmptyHistoryItemDetailList;
import ch.ige.ipobjects.report.history.impl.AbstractHistoryItemDetailBuilder;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.jipps.history.detail.BaseHistoryItemDetail;
import ch.ige.jipps.history.detail.Detail;
import ch.ige.jipps.text.ConvertText;
import ch.ige.util.constants.ReportTextConstants;

/**
 * Creates the detail lines for a design's items.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: DesignItemDetail.java,v 1.8 2007/12/19 08:57:08 lewis_lee Exp $
 */
public class DesignItemDetail extends BaseHistoryItemDetail<DesignItemsItem>
{

  /**
   * Creates this with all required data.
   * @param designDescription The design descriptions
   */
  public DesignItemDetail( TimePeriodCollection<DesignItemsItem> designDescription )
  {
    super( designDescription, null, null );
  }

  // See superclass
  @Override
  public boolean shouldGroupEndDates()
  {
    return true;
  }

  // See abstract method
  @Override
  protected HistoryItemDetailBuilder<DesignItemsItem, Detail> createBuilder()
  {
    return new DesignItemDetailBuilder();
  }

  /**
   * The class that actually formats the items.
   */
  public class DesignItemDetailBuilder extends AbstractHistoryItemDetailBuilder<DesignItemsItem, Detail>
  {
    /**
     * Creates the builder.
     */
    public DesignItemDetailBuilder()
    {
      super();
    }

    // @see
    // ch.ige.ipobjects.report.history.HistoryItemDetailBuilder#buildDetails(ch.ige.ipobjects.time.TimePeriodCollection)
    public HistoryItemDetailList<Detail> buildDetails( TimePeriodCollection<DesignItemsItem> detailData )
    {
      final DesignItemsItem designItem = detailData.get( 0 );

      if ( !designItem.isEmpty() )
      {
        List<String> orderNumberLines = new ArrayList<String>();
        List<String> orderNumberCodes = new ArrayList<String>();
        boolean containsText = false;
        for ( ModOrdnrVO orderNumberVO: designItem.getItems() )
        {
          orderNumberLines.add( orderNumberVO.getOrdn_nr() );
          orderNumberCodes.add( orderNumberVO.getOrdn_nr() );
          if ( StringUtils.isNotEmpty( orderNumberVO.getBeschreib_100() ) )
          {
            containsText = true;
            orderNumberLines.addAll( ConvertText.splitLines( orderNumberVO.getBeschreib_100() ) );
          }
        }

        Detail detail = new Detail( designItem, DesignItemDetail.this );
        detail.reportTextNumber = ReportTextConstants.IMAGE_ORDER_NUMBER_INTEGER;
        detail.textLines = new ArrayList<String>();

        if ( containsText )
        {
          detail.textLines.addAll( orderNumberLines );
        }
        else
        {
          detail.textLines.add( StringUtils.join( orderNumberCodes.iterator(), "; " ) );
        }

        return createListManyDataItemsSingleRepresentation( detailData, detail );
      }
      else
      {
        return EmptyHistoryItemDetailList.getInstance();
      }
    }
  }
}
