package ch.ige.jipps.history.detail.design;

import java.util.ArrayList;

import ch.ige.ipobjects.common.history.HistoryItem;
import ch.ige.ipobjects.common.history.ValueHistory;
import ch.ige.ipobjects.report.history.HistoryItemDetailBuilder;
import ch.ige.ipobjects.report.history.HistoryItemDetailList;
import ch.ige.ipobjects.report.history.HistoryItemDetailList.EmptyHistoryItemDetailList;
import ch.ige.ipobjects.report.history.impl.AbstractHistoryItemDetailBuilder;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.jipps.history.detail.BaseHistoryItemDetail;
import ch.ige.jipps.history.detail.Detail;
import ch.ige.util.constants.ReportTextConstants;

/**
 * Creates the detail lines for a design's number of items.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: NumberOfItemsDetail.java,v 1.6 2007/12/19 08:57:08 lewis_lee Exp $
 */
public class NumberOfItemsDetail extends BaseHistoryItemDetail<HistoryItem<Integer>>
{
  /**
   * Creates this with all the data.
   * @param numberOfItems The number of items
   */
  public NumberOfItemsDetail( ValueHistory<HistoryItem<Integer>> numberOfItems )
  {
    super( numberOfItems, null, null );
  }

  // See abstract method
  @Override
  protected HistoryItemDetailBuilder<HistoryItem<Integer>, Detail> createBuilder()
  {
    return new NumberOfItemsDetailBuilder();
  }

  // See superclass
  @Override
  public boolean shouldGroupEndDates()
  {
    return true;
  }

  /**
   * The class that actually formats the number of items.
   */
  public class NumberOfItemsDetailBuilder extends AbstractHistoryItemDetailBuilder<HistoryItem<Integer>, Detail>
  {
    /**
     * Creates the builder.
     */
    public NumberOfItemsDetailBuilder()
    {
      super();
    }

    // @see
    // ch.ige.ipobjects.report.history.HistoryItemDetailBuilder#buildDetails(ch.ige.ipobjects.time.TimePeriodCollection)
    public HistoryItemDetailList<Detail> buildDetails( TimePeriodCollection<HistoryItem<Integer>> detailData )
    {
      HistoryItem<Integer> numberOfItems = detailData.get( 0 );
      if ( numberOfItems.getValue() != null )
      {
        Detail detail = new Detail( numberOfItems, NumberOfItemsDetail.this );
        detail.reportTextNumber = ReportTextConstants.DESIGN_NUMBER_OF_ITEMS_INTEGER;
        detail.textLines = new ArrayList<String>();
        Integer number = numberOfItems.getValue();
        detail.textLines.add( number.toString() );
        return createListSingleDataItemSingleRepresentation( numberOfItems, detail );
      }
      else
      {
        return EmptyHistoryItemDetailList.getInstance();
      }
    }
  }
}
