package ch.ige.jipps.history.detail.trademark;

import java.util.Collections;

import ch.ige.ipobjects.common.vo.ImageVO;
import ch.ige.ipobjects.exceptions.ImageDiscardedException;
import ch.ige.ipobjects.report.history.HistoryItemDetailBuilder;
import ch.ige.ipobjects.report.history.HistoryItemDetailList;
import ch.ige.ipobjects.report.history.HistoryItemDetailList.EmptyHistoryItemDetailList;
import ch.ige.ipobjects.report.history.impl.AbstractHistoryItemDetailBuilder;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.jipps.history.detail.BaseHistoryItemDetail;
import ch.ige.jipps.history.detail.Detail;
import ch.ige.util.constants.ReportTextConstants;

/**
 * Creates the detail lines for a trademark's image.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: ImageDetail.java,v 1.6 2009/03/23 09:51:14 locher Exp $
 */
public class ImageDetail extends BaseHistoryItemDetail<ImageVO>
{
  /**
   * Creates this with its detail data.
   * @param images The images to format
   */
  public ImageDetail( TimePeriodCollection<ImageVO> images )
  {
    super( images, null, null );
  }

  // See abstract method
  @Override
  protected HistoryItemDetailBuilder<ImageVO, Detail> createBuilder()
  {
    return new ImageDetailBuilder();
  }

  /**
   * The class that actually formats the image.
   */
  public class ImageDetailBuilder extends AbstractHistoryItemDetailBuilder<ImageVO, Detail>
  {
    /**
     * Creates the builder.
     */
    public ImageDetailBuilder()
    {
      super();
    }

    // @see
    // ch.ige.ipobjects.report.history.HistoryItemDetailBuilder#buildDetails(ch.ige.ipobjects.time.TimePeriodCollection)
    public HistoryItemDetailList<Detail> buildDetails( TimePeriodCollection<ImageVO> detailData )
    {
      if ( detailData.size() > 1 )
      {
        throw new IllegalArgumentException( "Got more than one image: " + detailData );
      }
      ImageVO image = detailData.get( 0 );
      try
      {
        if ( image.getImage() != null )
        {
          Detail detail = new Detail( image, ImageDetail.this );
          detail.reportTextNumber = ReportTextConstants.TRADEMARK_IMAGE_INTEGER;
          detail.textLines = Collections.emptyList();
          detail.image = image;
          return createListSingleDataItemSingleRepresentation( image, detail );
        }
        else
        {
          return EmptyHistoryItemDetailList.getInstance();
        }
      }
      catch ( ImageDiscardedException e )
      {
        throw e.throwAsSystemException();
      }
    }
  }
}
