package ch.ige.jipps.history.detail.trademark;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import ch.ige.ipobjects.common.history.HistoryItem;
import ch.ige.ipobjects.common.history.ValueHistory;
import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.report.history.HistoryDescription;
import ch.ige.ipobjects.report.history.HistoryItemDetailBuilder;
import ch.ige.ipobjects.report.history.HistoryItemDetailList;
import ch.ige.ipobjects.report.history.HistoryItemDetailList.EmptyHistoryItemDetailList;
import ch.ige.ipobjects.report.history.impl.AbstractHistoryItemDetailBuilder;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.jipps.history.detail.BaseHistoryItemDetail;
import ch.ige.jipps.history.detail.Detail;
import ch.ige.jipps.vo.HistoryItemVO;
import ch.ige.util.constants.ReportTextConstants;

/**
 * Creates the detail lines for a trademark's publication remark.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: PublicationRemarkDetail.java,v 1.5 2007/12/19 08:57:21 lewis_lee Exp $
 */
public class PublicationRemarkDetail extends BaseHistoryItemDetail<HistoryItem<String>>
{
  /**
   * Creates this with all the data.
   * @param publicationRemark The publication remarks
   */
  public PublicationRemarkDetail( ValueHistory<HistoryItem<String>> publicationRemark )
  {
    super( publicationRemark, null, null );
  }

  // See abstract method
  @Override
  protected HistoryItemDetailBuilder<HistoryItem<String>, Detail> createBuilder()
  {
    return new PublicationRemarkDetailBuilder();
  }

  @Override
  public HistoryItemDetailList<Detail> createDetails( HistoryDescription<List<HistoryItemVO>, HistoryItemVO> histDescription, Event event )
  {
    return super.createDetails( histDescription, event );
  }

  /**
   * The class that actually formats the publication remark.
   */
  public class PublicationRemarkDetailBuilder extends AbstractHistoryItemDetailBuilder<HistoryItem<String>, Detail>
  {
    /**
     * Creates the builder.
     */
    public PublicationRemarkDetailBuilder()
    {
      super();
    }

    // @see
    // ch.ige.ipobjects.report.history.HistoryItemDetailBuilder#buildDetails(ch.ige.ipobjects.time.TimePeriodCollection)
    public HistoryItemDetailList<Detail> buildDetails( TimePeriodCollection<HistoryItem<String>> detailData )
    {
      HistoryItem<String> publicationRemark = detailData.get( 0 );
      if ( publicationRemark.getValue() != null )
      {
        Detail detail = new Detail( publicationRemark, PublicationRemarkDetail.this );
        detail.reportTextNumber = ReportTextConstants.TRADEMARK_REMARKS_INTEGER;
        detail.textLines = new ArrayList<String>();
        String[] publicationRemarkLines = StringUtils.split( publicationRemark.getValue(), '\n' );
        detail.textLines.addAll( Arrays.asList( publicationRemarkLines ) );
        return createListSingleDataItemSingleRepresentation( publicationRemark, detail );
      }
      else
      {
        return EmptyHistoryItemDetailList.getInstance();
      }
    }
  }
}
