package ch.ige.jipps.history.test;

import java.util.List;

import org.w3c.dom.Node;

import ch.ige.jipps.imageconv.ImageProcessor;
import ch.ige.jipps.test.AbstractTestFacadeXml;
import ch.ige.jipps.vo.ReferenceXMLVO;
import ch.ige.util.text.Replacer;

/**
 * Gets the reference XML documents and compares the history to what is generated on the fly.
 * <p>
 * Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Eric Lewis
 * @version FacadeXmlHistoryTest.java,v 1.3 2006/09/06 13:14:45 lewis_lee Exp
 */
public class FacadeXmlHistoryTest extends AbstractTestFacadeXml
{
  /**
   * The default constructor
   * @param name The test's name
   */
  public FacadeXmlHistoryTest( String name )
  {
    super( name );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  public void setUp() throws Exception
  {
    super.setUp();
    this.checkAllIpTypes = true;

    // Force the images to be written, since there are images in the trademark history.
    System.setProperty( ImageProcessor.PROCESSING_MODE, ImageProcessor.PROCESS_IMAGES );
  }

  /**
   * Gets the XML and tests part of it in a subclass. This is required because otherwise JUnit doesn't find any test.
   * @throws Exception if the test fails
   */
  public void testGetXml() throws Exception
  {
    super.testGetXml( false );
  }

  /**
   * Compares the expected header with the actual header.
   * @param expectedXml The XML document as it should be
   * @param actualXml The XML document as it was returned
   * @param referenceXmlVo The VO containing the data for the test case
   * @throws Exception If anything goes wrong during the process
   */
  @Override
  public void compareXml( String expectedXml, String actualXml, ReferenceXMLVO referenceXmlVo ) throws Exception
  {
    String correctedXml = expectedXml;
    // Special characters
    if ( referenceXmlVo.getIpKey() == 1200594499 )
    {
      // TODO The JIPPS database has ISO 8859-1, which can't use the character 'œ'. So it should be changed to UTF-8 one
      // day.
      // Only first oe in hist is a problem, rest is not written as a special character
      correctedXml = Replacer.replaceFirst( correctedXml, "d'¿ufs", "d'œufs" );
    }

    //TODO Remove?
    // This 'valid to...' text used to have a space, but that's done in the XSL now.
    correctedXml = Replacer.replaceAll( correctedXml, "(<gueltigTxt>gültig bis:) (</gueltigTxt>)", "$1$2" );

    final String xPath = "/auszug/*[local-name()='histData']";
    List<Node> expectedNodes = getNodes( correctedXml, xPath );
    List<Node> actualNodes = getNodes( actualXml, xPath );
    for ( int i = 0; i < expectedNodes.size(); i++ )
    {
      Node expectedNode = expectedNodes.get( i );
      Node actualNode = null;
      try
      {
        actualNode = actualNodes.get( i );
        compareHistoryNodes( expectedNode, actualNode, correctedXml, actualXml, referenceXmlVo );

        if ( expectedNode.equals( actualNode ) )
        {
          fail( "Expected node <" + expectedNode + ">, but found <" + actualNode + ">", correctedXml, actualXml, getInformation( referenceXmlVo ) );
          break;
        }
      }
      catch ( IndexOutOfBoundsException ex )
      {
        fail( "Expected, but could not find history node " + expectedNode, correctedXml, actualXml, getInformation( referenceXmlVo ) );
        break;
      }
    }

    if ( actualNodes.size() > expectedNodes.size() )
    {
      fail( "Generated XML contains extra nodes: " + actualNodes.subList( expectedNodes.size(), actualNodes.size() ) );
    }

    // The XML code is formatted differently, so remove all carriage returns and stuff.
    // The two XML parts should now be on one line only and thus be compared more easily.
    flattenAndCheckNode( correctedXml, actualXml, "histData", referenceXmlVo );
  }

  /**
   * Fully compares two history nodes.
   * @param expectedNode The history node as it should be
   * @param actualNode The history node as it was returned
   * @param expectedXml The XML document as it should be
   * @param actualXml The XML document as it was returned
   * @param referenceXmlVo The VO containing the data for the test case
   * @throws Exception if comparing fails
   */
  private void compareHistoryNodes( Node expectedNode, Node actualNode, String expectedXml, String actualXml, ReferenceXMLVO referenceXmlVo ) throws Exception
  {
    compareChildNodeTexts( expectedNode, actualNode, "vonEreigDat", expectedXml, actualXml, referenceXmlVo );
    compareChildNodeTexts( expectedNode, actualNode, "bisEreigDat", expectedXml, actualXml, referenceXmlVo );
    compareChildNodeTexts( expectedNode, actualNode, "artLbl", expectedXml, actualXml, referenceXmlVo );
    compareChildNodeTexts( expectedNode, actualNode, "regTxtLbl", expectedXml, actualXml, referenceXmlVo );
    compareChildNodeTexts( expectedNode, actualNode, "gueltigTxt", expectedXml, actualXml, referenceXmlVo );
    compareChildNodeTexts( expectedNode, actualNode, "ereigTxt", expectedXml, actualXml, referenceXmlVo );
    compareChildNodeTexts( expectedNode, actualNode, "publJahr", expectedXml, actualXml, referenceXmlVo );
    compareChildNodeTexts( expectedNode, actualNode, "publDat", expectedXml, actualXml, referenceXmlVo );
    compareChildNodeTexts( expectedNode, actualNode, "publNr", expectedXml, actualXml, referenceXmlVo );
    compareChildNodeTexts( expectedNode, actualNode, "publPflicht", expectedXml, actualXml, referenceXmlVo );
    compareChildNodeTexts( expectedNode, actualNode, "publTxt", expectedXml, actualXml, referenceXmlVo );
    compareImages( getChildNode( expectedNode, "bild" ), getChildNode( actualNode, "bild" ), true );
    compareAllChildNodeTexts( expectedNode, actualNode, "histAuszTxt", "histTxt", expectedXml, actualXml, referenceXmlVo );
  }
}
