package ch.ige.jipps.imageconv;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.Map;

import org.apache.log4j.Logger;

import ch.ige.image.EncodedImage;
import ch.ige.image.ImageException;
import ch.ige.image.ImageFactory;
import ch.ige.image.ImageFactoryImpl;
import ch.ige.image.Metadata;
import ch.ige.jipps.constants.FileConstants;
import ch.ige.jipps.constants.ImageTypeConstants;

import com.sun.media.jai.codec.MemoryCacheSeekableStream;

/**
 * Writes PNG images.
 *
 * @author lewis_lee
 * @version $Id: PNGImageWriter.java,v 1.2 2009/05/06 10:54:19 lewis_lee Exp $
 */
public class PNGImageWriter extends ImageWriter
{
  /**
   * The log.
   */
  private static final Logger LOG = Logger.getLogger( PNGImageWriter.class );

  /**
   * Creates this.
   */
  public PNGImageWriter()
  {
    super();
  }

  // @see ch.ige.jipps.imageconv.ImageWriter#processImage(java.lang.String, byte[])
  @Override
  public Map<String, Object> processImage( String pathWithoutFileType, byte[] image ) throws IOException
  {
    try
    {
      ImageFactory imageFactory = new ImageFactoryImpl();
      EncodedImage renderedImage = imageFactory.loadStream( new MemoryCacheSeekableStream( new ByteArrayInputStream( image ) ) );
      Metadata metadata = renderedImage.getMetadata();

      this.imageHeigthInPixel = Double.valueOf( metadata.getSize().getHeight() ).intValue();
      this.imageWidthInPixel = Double.valueOf( metadata.getSize().getWidth() ).intValue();
      try
      {
        // This may or may not work.
        this.xDensity = metadata.getDPI();
        this.yDensity = metadata.getDPI();
      }
      catch ( ImageException e )
      {
        LOG.warn( "Couldn't get DPI from image " + pathWithoutFileType + FileConstants.EXTENSION_PNG );
      }
      Map<String, Object> map = getDimensions();

      String sourceFilePath = pathWithoutFileType + FileConstants.EXTENSION_PNG;
      writeToFile( image, sourceFilePath );
      String destinationImageType = callServlet( sourceFilePath, ImageTypeConstants.IMAGE_TYPE_PNG );
      map.put( ImageProcessor.IMAGE_TYPE, destinationImageType );
      LOG.debug( "Resulting image type: " + destinationImageType );
      return map;
    }
    catch ( ImageException e )
    {
      IOException ex = new IOException( "Couldn't read image " + pathWithoutFileType );
      ex.initCause( e );
      throw ex;
    }
  }
}
