package ch.ige.jipps.test;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

import org.apache.commons.io.FileUtils;

import ch.ige.ipobjects.ServicesFactory;
import ch.ige.jipps.JIPPSFacadeServices;
import ch.ige.jipps.JIPPSFacadeServicesImpl;
import ch.ige.jipps.configuration.PathConfiguration;
import ch.ige.jipps.constants.FileConstants;
import ch.ige.jipps.vo.PrintJobVO;

/**
 * Tests {@link ch.ige.jipps.PatentSearchReportControl}.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author Eric Lewis
 * @version $Id: PatentSearchReportControlTest.java,v 1.16 2009/01/12 10:24:02 lewis_lee Exp $
 */
public class PatentSearchReportControlTest extends TestCase
{
  static
  {
    ServicesFactory.initStandaloneClient();
  }

  /**
   * The unit under test
   */
  private JIPPSFacadeServices jippsFacadeServices;

  /**
   * The print jobs to execute
   */
  private List<PrintJobVO> printJobs;

  /**
   * The IDs of the reports to generate.
   */
  private static final int[] REPORT_IDS = new int[] { 201, 221, 261, 281 };

  /**
   * The only constructor.
   * 
   * @param name The test's name
   */
  public PatentSearchReportControlTest( String name )
  {
    super( name );
    PathConfiguration.setPathsForLocalTest();
    this.jippsFacadeServices = new JIPPSFacadeServicesImpl();
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  protected void setUp() throws Exception
  {
    super.setUp();

    this.printJobs = new ArrayList<PrintJobVO>();
    for ( int reportId: REPORT_IDS )
    {
      PrintJobVO printJob = new PrintJobVO();
      printJob.reportId = reportId;
      printJob.regauszTyp = PrintJobVO.PATENT_SEARCH_REPORT_NUMERIC;
      printJob.formTyp = PrintJobVO.PATENT_SEARCH_REPORT;
      this.printJobs.add( printJob );
    }
  }

  /**
   * Tests {@link ch.ige.jipps.LetterControl#makePdf(List)}.
   * 
   * @throws Exception if the test fails
   */
  public void testMakePdf() throws Exception
  {
    byte[] pdfContent = this.jippsFacadeServices.getMergePdf( this.printJobs );
    assertNotNull( "Checking that a PDF exists", pdfContent );

    File pdfFile = File.createTempFile( "patent_search_report_", FileConstants.EXTENSION_PDF );
    FileUtils.writeByteArrayToFile( pdfFile, pdfContent );
    System.out.println( "PDF written to " + pdfFile );
  }
}
