package ch.ige.jipps.test;

import java.sql.Timestamp;

import junit.framework.TestCase;
import ch.ige.dbutils.dao.AbstractDAO;
import ch.ige.dbutils.dao.DAO;
import ch.ige.jipps.PrintJobStoreFactory;
import ch.ige.jipps.PrintJobStoreServices;
import ch.ige.jipps.dao.PrintjobDAO;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.util.client.ClientLogInitalizer;
import ch.ige.util.constants.PrintStatusConstants;

/**
 * Tests {@link ch.ige.jipps.PrintJobStoreServices}
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Marc Bouquet
 * @author Priska Steiner
 * @author Esther Bandschapp
 * @author Eric Lewis
 * @version $Id: PrintJobStoreBeanTest.java,v 1.23 2009/03/04 09:22:47 lewis_lee Exp $
 */
public class PrintJobStoreBeanTest extends TestCase
{
  static
  {
    ClientLogInitalizer.initializeLoggingForTests( true, true );
    AbstractDAO.setRunningInJ2EEServer( false );
    System.setProperty( DAO.STANDALONE_CONFIG_PROPERTY_NAME, "true" );
  }

  /**
   * If this is <code>true</code>, the print job was inserted and must be deleted during {@link #tearDown()}.
   */
  private boolean insertSuccessful;

  /**
   * The print job to insert
   */
  private PrintJobVO printJobVO;

  /**
   * The unit under test
   */
  private PrintJobStoreServices home;

  /**
   * Used for selecting and deleting, not for inserting
   */
  private PrintjobDAO printJobDAO;

  /**
   * The only constructor
   * @param name The test's name
   */
  public PrintJobStoreBeanTest( String name )
  {
    super( name );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  protected void setUp() throws Exception
  {
    super.setUp();

    this.home = PrintJobStoreFactory.getInstance();
    this.printJobDAO = new PrintjobDAO();
    this.insertSuccessful = false;
  }

  /**
   * Tests {@link ch.ige.jipps.PrintJobStoreServices#insPrintJob(PrintJobVO)}. Note that a print job is inserted into
   * the DB and must be deleted during {@link #tearDown()}.
   * @throws Exception if the test fails
   */
  public void testInsertPrintJob() throws Exception
  {
    this.printJobVO = new PrintJobVO();
    this.printJobVO.anzKopien = PrintJobVO.NO_COPIES;
    this.printJobVO.reportId = 0;
    this.printJobVO.formTyp = PrintJobVO.AUSZUG;
    this.printJobVO.druckStatus = PrintStatusConstants.PREVIEW_PRINT;
    this.printJobVO.druckerId = 16;
    this.printJobVO.erstDatum = new Timestamp( System.currentTimeMillis() );
    this.printJobVO.printJobFile = new byte[] { '1', '2', '3' };
    this.printJobVO.printJobId = 5;

    // If the insert is successful, the print job will be deleted later
    this.home.insPrintJob( this.printJobVO );
    this.insertSuccessful = true;

    PrintJobVO loadedPrintJob = this.printJobDAO.readPrintjob( this.printJobVO.printJobId );
    assertEquals( "Checking number of copies", this.printJobVO.anzKopien, loadedPrintJob.anzKopien );
    assertEquals( "Checking report ID", this.printJobVO.reportId, loadedPrintJob.reportId );
    assertEquals( "Checking print status", this.printJobVO.druckStatus, loadedPrintJob.druckStatus );
    assertEquals( "Checking printer ID", this.printJobVO.druckerId, loadedPrintJob.druckerId );
    assertEquals( "Checking print job ID", this.printJobVO.printJobId, loadedPrintJob.printJobId );

    long millisecondDifference = loadedPrintJob.erstDatum.getTime() - this.printJobVO.erstDatum.getTime();
    assertTrue( "Checking creation date", millisecondDifference > 0 );
    assertTrue( "Checking creation date difference between inserted and loaded print job: Shouldn't be greater than one minute.",
        millisecondDifference <= 60000 );

    assertEquals( "Checking print job file length", this.printJobVO.printJobFile.length, loadedPrintJob.printJobFile.length );
    for ( int i = 0; i < this.printJobVO.printJobFile.length; i++ )
    {
      assertEquals( "Checking print job byte " + i, this.printJobVO.printJobFile[i], loadedPrintJob.printJobFile[i] );
    }
  }

  // @see junit.framework.TestCase#tearDown()
  @Override
  protected void tearDown() throws Exception
  {
    if ( this.insertSuccessful )
    {
      this.printJobDAO.deletePrintjob( this.printJobVO.printJobId );
    }

    super.tearDown();
  }
}
