package ch.ige.jipps.text;

import java.math.BigDecimal;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.log4j.Logger;

import ch.ige.util.date.DateFormatter;
import ch.ige.util.server.replace.P2d2;

/**
 * Utilities for replacement of {@link P2d2}.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   lewis_lee
 * @version  $Id: P2d2Utils.java,v 1.1 2008/07/23 14:15:30 lewis_lee Exp $
 */
public class P2d2Utils
{
  //TODO Integrate in P2d2?

  /**
   * The log
   */
  public static final Logger LOG = Logger.getLogger( P2d2Utils.class );

  /**
   * Sets the <code>value</code> into the <code>tags</code>.
   * @param tags The P2D2 tags
   * @param placeholder The tag name
   * @param value The value for <code>placeholder</code>
   */
  public static void setStringReplacement( Map<String, String> tags, String placeholder, String value )
  {
    if ( tags.containsKey( placeholder ) )
    {
      tags.put( placeholder, value );
    }
  }

  /**
   * Sets the HTML tag value into the <code>tags</code>: The tag name is converted to the bare tag
   * (that is, without {@link P2d2#TAG_PREFIX}) and then converted to a pseudo HTML tag.
   * For instance, the tag <code>%%$$bla</code> gets the value <code>&lt;bla&gt;</code>.
   * @param tags The P2D2 tags
   * @param placeholder The tag name
   */
  public static void setTagReplacement( Map<String, String> tags, String placeholder )
  {
    String value = placeholder.replace( P2d2.TAG_PREFIX, "<" );
    value += ">";
    setStringReplacement( tags, placeholder, value );
  }

  /**
   * Sets the formatted date into the <code>tags</code>, but only if <code>value</cide> is not <code>null</code>.
   * @param tags The P2D2 tags
   * @param placeholder The tag name
   * @param value The value for <code>placeholder</code>
   */
  public static void setDateReplacement( Map<String, String> tags, String placeholder, java.util.Date value )
  {
    if ( value != null )
    {
      setStringReplacement( tags, placeholder, DateFormatter.shortFormat( value ) );
    }
    else if ( tags.containsKey( placeholder ) )
    {
      LOG.warn( "Found no Date value for " + placeholder );
    }
  }

  /**
   * Sets the formatted date into the <code>tags</code>, but only if <code>value</cide> is not <code>null</code>.
   * @param tags The P2D2 tags
   * @param placeholder The tag name
   * @param value The value for <code>placeholder</code>
   */
  public static void setBigDecimalReplacement( Map<String, String> tags, String placeholder, BigDecimal value )
  {
    if ( value != null )
    {
      setStringReplacement( tags, placeholder, Integer.toString( value.intValue() ) );
    }
    else if ( tags.containsKey( placeholder ) )
    {
      LOG.warn( "Found no Date value for " + placeholder );
    }
  }

  /**
   * Sets the formatted Integer into the <code>tags</code>, but only if <code>value</cide> is not <code>null</code>.
   * @param tags The P2D2 tags
   * @param placeholder The tag name
   * @param value The value for <code>placeholder</code>
   */
  public static void setIntegerReplacement( Map<String, String> tags, String placeholder, Integer value )
  {
    if ( value != null )
    {
      setStringReplacement( tags, placeholder, value.toString() );
    }
    else if ( tags.containsKey( placeholder ) )
    {
      LOG.warn( "Found no Integer value for " + placeholder );
    }
  }

  /**
   * Checks all tags which were created from the letter and looks if there are associated values for them. If there's no
   * value, it defaults the missing value to an empty String.
   * @param tags The P2D2 tags, where key and value are Strings
   */
  public static void checkAndDefaultValuelessTags( Map<String, String> tags )
  {
    for ( Entry<String, String> tag: tags.entrySet() )
    {
      if ( tag.getValue() == null )
      {
        LOG.debug( "No value was set for " + tag.getKey() + ", defaulting to empty String" );
        tags.put( tag.getKey(), "" );
      }
    }
  }

}
