package ch.ige.jipps.vo;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import ch.ige.ipobjects.common.vo.ImageVO;
import ch.ige.jipps.currentstate.CurrentStateElement;
import ch.ige.jipps.header.IpHeader;
import ch.ige.jipps.header.PrintHeader;
import ch.ige.jipps.signature.stamp.Stamp;
import ch.ige.jipps.snapshot.SnapshotVO;

/**
 * Contains all the data for one report.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 *
 * @author Priska Steiner
 * @author Elmar Isler
 * @author Eric Lewis
 * @version   $Id: AuszugVO.java,v 1.42 2008/10/23 13:33:49 lewis_lee Exp $
 */

public class AuszugVO implements Serializable
{
  /**
   * If this is {@code true}, the IPI header is printed
   */
  private boolean printIpiHeader = false;

  /**
   * The print header for the report
   */
  private PrintHeader printHeader;

  /**
   * The IP header for the report
   */
  private IpHeader ipHeader;

  /**
   * The image
   */
  private ImageVO image = null;

  /**
   * Contains the elements of the current state
   */
  private List<CurrentStateElement> currentStateElements;

  /**
   * The registration snapshot
   */
  private SnapshotVO registrationSnapshot;

  /**
   * The history's title
   */
  private String historyTitle = "";

  /**
   * The history items
   */
  private List<HistoryItemVO> historyItems;

  /**
   * The individual signature lines
   */
  private List<String> signatureLines;

  /**
   * The path of the stamp image as part of the signature.
   */
  private Stamp signatureStamp;

  /**
   * The optional watermark.
   */
  private String watermark;

  /**
   * Creates this and initializes the <code>List</code>s
   */
  public AuszugVO()
  {
    this.currentStateElements = new ArrayList<CurrentStateElement>();
    this.historyItems = new ArrayList<HistoryItemVO>();
    this.signatureLines = new ArrayList<String>();
  }

  /**
   * Shows whether the IPI header must be printed.
   * @return {@code true} if the IPI header must be printed, otherwise {@code false}
   */
  public boolean printIpiHeader()
  {
    return this.printIpiHeader;
  }

  /**
   * Sets the flag showing whether the IPI header must be printed.
   * @param printIpiHeader {@code true} if the IPI header must be printed, otherwise {@code false}
   */
  public void setPrintIpiHeader( boolean printIpiHeader )
  {
    this.printIpiHeader = printIpiHeader;
  }

  /**
   * Gets the print header for the report.
   * @return The print header for the report
   */
  public PrintHeader getPrintHeader()
  {
    return this.printHeader;
  }

  /**
   * Sets the print header for the report.
   * @param printHeader The print header for the report
   */
  public void setPrintHeader( PrintHeader printHeader )
  {
    this.printHeader = printHeader;
  }

  /**
   * Gets the IP header for the report.
   * @return The IP header for the report
   */
  public IpHeader getIpHeader()
  {
    return this.ipHeader;
  }

  /**
   * Sets the IP header for the report.
   * @param ipHeader The IP header for the report
   */
  public void setIpHeader( IpHeader ipHeader )
  {
    this.ipHeader = ipHeader;
  }

  /**
   * Gets the image.
   * @return the image
   */
  public ImageVO getImage()
  {
    return this.image;
  }

  /**
   * Sets the image.
   * @param image The image to set
   */
  public void setImage( ImageVO image )
  {
    this.image = image;
  }

  /**
   * Gets the elements of the current state.
   * @return the elements of the current state
   */
  public List<CurrentStateElement> getCurrentStateElements()
  {
    return this.currentStateElements;
  }

  /**
   * Sets the elements of the current state.
   * @param currentStateElements The elements of the current state to set
   */
  public void setCurrentStateElements( List<CurrentStateElement> currentStateElements )
  {
    this.currentStateElements = currentStateElements;
  }

  /**
   * Gets the registration snapshot, which is the state of the IP at registration time.
   * @return the registration snapshot
   */
  public SnapshotVO getRegistrationSnapshot()
  {
    return this.registrationSnapshot;
  }

  /**
   * Sets the registration snapshot.
   * @param registrationSnapshot The registration snapshot
   */
  public void setRegistrationSnapshot( SnapshotVO registrationSnapshot )
  {
    this.registrationSnapshot = registrationSnapshot;
  }

  /**
   * Gets the history title.
   * @return the history title
   */
  public String getHistoryTitle()
  {
    return this.historyTitle;
  }

  /**
   * Sets the history title.
   * @param historyTitle The history title to set
   */
  public void setHistoryTitle( String historyTitle )
  {
    this.historyTitle = historyTitle;
  }

  /**
   * Gets the history items.
   * @return the history items
   */
  public List<HistoryItemVO> getHistoryItems()
  {
    return this.historyItems;
  }

  /**
   * Sets the history items.
   * @param historyItems The history items to set
   */
  public void setHistoryItems( List<HistoryItemVO> historyItems )
  {
    this.historyItems = historyItems;
  }

  /**
   * Gets the signature lines.
   * @return the signature lines
   */
  public List<String> getSignatureLines()
  {
    return this.signatureLines;
  }

  /**
   * Sets the signature lines.
   * @param signatureLines The signature lines to set
   */
  public void setSignatureLines( List<String> signatureLines )
  {
    this.signatureLines = signatureLines;
  }

  /**
   * Gets the stamp image.
   * @return the stamp image
   */
  public Stamp getSignatureStamp()
  {
    return this.signatureStamp;
  }

  /**
   * Sets the stamp image.
   * @param signatureStamp The stamp image to set
   */
  public void setSignatureStamp( Stamp signatureStamp )
  {
    this.signatureStamp = signatureStamp;
  }

  /**
   * Gets the watermark.
   * @return the watermark
   */
  public String getWatermark()
  {
    return this.watermark;
  }

  /**
   * Sets the watermark.
   * @param watermark The watermark to set
   */
  public void setWatermark( String watermark )
  {
    this.watermark = watermark;
  }
}
