package ch.ige.jipps.xml;

import java.io.CharArrayWriter;
import java.io.Writer;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.PropertyException;
import javax.xml.bind.Validator;

import ch.ige.jipps.constants.EncodingConstants;

/**
 * Contains utilities for marshalling XML documents.
 *
 * <p>Copyright (c) 2006, Swiss Federal Institute of Intellectual Property</p>
 * @author   lewis_lee
 * @version  $Id: XmlMarshaller.java,v 1.1 2006/12/12 08:33:33 lewis_lee Exp $
 */
//TODO This could be done with xmlutil
public class XmlMarshaller
{
  /**
   * Marshals an object using JAXB and returns the corresponding XML document as String.
   * Note that this sets the following in the marshaller
   * <ul>
   * <li>The encoding is UTF-8</li>
   * <li>The output is formatted</li>
   * </ul>
   * It also validates the result using <code>schemaLocation</code>.
   * 
   * @param context The JAXB context
   * @param marshaller The XML marshaller to use
   * @param schemaLocation The XML schema to use for validation
   * @param object The object to marshal
   * @return The String containing the full marshalled XML document
   * @throws PropertyException if the conversion fails
   * @throws JAXBException if the conversion fails
   */
  public static String toString(JAXBContext context, Marshaller marshaller, String schemaLocation, Object object) throws PropertyException, JAXBException
  {
    Writer result = new CharArrayWriter();
    marshaller.setProperty( Marshaller.JAXB_ENCODING, EncodingConstants.XML_ENCODING_UTF8 );
    marshaller.setProperty( Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE );
    marshaller.setProperty( Marshaller.JAXB_NO_NAMESPACE_SCHEMA_LOCATION, schemaLocation );
    marshaller.marshal( object, result );

    Validator valid = context.createValidator();
    valid.validate( object );

    return result.toString();
  }
}
