package ch.ige.pctedi.client.wpa;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;
import org.jdom.Element;

import ch.ige.pctedi.client.XMLDoc;
import ch.ige.pctedi.server.PctDelegate;
import ch.ige.pctedi.vo.IpcVO;
import ch.ige.pctedi.vo.PatIpcVO;

/**
 *
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: PatIpc.java,v 1.18 2008/03/11 14:55:12 ruegsegger Exp $
 */

public class PatIpc
{
  private static final Logger LOG = Logger.getLogger( PatIpc.class );

  private int countIpcCdH; // Zähler für Hauptklassen

  private int countIpcCdN; // Zähler für Nebenklassen

  private int countIpcCdF; // Zähler für Fakultative Klasse

  private int countIpcCdI; // Zähler für Indexierte Fakultative Klasse

  private XMLDoc xml;

  private WpaLogFile logFile = WpaLogFile.getInstance();

  private SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd" );

  public PatIpc()
  {
    sdf.setLenient( false );
  }

  public List<PatIpcVO> extract( XMLDoc xmlDoc ) throws Exception
  {
    this.xml = xmlDoc;

    // Zähler initialisieren
    countIpcCdH = 0;
    countIpcCdN = 0;
    countIpcCdF = 0;
    countIpcCdI = 0;

    List<PatIpcVO> listPatIpcVO = new ArrayList<PatIpcVO>();

    // IPC-Klassen sollten ab 01.01.2006 im Element classification-ipcr in der neuen Struktur geliefert werden
    if ( xml.exists( "classifications-ipcr" ) )
    {
      listPatIpcVO = extractIpcr();
    }
    // Bis 31.12.2006 werden die IPC-Klassen noch im Element classification-ipc geliefert (Testdaten)
    // Auch nach dem 010106 kann es möglich sein, dass ompi die IPC-Daten in der alten Struktur liefert
    else if ( xml.exists( "classification-ipc" ) )
    {
      listPatIpcVO = extractIpc();
    }

    // Überprüft die IPC-Symbole
    checkPatIpcVOs( listPatIpcVO );

    // Überprüft die Anzahl der Hauptklassen
    // Bei den PCT's sind nicht in allen Fällen IPC-Klassen erfasst
    if ( !listPatIpcVO.isEmpty() )
    {
      checkNumberOfFirstClass( countIpcCdH );
    }

    return listPatIpcVO;
  }

  private List<PatIpcVO> extractIpcr() throws Exception
  {
    List<PatIpcVO> listPatIpcVO = new ArrayList<PatIpcVO>();
    PatIpcVO patIpcVO = null;
    ClassificationIpcr ipcr = new ClassificationIpcr( this.xml );

    List<Element> elementList = xml.getElements( "classification-ipcr" );
    for ( int i = 0; i < elementList.size(); i++ )
    {
      String ipcText = xml.getValueOfElement( elementList.get( i ), "text" );

      if ( ipcText == null || ipcText.equals( "" ) )
      {
        try
        {
          // Extrahiert PatIpc-Werte aus dem XML
          patIpcVO = ipcr.extractPatIpcVO( elementList.get( i ) );
          listPatIpcVO.add( patIpcVO );
        }
        catch ( Exception ex )
        {
          logFile.appendMessage( WpaLogFile.ERROR, "PCT " + xml.getSchutitGesuchNr() + " Unerwarteter Fehler beim Extrahieren der IPC-Klasse(" + i + ")" );
          LOG.error( ex.getMessage() );
        }
      }
      // Enthält eine PCT-Anmeldung keine IPC-Klassen benutzt das OMPI den freien Text
      // um dies mitzuteilen. Ist in der Spezifikation vom OMPI jeoch nicht enthalten
      else if ( !ipcText.equalsIgnoreCase( "none" ) )
      {
        logFile.appendMessage( WpaLogFile.ERROR, "PCT " + xml.getSchutitGesuchNr() + " Ungueltiger Eintrag in IPC-Klassen-Text <" + ipcText + ">" );
      }
    }

    return listPatIpcVO;
  }

  /**
   * <!ELEMENT classification-ipc (edition , main-classification , further-classification* ,
   * (additional-info | linked-indexing-code-group | unlinked-indexing-code)* , text?)>
   * @return List
   */
  private List<PatIpcVO> extractIpc()
  {
    List<PatIpcVO> listPatIpcVO = new ArrayList<PatIpcVO>();
    PatIpcVO patIpcVO = null;
    String ipcSymb = null;

    try
    {
      String ipcClass = xml.getValueOf( "classification-ipc", "main-classification" );

      // OMPI liefert zum Teil keinen Inhalt oder den Text none im IPC-Element
      if ( ipcClass != null && !ipcClass.equals( "" ) && !ipcClass.equalsIgnoreCase( "none" ) )
      {
        ipcSymb = getIpcSymbol( ipcClass );
        if ( ipcSymb != null )
        {
          patIpcVO = new PatIpcVO();
          patIpcVO.ipcSymb = ipcSymb;
          patIpcVO.patIpcCd = PatIpcVO.IPC_CD_H;
          patIpcVO.classValue = PatIpcVO.CLASS_VALUE_INVENTIVE;
          patIpcVO.erfassDat = getErfassDat( ipcSymb );
          listPatIpcVO.add( patIpcVO );
        }
      }

      List<String> ipcClassList = xml.getValuesOf( "classification-ipc", "further-classification" );
      for ( int i = 0; i < ipcClassList.size(); i++ )
      {
        ipcClass = ipcClassList.get( i );

        // OMPI liefert zum Teil keinen Inhalt oder den Text none im IPC-Element
        if ( ipcClass != null && !ipcClass.equals( "" ) && !ipcClass.equalsIgnoreCase( "none" ) )
        {
          ipcSymb = getIpcSymbol( ipcClass );
          if ( ipcSymb != null )
          {
            patIpcVO = new PatIpcVO();
            patIpcVO.ipcSymb = ipcSymb;
            patIpcVO.patIpcCd = PatIpcVO.IPC_CD_N;
            patIpcVO.erfassDat = getErfassDat( ipcSymb );
            listPatIpcVO.add( patIpcVO );
          }
        }
      }
    }
    catch ( Exception ex )
    {
      logFile.appendMessage( WpaLogFile.ERROR, "PCT " + xml.getSchutitGesuchNr() + " Unerwarteter Fehler beim Extrahieren der IPC-Klassen " );
      LOG.error( ex.getMessage() );
    }
    return listPatIpcVO;
  }

  /**
   * Extrahiert Attribut erfassDat aus dem XML-Element action-date
   * Falls kein Action-Date vorhanden, wird das Erteilungsdatum/Publikationsdatum oder das Anmeldedatum
   * Ist weder Action-Date noch Anmeldedatum vorhanden wird das Minimal-Date abgefüllt
   * @param tag String XML-Element
   * @return String Attribut erfassDat
   */
  private Date getErfassDat( String ipcSymbol ) throws Exception
  {
    String actionDate = xml.getValueOf( "application-reference", "document-id/date" );
    Date erfassDat = this.parseDate( actionDate );

    if ( erfassDat == null )
    {
      // Ist weder Action-Date noch Anmeldedatum vorhanden wird Tagesdatum abgefüllt,
      // da laut Datenbankdefinition das Feld nicht leer sein darf
      erfassDat = new Date();

      String line = "PCT " + xml.getSchutitGesuchNr() + " Bei IPC Klasse <" + ipcSymbol + "> kein oder ungültiges Erfassdatum vorhanden. Tagesdatum eingefügt";

      logFile.appendMessage( WpaLogFile.DEBUG, line );
    }

    return erfassDat;
  }

  /**
   * Wandelt das action Date im Format yyyyMMdd in ein gültiges Datum um
   * @param actionDate String
   * @return Date oder null falls Datum nicht geparsed werden kann
   */
  private Date parseDate( String actionDate )
  {
    try
    {
      return sdf.parse( actionDate );
    }
    catch ( Exception ex )
    {
      return null;
    }
  }

  /**
   * Konvertierung Ipc-Symbol von PCT-File-Format nach Bagis-Format
   * @param ipcClass String Ipc-Symbol im Format "A 01D   1/00"
   * @return String Ipc-Symbol im Format "A01D000100"
   */
  private String getIpcSymbol( String ipcClass )
  {
    String ipcSymb = null;

    try
    {
      IpcSymbol ipcSymbol = new IpcSymbol( ipcClass );
      ipcSymb = ipcSymbol.getIpcSymbol();
    }
    catch ( Exception ex )
    {
      logFile.appendMessage( WpaLogFile.ERROR, "PCT " + xml.getSchutitGesuchNr() + " Ungueltiges Format fuer IPC-Klasse <" + ipcClass + ">" );
      LOG.error( ex );
    }
    return ipcSymb;
  }

  /**
   * Überprüft ob die Vorschriften der IPC-Klassen eingehalten werden
   * @param listPatIpcVO List
   */
  private void checkPatIpcVOs( List<PatIpcVO> listPatIpcVO )
  {
    IpcVO ipcVO = null;

    for ( int i = 0; i < listPatIpcVO.size(); i++ )
    {
      // Extrahiert PatIpc-Werte
      PatIpcVO patIpcVO = listPatIpcVO.get( i );

      // Überprüft ob Symbol vorhanden
      ipcVO = getIpcVO( patIpcVO );

      if ( ipcVO != null )
      {
        if ( patIpcVO.patIpcCd == PatIpcVO.IPC_CD_H )
        {
          patIpcVO.patIpcLnr = ++countIpcCdH;

          // Überprüft die Vorschriften für Hauptklassen
          checkFirstClass( patIpcVO, ipcVO.zusatz );
        }
        else if ( patIpcVO.patIpcCd == PatIpcVO.IPC_CD_N )
        {
          // ermittelt classvalue falls nicht vorhanden
          checkClassValue( patIpcVO, ipcVO.zusatz );

          // Überprüft die Vorschriften für Nebenklassen
          checkLaterClass( patIpcVO, ipcVO.zusatz );
        }
      }
      // Falld IPC-Symbol in DB nicht vorhanden, dieses aus der Liste entfernen
      // da es sich um eine foreign-key verbindung handelt.
      // Die Information ist im Logfile vermerkt worden
      else
      {
        listPatIpcVO.remove( i );
        i--;
      }
    }
  }

  /**
   * Überprüft die Vorschriften die bei einer Hauptklasse gelten
   *  - eine Hauptklasse darf nicht indexiert sein.
   *  - eine Hauptklasse muss inventive sein
   * Bei Verletzungen wird ein Eintrag ins Logfile vorgenommen, die Werte werden unverändert übernommen
   * @param patIpcVO PatIpcVO
   */
  private void checkFirstClass( PatIpcVO patIpcVO, String zusatz )
  {
    // Beim Auftreten einer indexierten Hauptklasse wird ein Fehler ins Logfile geschrieben.
    if ( zusatz.equals( IpcVO.ZUSATZ_INDEXING ) )
    {
      String message = "PCT " + xml.getSchutitGesuchNr() + " Eine Hauptklasse darf nicht indexiert sein. " + "IPC Klasse <" + patIpcVO.ipcSymb
          + "> ist indexiert";
      logFile.appendMessage( WpaLogFile.WARN, message );
    }
    // Beim Auftreten einer non-inventive Hauptklasse wird ein Fehler ins Logfile geschrieben
    if ( patIpcVO.classValue.equals( PatIpcVO.CLASS_VALUE_NON_INVENTIVE ) )
    {
      String message = "PCT " + xml.getSchutitGesuchNr() + " Eine Hauptklasse muss inventive sein. " + "IPC Klasse <" + patIpcVO.ipcSymb
          + "> ist non-inventive";
      logFile.appendMessage( WpaLogFile.WARN, message );
    }
  }

  /**
   * Überprüft die Vorschriften die bei einer Nebenklasse gelten
   *  - eine Nebenklasse darf nicht indexiert sein.
   *  - eine Nebenklasse muss inventive sein
   * Bei Verletzungen wird eine Konvertierung durchgeführt oder ein Eintrag ins Logfile vorgenommen
   *
   * N(Nebenklasse)    I(inventive)	 K,D(not indexed)  ->    N(Nebenklasse)	        I(inventive)	  okay
   * N(Nebenklasse)    I(inventive)	 I  (indexed)      ->    N(Nebenklasse)	        I(inventive)	  Logmeldung (debug)
   * N(Nebenklasse)    N(non-inventive)  K  (not indexed)  ->   F(fakultative Klasse)  N(non-inventive)
   * N(Nebenklasse)    N(non-inventive)  I,D(indexed)      ->   I(indexierte Klasse)   N(non-inventive)
   *
   * @param patIpcVO PatIpcVO
   */
  private void checkLaterClass( PatIpcVO patIpcVO, String zusatz )
  {
    if ( patIpcVO.classValue.equals( PatIpcVO.CLASS_VALUE_INVENTIVE ) )
    {
      patIpcVO.patIpcLnr = ++countIpcCdN;

      if ( zusatz.equals( IpcVO.ZUSATZ_INDEXING ) )
      {
        String line = "PCT " + xml.getSchutitGesuchNr() + " Eine Nebenklasse darf nicht indexiert sein. IPC Klasse <" + patIpcVO.ipcSymb + "> ist indexiert";
        logFile.appendMessage( WpaLogFile.DEBUG, line );
      }
    }
    else
    {
      // Eine Nebenklasse darf nicht non-inventive sein
      // Konvertierung durchführen nach Code=F (fakultativ) da Typ classified
      if ( zusatz.equals( IpcVO.ZUSATZ_CLASSIFICATION ) )
      {
        patIpcVO.patIpcCd = PatIpcVO.IPC_CD_F;
        patIpcVO.patIpcLnr = ++countIpcCdF;
      }
      // Eine Nebenklasse darf nicht non-inventive sein
      // Konvertierung durchführen nach Code=I (indexiert) da Typ indexed or double purpose
      else
      {
        patIpcVO.patIpcCd = PatIpcVO.IPC_CD_I;
        patIpcVO.patIpcLnr = ++countIpcCdI;
      }
    }
  }

  /**
   * Bestimmt den class-value anhand der anderen Werte falls keine Information geliefert wird
   * I  (indexed)     --> N(non-inventive) ->   I(indexierte Klasse)
   * K,D(not indexed) --> I(inventive)     ->   N(Nebenklasse)
   * @param patIpcVO PatIpcVO
   * @param zusatz String
   */
  private void checkClassValue( PatIpcVO patIpcVO, String zusatz )
  {
    if ( patIpcVO.classValue == null || patIpcVO.classValue.equals( "" ) )
    {
      if ( zusatz.equals( IpcVO.ZUSATZ_INDEXING ) )
      {
        patIpcVO.classValue = PatIpcVO.CLASS_VALUE_NON_INVENTIVE;
      }
      else
      {
        patIpcVO.classValue = PatIpcVO.CLASS_VALUE_INVENTIVE;
      }

    }
  }

  /**
   * Überprüft die Anzahl erfasster Hauptklassen
   * Ein EP muss genau eine Hauptklasse zugewiesen haben
   * @param countIpcCdH int Anzahl Hauptklassen
   */
  private void checkNumberOfFirstClass( int countCdH )
  {
    if ( countCdH < 1 )
    {
      String line = "PCT " + xml.getSchutitGesuchNr() + " Keine IPC-Hauptklasse erfasst";
      logFile.appendMessage( WpaLogFile.DEBUG, line );
    }
    else if ( countCdH > 1 )
    {
      String line = "PCT " + xml.getSchutitGesuchNr() + " Mehr als eine IPC-Hauptklasse erfasst";
      logFile.appendMessage( WpaLogFile.DEBUG, line );
    }
  }

  /**
   * Liest das IPC-Symbol und dessen Eingenschaften ab der Tabelle IPC
   * Falls das Symbol mit dem entsprechenden Gültig von nicht existiert
   * wird das jüngste Versionsdatum gesetzt und eine Meldung ins Logfile geschrieben
   * @param ipcSymb String zu lesendes Symbol
   * @return IpcVO
   */
  private IpcVO getIpcVO( PatIpcVO patIpcVO )
  {
    IpcVO ipcVO = null;

    try
    {
      if ( patIpcVO.ipcGueltigVon != null )
      {
        ipcVO = PctDelegate.getInstance().getIpc( patIpcVO.ipcSymb, patIpcVO.ipcGueltigVon );
      }

      if ( ipcVO == null )
      {
        ipcVO = PctDelegate.getInstance().getYoungestIpc( patIpcVO.ipcSymb );

        if ( ipcVO != null )
        {
          // Falls kein Gültig von vorhanden ist, wird keine Meldung geschrieben,
          // da zum jetzigen Zeitpunkt (04.2006) zuviele davon betroffen sind
          if ( patIpcVO.ipcGueltigVon != null )
          {
            String message = "PCT " + xml.getSchutitGesuchNr() + " IPC-Klasse <" + patIpcVO.ipcSymb + "> mit Gültig von <"
                + ( patIpcVO.ipcGueltigVon == null ? "null" : sdf.format( patIpcVO.ipcGueltigVon ) ) + "> auf Bagis nicht vorhanden. Jüngstes Datum <"
                + sdf.format( ipcVO.gueltigVon ) + "> wird als Gültig von eingesetzt";

            logFile.appendMessage( WpaLogFile.WARN, message );
          }

          patIpcVO.ipcGueltigVon = ipcVO.gueltigVon;
        }
        else
        {
          logFile.appendMessage( WpaLogFile.ERROR, "PCT " + xml.getSchutitGesuchNr() + " IPC-Klasse <" + patIpcVO.ipcSymb
              + "> auf Bagis nicht vorhanden. IPC-Klasse kann nicht gespeichert werden" );
        }
      }
    }
    catch ( Exception ex )
    {
      LOG.error( "getIpc <" + patIpcVO.ipcSymb + ">" + ex.getMessage() );
    }

    return ipcVO;
  }

}
