package ch.ige.pctedi.client.wpa;

import java.util.ArrayList;
import java.util.List;

import org.jdom.Element;

import ch.ige.pctedi.client.MappingUnicode;
import ch.ige.pctedi.client.XMLDoc;
import ch.ige.pctedi.server.PctDelegate;
import ch.ige.pctedi.vo.PctAdrVO;

/**
 *
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: PctAdr.java,v 1.21 2008/03/11 14:55:12 ruegsegger Exp $
 */

public class PctAdr
{
  private List<String> listBland;

  private XMLDoc xml;

  private PctAdrVO pctAdrVO;

  private WpaLogFile logFile = WpaLogFile.getInstance();

  public PctAdr() throws Exception
  {
    if ( listBland == null )
    {
      listBland = PctDelegate.getInstance().getAllBland();
    }
  }

  public PctAdr( List<String> listBland )
  {
    this.listBland = listBland;
  }

  /**
   * <!ELEMENT parties (applicants , inventors? , correspondence-address? , agents?)>
   *
   * <!ELEMENT applicants (applicant+)>
   * <!ELEMENT applicant  (addressbook+ , nationality , residence , us-rights* , designated-states? , designated-states-as-inventor?)>
   * <!ATTLIST applicant  sequence    CDATA  #REQUIRED
   *                      app-type           (applicant | applicant-inventor )  #REQUIRED
   *                      designation        (all | all-except-us | us-only | as-indicated )  #REQUIRED >
   *
   * <!ELEMENT agents     (customer-number | agent+)>
   * <!ELEMENT agent      (addressbook+)>
   * <!ATTLIST agent      sequence CDATA  #REQUIRED
   *                      rep-type        (agent | attorney | common-representative )  #REQUIRED >
   *
   * <!ELEMENT inventors  (inventor | deceased-inventor)+>
   * <!ELEMENT inventor   (addressbook+ , designated-states?)>
   * <!ATTLIST inventor   sequence    CDATA  #REQUIRED
   *                      designation        (all | all-except-us | us-only | as-indicated )  #IMPLIED >
   *
   * @param xml WPAXml
   * @return List
   */
  public List<PctAdrVO> extract( XMLDoc xmlDoc ) throws Exception
  {
    this.xml = xmlDoc;
    List<PctAdrVO> listPctAdrVO = new ArrayList<PctAdrVO>();

    // applicant = Hinterleger
    addApplicants( listPctAdrVO );

    // inventor = Erfinder
    addInventors( listPctAdrVO );

    return listPctAdrVO;
  }

  /**
   * Die Vertreter (agents) sind nicht mehr zu laden.
   * Inhaber und Erfinder mit designation=“us-only“ werden nicht eingelesen.
   * Der Erfinder muss in den USA auch Anmelder sein nicht nur Erfinder, deshalb wird diese Unterscheidung gemacht.
   * In USA sind zudem keine juristischen Personen als Anmelder erlaubt.
   *
   * Falls eine Adresse mit dem app-type=applicant-inventor daherkommt
   * wird diese Adresse zweimal in die Datenbank geschrieben.
   * Einmal als Inhaber und einmal als Erfinder.
   *
   * Adressen mit app-type=applicant-inventor designation=“us-only“
   * werden nur als Erfinder in die Datenbank geschrieben (designation bezieht sich auf applicant)
   * @param listPctAdrVO List
   */
  private void addApplicants( List<PctAdrVO> listPctAdrVO ) throws Exception
  {
    List<Element> applicants = xml.getElements( "applicant" );

    for ( int i = 0; i < applicants.size(); i++ )
    {
      String appType = xml.getAttributeOfElement( applicants.get( i ), "app-type" );
      String designation = xml.getAttributeOfElement( applicants.get( i ), "designation" );

      if ( !designation.equalsIgnoreCase( "us-only" ) )
      {
        pctAdrVO = getAddressBook( applicants.get( i ) );
        pctAdrVO.pctAdrLnr = listPctAdrVO.size() + 1;
        pctAdrVO.adrRolleCd = PctAdrVO.INHABER;
        listPctAdrVO.add( pctAdrVO );
      }

      if ( appType.equalsIgnoreCase( "applicant-inventor" ) )
      {
        pctAdrVO = getAddressBook( applicants.get( i ) );
        pctAdrVO.pctAdrLnr = listPctAdrVO.size() + 1;
        pctAdrVO.adrRolleCd = PctAdrVO.ERFINDER;
        listPctAdrVO.add( pctAdrVO );
      }
    }
  }

  private void addInventors( List<PctAdrVO> listPctAdrVO ) throws Exception
  {
    List<Element> inventors = xml.getElements( "inventor" );
    for ( int i = 0; i < inventors.size(); i++ )
    {
      String designation = xml.getAttributeOfElement( inventors.get( i ), "designation" );
      if ( !designation.equalsIgnoreCase( "us-only " ) )
      {
        pctAdrVO = getAddressBook( inventors.get( i ) );
        pctAdrVO.adrRolleCd = PctAdrVO.ERFINDER;
        pctAdrVO.pctAdrLnr = listPctAdrVO.size() + 1;
        listPctAdrVO.add( pctAdrVO );
      }
    }

    List<Element> deceasedInventors = xml.getElements( "deceased-inventor" );
    for ( int i = 0; i < deceasedInventors.size(); i++ )
    {
      PctAdrVO adrVO = new PctAdrVO();
      String name = xml.getValueOfElement( deceasedInventors.get( i ), "name" );
      adrVO.pctName = MappingUnicode.getInstance().replaceUnicodeValues( name );
      if ( adrVO.pctName.length() > 50 )
      {
        adrVO.pctName = adrVO.pctName.substring( 0, 50 );
      }
      adrVO.pctAdresse = "verstorben";
      adrVO.blandCd = "XX";
      adrVO.adrRolleCd = PctAdrVO.ERFINDER;
      adrVO.pctAdrLnr = listPctAdrVO.size() + 1;
      listPctAdrVO.add( adrVO );
    }
  }

  /** TODO evtl. adressbook auf sprache abfrage bis jetzt wird immer das erste Element genommen*/
  private PctAdrVO getAddressBook( Object element ) throws Exception
  {
    PctAdrVO adrVO = new PctAdrVO();
    String name = xml.getValueOfElement( element, new String[] { "addressbook", "name" } );
    adrVO.pctName = MappingUnicode.getInstance().replaceUnicodeValues( name );
    if ( adrVO.pctName.length() > 50 )
    {
//      Logmeldung nicht schreiben, da sehr viele davon betroffen sind und somit das Logfile gefüllt wird
      adrVO.pctName = adrVO.pctName.substring( 0, 50 );
    }

    adrVO.blandCd = getCountry( element );
    adrVO.pctAdresse = getAddress( element );

    return adrVO;
  }

  /**
   *
   * @param element Object
   * @return String
   */
  private String getAddress( Object element ) throws Exception
  {
    StringBuffer addressBuffer = new StringBuffer();
    String addressrow;
    String address;

    for ( int i = 0; i < 5; i++ )
    {
      addressrow = xml.getValueOfElement( element, new String[] { "addressbook", "address", "address-" + ( i + 1 ) } );
      if ( addressrow != null && !addressrow.equals( "" ) )
      {
        addressBuffer.append( addressrow );
      }
    }

    address = MappingUnicode.getInstance().replaceUnicodeValues( addressBuffer.toString() );
    if ( address.length() > 250 )
    {
//      Logmeldung nicht schreiben, da sehr viele davon betroffen sind und somit das Logfile gefüllt wird
      address = address.substring( 0, 250 );
    }
    return address;
  }

  private String getCountry( Object element ) throws Exception
  {
    String country = xml.getValueOfElement( element, new String[] { "addressbook", "address", "country" } );

    // Bei Erfinder (meist JP und US) fehlen oftmals Adresse und Ort
    // Kein Eintrag ins Logfile, da zuviele davon betroffen sind
    if ( country == null || country.equals( "" ) )
    {
      country = "XX";
    }
    else if ( !listBland.contains( country ) )
    {
      logFile.appendMessage( WpaLogFile.WARN, "PCT " + xml.getSchutitGesuchNr() + " Country : " + country + " nicht vorhanden in Bagis. XX eingesetzt" );
      country = "XX";
    }
    return country;
  }

}
