package ch.ige.pctedi.client.wpa;

import java.util.List;

import org.apache.log4j.Logger;

import ch.ige.pctedi.BuildInfo;
import ch.ige.pctedi.client.PctZipFile;
import ch.ige.pctedi.client.batch.Process;
import ch.ige.pctedi.client.wpa.file.WpaFile;

/**
 *
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version Id:
 */
public class Wpa extends Process
{
  private static final Logger LOG = Logger.getLogger( "ch.ige.pctedi.client.wpa" );

  private WpaLogFile logFile = null;

  private String titel = " Logfile WO-Published-Applications";

  @Override
  protected String getAttachement()
  {
    return logFile != null ? logFile.getLogFile().getPath() : null;
  }

  @Override
  protected String getSubject()
  {
    StringBuffer buf = new StringBuffer();

    buf.append( BuildInfo.BUILD_ENV_NAME );
    buf.append( " - " );
    buf.append( "PCT Verarbeitung " + zipFile.extractProcessWeek() );
    buf.append( titel );

    return buf.toString();
  }

  @Override
  protected void process( PctZipFile pctZipFile ) throws Exception
  {
    LOG.debug( "wpa process started..." );

    logFile = WpaLogFile.getInstance();
    logFile.setLogFileName( pctZipFile.extractProcessWeek() );
    logFile.writeHeader( pctZipFile );

    WpaProcess process = new WpaProcess();

    WpaFile startDir = new WpaFile( pctZipFile.getUnzipDirectory() );
    List<WpaFile> listWpaFile = startDir.listWpaFiles();

    if ( !listWpaFile.isEmpty() )
    {
      LOG.debug( "number of files found to process: " + listWpaFile.size() );

      for ( WpaFile wpaFile: listWpaFile )
      {
        LOG.debug( "process file: " + wpaFile.getPath() );
        process.process( wpaFile );
      }

      logFile.writeStatistik( process.getStatistik() );
      logFile.writeFooter();

      if ( process.isErrorOccured() )
      {
        error = true;
        logAndAppendMsg( process.getLogMessage() );
      }
    }
    else
    {
      error = true;
      logAndAppendMsg( "no wo-published-application files found to process in " + pctZipFile.getPath() );
    }

    LOG.debug( "wpa process finished..." );
  }
}
