package ch.ige.pctedi.client.wpa.file;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.apache.log4j.Logger;

import ch.ige.pctedi.client.PctProperties;

/**
 * ZipFile wird momentan nicht verwendet
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version Id:
 */
public class WpaZipFile extends File
{
  /**
   * Default id.
   */
  private static final long serialVersionUID = 1L;

  private static final Logger LOG = Logger.getLogger( WpaZipFile.class );

  private String contentType = "application/zip";

  private String url = null;

  private File unzipTarget = null;

  public WpaZipFile( String filename )
  {
    super( PctProperties.getInstance().getProperty( PctProperties.ZIPFILE_TARGET ), filename );
    this.getParentFile().mkdirs();

    url = PctProperties.getInstance().getProperty( PctProperties.DOWNLOAD_URL );

    unzipTarget = new File( PctProperties.getInstance().getProperty( PctProperties.UNZIP_DIRECTORY ) );
  }

  public String getUrl()
  {
    return url;
  }

  public void checkContentType( String type ) throws Exception
  {
    if ( !this.contentType.equals( type ) )
    {
      throw new Exception( "content type is: " + type + " expected type is: " + contentType );
    }
  }

  public String unzip() throws Exception
  {
    ZipFile zipFile = new ZipFile( this );
    ZipEntry entry;
    File outFile;
    InputStream is = null;

    try
    {
      unzipTarget.mkdirs();

      Enumeration<? extends ZipEntry> enumeration = zipFile.entries();

      while ( enumeration.hasMoreElements() )
      {
        entry = enumeration.nextElement();
        is = zipFile.getInputStream( entry );

        outFile = new File( unzipTarget, entry.getName() );
        copyToFile( is, outFile );

        LOG.debug( "Extracting file: " + entry.getName() );
      }
      zipFile.close();
    }
    catch ( Exception ex )
    {
      LOG.error( "Exception in unzip: " + ex );
      throw ex;
    }
    return unzipTarget.getPath();
  }

  public void copyToFile( InputStream in, File file ) throws Exception
  {
    BufferedOutputStream out = new BufferedOutputStream( new FileOutputStream( file ) );

    int size = 0;
    byte[] buf = new byte[1024];

    while ( ( size = in.read( buf, 0, buf.length ) ) != -1 )
    {
      out.write( buf, 0, size );
    }

    in.close();
    out.close();
  }

  /**
   * Verschiebt Zipfile in Unterverzeichnis done
   */
  public void moveToDoneDirectory()
  {
    File doneDirectory = new File( this.getParent(), "done" );
    doneDirectory.mkdir();

    File destination = new File( doneDirectory, this.getName() );

    this.renameTo( destination );
  }

  /**
   * Löscht das unzip-Verzeichnis und die darin enthaltenen Files
   */
  public void deleteUnzippedFiles()
  {
    File[] files = unzipTarget.listFiles();
    for ( int i = 0; i < files.length; i++ )
    {
      files[i].delete();
    }
    unzipTarget.delete();
  }
}
