package ch.ige.pctedi.server;

import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import ch.ige.pctedi.vo.IpcVO;
import ch.ige.pctedi.vo.PctVO;

/**
 * * <!-- begin-xdoclet-definition -->
 * 
 * @ejb.bean name="PctFacade" description="An EJB named PctFacade" 
 * display-name="PctFacade" 
 * view-type="remote"
 * jndi-name="PctFacade" 
 * type="Stateless" 
 * transaction-type="Container"
 * 
 * @ejb.transaction type="Required"
 * @ejb.ejb-ref ejb-name="PctXxIdentNumber" 
 * view-type="local"
 *  
 * <!-- end-xdoclet-definition -->
 */
public class PctFacadeBean implements SessionBean
{
  /**
   * Default id.
   */
  private static final long serialVersionUID = 1L;
  SessionContext sessionContext;

  public void setSessionContext( SessionContext sessionContext )
  {
    this.sessionContext = sessionContext;
  }

  /**
   * Kreiert ein PCT.
   * @param pctVO Value Object der Spidi
   * 
   * @ejb.interface-method
   */
  public void createPct( PctVO pctVO )
  {
    new PctBO().createPct( pctVO );
  }

  /**
   * Testet, ob das PCT bereits vorhanden ist.
   * @param schutitNr des gesuchten PCT
   * @return int schutitKey falls notfound 0
   * 
   * @ejb.interface-method
   */
  public int selectSchutitKey( String schutitNr )
  {
    return new PctBO().selectSchutitKey( schutitNr );
  }

  /**
   * Testet, ob das PCT bereits vorhanden ist.
   * @param pctVO gesuchNr und schutitNr reichen als Inhalt
   * @return  int 0: kein Schutztitel vorhanden
   *              1: Schutztitel vorhanden; Suche über schutitNr und gesuchNr ergaben das selbe Resultat.
   *              2: Schutztitel vorhanden; nur Suche über schutitNr ergab ein Resultat.
   *              3: Schutztitel vorhanden; nur Suche über GesuchNr ergab ein Resultat.
   *              4: Schutztitel vorhanden; Suche über schutitNr und gesuchNr ergaben unterschiedliche Resultate.
   * 
   * @ejb.interface-method
   */
  public int findPct( PctVO pctVO )
  {
    return new PctBO().findPct( pctVO );
  }

  /**
   * Holt alle BLänder.
   *
   * @return List<String> BlandWork mit allen "Bländer"
   * 
   * @ejb.interface-method
   */
  public List<String> getAllBland()
  {
    return new PctBO().getAllBland();
  }

  /**
   * Holt alle unicode iso mappings
   * @return Map<String, String>
   * 
   * @ejb.interface-method
   */
  public Map<String, String> getAllMappingUnicode()
  {
    return new PctBO().getAllMappingUnicode();
  }

  /**
   * Holt einen IPC-Eintrag aus der Tabelle IPC
   * @return IpcVO IPC-Eintrag
   * 
   * @ejb.interface-method
   */
  public IpcVO getIpc( String ipcSymb, Date gueltigVon )
  {
    return new PctBO().getIpc( ipcSymb, gueltigVon );
  }

  /**
   * Holt einen IPC-Eintrag aus der Tabelle IPC
   * @return IpcVO IPC-Eintrag
   * 
   * @ejb.interface-method
   */
  public IpcVO getYoungestIpc( String ipcSymb )
  {
    return new PctBO().getYoungestIpc( ipcSymb );
  }

  /**
   * 
   * @param schutitKey
   * @param publDat
   * @return Meldung, was gemacht wurde.
   * 
   * @ejb.interface-method
   */
  public String createExtension( int schutitKey, java.util.Date publDat )
  {
    return new PctBO().createExtension( schutitKey, publDat );
  }

  public void ejbCreate() throws EJBException
  {
  }

  // @see javax.ejb.SessionBean#ejbActivate()
  public void ejbActivate() throws EJBException
  {
    // TODO Auto-generated method stub

  }

  // @see javax.ejb.SessionBean#ejbPassivate()
  public void ejbPassivate() throws EJBException
  {
    // TODO Auto-generated method stub

  }

  // @see javax.ejb.SessionBean#ejbRemove()
  public void ejbRemove() throws EJBException
  {
    // TODO Auto-generated method stub

  }
}
