package ch.ige.rbbagisexport.server.ejb;

import java.io.IOException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.xml.bind.JAXBException;

import org.apache.log4j.Logger;

import ch.ige.rbbagisexport.common.RBBagisExportConfiguration;
import ch.ige.rbbagisexport.common.RBBagisExportSummary;
import ch.ige.rbbagisexport.server.config.RBBagisExportConfigReader;
import ch.ige.rbbagisexport.server.dao.RBExportDAO;
import ch.ige.rbbagisexport.server.dao.RBMultipleExportDAO;
import ch.ige.rbbagisexport.server.datacontainer.ExportContainerFactory;
import ch.ige.rbbagisexport.server.datacontainer.ExportDataContainer;
import ch.ige.rbbagisexport.server.filetransfer.ExportFileHelper;
import ch.ige.rbbagisexport.server.vo.RBExportVO;

/**
 * <p></p>
 * Copyright © 2005, Swiss Federal Institute of Intellectual Property
 * @author Michael Künzli
 * @version  $Id: RBBagisExportBean.java,v 1.22 2008/10/03 11:32:16 kuenzli Exp $
 * <!-- begin-user-doc --> A generated session bean <!-- end-user-doc --> * 
 * <!-- begin-xdoclet-definition --> 
 * @ejb.bean 
 *  name="RBBagisExport"  
 *  description="An EJB named FirstTest"
 *  display-name="RBBagisExport"
 *  view-type="remote"
 *  jndi-name="RBBagisExport"
 *  type="Stateless"
 *  transaction-type="Container"             
 * @ejb.interface
 *  local-class="ch.ige.rbbagisexport.server.ejb.test.RBBagisExport"
 * @ejb.home
 *  local-class="ch.ige.rbbagisexport.server.ejb.test.RBBagisExportHome"
 * @ejb.transaction
 *   type="RequiresNew"
 * <!-- end-xdoclet-definition --> 
 */
public class RBBagisExportBean implements SessionBean
{
  //----------------------------------------------------------------------------
  // constants
  //----------------------------------------------------------------------------
  private static final Logger LOG = Logger.getLogger( RBBagisExportBean.class );
  private static final long serialVersionUID = 1L;
  //----------------------------------------------------------------------------
  // private members
  //----------------------------------------------------------------------------
  private RBExportDAO rbExpDAO = null;

  private ExportContainerFactory containerFactory = null;

  private ExportFileHelper fileHelper = null;

  //----------------------------------------------------------------------------
  // local home interfaces
  //----------------------------------------------------------------------------

  public void ejbCreate() throws CreateException
  {
  }

  public void ejbActivate() throws EJBException, RemoteException
  {
  }

  public void ejbPassivate() throws EJBException, RemoteException
  {
  }

  public void ejbRemove() throws EJBException, RemoteException
  {
  }

  public void setSessionContext( SessionContext arg0 ) throws EJBException, RemoteException
  {
    try
    {
      this.rbExpDAO = new RBExportDAO();
      this.containerFactory = new ExportContainerFactory();
      this.fileHelper = new ExportFileHelper();
    }
    catch ( Exception e )
    {
      LOG.error( "setSessionContext(): Error creating DAO; " + e.getMessage() );
    }

  }

  /**
   * 
   * 
   * @throws java.lang.Exception
   * <!-- begin-xdoclet-definition -->
   * @ejb.interface-method
   * <!-- end-xdoclet-definition -->
   */
  public RBBagisExportConfiguration getConfiguration() throws Exception
  {
    RBBagisExportConfiguration config = RBBagisExportConfigReader.instance().getConfiguration();
    return config;
  }

  /**
   * 
   * @param schutitKey Integer Schutit_key
   * @throws java.lang.Exception
   * <!-- begin-xdoclet-definition -->
   * @ejb.interface-method
   * <!-- end-xdoclet-definition -->
   */
  public String exportPatentApplication( Integer schutitKey ) throws Exception
  {
    RBExportVO rbExportData;
    String xml = null;
    ExportDataContainer expDataContainer = null;
    try
    {
      rbExportData = this.rbExpDAO.selectRBExportData( schutitKey );
      expDataContainer = this.containerFactory.createContainer( rbExportData );
      this.fileHelper.writeFile( expDataContainer );
    }
    catch ( Exception e )
    {
      LOG.error( "exportPatentApplication(): " + e.getMessage(), e );
      throw e;
    }
    if ( LOG.isDebugEnabled() )
    {
      LOG.debug( xml );
    }
    return expDataContainer.getFileName();
  }

  /**
   * Exports data of all Patent Applications that have a order for a Search 
   * Report.
   * @throws java.lang.Exception, java.rmi.RemoteException
   * <!-- begin-xdoclet-definition -->
   * @ejb.interface-method
   * <!-- end-xdoclet-definition -->
   */
  public RBBagisExportSummary exportAllPatentApplications() throws Exception
  {
  	RBBagisExportSummary expSummary = new RBBagisExportSummary();
    // get DAO
    RBMultipleExportDAO rbExpDAO;
    try
    {
      rbExpDAO = new RBMultipleExportDAO();
    }
    catch ( Exception e )
    {
      LOG.error( "exportAllPatentApplications(): Could not create DAO.", e );
      throw e;
    }
    // get VO List
    List<RBExportVO> exportVOList;
    try
    {
      exportVOList = rbExpDAO.selectExportData();
    }
    catch ( Exception e )
    {
      LOG.error( "exportAllPatentApplications(): Error selecting data.", e );
      throw e;
    }
    // fill container list
    ExportContainerFactory factory = new ExportContainerFactory();
    List<ExportDataContainer> dataContainerList = new ArrayList<ExportDataContainer>();
    for ( RBExportVO exportVO: exportVOList )
    {
      LOG.debug( exportVO.toString() );

      ExportDataContainer container = null;
      try
      {
        container = factory.createContainer( exportVO );
        expSummary.addToExportedFileNames(getConfiguration().getExportFilePath() + container.getFileName());
      }
      catch ( JAXBException e )
      {
        LOG.error( "exportAllPatentApplications(): Could not create Container:" + exportVO.toString(), e );
        throw new RemoteException( e.getMessage(), e );
      }
      dataContainerList.add( container );
      
    }
    // export files
    ExportFileHelper fileHelper = new ExportFileHelper();
    try
    {
      fileHelper.writeFiles( dataContainerList );
    }
    catch ( IOException e )
    {
      LOG.error( "exportAllPatentApplications(): Error transfering files.", e );
      throw e;
    }
    return expSummary;
  }
}
