package ch.ige.rbbagisimport.server;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;

import ch.ige.rbbagisimport.common.vo.SearchReportFileVO;
import ch.ige.rbbagisimport.common.vo.XxSysparamsVO;

/**
 * EJB that provides methods to store searchreports in bagis
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   bolzli
 * @version  $Id: RBWorkBean.java,v 1.15 2008/05/06 13:54:36 wild Exp $
 * 
 *  @ejb.bean 
 *  name="RBWork" 
 *  display-name="RBWork"
 *  view-type="remote"
 *  jndi-name="RBWork"
 *  remote-jndi-name="RBWorkRemote"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="Required"
 * 
 */

@SuppressWarnings( "serial" )
public class RBWorkBean implements SessionBean
{

  //
  // constants
  //
  private static final Logger LOG = Logger.getLogger( RBWorkBean.class );

  //
  // fields
  //
  private SessionContext sessionContext;

  // @see javax.ejb.SessionBean#ejbCreate()
  public void ejbCreate()
  {
    // lazy bone
  }

  // @see javax.ejb.SessionBean#ejbRemove()
  public void ejbRemove() throws EJBException
  {
    // lazy bone
  }

  // @see javax.ejb.SessionBean#ejbActivate()
  public void ejbActivate() throws EJBException
  {
    // lazy bone
  }

  // @see javax.ejb.SessionBean#ejbPassivate()
  public void ejbPassivate() throws EJBException
  {
    // lazy bone
  }

  // @see javax.ejb.SessionBean#setSessionContext(javax.ejb.SessionContext)
  public void setSessionContext( SessionContext sessionContext ) throws EJBException
  {
    this.sessionContext = sessionContext;
  }

  /**
   * Inserts the Searchreport in BAGIS
   * @param srFile
   * @return errormessage
   * 
   * @ejb.interface-method
   */
  public SearchReportFileVO insertIntoBAGIS( SearchReportFileVO srFile )
  {
    RBWorkBO bo = new RBWorkBO();
    try
    {
      bo.insertIntoBAGIS( srFile );
      srFile.setProcessState( SearchReportFileVO.PROCESS_STATE_SUCCEEDED );
    }
    catch ( RBServerException e )
    {
      sessionContext.setRollbackOnly();
      srFile.setErrorMsg( e.getErrorMsg() );
      LOG.error( "Error: " + srFile.getErrorMsg() + "Datei: " + srFile.getFilePath() );
    }
    return srFile;
  }

  /**
   * Laden der Konfiguration und Rückgabe in einem einzigen VO
   * @return XxSysparamsVO gefüllt mit den nötigen Werten.
   * 
   * @ejb.interface-method
   */
  public XxSysparamsVO getConfiguration()
  {
    RBWorkBO bo = new RBWorkBO();
    XxSysparamsVO sysParams = new XxSysparamsVO();
    try
    {
      sysParams = bo.getConfiguration();
    }
    catch ( RBServerException e )
    {
      String message = "Error on loading Configuration: " + e.getErrorMsg();
      LOG.error( message );
    }
    return sysParams;
  }
} // end class
