package ch.ige.rbbagisimport.server;

import java.io.CharArrayWriter;
import java.sql.Date;
import java.util.ArrayList;

import javax.xml.bind.JAXBException;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.stream.StreamResult;

import org.apache.log4j.Logger;
import org.xml.sax.SAXException;

import ch.ige.chsearchreport.ApplicationReferenceType;
import ch.ige.chsearchreport.ClassificationIpcr;
import ch.ige.chsearchreport.ClassificationsIpcrType;
import ch.ige.chsearchreport.IpcVersionIndicatorType;
import ch.ige.chsearchreport.ObjectFactory;
import ch.ige.chsearchreport.SearchReportData;
import ch.ige.chsearchreport.impl.IpcVersionIndicatorTypeImpl;
import ch.ige.rbbagisimport.common.vo.SearchReportFileVO;
import ch.ige.rbbagisimport.server.vo.IPCClassVO;
import ch.ige.xmlutil.bind.Bindings;

/**
 * Helper Methods for XML Data
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   bolzli
 * @version  $Id: XMLHelper.java,v 1.21 2008/05/06 11:37:47 wild Exp $
 */
public class XMLHelper
{
  private static final Logger LOG = Logger.getLogger( XMLHelper.class );

  private SearchReportData searchReportData = null;

//  private static final String schema = "http://schema.ige.ch/ch-search-report-v1-2-ige1-0.xsd";

  private ObjectFactory of = new ObjectFactory();

  /**
   * Constructor. Creates xml-structure
   * @param xmlString XML as String
   * @throws RBServerException
   */
  public XMLHelper( String xmlString ) throws RBServerException
  {
    try
    {
      this.searchReportData = (SearchReportData) Bindings.unmarshalString( Bindings.CHSEARCHREPORT, xmlString, false );
    }
    catch ( JAXBException e )
    {
      LOG.error( "Error in unmarshal of XML-String" + "\n" + e.getMessage(), e );
      throw new RBServerException( SearchReportFileVO.MSG_NOT_VALID );
    }
    catch ( SAXException e )
    {
      LOG.error( "Error in JAXB unmarshal of XML-String" + "\n" + e.getMessage(), e );
      throw new RBServerException( SearchReportFileVO.MSG_NOT_VALID );
    }
    catch ( ParserConfigurationException e )
    {
      LOG.error( "Error ParserConfiguration of XML-String" + "\n" + e.getMessage(), e );
      throw new RBServerException( SearchReportFileVO.MSG_NOT_VALID );
    }
  }

  /**
   * Returns the xml structure
   * @return SearchReportDataImpl the xml structure
   */
  public SearchReportData getXMLData()
  {
    return searchReportData;
  }

  /**
   * Inserts IPC-Classes
   * @param ipcClasses ArrayList containing the classes to insert
   * @return String the XML as String
   * @throws JAXBException 
   */
  @SuppressWarnings( "unchecked" )
  public String addIPCClasses( ArrayList<IPCClassVO> ipcClasses ) throws JAXBException
  {
    int sequence = 1;
    for ( IPCClassVO ipc: ipcClasses )
    {
      ClassificationIpcr ipcClass = of.createClassificationIpcr();
      ipcClass.setSequence( sequence );
      ipcClass.setClassificationLevel( ipc.getClassificationLevel() );
      ipcClass.setClassificationValue( ipc.getClassificationValue() );
      ipcClass.setClazz( ipc.getClazz() );
      ipcClass.setIpcVersionIndicator( this.getIndicatorType( ipc.getVersionDate() ) );
      ipcClass.setMainGroup( Integer.valueOf( ipc.getMainGroup() ) );
      ipcClass.setSection( ipc.getSection() );
      ipcClass.setSubclass( ipc.getSubClass() );
      ipcClass.setSubgroup( String.valueOf( ipc.getSubGroup() ) );
      ipcClass.setSymbolPosition( ipc.getSymbolPosition() );

      if ( searchReportData.getSrepForPub().getClassificationsIpcr() == null )
      {
        ClassificationsIpcrType classifications = of.createClassificationsIpcrType();
        classifications.getClassificationIpcr().add( ipcClass );
        searchReportData.getSrepForPub().setClassificationsIpcr( classifications );
      }
      else
      {
        searchReportData.getSrepForPub().getClassificationsIpcr().getClassificationIpcr().add( ipcClass );
      }
      sequence++;
    }
    return getXml();
  }

  /**
   * Extracts the application number from the xml
   * @return String application number
   */
  public String extractApplNr()
  {
    ApplicationReferenceType applType = (ApplicationReferenceType) searchReportData.getSrepInfo().getApplicationReference().get( 0 );
    return applType.getDocumentId().getDocNumber();
  }

  /**
   * Writes the xml to the console or/and to the logfile
   * @param console writes the xml to the console if true
   * @param log writes the xml to the logfile if true
   */
  public void printXML( boolean console, boolean log )
  {
    String xml = "";
    try
    {
      xml = getXml();
    }
    catch ( JAXBException e )
    {
      LOG.error( "Error: " + e.getMessage(), e );
    }

    if ( console )
    {
      System.out.println( "XML:\n\n" + xml );
    }
    if ( log )
    {
      LOG.info( "XML:\n\n" + xml );
    }
  }

  /**
   * Returns the XML-Data as String
   * @return String XML as String
   * @throws JAXBException
   */
  public String getXml() throws JAXBException
  {
    CharArrayWriter cw = new CharArrayWriter();
    Bindings.validate( Bindings.CHSEARCHREPORT, searchReportData );
    Bindings.marshal( Bindings.CHSEARCHREPORT, searchReportData, new StreamResult( cw ) );
    return cw.toString();
  }

  private IpcVersionIndicatorType getIndicatorType( Date date )
  {
    IpcVersionIndicatorType ivit = new IpcVersionIndicatorTypeImpl();
    ivit.setDate( date );
    return ivit;
  }

} // end class
