package ch.ige.rbbagisimport.server.dao;

import java.sql.Connection;
import java.sql.SQLException;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.log4j.Logger;

import ch.ige.dbutils.runner.IGEQueryRunner;
import ch.ige.util.server.DBHelper;

/**
 *  Superclass for BAGIS-Accesses 
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   bolzli
 * @version  $Id: BagisDAO.java,v 1.3 2008/04/23 13:28:08 wild Exp $
 */
public class BagisDAO
{
  //
  // const
  //
  /** Bagis Datasource Name (für die neue Oracle DB) */
  public static final String DSN = DBHelper.BAGIS;

  // query runner to execute sql statements
  protected IGEQueryRunner run;

  private static final Logger LOG = Logger.getLogger( BagisDAO.class );

  //
  // fields
  //
  public StringBuffer query = new StringBuffer();

  //
  // constructors
  //
  public BagisDAO() throws SQLException
  {
    this( DBHelper.BAGIS );
  }

  public BagisDAO( String dsn ) throws SQLException
  {
    InitialContext ctx;
    try
    {
      ctx = new InitialContext();
      DataSource ds = (DataSource) ctx.lookup( dsn );
      this.run = new IGEQueryRunner( ds );
    }
    catch ( NamingException ex )
    {
      String message = "Constructor of BagisDAO " + "  " + ex.getMessage();
      LOG.error( message );
      throw new SQLException( message );
    }
  }

  public Connection getConnection()
  {
    return DBHelper.getConnection( DSN );
  }

  //---------------------------------------------------------------------------
  // methods - getter/setter
  //---------------------------------------------------------------------------

  public void resetQuery()
  {
    query.delete( 0, query.length() );
  }

  public void setQuery( String query )
  {
    resetQuery();
    this.query.append( query );
  }

  public void setQuery( StringBuffer query )
  {
    setQuery( query.toString() );
  }

  public String getQuery()
  {
    return query.toString();
  }

} // end class
