package ch.ige.rbbagisimport.server.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.util.GregorianCalendar;

import org.apache.commons.dbutils.ResultSetHandler;
import org.apache.log4j.Logger;

import ch.ige.ipobjects.patent.vo.SearchReportVO;
import ch.ige.rbbagisimport.server.vo.ReportVO;

/**
 * Data Access Object for table SEARCHREPORT
 * 
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author bolzli
 * @version $Id: SearchReportDAO.java,v 1.17 2008/04/25 13:18:54 wild Exp $
 */
public class SearchReportDAO extends BagisDAO
{
  private static final String STMT_SELECT_SEARCHREPORT = "Select * From SEARCHREPORT Where schutit_key = ? And type = ? And State = ?";

  private static final String STMT_SELECT_PATDOCSCHEMAMAPPING_ID = "select PATDOCSCHEMAMAPPING_ID  From PATDOCSCHEMAMAPPING WHERE PRODUCTIVE = 1";

  // Workaround, da des zu beginn keine gültige einträge in der Tabelle patdocschemamapping hat.
// private static final String STMT_SELECT_PATDOCSCHEMAMAPPING_ID = "select decode (PATDOCSCHEMAMAPPING_ID,4,5, PATDOCSCHEMAMAPPING_ID) PATDOCSCHEMAMAPPING_ID From PATDOCSCHEMAMAPPING WHERE PRODUCTIVE = 1";

  private static final Logger LOG = Logger.getLogger( SearchReportDAO.class );

  /**
   * default constructor
   * 
   * @throws SQLException
   */
  public SearchReportDAO() throws SQLException
  {
    super();
  }

  public void insert( ReportVO report ) throws SQLException
  {
    try
    {
      Boolean isFinal = false;
      Boolean isForThirdParty = false;
      Boolean isSent = true;
      Date importDate = new Date( new GregorianCalendar().getTimeInMillis() );

      if ( report.getType() == ReportVO.TYPE_FINAL )
      {
        isFinal = true;
      }
      if ( report.getState() == ReportVO.STATE_NOT_SENT )
      {
        isSent = false;
      }
      else
      {
        throw new SQLException( "Insert with State Sent is not allowed" );
      }
      if ( report.getState() == ReportVO.STATE_SENT_TO_THIRD_PARTY )
      {
        isForThirdParty = true;
      }
      SearchReportVO sRepVO = new SearchReportVO( this.selectPatDocSchemaMappingId(), isFinal, isForThirdParty, isSent, importDate, report.getXml() );
      SearchRepDAOWraper ipioDAO = new SearchRepDAOWraper();
      ipioDAO.insert( sRepVO, report.getSchutitKey() );
    }
    catch ( SQLException ex )
    {
      String message = "error in insert; schutit_key: " + report.getSchutitKey() + "  " + ex.getMessage();
      LOG.error( message );
      throw new SQLException( message );
    }
  }

  /**
   * Deletes a Row of Searchreport with ipiObject Functionality
   *  
   * @param searchReportId 
   */
  public void delete( int searchReportId )
  {
    SearchRepDAOWraper ipioDAO = new SearchRepDAOWraper();
    ipioDAO.delete( searchReportId );

  }

  private int selectPatDocSchemaMappingId() throws SQLException
  {
    try
    {
      class RSHandler implements ResultSetHandler
      {
        public Object handle( ResultSet rs ) throws SQLException
        {
          int mySchemaId = 0;
          if ( rs.next() )
          {
            mySchemaId = rs.getInt( "patdocschemamapping_id" );
          }
          return mySchemaId;
        }
      }
      String ps = STMT_SELECT_PATDOCSCHEMAMAPPING_ID;
      ResultSetHandler rsh; // result set handler
      rsh = new RSHandler();

      return (Integer) run.query( ps, rsh );
    }
    catch ( SQLException ex )
    {
      String message = "error in selectPatDocSchemaMappingId " + ex.getMessage();
      LOG.error( message );
      throw new SQLException( message );
    }
  }

  /**
   * Get a row of the table searchreport. 
   * 
   * @param report (ReportVO)
   * @return ReportVO 
   * @throws SQLException
   */
  @SuppressWarnings( "hiding" )
  public ReportVO select( ReportVO report ) throws SQLException
  {
    try
    {
      // Handler Class
      class RSHandler implements ResultSetHandler
      {
        public Object handle( ResultSet rs ) throws SQLException
        {
          ReportVO myReport = null;
          if ( rs.next() )
          {
            myReport = new ReportVO( rs.getInt( "searchreport_id" ), rs.getInt( "schutit_key" ), rs.getInt( "type" ), rs.getInt( "state" ), rs
                .getDate( "import_date" ), rs.getString( "xml" ) );
          }
          return myReport;
        }
      }

      String ps; // prepared statement
      ResultSetHandler rsh; // result set handler
      Object[] params; // parameters

      ps = STMT_SELECT_SEARCHREPORT;
      params = new Object[] { report.getSchutitKey(), report.getType(), report.getState() };
      rsh = new RSHandler();

      ReportVO existingReport = (ReportVO) run.query( ps, params, rsh );

      return existingReport;
    }
    catch ( SQLException ex )
    {
      String message = "error in select Searchreport; schutitKey: " + report.getSchutitKey() + " Type: " + report.getType() + " " + ex.getMessage();
      LOG.error( message );
      throw new SQLException( message );
    }

  }

} // end class
