package ch.ige.rbbagisimport.server.dao.test;

import java.sql.Date;
import java.sql.SQLException;
import java.util.GregorianCalendar;

import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;

import ch.ige.dbutils.util.ClientInitializer;
import ch.ige.rbbagisimport.server.dao.SearchReportDAO;
import ch.ige.rbbagisimport.server.vo.ReportVO;
import ch.ige.util.server.DBHelper;

/**
 * Junit Testcase for testing DAO run on client 
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   ruegsegger
 * @version  $Id: DAOTestCase.java,v 1.7 2008/05/05 16:14:09 wild Exp $
 */
public class DAOTestCase extends TestCase
{
  private static final String SOME_GREEK_LETTERS = "\u03A6\u0399\u039B\u0397\u039C\u039F\u039D\u0391";

  private static final String THOUSAND_BYTES = "Lorem ipsum dolor sit amet, consectetuer sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie consequat, velillum dolore GAGA";

  /**
   * Konstruktor
   * @param s
   */
  public DAOTestCase( String s )
  {
    super( s );
  }

  static
  {
    BasicConfigurator.configure();
    DBHelper.setRunOnServer( false );
    ClientInitializer.initStandaloneClient();
  }

  /**
   * Einfacher Test, ob die Daten geschrieben werden.
   */
  public void test00Insert()
  {
    String xmlString = "gaga";
    ReportVO report = new ReportVO( 123, 1201634757, 1, 0, new Date( new GregorianCalendar().getTimeInMillis() ), xmlString );
    ReportVO checkReport;
    try
    {
      SearchReportDAO srDAO = new SearchReportDAO();
      srDAO.insert( report );
      checkReport = srDAO.select( report );

      // cleanup
      srDAO.delete( checkReport.getSearchReportId() );

      assertEquals( report.getXml(), checkReport.getXml() );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  /**
   * Einfacher Test, mit einem Spezialfall aus den Tests
   */
  public void test01Insert()
  {
    String xmlString = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><search-report-data xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""
        + " xsi:noNamespaceSchemaLocation=\"http://schema.ige.ch/ch-search-report-v1-2-ige0-4.xsd\" id=\"srep\" lang=\"de\" status="
        + "\"final\" srep-office=\"CH\" date-produced=\"2007-09-10\" srep-type=\"ch-sr\"><srep-info><application-reference><document-id>"
        + "<date-of-earliest-priority><!-- Das Prioritätsdatum stammt aus BAGIS (ist hier rein fiktiv... ) --><date>2007-08-15</date>"
        + "</date-of-earliest-priority><applicant-name><name>Die Firma AG</name></applicant-name><srep-unity-of-invention><p"
        + " num=\"0001\">   <!-- wird durchnummeriert -->Begründung für mangelnde Erfindung</p><srep-search-fees><srep-fee-1 />  "
        + " <!-- \"Alle weiteren Recherchegebühren...\" gemäss Tagging-Dokument -->  </srep-search-fees> </srep-unity-of-invention> "
        + "<srep-invention-title title-approval=\"yes\" /> <!-- keine weiteren Informationen --> <srep-abstract abs-approval=\"yes\" />  "
        + "<!-- keine weiteren Informationen --> <srep-figure-to-publish figinfo=\"1\" />  <!-- immer 1 --> <srep-info-admin>  "
        + "<srep-office>  <addressbook>   <name>Nyfeler, Theodor</name>   <address>   <city>Bern</city>   <country>CH</country> "
        + "  </address>  </addressbook>  </srep-office> </srep-info-admin> </srep-info> <srep-for-pub> <classifications-ipcr>   "
        + " <!-- Klassifikation der Anmeldung - wird erst später durch BAGIS hinzugefügt! -->  <classification-ipcr sequence=\"1\"> "
        + " <section>F</section>  <class>41</class>  <subclass>H</subclass>  <main-group>11</main-group>  <subgroup>16</subgroup>  "
        + "<ipc-version-indicator>   <date>2006-01-01</date>  </ipc-version-indicator>  <classification-level>ipcr-advanced"
        + "</classification-level>  <symbol-position>first-symbol-position</symbol-position>  <classification-value>invention"
        + "</classification-value>  </classification-ipcr>  <classification-ipcr sequence=\"2\">  <section>A</section>  "
        + "<class>01</class>  <subclass>B</subclass>  <main-group>33</main-group>  <subgroup>02</subgroup>  <ipc-version-indicator>  "
        + " <date>2006-01-01</date>  </ipc-version-indicator>  <classification-level>ipcr-advanced</classification-level>  "
        + "<symbol-position>later-symbol-position</symbol-position>  <classification-value>invention</classification-value> "
        + " </classification-ipcr> </classifications-ipcr> <srep-fields-searched>  <minimum-documentation>  <classifications-ipcr> "
        + "  <classification-ipcr sequence=\"1\">   <text>F41H</text>   </classification-ipcr>   <classification-ipcr sequence=\"2\">  "
        + " <text>A01B</text>   </classification-ipcr>  </classifications-ipcr>  </minimum-documentation> </srep-fields-searched> "
        + "<srep-citations>  <citation id=\"CI0001\">   <!-- id immer CI + Laufnummer! -->  <patcit dnum=\"US2003145716\" "
        + "url=\"http://v3.espacenet.com/textdoc?DB=EPODOC&amp;IDX=US2003145716&amp;CY=ep\">   <document-id>   <country>US</country>  "
        + " <doc-number>2003145716</doc-number>   <kind>A1</kind>   <name>DIXON ROY</name>   <date>2003-08-07</date>   </document-id>"
        + "  </patcit>  <!-- Ein <rel-passage> pro Kategorie-Abschnitt. -->  <rel-passage>   <passage>* Absätze [0012], [0054], [0037], "
        + "[0018], [0044], [0020], Abbildung 10 *</passage>   <category> X</category>   <rel-claims>1, 2, 4-7, 9</rel-claims>  "
        + "</rel-passage>  <rel-passage>   <passage>* Absätze [0013], [0027], Fig. 13 *</passage>   <category>Y</category> "
        + "  <rel-claims>3, 8, 10-15</rel-claims>  </rel-passage>  </citation>  <citation id=\"CI0002\"> "
        + " <patcit dnum=\"EP0717216\" url=\"http://v3.espacenet.com/textdoc?DB=EPODOC&amp;IDX=EP0717216&amp;CY=ep\">   "
        + "<document-id>   <country>EP</country>   <doc-number>0717216</doc-number>   <kind>A1</kind>   <name>NIPPON KOGAKU KK</name>  "
        + " <date>1997-06-04</date>   </document-id>  </patcit>  <rel-passage>   <passage>* ganzes Dokument *</passage>   "
        + "<category>D, A</category>   <rel-claims>1, 11</rel-claims>  </rel-passage>  <rel-passage>   <passage>* Seite 5 Zeilen 29-35,"
        + " Fig. 2 *</passage>   <category>Y</category>   <rel-claims>3, 8, 10-15</rel-claims>  </rel-passage>  </citation>  "
        + "<citation id=\"CI0003\">   <nplcit>   <text>[ONLINE] PATENT ABSTRACTS OF JAPAN, vol. 1997, no. 02, 28.02.1997</text> "
        + " </nplcit>  <rel-passage>   <passage>* Zusammenfassung, Fig. 1 *</passage>   <category>D, A</category>   "
        + "<rel-claims>1, 11</rel-claims>  </rel-passage>  <corresponding-docs>   <patcit dnum=\"JP08273202\""
        + " url=\"http://v3.espacenet.com/textdoc?DB=EPODOC&amp;IDX=JP08273202&amp;CY=jp\">   <document-id>    "
        + "<country>JP</country>    <doc-number>08273202</doc-number>    <kind>A</kind>    <name>SONY CORP</name>    "
        + "<date>1996-10-18</date>   </document-id>   </patcit>  </corresponding-docs>  </citation>  <citation id=\"CI0004\">  "
        + "<nplcit>   <text>Example of NPL-Citation including a weblink: http://www.vdmais.kiev.ua/pdf_files/tektronix5000-8000.pdf#search='p7380sma%20tektronix and a link to espacenet including"
        + " a patent number: http://v3.espacenet.com/results?sfsf=a&amp;DBEPODOC=&amp;PN=ep100001</text>"
        + "  </nplcit>  <rel-passage>   <passage>* Footnote in right column on page 2 showing Copyright "
        + "from 2004Section: &quot;Complete Probe Solutions&quot;. Subsection: Differential Probes&quot;;"
        + " P7380SMA, 8GHz differential probe *</passage>   <category>X</category>   <rel-claims>1-10</rel-claims> "
        + " </rel-passage>  </citation>   </srep-citations> <srep-admin>  <srep-office>  <addressbook>  "
        + " <name>Nyfeler, Theodor</name>   <address>   <city>Bern</city>   <country>CH</country>   </address>  "
        + "</addressbook>  </srep-office>  <date-search-completed>  <date>2006-05-31</date>  </date-search-completed>"
        + " </srep-admin> <srep-patent-family>  <patent-family>  <priority-application>   <document-id>   <country>US</country> "
        + "  <doc-number>2003145716</doc-number>   <kind>A1</kind>   <date>2003-08-07</date>   </document-id>  </priority-application> "
        + " <text>NONE</text>  <!-- wenn keine Familienmitglieder vorhanden sind, kommt ein <text> Element mit NONE -->  </patent-family> "
        + " <patent-family>  <priority-application>   <document-id>   <country>EP</country>   <doc-number>0717216</doc-number>  " + " <kind>A1</kind>   <"
        + "date>1997-06-04</date>   </document-id>  </priority-application>  <family-member>   <document-id>"
        + "   <country>CN</country>   <doc-number>1414566</doc-number>   <kind>A</kind>   <date>2003-04-30</date>   </document-id>  "
        + "</family-member>  <family-member>   <document-id>   <country>EP</country>   <doc-number>1271553</doc-number>  "
        + " <kind>A2</kind>   <date>2003-01-02</date>   </document-id>  </family-member>  <family-member>   <document-id>   "
        + "<country>US</country>   <doc-number>2003128586</doc-number>   <kind>A1</kind>   <date>2003-01-02</date>   </document-id>  "
        + "</family-member>  <family-member>   <document-id>   <country>JP</country>   <doc-number>2003109386</doc-number>   <kind>A</kind> "
        + "  <date>2003-04-11</date>   </document-id>  </family-member>  </patent-family> </srep-patent-family> </srep-for-pub>" + "</search-report-data>";
    ReportVO report = new ReportVO( 123, 1201634757, 1, 0, new Date( new GregorianCalendar().getTimeInMillis() ), xmlString );
    ReportVO checkReport;
    try
    {
      SearchReportDAO srDAO = new SearchReportDAO();
      srDAO.insert( report );
      checkReport = srDAO.select( report );

      // cleanup
      srDAO.delete( checkReport.getSearchReportId() );

      assertEquals( report.getXml(), checkReport.getXml() );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  /**
   * Insert mit Status gesendet darf nicht eingefügt werden.
   */
  public void test02Insert()
  {
    String xmlString = "gaga";
    ReportVO report = new ReportVO( 123, 1201634757, 1, 1, new Date( new GregorianCalendar().getTimeInMillis() ), xmlString );
    try
    {
      SearchReportDAO srDAO = new SearchReportDAO();
      srDAO.insert( report );
      fail( "Eintrag mit Status gesendet darf nicht gespeichert werden" );
    }
    catch ( SQLException ex )
    {
      assertEquals( ex.getMessage(), "error in insert; schutit_key: 1201634757  Insert with State Sent is not allowed" );
    }
  }

  /**
   * Einfügen und löschen 
   */
  public void test00Delete()
  {
    String xmlString = "gaga";
    ReportVO report = new ReportVO( 123, 1201634757, 1, 0, new Date( new GregorianCalendar().getTimeInMillis() ), xmlString );
    ReportVO checkReport;
    try
    {
      SearchReportDAO srDAO = new SearchReportDAO();
      srDAO.insert( report );
      checkReport = srDAO.select( report );
      if ( checkReport == null )
      {
        fail( "Insert nicht erfolgreich" );
      }
      srDAO.delete( checkReport.getSearchReportId() );

      checkReport = srDAO.select( report );
      assertNull( checkReport );
    }

    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  /**
   * Test, ob die Spezialzeichen geschrieben werden.
   */
  public void testSpezCharInsert()
  {
    String xmlString = "1234567890+\"*ç%&/()=?éà£!è´'´´~´C'est ci bon" + SOME_GREEK_LETTERS;

    ReportVO report = new ReportVO( 123, 1201634757, 1, 0, new Date( new GregorianCalendar().getTimeInMillis() ), xmlString );
    ReportVO checkReport;
    try
    {
      SearchReportDAO srDAO = new SearchReportDAO();
      srDAO.insert( report );
      checkReport = srDAO.select( report );

      // cleanup
      srDAO.delete( checkReport.getSearchReportId() );

      System.out.println( "Exp " + report.getXml() );
      System.out.println( "Res " + checkReport.getXml() );
      assertEquals( report.getXml(), checkReport.getXml() );

    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  /**
   * Test, ob ein String > 32KB geschrieben wird. 
   */
  public void testLongStringInsert()
  {
    GregorianCalendar cal = new GregorianCalendar();
    cal.set( 2008, GregorianCalendar.MARCH, 6 );
    Date myDate = new Date( cal.getTimeInMillis() );

    StringBuffer sb = new StringBuffer();
    for ( int i = 0; i < 34; i++ )
    {
      sb.append( THOUSAND_BYTES );
    }

    String xmlString = sb.toString();

    ReportVO report = new ReportVO( 123, 1201634757, 1, 0, myDate, xmlString );
    ReportVO checkReport;
    try
    {
      SearchReportDAO srDAO = new SearchReportDAO();
      srDAO.insert( report );
      checkReport = srDAO.select( report );

      // cleanup
      srDAO.delete( checkReport.getSearchReportId() );

      System.out.println( "Länge " + xmlString.length() );
      assertEquals( report.getXml(), checkReport.getXml() );

    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }
}
