package ch.ige.st36xmlgenerator.batch.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import junit.framework.TestCase;
import ch.ige.util.client.batch.BatchResult;
import ch.ige.util.client.batch.NewAbstractBatch;

public class AbstractBatchTestClient extends TestCase
{

  protected NewAbstractBatch client;
  
  protected static String TEST_PROPERTY = "test.mail.file";
  
  protected static String test_mail_path ;
  // init with inititalizer
  
  protected String nokfile_name;
    
  protected static File test_mail_file;
  
  protected static final String RETURN_CODE_ENTRY = "Return Code";
  
  protected List<String> skiplist = new ArrayList<String>(8);
  
  public AbstractBatchTestClient( String name )
  {
    super( name );
  }
  
  @Override
  protected void setUp() throws Exception
  {
    super.setUp();
    test_mail_file = new File( test_mail_path );
    this.test_mail_file.delete();
    this.test_mail_file.getParentFile().mkdirs();

    System.setProperty( "log4j.configuration", "file:///c:/jp/projects/st36xmlgenerator/install/programme/ige/st36xmlgenerator/config/st36biblgenerator_log_config.xml");
    System.setProperty( "java.naming.factory.initial", "org.jnp.interfaces.NamingContextFactory" );
    System.setProperty( "java.naming.provider.url", "jnp://ipiebgs010.ipie.ch:11099" );
    System.setProperty( "java.naming.factory.url.pkgs", "org.jnp.interfaces" );
    
    System.setProperty( TEST_PROPERTY, test_mail_path );
    
    this.skiplist.addAll( Arrays.asList(  new String[] {"Ende", "Dauer", "Start", "CVS-Tag", "Build-ID"} ) );
    // init client in subclass
    // increase skiplist
    System.gc();
  }
  

  protected void executeTestAndCompareResultWithExpected(String[] args, boolean statusSuccess, int expectedReturnCode, String resourceExpectedResultFile) throws Exception
  {
    NewAbstractBatch.start( client, args );
    checkStatusFile( statusSuccess,expectedReturnCode );
    assertTrue( compareFiles( test_mail_file, this.getClass().getResourceAsStream( resourceExpectedResultFile )) );
  }

 

  
  /**
   * Checks the local status file.
   * @throws Exception if the status file isn't OK
   */
  protected void checkStatusFile(boolean success, int returnCode) throws Exception
  {
    String filename  = success ? nokfile_name +".ok" : nokfile_name +".nok";
    File statusFile = new File( BatchResult.BATCH_RESULT_DIRECTORY, filename );
    if ( !statusFile.exists() )
    {
      throw new IllegalStateException( "Missing OK status file " + statusFile.getAbsolutePath() );
    }
    BufferedReader reader = new BufferedReader(new FileReader (statusFile));
    String line = reader.readLine();
    while (line != null  )
    { 
      if ( returnCode == 0)
      {
        assertEquals(Integer.valueOf( line.trim()).intValue(),0);
        line = reader.readLine();
        continue;
      }
      int indexOfRC = line.indexOf( RETURN_CODE_ENTRY );
      if ( indexOfRC < -1) // should be in first line
      {
        throw new IllegalArgumentException("No return code in file " + filename);
      }
      int substringStart = indexOfRC + RETURN_CODE_ENTRY.length() + 1;
      int returncodeRead = Integer.valueOf( line.substring( substringStart , substringStart+ 2 ).trim());
      assertEquals(returnCode, returncodeRead);
      line = reader.readLine();
    }
    statusFile.delete();
  }
  
  protected boolean compareFiles(File file1, InputStream file2) throws Exception
  {
    
    BufferedReader reader1 = new BufferedReader(new FileReader (file1));
    BufferedReader reader2 = new BufferedReader(new InputStreamReader(file2));
    String line1 = reader1.readLine();
    String line2 = reader2.readLine();

    while (line1 != null  )
    {
      if ( line2 == null  )
      {
        return false;
      }
      boolean skip = false;
      for (String skipMarker : skiplist )
      {
        if ( line1.contains( skipMarker ) && line2.contains( skipMarker ))
        {
          skip = true;
          break;
        }
      }
      if ( !skip )
      {
        if (!line1.equals( line2 ))
        {
          return false;
        }
      }
      line1 = reader1.readLine();
      line2 = reader2.readLine();

    }
    if ( line2 != null  )
    {
      return false;
    }
    return true;
    
  }
}
