package ch.ige.st36xmlgenerator.metadata;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

import ch.ige.ipobjects.patent.vo.PatentDocumentSchemaVO;

/**
 * The factory for the metadata extractors, that are created for a specified schema 
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author fabrizioli
 * @version $Id: JAXBMetadataExtractorFactory.java,v 1.5 2008/09/25 07:29:27 fabrizioli Exp $
 */
public class JAXBMetadataExtractorFactory
{
  private static final Logger LOG = Logger.getLogger( JAXBMetadataExtractorFactory.class );

  private static JAXBMetadataExtractorFactory uniqueInstance = new JAXBMetadataExtractorFactory();

  /**
   * Get the singleton
   */
  public static final JAXBMetadataExtractorFactory getInstance()
  {
    return JAXBMetadataExtractorFactory.uniqueInstance;
  }

  /**
   * Initialize the metadata with the schema and create the extractors
   */
  public static final void initialize( PatentDocumentSchemaVO productiveSchema )
  {
    LOG.debug( "initialize factory with schemas" );
    JAXBMetadataExtractorFactory.uniqueInstance.setSchema( productiveSchema );
  }

  /**
   * Clear the extractors
   */
  public static final void releaseSchemas()
  {
    LOG.debug( "release all schemas from factory" );
    JAXBMetadataExtractorFactory.uniqueInstance.releaseAll();
  }

  private void releaseAll()
  {
    this.extractors.clear();
  }

  private Map<String, XMLGeneratorMetadataExtractor> extractors;

  private JAXBMetadataExtractorFactory()
  {
    this.extractors = Collections.<String, XMLGeneratorMetadataExtractor>emptyMap();
  }

  public JAXBMetadataExtractorFactory( PatentDocumentSchemaVO schema )
  {
    this.setSchema( schema );
  }

  private void setSchema( PatentDocumentSchemaVO schema )
  {
    this.extractors = createExtractors( schema );
  }

  private Map<String, XMLGeneratorMetadataExtractor> createExtractors( PatentDocumentSchemaVO schema )
  {
    if ( schema == null )
    {
      throw new JAXBMetadataException( "Cannot initialize the MetadataExtractorFactory without any schema information" );
    }
    Map<String, XMLGeneratorMetadataExtractor> result = new HashMap<String, XMLGeneratorMetadataExtractor>( 1 );
    XMLGeneratorMetadataExtractor extractor = new JAXBXMLGeneratorMetadataExtractor( schema );
    result.put( schema.getId_string(), extractor );
    return result;
  }

  public Set<String> getAvailableSchemas()
  {
    return Collections.unmodifiableSet( this.extractors.keySet() );
  }

  public XMLGeneratorMetadataExtractor getExtractor( String schemaIdentifier )
  {
    return this.extractors.get( schemaIdentifier );
  }

}
