package ch.ige.st36xmlgenerator.metadata;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import ch.ige.st36xmlgenerator.builder.Factory;
import ch.ige.st36xmlgenerator.exception.MissingMandatoryValueException;
import ch.ige.st36xmlgenerator.metadata.XPathXMLAccessor.AttributeDefinition;

/**
 * 
 * @author fabrizioli
 *
 */
public class XMLAccessorContainer implements XMLAccessor
{

  public XMLAccessorContainer( List<XMLAccessor> xmlAccessors, Boolean isMandatory )
  {
    super();
    this.isMandatory = isMandatory;
    this.xmlAccessors = xmlAccessors;
    this.size = xmlAccessors.size();
  }

  public XMLAccessorContainer( int size )
  {
    this.xmlAccessors = new ArrayList<XMLAccessor>( size );
    this.size = size;
  }

  public void setMandatory( boolean mandatory )
  {
    this.isMandatory = mandatory;
  }

  private boolean isMandatory;

  private List<XMLAccessor> xmlAccessors;

  private int size;

  public int getExpectedNumberOfValues()
  {
    return size;
  }

  public boolean createPathAndSetValue( Factory<?> factory, Object[] values ) throws ClassNotFoundException, MissingMandatoryValueException
  {
    return createPathAndSetValue( factory, values, (Map[]) null );
  }

  private boolean containsNull( Object[] values )
  {
    for ( int i = 0; i < values.length; i++ )
    {
      if ( values[i] == null )
      {
        return true;
      }
    }
    return false;
  }

  public void add( XMLAccessor xmlAcc )
  {
    //isMndatory not introduced to elements : lesser objects are created xmlAcc.setMandatory(isMandatory);
    xmlAccessors.add( xmlAcc );

  }

  public boolean createPathAndSetValue( Factory<?> factory, Object value ) throws ClassNotFoundException, MissingMandatoryValueException
  {

    return createPathAndSetValue( factory, new Object[] { value } );
  }

  public boolean isMandatory()
  {
    return isMandatory;
  }

  public boolean createPathAndSetValue( Factory<?> factory, Object[] values, Map<String, List<AttributeDefinition>>[] attributes ) throws ClassNotFoundException,
      MissingMandatoryValueException
  {
    if ( values.length != size || ( attributes != null && attributes.length != size ) )
    {
      throw new IllegalArgumentException( "The number of values (" + values.length + ") or attributes (" + ( attributes != null ? attributes.length : 0 )
          + ") must be equal to the number of xpaths available (" + size + ")" );
    }
    if ( isMandatory && containsNull( values ) )
    {
      throw new MissingMandatoryValueException( "Missing a mandatory value for the st36 generation" );
    }
    int i = 0;
    boolean success = true;
    for ( XMLAccessor xmlaccessor: xmlAccessors )
    {
      int index = i++;
      Object next = values[index];
      Map attribute = null;
      if ( attributes != null )
      {
        attribute = attributes[index];
      }
      if ( next == null )
      {
        continue;
      }
      else
      {
        success &= xmlaccessor.createPathAndSetValue( factory, next, attribute );
      }
    }
    return success;
  }

  public boolean createPathAndSetValue( Factory<?> factory, Object value, Map<String, List<AttributeDefinition>> attributes ) throws ClassNotFoundException,
      MissingMandatoryValueException
  {
    return createPathAndSetValue( factory, new Object[] { value }, new Map[] { attributes } );
  }

}
