package ch.ige.st36xmlgenerator.test;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.custommonkey.xmlunit.DetailedDiff;
import org.custommonkey.xmlunit.Diff;
import org.custommonkey.xmlunit.Difference;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import ch.ige.ipobjects.patent.KindCode;
import ch.ige.xmlutil.doc.DocumentTypes;
import ch.ige.xmlutil.doc.XMLDocument;

public abstract class AbstractXMLTestCase extends TestCase
{

  protected String folderPath; 
  
  public AbstractXMLTestCase( String name, String folderPath )
  {
    super( name );
    this.folderPath = folderPath;
  }

  protected void compareWithStored( Document doc, Integer schutit, KindCode kind ) throws Exception
  {
    Document refDoc = getReference( schutit + "_" + kind + ".xml" );
    assertXMLEqual( doc, refDoc );
  }
  
  protected void compareWithStored( Document doc, String filename ) throws Exception
  {
    Document refDoc = getReference( filename + ".xml" );
    assertXMLEqual( doc, refDoc );
  }

  protected Document getReference( String fileName ) throws SAXException, IOException, ParserConfigurationException
  {
    File folder = new File( folderPath );
    InputStream expectedXmlStream = new FileInputStream( new File( folder, fileName ) );
    assertNotNull( "Checking that the expected XML file '" + fileName + "' exists", expectedXmlStream );
    Document doc = XMLDocument.fromUTF8InputStream( DocumentTypes.GENERIC, expectedXmlStream ).getDOM();
    doc.normalizeDocument();
    return doc;
  }
  
  protected void assertXMLEqual( Document actual, Document expected )
  {
    Diff diff = new Diff( expected, actual );
    if ( !diff.similar() )
    {
      Assert.fail( detailedDifferenceMessage( diff, actual, expected ) );
    }
  }

  @SuppressWarnings( "unchecked" )
  private String detailedDifferenceMessage( Diff prototype, Document actual, Document expected )
  {
    DetailedDiff detailed = new DetailedDiff( prototype );
    List<Difference> diffs = detailed.getAllDifferences();
    StringBuilder msg = new StringBuilder( "<failure>" );
    msg.append( "<differences>" );
    for ( Difference difference: diffs )
    {
      msg.append( "<difference><![CDATA[" );
      msg.append( difference.toString() );
      msg.append( "]]></difference>" );
    }
    msg.append( "</differences>" );
    msg.append( "<expected>" );
    try
    {
      msg.append( "<![CDATA[" );
      msg.append( XMLDocument.fromDOM( DocumentTypes.GENERIC, expected ).getContent() );
      msg.append( "]]>" );
    }
    catch ( TransformerException e )
    {
      StringWriter trace = new StringWriter();
      e.printStackTrace( new PrintWriter( trace ) );
      msg.append( "<![CDATA[" );
      msg.append( trace );
      msg.append( "]]>" );
    }
    msg.append( "</expected>" );
    msg.append( "<actual>" );
    try
    {
      msg.append( "<![CDATA[" );
      msg.append( XMLDocument.fromDOM( DocumentTypes.GENERIC, actual ).getContent() );
      msg.append( "]]>" );
    }
    catch ( TransformerException e )
    {
      StringWriter trace = new StringWriter();
      e.printStackTrace( new PrintWriter( trace ) );
      msg.append( "<![CDATA[" );
      msg.append( trace );
      msg.append( "]]>" );
    }
    msg.append( "</actual>" );
    msg.append( "</failure>" );
    return msg.toString();
  }

}
