package ch.ige.ses.client.batch;

import java.io.*;
import org.apache.log4j.*;
import ch.ige.util.client.*;
import ch.ige.ses.BuildInfo;
import ch.ige.ses.server.batchctrl.*;

/**
 * SES Batch Programm zur reorganisieren der Datenbank
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author   Stefan Nydegger, thomas Iten
 * @version $Id: SesReorganisation.java,v 1.10 2007/01/10 12:25:25 iten Exp $
 */
public class SesReorganisation extends BatchBasis
{
  //
  // const
  //
  private static final Logger LOG      = Logger.getLogger("ch.ige.ses.client.batch");
  private static final String JOB_NAME = "SesReorganisation";

  private static final int    MODUS_HELP          = 0;    // program modes
  private static final int    MODUS_AUTO          = 1;
  private static final int    MODUS_MAN           = 2;

  private static final String OK_NUM_SUCCESS      = "0";
  private static final String OK_MSG_SUCCESS      = "Batch Job erfolgreich beendet: ";

  private static final String ERR_NUM_EXCEPTION   = "-1";
  private static final String ERR_MSG_EXCEPTION   = "Eine unerwartete Exception ist aufgetreten: ";

  private static final String ERR_NUM_WRONG_ARG   = "-2";
  private static final String ERR_MSG_WRONG_ARG   = "Aufruf Program mit ungültigen Parametern";

  //
  // fields
  //
  private int          fMode      = MODUS_HELP;
  private StringBuffer fActionMsg = null;


  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public SesReorganisation()
  {
    super(JOB_NAME);
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  public int getMode()
  {
    return fMode;
  }

  public boolean hasActionMsg()
  {
    if (fActionMsg == null)
    {
      return false;
    }
    else
    {
      return true;
    }
  }

  public String getActionMsg()
  {
    if (hasActionMsg())
    {
      return fActionMsg.toString();
    }
    else
    {
      return "";
    }
  }

  /**
   * Ausgabe Programmhilfe
   * Zeigt mit welchen Parametern das Program gestartet werden kann.
   */
  private void printUsage()
  {
    System.out.println();
    System.out.println("Usage: " + BuildInfo.CVS_TAG);
    System.out.println("SesReorganisation modus [-h] [-?] [help] [?]");
    System.out.println("  -h, -?, help, ?   Anzeige Hilfe (Usage)");
    System.out.println("  modus             Programm Modus [auto,man]" );
    System.out.println();
  }


  /**
   * Liest die Programm Parameter ein und setzt (falls ein Wert vorhanden) die
   * entsprechenden Attribute.
   *
   * @param  args             programm arguments
   * @throws BatchException   in case of wrong arguments
   */
  private void loadParams(String[] args) throws BatchException
  {
    if (args.length ==1)
    {
      if ( args[0].equalsIgnoreCase("auto") )
      {
        fMode = MODUS_AUTO;
      }
      else if ( args[0].equalsIgnoreCase("man") )
      {
        fMode = MODUS_MAN;
      }
      else if ( args[0].equalsIgnoreCase("-h")   || args[0].equals("-?") ||
                args[0].equalsIgnoreCase("help") || args[0].equals("?") )
      {
        fMode = MODUS_HELP;
      }
      else 
      {
        setNotifyConsole(true);
        throw new BatchException (ERR_NUM_WRONG_ARG,ERR_MSG_WRONG_ARG);
      }
    }
    else
    {
      setNotifyConsole(true);
      throw new BatchException (ERR_NUM_WRONG_ARG,ERR_MSG_WRONG_ARG);
    }
  }


  /**
   * Start manueller Modus
   *
   * @throws Exception  unerwarteter Fehler
   */
  public void manStart() throws Exception
  {
    String  table   = null;
    int     month   = 0;
    String  result  = null;

    System.out.println();
    System.out.println("SesReorganisation - manueller Modus:");
    System.out.println("SesReorganisation - manueller Modus:");

    System.out.println("Eingabe Parameter:");
    table = readString("  Tabelle [jgb_lauf,amv_lauf,stako_kontrolle,trace] = ");
    month = readInt   ("  Anzahl Monate                                     = ");
    System.out.println();

    result = BatchDelegate.getInstance().reorgDatabase(table,month);

    System.out.println("");
    System.out.println("Resultat:");
    setMessage ( MSG_TYPE_OK, OK_NUM_SUCCESS, OK_MSG_SUCCESS + "\n" + result );
    System.out.println("");
  }


  /**
   * Start Produktiv Modus.
   *
   * @throws Exception  unerwarteter Fehler
   */
  public void autoStart() throws Exception
  {
    int     months  = 0;
    String  result  = null;

    LOG.debug("autoStart - start cleaning table jgb_lauf...");
    months = Integer.parseInt(BatchDelegate.getInstance().getProperty("batch.SesReorganisation.keepJgb"));
    result = BatchDelegate.getInstance().reorgDatabase("jgb_lauf",months);
    addActionMsg(result);

    LOG.debug("autoStart - start cleaning table amv_lauf...");
    months = Integer.parseInt(BatchDelegate.getInstance().getProperty("batch.SesReorganisation.keepAmv"));
    result = BatchDelegate.getInstance().reorgDatabase("amv_lauf",months);
    addActionMsg(result);

    LOG.debug("autoStart - start cleaning table stako_kontrolle...");
    months = Integer.parseInt(BatchDelegate.getInstance().getProperty("batch.SesReorganisation.keepStako"));
    result = BatchDelegate.getInstance().reorgDatabase("stako_kontrolle",months);
    addActionMsg(result);

    LOG.debug("autoStart - start cleaning table trace...");
    months = Integer.parseInt(BatchDelegate.getInstance().getProperty("batch.SesReorganisation.keepTrace"));
    result = BatchDelegate.getInstance().reorgDatabase("trace",months);
    addActionMsg(result);

    setMessage ( MSG_TYPE_OK, OK_NUM_SUCCESS,
                 OK_MSG_SUCCESS + "\n" + getActionMsg() );

  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  /**
   * Ausgabe Prompt Meldung und Einlesen eines Strings.
   *
   * @param   prompt        Meldung für Ausgabe
   * @throws  Exception     Unerwarteter Fehler
   * @return  String        Eingelesener String
   */
  private String readString(String prompt) throws Exception
  {
    BufferedReader  stdin = new BufferedReader(new InputStreamReader(System.in));

    System.out.print(prompt);
    return stdin.readLine();
  }

  /**
   * Ausgabe Prompt Meldung und Einlesen einer Zahl.
   *
   * @param   prompt        Meldung für Ausgabe
   * @throws  Exception     Unerwarteter Fehler
   * @return  int           Eingelesene Zahl
   */
  private int readInt(String prompt) throws Exception
  {
    return Integer.parseInt(readString(prompt));
  }


  private void addActionMsg(String msg)
  {
    if (fActionMsg == null)
    {
      fActionMsg = new StringBuffer();
    }

    fActionMsg.append(msg);
  }

  //***************************************************************************
  // main
  //***************************************************************************

  public static void main(String[] args)
  {
    SesReorganisation job = new SesReorganisation();

    try
    {
      //
      // init
      //
      ClientLogInitalizer.initalizeLogging(ClientLogInitalizer.BATCH_CONFIG);
      job.loadParams(args);


      //
      // run job
      //
      switch (job.getMode())
      {
        case MODUS_AUTO:
          job.autoStart();                      // start job
          job.notifyMessage();                  // notify that job successfully finished
          break;

        case MODUS_MAN:
          job.setNotifyAll(false,false,true);   // set notif options to console only
          job.manStart();                       // start test
          job.notifyMessage();                  // notify that job successfully finished
          break;

        default:
          job.printUsage();
          break;
      }

      //
      // End
      //
      System.exit(0);

    }
    catch (BatchException bex)
    {
      LOG.error("main - BatchException occured: " + bex.getErrorMsg());
      job.notifyMessage ( MSG_TYPE_NOK, bex.getErrorNr(),
                          job.getActionMsg() + bex.getErrorMsg() );
      System.exit( Integer.parseInt(bex.getErrorNr()) );
    }
    catch (Exception ex)
    {
      LOG.error("main - exception occured: " + ex.getMessage());
      job.notifyMessage ( MSG_TYPE_NOK, ERR_NUM_EXCEPTION,
                          job.getActionMsg() + ERR_MSG_EXCEPTION + ex.toString() );
      System.exit( Integer.parseInt(ERR_NUM_EXCEPTION) );
    }

  } // end main


} // end class

