package ch.ige.ses.client.sapinbound;

import java.util.*;
import java.lang.*;
import com.sap.mw.jco.*;
import org.apache.log4j.*;
import ch.ige.ses.client.saputil.*;

/**
 * SAP Remote Function Module _BAPI_MATERIAL_EXISTENCECHECK
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * <p>Organisation: Eidgenössisches Institut für Geistiges Eigentum
 * @author Patrick Bolzli
 * @version $Id: RFM_BAPI_MATERIAL_EXISTENCECHECK.java,v 1.5 2006/12/11 12:43:32 iten Exp $
 */

public class RFM_BAPI_MATERIAL_EXISTENCECHECK extends RFM
{
  //
  // const
  //
  private static final Logger LOG = Logger.getLogger(RFM_Z_BAPI_ZVL_GET_CUST_BALANCE.class);

  //
  // fields
  //
  public String fNextStep = "";  // next step in process

  //------------------------------------------------------------------
  // constructors
  //------------------------------------------------------------------
  public RFM_BAPI_MATERIAL_EXISTENCECHECK (IRepository repository)
  {
    super ("BAPI_MATERIAL_EXISTENCECHECK" ,repository);
  }

  //------------------------------------------------------------------
  // public methods
  //------------------------------------------------------------------
  public void existenceCheck( String art_nr )
  {
    try
    {
      setParams(art_nr);
    }
    catch (Exception ex)
    {
      LOG.error("exception: " + ex.toString());
      LOG.error("hint (next step): " + fNextStep);
      throw new RfmException(ex.toString());
    }
  }

// Diese Methode muss überschrieben werden, da in der Parentklasse mit Tabellen gearbeitet wird.
// In diesem Fall kommt jedoch eine Structure zurück
  public boolean isReturnOk()
  {
    JCO.Structure str = this.fJCoFunc.getExportParameterList().getStructure("RETURN");
    String delflg = this.fJCoFunc.getExportParameterList().getString("DELETION_FLAG");
    String    buf;

    buf = str.getString("TYPE");
    if ( !(buf.equalsIgnoreCase("S") || buf.equals("")) || delflg.equalsIgnoreCase("X") )
    {
      return false;
    }
    return true;
  }



// Diese Methode muss überschrieben werden, da in der Parentklasse mit Tabellen gearbeitet wird.
// In diesem Fall kommt jedoch eine Structure zurück
  public String toStringReturn()
  {
    JCO.Structure str = this.fJCoFunc.getExportParameterList().getStructure("RETURN");
    String delflg = this.fJCoFunc.getExportParameterList().getString("DELETION_FLAG");
    StringBuffer buf = new StringBuffer();

    buf.append("TYPE ID NUMBER MESSAGE \n");
    buf.append( Convert.toSapStr(str.getString("TYPE"),5) +
                Convert.toSapStr(str.getString("ID"),3) +
                Convert.toSapStr(str.getString("NUMBER"),7) +
                str.getString("MESSAGE") + "\n" +
                "DELETION_FLAG:" + delflg );

    return buf.toString();
  }

// Diese Methode muss überschrieben werden, da in der Parentklasse mit Tabellen gearbeitet wird.
// In diesem Fall kommt jedoch eine Structure zurück
  public String toStringReturnNOK()
  {
    JCO.Structure str = this.fJCoFunc.getExportParameterList().getStructure("RETURN");
    String delflg = this.fJCoFunc.getExportParameterList().getString("DELETION_FLAG");
    StringBuffer buf = new StringBuffer();
    String       type;

    buf.append("TYPE ID NUMBER MESSAGE \n");
    type = str.getString("TYPE");
    if ( !(type.equalsIgnoreCase("S") || buf.equals("")) || delflg.equalsIgnoreCase("X") )
    {
      buf.append ( Convert.toSapStr(str.getString("TYPE"),5) +
                   Convert.toSapStr(str.getString("ID"),3) +
                   Convert.toSapStr(str.getString("NUMBER"), 7) +
                   str.getString("MESSAGE") + "\n" +
                   "DELETION_FLAG:" + delflg );
    }
    return buf.toString();

  }

  //------------------------------------------------------------------
  // private methods
  //------------------------------------------------------------------
  private void setParams(String art_nr)
  {
    JCO.ParameterList input = this.fJCoFunc.getImportParameterList();

    fNextStep = "setParams - art_nr";
    input.setValue(Convert.toSapMaterial(art_nr), "MATERIAL");

  }

} // end class


