package ch.ige.ses.client.sapoutbound;

import java.util.*;
import java.io.*;
import java.math.BigDecimal;
import ch.ige.ses.BuildInfo;
import ch.ige.ses.vo.bagis.*;
import ch.ige.ses.client.saputil.*;
import ch.ige.ses.server.sapctrl.SapDelegateOutbound;

/**
 * Diese Klasse wird für den manuellen Modus verwendet.<br>
 * Sie ist nur für Testzwecke und schreibt direkt auf den Standard - Output.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: ManualMode.java,v 1.18 2007/01/09 16:48:29 iten Exp $
 */
public class ManualMode
{

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------
  public ManualMode()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  public void run() throws Exception
  {
    SapServerController   ssc = new SapServerController();
    boolean               go  = true;

    //
    // init
    //
    printBuildInfo();
    printParams();
    printProperties("SAP Outbound Client - Properties",SapOutbound.getProperties());
    printProperties("System - Properties",System.getProperties());

    //
    // run
    //
    while (go)
    {
      try
      {
        switch (printMenu())
        {
          case 1:
            printBuildInfo();
            printParams();
            printProperties("SAP Outbound Client - Properties",SapOutbound.getProperties());
            printProperties("System - Properties",System.getProperties());
            break;

          case 10:
            ssc.startServers();
            break;

          case 11:
            ssc.stopServers();
            break;

          case 20:
          {
            int                       cnt   = 0;
            SAPRechnungStatusListVO   list  = new SAPRechnungStatusListVO();

            cnt = readLineInt("Anzahl Status Rückmeldungen = ");
            for (int i=0; i<cnt; i++)
            {
              System.out.println("\nEinlesen Record #" + i + ":");
              list.statusList.add(readSAPRechnungStatusVO());
            }

            System.out.println("\nFolgende Daten wurden eingegeben:");
            System.out.println(list.toStringValidDataOnly());

            System.out.println("Start update BAGIS...");
            SapDelegateOutbound.getInstance().receiveStatus(list);
            break;
          }

          case 21:
          {
            int                       cnt   = 0;
            SAPRechnungStatusListVO   list  = new SAPRechnungStatusListVO();

            cnt = readLineInt("Anzahl Status Rückmeldungen = ");
            for (int i=0; i<cnt; i++)
            {
              System.out.println("\nEinlesen Record #" + i + ":");
              list.statusList.add(readSAPRechnungStatusVO());
            }

            System.out.println("\nFolgende Daten wurden eingegeben:");
            System.out.println(list.toStringValidDataOnly());

            System.out.println("Start update SES...");
            SapDelegateOutbound.getInstance().receiveStatusJGB(list);
            break;
          }

          case 99:
            ssc.stopServers();
            System.out.println("Exit SAP Inbound Client, good by...");
            go = false;
            break;

          case -1:
            System.out.println("Fehler beim einlesen des Befehls !");
            break;

          default:
            System.out.println("Unbekannter Befehl !");
            break;
        } // end switch
      } // end try

      catch (Exception ex)
      {
        System.out.println("");
        System.out.println("### Achtung folgende Exception ist aufgetreten:");
        System.out.println(ex.getMessage());
        System.out.println("Stack Trace:");
        ex.printStackTrace();
      }

    } // end while
  } // end run

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private int printMenu()
  {
    String         cmd;

    try
    {
      System.out.println();
      System.out.println("SAP Outbound Server - Manual Mode");
      System.out.println("***************************************************");
      System.out.println("    1    show program info, parameters & properties");
      System.out.println("   10    start servers ");
      System.out.println("   11    stop servers ");
      System.out.println("   20    update status on bagis db ");
      System.out.println("   21    update status on ses db (for jgb only)");
      System.out.println("   99    exit");
      System.out.println();

      cmd = readLine("   Comand: ");
      System.out.println();

      return (Integer.parseInt(cmd));
    }
    catch (Exception ex)
    {
      return -1;
    }
  } // end method

  private void printBuildInfo()
  {
    System.out.println();
    System.out.println("SAP Outbound Server - BuildInfo");
    System.out.println("   CVS Tag  : " + BuildInfo.CVS_TAG  );
    System.out.println("   Build Id : " + BuildInfo.BUILD_ID );
    System.out.println("   Default Provider Url used, when java.naming.provider.url is set to <default>:" );
    System.out.println("   Provider Url : " + ch.ige.util.BuildInfo.PROVIDER_URL );    
  } // end method

  private void printParams()
  {
    System.out.println();
    System.out.println("SAP Outbound Server - Params");
    System.out.println("   Program Id          : " + SapOutbound.getProgramId() );
    System.out.println("   Program Action      : " + SapOutbound.getProgramAction() );
    System.out.println("   Program Environment : " + SapOutbound.getProgramEnv() );
    System.out.println("   Properties File     : " + SapOutbound.getPropertiesFile() );
  }


  private void printProperties(String title, Properties prop)
  {
    Enumeration eNames = prop.propertyNames();
    TreeSet     set  = new TreeSet();
    String      key;

    System.out.println();
    System.out.println(title);
    while ( eNames.hasMoreElements() )
    {
      key = eNames.nextElement().toString();
      set.add(key);
    }

    for (Iterator i = set.iterator(); i.hasNext(); )
    {
      key = (String) i.next();
      System.out.println("  " + Convert.toSapStr(key,40) + ": <" + prop.getProperty(key) + ">");
    }

  } // end method


  private String readLine(String prompt)
  {
    try
    {
      BufferedReader stdin  = new BufferedReader(new InputStreamReader(System.in));

      System.out.print (prompt);
      return(stdin.readLine());
    }
    catch (Exception ex)
    {
      return ("");
    }
  } // end method


  private int readLineInt(String prompt)
  {
    return Integer.parseInt(readLine(prompt));
  } // end method


  private SAPRechnungStatusVO readSAPRechnungStatusVO()
  {
    SAPRechnungStatusVO vo = new SAPRechnungStatusVO();

    vo.bfakt_id  = readLineInt("bfakt_id  = ");
    vo.jgbLaufNr = readLine   ("jgbLaufNr = ");

    if ( readLineInt("Rückmeldungsart (0=Druck, 1=Zahl) = ") == 0)
    {
      vo.status_druck  = SAPRechnungStatusVO.STATUS_DRUCK_GEDRUCKT;
      vo.bfakt_dat     = new Date();
    }
    else
    {
      vo.status_zahlg   = readLine("Zahlstatus " + SAPRechnungStatusVO.LEGENDE_STATUS_ZAHLG + " = ");
      vo.status_datum   = new Date();             // datum zahlung oder storno
      vo.buch_datum     = new Date();             // buchungsdatum für postgrant
      vo.fakt_betrag    = new BigDecimal(100);
    }

    return vo;
  } // end method

} // end ManualMode


