package ch.ige.ses.client.saputil.test;

import java.util.*;
import junit.framework.*;

import ch.ige.ses.vo.bagis.*;
import ch.ige.ses.client.saputil.Convert;

/**
 * JUnit Test Class
 * <p>
 * Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: ConvertTest.java,v 1.6 2006/12/13 14:54:56 iten Exp $
 */
public class ConvertTest extends TestCase
{

  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------

  public ConvertTest(String s)
  {
    super(s);
  }

  //---------------------------------------------------------------------------
  // init / clean-up
  //---------------------------------------------------------------------------

  public static Test suite()
  {
    return new TestSuite(ConvertTest.class);
  }

  protected void setUp()
  {
    // lazy bone
  }

  protected void tearDown()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // tests
  //---------------------------------------------------------------------------

  public void testToSapNr()
  {
    // example: value = 5, len = 3 --> sap nr = "005"
    String sapNr = Convert.toSapNr(5, 3);
    assertEquals(sapNr, "005");
  }

  public void testToSapStr()
  {
    // example: value = hello, len = 10 --> sap string = "hello     "
    String sapStr = Convert.toSapStr("hello", 10);
    assertEquals(sapStr, "hello     ");
  }

  public void testToBagisArtNr()
  {
    // führenden Nullen entfernen
    String buf = Convert.toBagisArtNr("007");
    assertEquals(buf, "7");
  }

  public void testToSapBillDate()
  {
    // return sap bill date. returns current date if bfakt_dat is null.
    Date d = Convert.toSapBillDate(null);
    assertNotNull(d);
  }

  public void testToSapBillBlockESR()
  {
    SAPRechnungVO re = new SAPRechnungVO();
    assertEquals("Z1", Convert.toSapBillBlock(re));
  }

  public void testToSapBillBlockDepKto()
  {
    SAPRechnungVO re = new SAPRechnungVO();
    re.regulierer = 1;
    assertEquals("Z2", Convert.toSapBillBlock(re));
  }

  public void testToSapBillBlockJGB()
  {
    SAPRechnungVO re = new SAPRechnungVO();
    re.jgbLaufId = 1;
    assertEquals("Z3", Convert.toSapBillBlock(re));
  }

  public void testToSapBillBlockAMV()
  {
    SAPRechnungVO re = new SAPRechnungVO();
    re.amvLaufId = 1;
    assertEquals("Z4", Convert.toSapBillBlock(re));
  }

  public void testToSapRegMail()
  {
    assertEquals(" ", Convert.toSapRegMail(null));
    assertEquals(" ", Convert.toSapRegMail("0"));
    assertEquals("X", Convert.toSapRegMail("1"));
  }


  public void testToSapMessageType()
  {
    assertEquals(Convert.SAP_MESSAGE_TYPE_ESR,
                 Convert.toSapMessageType(new SAPRechnungVO(),null));
  }

  public void testToSapMessageTypeCopy()
  {
    assertEquals(Convert.SAP_MESSAGE_TYPE_ESR_COPY,
                 Convert.toSapMessageTypeCopy(0));

    assertEquals(Convert.SAP_MESSAGE_TYPE_DEPKTO_COPY,
                 Convert.toSapMessageTypeCopy(1));
  }


  public void testToSapTargetQu()
  {
    assertEquals("ST",Convert.toSapTargetQu("ANZ"));
    assertEquals("STD",Convert.toSapTargetQu("STD"));
    assertEquals("ST",Convert.toSapTargetQu("STK"));
  }


  public void testToSapLangu()
  {
    assertEquals("D",Convert.toSapLangu(null));
    assertEquals("D",Convert.toSapLangu("D"));
    assertEquals("F",Convert.toSapLangu("F"));
    assertEquals("E",Convert.toSapLangu("E"));
    assertEquals("I",Convert.toSapLangu("I"));
    assertEquals("E",Convert.toSapLangu("USA"));
  }


  public void testToSapDispatchTime()
  {
    assertEquals("4",Convert.toSapDispatchTime("1"));
    assertEquals("3",Convert.toSapDispatchTime("2"));
    assertEquals("3",Convert.toSapDispatchTime("3"));
    assertEquals("3",Convert.toSapDispatchTime("4"));
    assertEquals("4",Convert.toSapDispatchTime("5"));
  }


  public void testToSapListMode()
  {
    assertEquals("0",Convert.toSapListMode(null));
    assertEquals("2",Convert.toSapListMode("2"));
    assertEquals("6",Convert.toSapListMode("6"));
    assertEquals("7",Convert.toSapListMode("7"));
  }


  public void testToSapPartnerRole()
  {
    assertEquals("R0",Convert.toSapPartnerRole(0));
    assertEquals("R7",Convert.toSapPartnerRole(7));
    assertEquals("R9",Convert.toSapPartnerRole(9));
    assertEquals("Z0",Convert.toSapPartnerRole(10));
    assertEquals("Z4",Convert.toSapPartnerRole(14));
    assertEquals("Z9",Convert.toSapPartnerRole(19));
  }


  public void testToSapReasonRej()
  {
    assertEquals("Z1",Convert.toSapReasonRej("9"));
    assertEquals("Z2",Convert.toSapReasonRej("A"));
  }


  public void testToBagisStatusDruck()
  {
    assertEquals("9",Convert.toBagisStatusDruck("1"));
  }


  public void testToBagisStatusZahlg()
  {
    assertEquals("4",Convert.toBagisStatusZahlg("2"));
    assertEquals("2",Convert.toBagisStatusZahlg("3"));
    assertEquals("9",Convert.toBagisStatusZahlg("4"));
    assertEquals("A",Convert.toBagisStatusZahlg("5"));
  }


  public void testToSapCustGrp1()
  {
    assertEquals(" ",Convert.toSapCustGrp1(null));
    assertEquals("001",Convert.toSapCustGrp1("001"));
  }


} // end class

