package ch.ige.ses.exception;

/**
 * SAP Call Exception
 * Kappselt alle möglichen Fehlermeldungen die bei einem SAP Call
 * entstehen können. Dies beinhaltet auch alle im Zusammenhang mit der
 * Kommunikation möglichen Fehlerfälle.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: SapCallException.java,v 1.4 2006/12/11 12:43:36 iten Exp $
 */
public class SapCallException extends Exception
{
  private String  fUserMsg    = null;
  private String  fDetailMsg  = null;


  /**
   * Constructs an SapRequestException with no specified detail message.
   */
  public SapCallException()
  {
    super();
  }


  /**
   * Constructs an Exception with the specified detail message.
   *
   * @param s detail message
   */
  public SapCallException(String s)
  {
    super(s);
    fDetailMsg = s;
  }


  /**
   * Constructs an Exception with the specified detail message.
   *
   * @param userMessage user message
   * @param detailMessage detail message (i.e. exception text)
   */
  public SapCallException(String userMessage, String detailMessage)
  {
    super(userMessage + ": " + detailMessage);
    fUserMsg    = userMessage;
    fDetailMsg  = detailMessage;
  }


  /**
   * get user message only
   *
   * @return message or empty string if user message is null
   */
  public String getUserMessage()
  {
    return ( fUserMsg != null ? fUserMsg : "" );
  }


  /**
   * get detail message only
   *
   * @return message or empty string if detail message is null
   */
  public String getDetailMessage()
  {
    return ( fDetailMsg != null ? fDetailMsg : "" );
  }

} // end class

