package ch.ige.ses.server.bagisinbound;

import java.util.ArrayList;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;

import ch.ige.ses.server.dao.bagis.BfakturDAO;
import ch.ige.ses.vo.bagis.SAPRechnungStatusListVO;
import ch.ige.ses.vo.bagis.SAPRechnungStatusVO;

/**
 * SES: Bean zur Verarbeitung der Rückmeldungen zu Fakturaanforderungen aus SAP
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author   Pirmin Suter
 * @version  $Id: SESRechnungStatusBean.java,v 1.40 2007/01/10 14:46:37 iten Exp $
 * <p>
 * 
 * @ejb.bean 
 *  name="SESRechnungStatus"  
 *  display-name="SESRechnungStatus"
 *  view-type="local"
 *  jndi-name="SESRechnungStatus"
 *  local-jndi-name="SESRechnungStatusLocal"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="Required"
 */
public class SESRechnungStatusBean implements SessionBean
{
  //
  // const
  //
  private static final Logger LOG   = Logger.getLogger(SESRechnungStatusBean.class);

  //
  // fields
  //
  SessionContext sessionContext;

  //---------------------------------------------------------------------------
  // bean methods
  //---------------------------------------------------------------------------

  /**
   * create ejb
   */
  public void ejbCreate() 
  {
    // lazy bone
  }

  public void ejbRemove()
  {
    // lazy bone
  }

  public void ejbActivate()
  {
    // lazy bone
  }

  public void ejbPassivate()
  {
    // lazy bone
  }

  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }


  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * @ejb.interface-method
   */
  public boolean RechnungStatusUpdate(SAPRechnungStatusListVO rechnungListVO)
  {
    BfakturDAO            bfakturDAO = new BfakturDAO();
    SAPRechnungStatusVO   vo         = null;

    try
    {
      for (int i = 0;i < rechnungListVO.statusList.size() ;i++)
      {
        //------------------------------------------------------------
        // init
        //------------------------------------------------------------

        vo = (SAPRechnungStatusVO)rechnungListVO.statusList.get(i);

        LOG.debug ( "RechnungStatusUpdate - bfakt_id=" + vo.bfakt_id +
                    ",status_druck=" + vo.status_druck +
                    ",status_zahlg=" + vo.status_zahlg );

        //------------------------------------------------------------
        // update zahl status
        //------------------------------------------------------------

        if (vo.status_druck.equals(""))
        {
          // STATUS_ZAHLG_TYPES - value range
          // [0] : 2 = Storniert
          // [1] : 4 = Bezahlt
          // [2] : 9 = Annulliert
          // [3] : A = Fristablauf
          if(vo.status_zahlg.equals(SAPRechnungStatusVO.STATUS_ZAHLG_TYPES[0]))
          {
            LOG.debug("RechnungStatusUpdate - storniert");
            bfakturDAO.updateBfakturStorniert(vo);
          }
          else if (vo.status_zahlg.equals(SAPRechnungStatusVO.STATUS_ZAHLG_TYPES[1]))
          {
            LOG.debug("RechnungStatusUpdate - bezahlt");
            bfakturDAO.updateBfakturBezahlt(vo);
          }
          else if (vo.status_zahlg.equals(SAPRechnungStatusVO.STATUS_ZAHLG_TYPES[2]))
          {
            LOG.debug("RechnungStatusUpdate - annuliert");
            bfakturDAO.updateBfakturAnnuliert(vo);
          }
          else if (vo.status_zahlg.equals(SAPRechnungStatusVO.STATUS_ZAHLG_TYPES[3]))
          {
            if (vo.jgbLaufNr.equals(""))
            {
              if (bfakturDAO.selectKzFristAbgelaufen(vo.bfakt_id) == 2)
              {
                // fristablauf design (halbautomat)
                LOG.debug("RechnungStatusUpdate - fristablauf design (halbautomat)");
                bfakturDAO.updateBfakturAnnuliertJGB(vo);
              }
              else
              {
                // fristablauf (batch)
                LOG.debug("RechnungStatusUpdate - fristablauf (batch)");
                bfakturDAO.updateBfakturFristAbgelaufen(vo.bfakt_id);
              }
            }
            else
            {
              // fristablauf jgb
              LOG.debug("RechnungStatusUpdate - fristablauf jgb");
              bfakturDAO.updateBfakturAnnuliertJGB(vo);
            }
          }
        }

        //------------------------------------------------------------
        // update druck status
        //------------------------------------------------------------
        else
        {
          if (vo.jgbLaufNr.equals(""))
          {
            LOG.debug("RechnungStatusUpdate - Einzel/AMV Rechnung (Druckstatus & Datum) " +
                      "bfakt_id=" + vo.bfakt_id);
            bfakturDAO.updateBfakturGedruckt(vo);
          }
          else
          {
            LOG.debug("RechnungStatusUpdate - JGB Rechnung (Druckstatus) " +
                      "bfakt_id=" + vo.bfakt_id);
            bfakturDAO.updateBfakturGedrucktJGB(vo);
          }
        }

      } // end for

      //
      // end
      //
      LOG.debug("RechnungStatusUpdate - Status der BFakturen aktualisiert");
      return true;

    }
    catch (Exception ex)
    {
      LOG.warn("RechnungStatusUpdate - Exception: " + ex.getMessage());
      throw new EJBException(ex);
    }
  }


  /**
   * Setzt den Rechnungssatus für die angegebene Liste mit Rechnungsnummern
   * (bfakt_id's) auf den angegebenen Status.
   *
   * @param   list      Liste mit Rechnungsnummern (Integer)
   * @throws Exception  Unerwarteter Fehler
   * 
   * @ejb.interface-method
   */
  public void setRechnungStatusDruck (ArrayList list, String status) throws Exception
  {
    BfakturDAO dao = new BfakturDAO();


    // check for invalid list
    if (list == null)
    {
      LOG.warn("setRechnungStatusDruck - Aufruf mit Liste = NULL !");
      return;
    }

    // check for empty list
    if (list.size() == 0)
    {
      LOG.warn("setRechnungStatusDruck - Aufruf mit Liste.size = 0 !");
      return;
    }

    // do update
    LOG.debug ( "setRechnungStatusDruck - start (list.size=" + list.size() +
                ", status=" + status + ")...");

    for (Iterator iter = list.iterator(); iter.hasNext(); )
    {
      Integer item = (Integer)iter.next();
      dao.updateBfakturDruckstatus(item.intValue(),status);
    }

    LOG.debug("setRechnungStatusDruck - successfully done");
  }

} // end class

