package ch.ige.ses.server.bagisoutbound;

/**
 * Exception für SES-Komponente
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Pirmin Suter
 * @version $Id: SESException.java,v 1.5 2006/12/11 12:43:32 iten Exp $
 */
public class SESException extends Exception
{
  //
  // const
  //
  private static final String   MASK_BEGIN  = "@@B";
  private static final String   MASK_END    = "E@@";

  public static final int       MASK        = 1;
  public static final int       UNMASK      = 2;

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public SESException(String message)
  {
    super(message);
  }

  public SESException(int mode, String message)
  {
    super( manipulate(mode,message) );
  }

  public SESException(int mode, String message, String messageHead)
  {
    super( messageHead + manipulate(mode,message) );
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private static String manipulate(int mode, String message)
  {
    switch (mode)
    {
      case MASK:
        return mask(message);

      case UNMASK:
        return unmask(message);

      default:
        return message;
    }
  }

  private static String mask (String message)
  {
    return (MASK_BEGIN + message + MASK_END);
  }

  private static String unmask (String message)
  {
    int startIdx = message.indexOf(MASK_BEGIN);
    int endIdx   = message.indexOf(MASK_END);

    if ( (startIdx > -1) && (endIdx > -1) )
    {
      return message.substring(startIdx + MASK_BEGIN.length(),endIdx);
    }
    else
    {
      return message;
    }
  }

} // end class


