package ch.ige.ses.server.bagisoutbound;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.ses.util.SESConstants;

/**
 * Dient zum Lesen der JMS Queue SES.jgbStartQueue
 * <p>
 * Copyright: Copyright (c) 2001-2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Pirmin Suter
 * @version $Id: SESJGBStartMDBBean.java,v 1.36 2007/01/10 18:21:18 iten Exp $
 * 
 * @ejb.bean name="SESJGBStartMDB" 
 *  destination-type="javax.jms.Queue"
 *  transaction-type="Container"
 *  destination-jndi-name="jms/SES.jgbStartQueue"
 *  connection-factory-jndi-name="jms/IGEQueueConnectionFactory" 
 * @jboss.destination-jndi-name name="jms/SES.jgbStartQueue" 
 * @ejb.transaction 
 *  type="NotSupported"
 * @ejb.ejb-ref
 *   ejb-name="SESJgbBagisRead"
 *   view-type="local"
 * @ejb.ejb-ref
 *   ejb-name="SESJgbWork"
 *   view-type="local"
 * @ejb.ejb-ref
 *   ejb-name="SapInboundCtrl"
 *   view-type="local"
 */
public class SESJGBStartMDBBean implements MessageDrivenBean, MessageListener
{
  //
  // const
  //
  private static final Logger LOG = Logger.getLogger(SESJGBStartMDBBean.class);


  //
  // fields
  //
  MessageDrivenContext messageDrivenContext;

  SESJgbBagisReadLocalHome sesJgbBagisReadLocalHome = null;
  SESJgbBagisReadLocal sesJgbBagisReadLocal         = null;

  SESJgbWorkLocalHome sesJgbWorkLocalHome           = null;
  SESJgbWorkLocal sesJgbWorkLocal                   = null;

  //---------------------------------------------------------------------------
  // bean methods
  //---------------------------------------------------------------------------

  /**
   * Holt mittels Lookup die HomeInterfaces und instanziert die
   * RemoteInterfaces der beiden Beans SESJgbBagisRead und SESJgbWork
   *
   */
  public void ejbCreate()
  {
    try
    {
      Context ctx = new InitialContext();

      Object ref = ctx.lookup(sesJgbBagisReadLocalHome.COMP_NAME);
      sesJgbBagisReadLocalHome = (SESJgbBagisReadLocalHome) ref;
      sesJgbBagisReadLocal = sesJgbBagisReadLocalHome.create();

      ref = ctx.lookup(sesJgbWorkLocalHome.COMP_NAME);
      sesJgbWorkLocalHome = (SESJgbWorkLocalHome) ref;
      sesJgbWorkLocal = sesJgbWorkLocalHome.create();

    }
    catch (CreateException ex)
    {
      LOG.error("ejbCreate - CreateException : " + ex.getMessage());
    }
    catch (NamingException ex)
    {
      LOG.error("ejbCreate - NamingException: " + ex.getMessage());
    }
  }


  public void ejbRemove()
  {
    // lazy bone
  }


  public void setMessageDrivenContext(MessageDrivenContext messageDrivenContext)
  {
    this.messageDrivenContext = messageDrivenContext;
  }

  //---------------------------------------------------------------------------
  // onMessage
  //---------------------------------------------------------------------------

  /**
   * Methode onMessage instanziert das BusinessObject SESJgbBagisBO
   * und übergibt die Referenzen der Beans SESJgbBagisRead SESJgbWork und
   * SESErrorProcess. Je nach Message wird eine andere Methode aufgerufen.
   *
   * @param msg Message als String
   */
  public void onMessage(Message msg)
  {
    SESJgbBagisBO           bo      = null;
    javax.jms.TextMessage   message = null;

    try
    {
      if ( msg instanceof javax.jms.TextMessage )
      {
        bo = new SESJgbBagisBO ( sesJgbBagisReadLocal,
                                 sesJgbWorkLocal );

        message = (TextMessage) msg;

        if ( message.getText().equals(SESConstants.TM_JGB_START) )
        {
          LOG.info("onMessage - TM_JGB_START");
          bo.start();
        }
        else if ( message.getText().equals(SESConstants.TM_JGB_TEST) )
        {
          LOG.info("onMessage - TM_JGB_TEST");
          bo.start ( message.getIntProperty("minBFaktId"),
                     message.getIntProperty("maxBFaktId") );
        }
        else if (message.getText().equals(SESConstants.TM_JGB_STOP))
        {
          LOG.info("onMessage - TM_JGB_STOP");
          bo.stop();
        }
        else if (message.getText().equals(SESConstants.TM_JGB_RESTART))
        {
          LOG.info("onMessage - TM_JGB_RESTART");

          if ( message.getIntProperty("minBFaktId") == 0 &&
               message.getIntProperty("maxBFaktId") == 0 )
          {
            bo.restart();
          }
          else
          {
            bo.restart ( message.getIntProperty("minBFaktId"),
                         message.getIntProperty("maxBFaktId") );
          }
        }
        else
        {
          LOG.error ( "onMessage - Ungültige TextMessage empfangen " +
                      "(serial://jms/SES.jgbStartQueue)" );
        }
      }
      else
      {
        LOG.error( "onMessage - Ungültige TextTyp empfangen " +
                   "(serial://jms/SES.jgbStartQueue)" );
      }
    }
    catch (Exception ex)
    {
      LOG.fatal( "onMessage - Ein unerwartete Fehler ist aufgetreten " +
                 "Exception : " + ex.getMessage() );

      throw new EJBException(ex);
    }
  }

} // end class


