package ch.ige.ses.server.dao.ses.test;

import java.sql.*;
import ch.ige.ses.server.dao.ses.*;
import ch.ige.ses.vo.ses.*;
import junit.framework.*;

/**
* Testklasse für die Klasse AmvLaufDAO
* <p>
* Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum</p>
* @author T. Iten
* @version $Id: AmvLaufDAOTest.java,v 1.8 2007/06/04 10:17:52 iten Exp $
*/
public class AmvLaufDAOTest extends SesTestCase
{
  //
  // const
  //
  private static final  String  LAUF_NR           = "2999061800";
  private static final  int     LAUF_STAT_INSERT  = AmvLaufVO.LAUF_STAT_STARTED;
  private static final  int     LAUF_STAT_UPDATE  = AmvLaufVO.LAUF_STAT_FINISHED;

  //
  // fields
  //
  public AmvLaufDAO dao = new AmvLaufDAO();
  public AmvLaufVO  vo  = null;


  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------

  public AmvLaufDAOTest(String s)
  {
    super(s);
  }

  //---------------------------------------------------------------------------
  // init / clean-up
  //---------------------------------------------------------------------------

  public static Test suite()
  {
     return new TestSuite(AmvLaufDAOTest.class);
  }

  protected void setUp()
  {
    // lazy bone
  }

  protected void tearDown()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // tests
  //---------------------------------------------------------------------------

  public void test01SelectEmpty() throws SQLException
  {
    try
    {
      vo = dao.select(LAUF_NR);
      assertNull(vo);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void test02Insert() throws SQLException
  {
    try
    {
      vo = new AmvLaufVO();
      vo.laufNr   = LAUF_NR;
      vo.laufStat = LAUF_STAT_INSERT;

      dao.insert(vo);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void test03SelectByLaufNr() throws SQLException
  {
    try
    {
      vo = dao.select(LAUF_NR);

      assertEquals("amvLaufId > 0", true, (vo.amvLaufId > 0));
      assertEquals("amvLaufNr == " + LAUF_NR, LAUF_NR, vo.laufNr);
      assertEquals("laufStat == " + LAUF_STAT_INSERT, LAUF_STAT_INSERT, vo.laufStat);
      assertNotNull("createdDat",vo.createdDat);
      assertNotNull("changedDat",vo.changedDat);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void test04SelectByAmvLaufId() throws SQLException
  {
    try
    {
      vo = dao.select(LAUF_NR);
      vo = dao.select(vo.amvLaufId);
      assertNotNull(vo);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void test05SelectCount() throws SQLException
  {
    try
    {
      int cnt = dao.selectCount();
      assertTrue(cnt > 0);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void test06SelectMaxLaufID() throws SQLException
  {
    try
    {
      // select max id
      int id = dao.selectMaxLaufID();

      // get last id
      vo = dao.select(LAUF_NR);
      vo = dao.select(vo.amvLaufId);

      // compare
      assertEquals("max id == last amvLaufId", id, vo.amvLaufId);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void test07UpdateLaufStat() throws SQLException
  {
    try
    {
      vo = dao.select(LAUF_NR);
      int affectedRows = dao.updateLaufStat(vo.amvLaufId,LAUF_STAT_UPDATE);
      assertEquals("affectedRows == 1", true, (affectedRows == 1));

      vo = dao.select(LAUF_NR);
      assertEquals("status == " + LAUF_STAT_UPDATE, LAUF_STAT_UPDATE, vo.laufStat);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void test08Delete() throws SQLException
  {
    try
    {
      vo = dao.select(LAUF_NR);
      int affectedRows = dao.delete(vo.amvLaufId);
      assertEquals("affectedRows == 1", true, (affectedRows == 1));
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      // ignore 
      // fail(ex.toString());
    }
  }



  //***************************************************************************
  // main
  //***************************************************************************

  public static void main(String[] args)
  {
    junit.textui.TestRunner.run(suite());
  }

} // end class

