package ch.ige.ses.server.dao.ses.test;

import java.sql.*;
import java.util.*;


import ch.ige.ses.client.sapinbound.*;
import ch.ige.ses.server.dao.ses.*;
import ch.ige.ses.vo.ses.*;
import ch.ige.util.server.*;
import ch.ige.ses.vo.bagis.*;
import junit.framework.*;
import java.io.*;


/**
 * <p>Test der Klasse ErrorSESDAO</p>
 * <p>Beschreibung: </p>
 * <p>Copyright: Copyright (c) Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author A.Rueegsegger
 * @version $Id: ErrorDAOTest.java,v 1.8 2007/01/04 15:58:08 iten Exp $
 */

public class ErrorDAOTest extends SesTestCase
{
  //
  // fields
  //
  String    blobChunk[] = new String[10000];        // data used for blob test

  ErrorVO   vo1   = new ErrorVO();                  // initialised by setUp()
  ErrorVO   vo2   = new ErrorVO();
  ErrorVO   vo3   = new ErrorVO();
  ErrorVO   vo4   = new ErrorVO();
  ErrorVO   vo5   = new ErrorVO();
  ErrorVO   vo6   = new ErrorVO();
  ErrorVO   vo7   = new ErrorVO();
  ErrorVO   vo8   = new ErrorVO();
  ErrorVO   vo9   = new ErrorVO();
  ErrorVO   vo10  = new ErrorVO();
  ErrorVO   vo11  = new ErrorVO();

  ErrorDAO  dao   = new ErrorDAO();                 // used by methods
  ErrorVO   vo    = new ErrorVO();

  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------

  public ErrorDAOTest(String s)
  {
    super(s);
  }


  //---------------------------------------------------------------------------
  // suot / setUp / tearDown
  //---------------------------------------------------------------------------

  public static Test suite()
  {
     return new TestSuite(ErrorDAOTest.class);
  }

  protected void setUp()
  {
    //
    // init blob data
    //
    for (int i = 0; i < blobChunk.length; i++)
    {
      blobChunk[i] = "Test String #" + i;
    }

    //
    // init vo's
    // errorID will be set after successfully insert
    //

    // vo #1
    vo1.source          = "ErrorDAOTest";
    vo1.employeeId      = "1314";
    vo1.employeeNotify  = ErrorVO.EMPL_NOTIFY_BENACHRICHTIGT;
    vo1.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_DEFAULT;
    vo1.description     = "Test Fehler #01";
    vo1.dataTypeCd      = ErrorVO.DATA_TYPE_CD_NO_DATA;
    vo1.data            = null;
    vo1.errorStat       = ErrorVO.ERROR_STAT_PENDENT;

    // vo #2
    vo2.source          = "ErrorDAOTest";
    vo2.employeeId      = "1314";
    vo2.employeeNotify  = ErrorVO.EMPL_NOTIFY_KEINE;
    vo2.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_MAX_EMPF;
    vo2.description     = "Test Fehler #02";
    vo2.dataTypeCd      = ErrorVO.DATA_TYPE_CD_STRING;
    vo2.data            = "Dies ist ein Test";
    vo2.errorStat       = ErrorVO.ERROR_STAT_PENDENT;

    // vo #3
    vo3.source          = "ErrorDAOTest";
    vo3.employeeId      = "1314";
    vo3.employeeNotify  = ErrorVO.EMPL_NOTIFY_PENDENT;
    vo3.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_DEFAULT;
    vo3.description     = "Test Fehler #03";
    vo3.dataTypeCd      = ErrorVO.DATA_TYPE_CD_SAP_RECHNUNG_VO;
    vo3.data            = TestData.createRechnung(112233,5,5);
    vo3.errorStat       = ErrorVO.ERROR_STAT_PENDENT;

    // vo #4
    vo4.source          = "ErrorDAOTest";
    vo4.employeeId      = "1314";
    vo4.employeeNotify  = ErrorVO.EMPL_NOTIFY_PENDENT;
    vo4.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_DEFAULT;
    vo4.description     = "Test Fehler #04";
    vo4.dataTypeCd      = ErrorVO.DATA_TYPE_CD_SAP_RECHNUNG_ANNULATION_VO;
    vo4.data            = TestData.createAnnullation(112233);
    vo4.errorStat       = ErrorVO.ERROR_STAT_PENDENT;

    // vo #5
    vo5.source          = "ErrorDAOTest";
    vo5.employeeId      = "1314";
    vo5.employeeNotify  = ErrorVO.EMPL_NOTIFY_PENDENT;
    vo5.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_DEFAULT;
    vo5.description     = "Test Fehler #05";
    vo5.dataTypeCd      = ErrorVO.DATA_TYPE_CD_SAP_RECHNUNG_STATUS_LIST_VO;
    vo5.data            = TestData.createStatusListe(5);
    vo5.errorStat       = ErrorVO.ERROR_STAT_PENDENT;

    // vo #6
    vo6.source          = "ErrorDAOTest";
    vo6.employeeId      = "1314";
    vo6.employeeNotify  = ErrorVO.EMPL_NOTIFY_PENDENT;
    vo6.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_DEFAULT;
    vo6.description     = "Test Fehler #06";
    vo6.dataTypeCd      = ErrorVO.DATA_TYPE_CD_SAP_JGB_JOB_VO;
    vo6.data            = TestData.createJGBJob(23564);
    vo6.errorStat       = ErrorVO.ERROR_STAT_PENDENT;

    // vo #7
    vo7.source          = "ErrorDAOTest";
    vo7.employeeId      = "1314";
    vo7.employeeNotify  = ErrorVO.EMPL_NOTIFY_PENDENT;
    vo7.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_DEFAULT;
    vo7.description     = "Test Fehler #07";
    vo7.dataTypeCd      = ErrorVO.DATA_TYPE_CD_SAP_VERS_ADR_VO;
    vo7.data            = TestData.createAdresse(112233);
    vo7.errorStat       = ErrorVO.ERROR_STAT_PENDENT;

    // vo #8
    vo8.source          = "ErrorDAOTest";
    vo8.employeeId      = "1314";
    vo8.employeeNotify  = ErrorVO.EMPL_NOTIFY_PENDENT;
    vo8.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_DEFAULT;
    vo8.description     = "Test Fehler #08";
    vo8.dataTypeCd      = ErrorVO.DATA_TYPE_CD_SAP_ADR_STATUS_VO;
    vo8.data            = TestData.createAdrStatus(21541);
    vo8.errorStat       = ErrorVO.ERROR_STAT_PENDENT;

    // vo #9
    vo9.source          = "ErrorDAOTest";
    vo9.employeeId      = "1314";
    vo9.employeeNotify  = ErrorVO.EMPL_NOTIFY_PENDENT;
    vo9.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_DEFAULT;
    vo9.description     = "Test Fehler #09";
    vo9.dataTypeCd      = ErrorVO.DATA_TYPE_CD_STAKO_VO;
    vo9.data            = TestData.createStako();
    vo9.errorStat       = ErrorVO.ERROR_STAT_PENDENT;

    // vo #10
    vo10.source          = "ErrorDAOTest";
    vo10.employeeId      = "1314";
    vo10.employeeNotify  = ErrorVO.EMPL_NOTIFY_PENDENT;
    vo10.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_DEFAULT;
    vo10.description     = "Test Fehler #10";
    vo10.dataTypeCd      = ErrorVO.DATA_TYPE_CD_SAP_STAKO_JOB_VO;
    vo10.data            = TestData.createStakoJob();
    vo10.errorStat       = ErrorVO.ERROR_STAT_PENDENT;

    // vo #11 mit BLOB Daten > 4 kb
    // ACHTUNG: dieses VO kann mit dem SES-Monitor nicht angeschaut werden!
    vo11.source          = "ErrorDAOTest-BLOB";
    vo11.employeeId      = "1284";
    vo11.employeeNotify  = ErrorVO.EMPL_NOTIFY_PENDENT;
    vo11.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_DEFAULT;
    vo11.description     = "Test Fehler #11";
    vo11.dataTypeCd      = ErrorVO.DATA_TYPE_CD_STRING;
    vo11.data            = blobChunk;
    vo11.errorStat       = ErrorVO.ERROR_STAT_PENDENT;
  }


  protected void tearDown()
  {
    // not used
  }

  //---------------------------------------------------------------------------
  // tests
  //---------------------------------------------------------------------------


  public void test01Insert() throws Exception
  {
    int lastId = this.selectMaxErrorId();

    // insert
    dao.insert(vo1);
    vo1.errorId = this.selectMaxErrorId();

    dao.insert(vo2);
    vo2.errorId = this.selectMaxErrorId();

    dao.insert(vo3);
    vo3.errorId = this.selectMaxErrorId();

    dao.insert(vo4);
    vo4.errorId = this.selectMaxErrorId();

    dao.insert(vo5);
    vo5.errorId = this.selectMaxErrorId();

    dao.insert(vo6);
    vo6.errorId = this.selectMaxErrorId();

    dao.insert(vo7);
    vo7.errorId = this.selectMaxErrorId();

    dao.insert(vo8);
    vo8.errorId = this.selectMaxErrorId();

    dao.insert(vo9);
    vo9.errorId = this.selectMaxErrorId();

    dao.insert(vo10);
    vo10.errorId = this.selectMaxErrorId();

    dao.insert(vo11);
    vo11.errorId = this.selectMaxErrorId();

    // check
    assertTrue("ErrorVO #01 eingefügt ", vo1.errorId != lastId);
    assertTrue("ErrorVO #02 eingefügt ", vo2.errorId == vo1.errorId + 1);
    assertTrue("ErrorVO #03 eingefügt ", vo3.errorId != lastId);
    assertTrue("ErrorVO #11 eingefügt ", vo11.errorId > 0);
  }



  public void test02SelectVo1() throws Exception
  {
    // init vo error id's
    setErrorIds();

    // select
    vo = dao.select(vo1.errorId);

    // check
    assertNotNull(vo);

    assertEquals("errorId: ",           vo.errorId,         vo1.errorId);
    assertEquals("source: ",            vo.source,          vo1.source);
    assertEquals("employeeId: ",        vo.employeeId,      vo1.employeeId);
    assertEquals("employeeNotify: ",    vo.employeeNotify,  vo1.employeeNotify);
    assertEquals("employee_msg_type: ", vo.employeeMsgType, vo1.employeeMsgType);
    assertEquals("description: ",       vo.description,     vo1.description);
    assertEquals("dataTypeCd: ",        vo.dataTypeCd,      vo1.dataTypeCd);
    assertEquals("errorStat: ",         vo.errorStat,       vo1.errorStat);

    // data field has to be null
    assertEquals(vo.data, null);
  }



  public void test03SelectVo3() throws Exception
  {
    // init vo error id's
    setErrorIds();

    // select
    vo = dao.select(vo3.errorId);

    // check
    assertNotNull(vo);

    assertEquals("errorId: ",           vo.errorId,         vo3.errorId);
    assertEquals("source: ",            vo.source,          vo3.source);
    assertEquals("employeeId: ",        vo.employeeId,      vo3.employeeId);
    assertEquals("employeeNotify: ",    vo.employeeNotify,  vo3.employeeNotify);
    assertEquals("employee_msg_type: ", vo.employeeMsgType, vo3.employeeMsgType);
    assertEquals("description: ",       vo.description,     vo3.description);
    assertEquals("dataTypeCd: ",        vo.dataTypeCd,      vo3.dataTypeCd);
    assertEquals("errorStat: ",         vo.errorStat,       vo3.errorStat);

    // data field
    SAPRechnungVO re  = (SAPRechnungVO)vo.data;
    SAPRechnungVO re3 = (SAPRechnungVO)vo3.data;

    assertEquals("bfaktid: ",     re.bfakt_id,              re3.bfakt_id);
    assertEquals("schutitNr: ",   re.schutit_nr,            re3.schutit_nr);
    assertEquals("rech empf: ",   re.zusatzRechEmpf.size(), re3.zusatzRechEmpf.size());
    assertEquals("rech pos: ",    re.posList.size(),        re3.posList.size());
  }


  /**
   * Test ob die Daten > 4KB auch wieder gelesen werden können
   *
   * @throws Exception
   */
  public void test04SelectVo11Blob() throws Exception
  {
    ErrorVO   errVO;
    String    buf[];
    int       bufLen = 0;


    // init vo error id's
    setErrorIds();

    // select
    errVO = dao.select(vo11.errorId);

    // check
    assertNotNull(vo);
    buf = (String[])errVO.data;

    assertEquals(buf.length, blobChunk.length);
    assertEquals(buf[5], "Test String #5");
    assertEquals(buf[50], "Test String #50");
    assertEquals(buf[500], "Test String #500");

    // show length

    for (int i = 0; i < buf.length; i++)
    {
      bufLen = bufLen + buf[i].length();
    }
    System.out.println("test12SelectVo11Blob - bufLen=" + bufLen);
  }



  public void _test05SelectAll() throws Exception
  {
    ArrayList list;

    // select all
    list = dao.selectAll();

    // check
    assertTrue("list size: ", list.size() >= 11);
  }



  public void test06UpdateVo3() throws Exception
  {
    SAPRechnungVO re,re2;
    int           ret;

    // init vo error id's
    setErrorIds();
    re   = (SAPRechnungVO) vo3.data;

    // modifiy some data
    re.bfakt_id++;
    re.unsere_ref = "updated reference";

    // update
    ret = dao.update(vo3);

    // check
    assertEquals(ret,1);

    vo = dao.select(vo3.errorId);
    re2= (SAPRechnungVO) vo.data;
    assertEquals(re.bfakt_id,re2.bfakt_id);
    assertEquals(re.unsere_ref,re2.unsere_ref);
  }

  public void test07UpdateVo11Blob() throws Exception
  {
    String[]  b1,b2;
    int       ret;

    // init vo error id's
    setErrorIds();
    b1 = (String[]) vo11.data;

    // modifiy some data
    b1[1] = b1[1] + " updated";
    b1[b1.length-1] = b1[b1.length-1] + " updated to";

    // update
    ret = dao.update(vo11);

    // check
    assertEquals(ret,1);

    vo = dao.select(vo11.errorId);
    b2 = (String[]) vo.data;
    assertEquals(b1[1],b2[1]);
    assertEquals(b1[b1.length-1],b2[b2.length-1]);
  }


  public void test08Delete() throws Exception
  {
    // init vo error id's
    setErrorIds();

    // delete
    dao.delete(vo1.errorId);
    dao.delete(vo2.errorId);
    dao.delete(vo3.errorId);
    dao.delete(vo4.errorId);
    dao.delete(vo5.errorId);
    dao.delete(vo6.errorId);
    dao.delete(vo7.errorId);
    dao.delete(vo8.errorId);
    dao.delete(vo9.errorId);
    dao.delete(vo10.errorId);
    dao.delete(vo11.errorId);
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  /**
   * Gibt die zuletzt eingefügte error_id zurück
   * @return int
   */
  private int selectMaxErrorId()
  {
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    int max_value = 0;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement("SELECT MAX(error_id) max_value from ses.error");
      rs = ps.executeQuery();

      if(rs.next())
      {
        max_value = rs.getInt("max_value");
      }
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
    return max_value;
  }


  /**
   * setzt die Error Id's für die VO's 1-11
   */
  private void setErrorIds()
  {
    int   lastId  = this.selectMaxErrorId();

    vo11.errorId  = lastId--;
    vo10.errorId  = lastId--;
    vo9.errorId   = lastId--;
    vo8.errorId   = lastId--;
    vo7.errorId   = lastId--;
    vo6.errorId   = lastId--;
    vo5.errorId   = lastId--;
    vo4.errorId   = lastId--;
    vo3.errorId   = lastId--;
    vo2.errorId   = lastId--;
    vo1.errorId   = lastId;
  }


  //***************************************************************************
  // main
  //***************************************************************************

  public static void main(String[] args)
  {
    junit.textui.TestRunner.run(suite());
  }

} // end class

