package ch.ige.ses.server.error;

import org.apache.log4j.Logger;

import ch.ige.ses.BuildInfo;
import ch.ige.ses.util.SESConstants;
import ch.ige.ses.vo.bagis.SAPRechnungVO;
import ch.ige.ses.vo.bagis.SAPRechnungsPositionVO;
import ch.ige.ses.vo.ses.ErrorVO;

/**
 * <p>Überschrift: SESErrorParseMail</p>
 * <p>Beschreibung: Ersetzt Platzhalter in den Mailkomponenten Subject,
 * Kopftext und Fusstext</p>
 * <p>Copyright: Copyright (c) 2001-2003, Eidgenössisches Institut für Geistiges Eigentum</p>
 * <p>Organisation: Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author  Stefan Nydegger
 * @version $Id: SESErrorParseMail.java,v 1.15 2007/01/19 14:28:37 iten Exp $
 */
public class SESErrorParseMail
{

  private static final Logger LOG = Logger.getLogger(SESErrorParseMail.class);


  /**
   * create SESErrorParseMail
   */  
  public SESErrorParseMail()
  {
    // lazy bone
  }

  /**
   * Die Methode parseMaxEmpfSubject ersetzt die Platzhalter bfakt_id und schutit_nr
   * mit den entsprechenden Werten des sapRechnungVO
   * @param subject       Titel des Mails
   * @param sapRechnungVO Value Object mit den Daten einer Bfaktur
   * @return subject als String
   */
  public static String parseMaxEmpfSubject(String subject, SAPRechnungVO sapRechnungVO)
  {
    try
    {
      StringBuffer bufSubject = new StringBuffer(subject);
      int posBfaktID = subject.indexOf(SESConstants.ERROR_MSG_BFAKTID);
      int posSchutitNr = subject.indexOf(SESConstants.ERROR_MSG_SCHUTIT_NR);
      //schutit_nr einsetzen
      bufSubject = bufSubject.replace(posSchutitNr, posSchutitNr + SESConstants.ERROR_MSG_SCHUTIT_NR.length(),
                                      "" + sapRechnungVO.schutit_nr);
      //bfakt_id einsetzen
      bufSubject = bufSubject.replace(posBfaktID, posBfaktID + SESConstants.ERROR_MSG_BFAKTID.length(),
                                      "" + sapRechnungVO.bfakt_id);
      return bufSubject.toString();
    }
    catch (Exception ex)
    {
      LOG.error("parseMaxEmpfSubject - unerwarteter Fehler. Exception=" + ex.getMessage());
      return subject;
    }
  }

  /**
   * Die Methode parseFussText ersetzt die Platzhalter Umgebung und employeeID
   * mit den entsprechenden Werten des errorVO und der SystemProperties
   * @param fussText      fussText des Mails
   * @param errorVO       Value Object mit den Daten des ErrorVO
   * @return subject als String
   */
  public static String parseFussText(String fussText, ErrorVO errorVO)
  {
    StringBuffer  bufFussText = new StringBuffer();
    int           pos         = 0;


    //
    // init
    //
    if (fussText == null)
    {
      LOG.error("parseFussText - fussText ist null");
      return "";
    }
    else
    {
      bufFussText.append( fussText );
    }

    //
    // Umgebung einsetzen
    //
    try
    {
      pos = fussText.indexOf(SESConstants.ERROR_MSG_UMGEBUNG);

      bufFussText = bufFussText.replace (pos, pos + SESConstants.ERROR_MSG_UMGEBUNG.length(),
                                         BuildInfo.BUILD_ENV_NAME );
    }
    catch (Exception ex)
    {
      LOG.error("parseFussText - Umgebung einsetzen, unerwarteter Fehler. Exception=" + ex.getMessage());
    }


    //
    // EmployeeID einsetzen
    //
    try
    {
      String employeeId = null;

      pos = fussText.indexOf(SESConstants.ERROR_MSG_EMPLOYEEID);

      if ( errorVO.employeeId == null )
      {
        employeeId = "keine vorhanden";
      }
      else
      {
        employeeId = errorVO.employeeId;
      }

      bufFussText = bufFussText.replace (pos, pos + SESConstants.ERROR_MSG_EMPLOYEEID.length(),
                                         employeeId);
    }
    catch (Exception ex)
    {
      LOG.error("parseFussText - EmployeeID einsetzen, unerwarteter Fehler. Exception=" + ex.getMessage());
    }

    //
    // end
    //
    return bufFussText.toString();
  }


  /**
   * Die Methode parseMaxEmpfKopfText ersetzt die Platzhalter empfaenger und
   * zusatz_empfaenger mit den adr_nr, welche die Rechnung erhalten habe und mit
   * den adr_nr, welche die Rechnung nicht erhalten haben
   * @param kopfText      kopfText des Mails
   * @param sapRechnungVO Value Object mit den Daten einer Bfaktur
   * @return kopfText als String
   */
  public static String parseMaxEmpfKopfText(String kopfText, SAPRechnungVO sapRechnungVO)
  {
    try
    {
      StringBuffer bufKopfText = null;
      bufKopfText = new StringBuffer(kopfText);
      StringBuffer artikel = new StringBuffer();
      StringBuffer empfaenger = new StringBuffer();
      StringBuffer empfaengerZusatz = new StringBuffer();
      //System.out.println(sapRechnungVO.toString());
      int posArtikel = kopfText.indexOf(SESConstants.ERROR_MSG_ARTIKEL);
      int posEmpf = kopfText.indexOf(SESConstants.ERROR_MSG_EMPFAENGER);
      int posZusEmpf = kopfText.indexOf(SESConstants.ERROR_MSG_ZUSATZ_EMPFAENGER);

      //art_nr und kurztext zusammenfügen
      for (int i = 0; i < sapRechnungVO.posList.size(); i++)
      {
        artikel.append(((SAPRechnungsPositionVO)sapRechnungVO.posList.get(i)).art_nr + " ");
        artikel.append(((SAPRechnungsPositionVO)sapRechnungVO.posList.get(i)).artKurztext + "\n");
      }
      // Hauptrechnungsempfänger anfügen
      empfaenger.append(sapRechnungVO.hauptRechEmpf + "\n");
      //Einfügen der Empfänger, welche die Rechnung erhalten haben
      for (int i = 0; i < sapRechnungVO.zusatzRechEmpf.size(); i++)
      {
        empfaenger.append(sapRechnungVO.zusatzRechEmpf.get(i) + "\n");
      }
      //Empfänger hinzufügen, welche die Rechnung nicht erhalten haben
      for (int i = 0; i < sapRechnungVO.zusatzRechEmpfNichtVerarbeitet.size(); i++)
      {
        empfaengerZusatz.append(sapRechnungVO.zusatzRechEmpfNichtVerarbeitet.get(i) + "\n");
      }
      //kopftext zusammensetzen
      bufKopfText = bufKopfText.replace(posZusEmpf, posZusEmpf + SESConstants.ERROR_MSG_ZUSATZ_EMPFAENGER.length(),
                                        empfaengerZusatz.toString());
      bufKopfText = bufKopfText.replace(posEmpf, posEmpf + SESConstants.ERROR_MSG_EMPFAENGER.length(),
                                        empfaenger.toString());
      bufKopfText = bufKopfText.replace(posArtikel, posArtikel + SESConstants.ERROR_MSG_ARTIKEL.length(),
                                        artikel.toString());
      return bufKopfText.toString();
    }
    catch (Exception ex)
    {
      LOG.error("parseMaxEmpfKopfText -  unerwarteter Fehler. Exception=" + ex.getMessage());
      return kopfText;
    }
  }

  /**
   * Die Methode parseAdrKopfText ersetzt den Platzhalter adrNr
   * mit dem entsprechenden
   * @param kopfText      Kopftext des Mails
   * @param adrNr         adrNr Adresse Nummer der fehlerhaften Adresse
   * @return subject als String
   */
  public static String parseAdrKopfText(String kopfText, String adrNr)
  {
    try
    {
      StringBuffer bufKopfText = new StringBuffer(kopfText);
      int posAdrNr = kopfText.indexOf(SESConstants.ERROR_MSG_ADR_NR);
      //adrNr einsetzen
      bufKopfText = bufKopfText.replace(posAdrNr, posAdrNr + SESConstants.ERROR_MSG_ADR_NR.length(), adrNr);
      return bufKopfText.toString();
    }
    catch (Exception ex)
    {
      LOG.error("parseAdrKopfText - unerwarteter Fehler. Exception=" + ex.getMessage());
      return kopfText;
    }
  }

  /**
   * Die Methode parseRechnungKopfText ersetzt den Platzhalter bfaktId
   * mit der entsprechenden bfaktId aus dem SAPRechnungsVO
   * @param kopfText      Kopftext des Mails
   * @param bfaktId       bfaktId der fehlerhaften Rechnung
   * @return subject als String
   */
  public static String parseRechnungKopfText(String kopfText, String bfaktId)
  {
    try
    {
      StringBuffer bufKopfText = new StringBuffer(kopfText);
      int posBfaktId = kopfText.indexOf(SESConstants.ERROR_MSG_BFAKTID);
      // bfakt_id einsetzen
      bufKopfText = bufKopfText.replace(posBfaktId, posBfaktId + SESConstants.ERROR_MSG_BFAKTID.length(), bfaktId);
      return bufKopfText.toString();
    }
    catch (Exception ex)
    {
      LOG.error("parseRechnungKopfText - unerwarteter Fehler. Exception=" + ex.getMessage());
      return kopfText;
    }
  }
}
