package ch.ige.ses.server.sapctrl;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.ses.server.bagisinbound.SESRechnungStatusLookup;
import ch.ige.ses.server.dao.ses.BfakturSESDAO;
import ch.ige.ses.server.dao.ses.TraceDAO;
import ch.ige.ses.server.error.SESErrorProcessLocal;
import ch.ige.ses.server.error.SESErrorProcessLocalHome;
import ch.ige.ses.vo.bagis.SAPRechnungStatusListVO;
import ch.ige.ses.vo.bagis.SAPRechnungStatusVO;
import ch.ige.ses.vo.ses.ErrorVO;
import ch.ige.ses.vo.ses.TraceVO;

/**
 * Speichert Meldungen vom SAP Inbound / Outbound Service in die SES DB
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: SapReceiveBean.java,v 1.25 2007/01/10 14:46:37 iten Exp $
 * 
 * @ejb.bean 
 *  name="SapReceive"  
 *  display-name="SapReceive"
 *  view-type="local"
 *  jndi-name="SapReceive"
 *  local-jndi-name="SapReceiveLocal"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="Required"
 * @ejb.ejb-ref
 *   ejb-name="SESErrorProcess"
 *   view-type="local"
 * @ejb.ejb-ref
 *   ejb-name="SESRechnungStatus"
 *   view-type="local"
 */
public class SapReceiveBean implements SessionBean
{
  //
  // const
  //
  private static final Logger   LOG = Logger.getLogger(SapRequestBean.class);


  //
  // fields
  //
  SessionContext                sessionContext;

  SESRechnungStatusLookup       rechnungStatus            = new SESRechnungStatusLookup();

  SESErrorProcessLocalHome      sesErrorProcessLocalHome  = null;
  SESErrorProcessLocal          sesErrorProcessLocal      = null;

  //---------------------------------------------------------------------------
  // bean methods
  //---------------------------------------------------------------------------

  /**
   * create ejb
   */
  public void ejbCreate() 
  {
    Context   ctx;
    Object    ref;

    try
    {
      ctx = new InitialContext();

      ref                           = ctx.lookup(sesErrorProcessLocalHome.COMP_NAME);
      sesErrorProcessLocalHome      = (SESErrorProcessLocalHome)ref;
      sesErrorProcessLocal          = sesErrorProcessLocalHome.create();

    }
    catch (CreateException ex)
    {
      LOG.error("ejbCreate - CreateException occured: " + ex.getMessage());
    }
    catch (NamingException ex)
    {
      LOG.error("ejbCreate - NamingException occured: " + ex.getMessage());
    }
  }

  public void ejbRemove()
  {
    // lazy bone
  }

  public void ejbActivate()
  {
    // lazy bone
  }

  public void ejbPassivate()
  {
    // lazy bone
  }

  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * @ejb.interface-method
   */
  public String receivePing (String request)
  {
    LOG.info("receivePing - received request: " + request);
    return new String ("echo from receivePing: " + request);
  }


  /**
   * @ejb.interface-method
   */
  public void receiveStatus(SAPRechnungStatusListVO list)
  {
    LOG.info("receiveStatus - update bagis db with rechnung status");
    rechnungStatus.getLocalInterface().RechnungStatusUpdate(list);
  }


  /**
   * @ejb.interface-method
   */
  public void receiveStatusJGB(SAPRechnungStatusListVO list)
  {
    SAPRechnungStatusVO   vo  = null;

    try
    {
      for (int i = 0; i < list.statusList.size() ; i++ )
      {
        vo = (SAPRechnungStatusVO) list.statusList.get(i);

        if ( ! vo.status_druck.equals("") && (vo.status_druck != null) )
        {
          LOG.info( "receiveStatusJGB - update ses db with rechnung status druck (jgb only)" +
                    "bfakt_id=" + vo.bfakt_id + ", status_druck=" + vo.status_druck );
          new BfakturSESDAO().updateStatusVerarbeitung(vo);
        }
        else
        {
          throw new Exception("Ungültiger status_druck für bfakt_id=" + vo.bfakt_id);
        }
      }
    }
    catch (Exception ex)
    {
      LOG.error("receiveStatusJGB - update ses db failed, Exception: " + ex.getMessage());
      sessionContext.setRollbackOnly();
      throw new EJBException(ex);
    }
  }


  /**
   * @ejb.interface-method
   */
  public void receiveError(ErrorVO err)
  {
    sesErrorProcessLocal.processError(err);
  }


  /**
   * @ejb.interface-method
   */
  public void receiveTrace(TraceVO trace) throws Exception
  {
    try
    {
      LOG.debug("receiveTrace - store TraceVO into ses db");

      if (trace.data.length() <= TraceDAO.FLD_LEN_DATA)
      {
        new TraceDAO().insert(trace);
      }
      else
      {
        int     begin      = 0;
        int     end        = 0;
        int     cnt        = 0;
        String  origAction = trace.action;
        String  origData   = trace.data;

        while ( begin < origData.length() )
        {
          end = begin + TraceDAO.FLD_LEN_DATA;
          if (end > origData.length())
            end = origData.length();

          trace.action = origAction + " (data part #"  + cnt + ")";
          trace.data   = origData.substring(begin,end);

          new TraceDAO().insert(trace);

          begin = begin+TraceDAO.FLD_LEN_DATA;
          cnt++;
        }
      }

      LOG.debug("receiveTrace - successfully done");
    }
    catch (Exception ex)
    {
      LOG.error("receiveTrace - Exception occured: " + ex.toString());
      throw new Exception(ex.getMessage());
    }
  }


} // end class


