package ch.ige.ses.util;

import java.util.*;
import ch.ige.ses.vo.bagis.*;
import ch.ige.ses.server.bagisoutbound.SESException;
import ch.ige.util.*;
import org.apache.log4j.*;
import ch.ige.util.server.replace.*;


public class ReplaceTexte
{
  //
  // const
  //
  private static final Logger LOG     = Logger.getLogger(ReplaceTexte.class);
  private static final String MESSAGE = "<< Diesen Platzhalter nicht mehr verwenden! >>";


  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public ReplaceTexte()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  //  public methods
  //---------------------------------------------------------------------------

 /**
  * Methode PrepareTexte ist verantwortlich für die Ersetzung der diversen
  * Platzhalter in den Formular- und Artikeltexten. Dazu wird die Funktionalität
  * des Packages ch.ige.util.server.replace benutzt.
  *
  * @param  al    Liste mit SAPRechnungVO Objekten
  */
  public void PrepareTexte(ArrayList al)
  {
    for (int i = 0; i < al.size(); i++)
    {
      try
      {
        PrepareTexte( (SAPRechnungVO)al.get(i) );
      }
      catch (Exception ex)
      {
        LOG.error ( "PrepareTexte - Ersetzung Platzhalter fehlgeschlagen " +
                    "für bfakt_id <" + ((SAPRechnungVO)al.get(i)).bfakt_id + ">." );

        //  error_source und error_description in SAPRechnungsVO setzen
        ((SAPRechnungVO)al.get(i)).error_source        = "PrepareTexte";
        ((SAPRechnungVO)al.get(i)).error_description   = "Ersetzung Platzhalter fehlgeschlagen";
        ((SAPRechnungVO)al.get(i)).status_verarbeitung = SAPRechnungVO.STATUS_VERARBEITUNG_ERROR;

      }
    } // end for
  }


 /**
  * Methode PrepareTexte ist verantwortlich für die Ersetzung der diversen
  * Platzhalter in den Formular- und Artikeltexten. Dazu wird die Funktionalität
  * des Packages ch.ige.util.server.replace benutzt.
  *
  * @param  sapRechnungVO   Rechnungsdaten
  * @throws Exception       Fehlerhafte Verarbeitung
  */
 public void PrepareTexte(SAPRechnungVO sapRechnungVO) throws Exception
  {
    P2d2            replace       = new P2d2();
    HashMap         mP2d2Formular = new HashMap();
    HashMap         mP2d2Position = new HashMap();
    java.util.Date  EreigDat      = null;
    String          buf           = null;


    //
    // Ersetzen aller Platzhalter in den Artikeltexten der einzelnen Positionen
    //
    for (int i=0; i < sapRechnungVO.posList.size(); i++)
    {
      SAPRechnungsPositionVO positionVO = (SAPRechnungsPositionVO)sapRechnungVO.posList.get(i);

      // Ereignis Datum
      if(positionVO.dat_ereignis != null)
      {
        EreigDat = positionVO.dat_ereignis;
      }
      mP2d2Position.putAll(replace.findP2d2(positionVO.artText));

      // Artikel Texte
      if (mP2d2Position.size() > 0)
      {
        mP2d2Position = ReplaceTagsWithData ( mP2d2Position, positionVO,
                                              sapRechnungVO, positionVO.dat_ereignis );
        try
        {
          positionVO.artText = replace.replaceP2d2(positionVO.artText, mP2d2Position);
           //LOG.debug("ReplaceTexte PrepareTexte: Platzhalter artText ersetzt; BFAKT_ID: "+ sapRechnungVO.bfakt_id + " BFAKT_POSNR: " + positionVO.bfakt_posnr +" ");
          sapRechnungVO.posList.set(i, positionVO);
        }
        catch (Exception ex)
        {
          buf = "Fehler beim Bearbeiten vom <artText> " +
                "für die Rechnung <" + sapRechnungVO.bfakt_id + "> : " + ex.getMessage();
          LOG.error("PrepareTexte - " + buf);
          throw new SESException(SESException.MASK,buf);
        }
      }

      // Artikel Zusatztexte
      mP2d2Position.clear();
      mP2d2Position.putAll(replace.findP2d2(positionVO.artZusatzText));
      if (mP2d2Position.size() > 0)
      {
        mP2d2Position = ReplaceTagsWithData(mP2d2Position, positionVO, sapRechnungVO, positionVO.dat_ereignis);
        try
        {
          positionVO.artZusatzText = replace.replaceP2d2(positionVO.artZusatzText, mP2d2Position);
           //LOG.debug("ReplaceTexte PrepareTexte: Platzhalter artZusatzText ersetzt; BFAKT_ID: "+ sapRechnungVO.bfakt_id + " BFAKT_POSNR: " + positionVO.bfakt_posnr +" ");
          sapRechnungVO.posList.set(i, positionVO);
        }
        catch (Exception ex)
        {
          buf = "Fehler beim Bearbeiten vom <artZusatzText> " +
                "für die Rechnung <" + sapRechnungVO.bfakt_id + "> : " + ex.getMessage();
          LOG.error("PrepareTexte - " + buf);
          throw new SESException(SESException.MASK,buf);
        }
      }
      mP2d2Position.clear();

    } // end for


    //
    // Ersetzen aller Platzhalter in den Formulartexten der Rechnung
    //
    mP2d2Formular.clear();
    if(sapRechnungVO.kopfText != "" || sapRechnungVO.kopfText != null)
    {
      mP2d2Formular.putAll(replace.findP2d2(sapRechnungVO.kopfText));
      if (mP2d2Formular.size() > 0)
      {
        mP2d2Formular = ReplaceTagsWithData(mP2d2Formular, null, sapRechnungVO, EreigDat);
        try
        {
           //LOG.debug("ReplaceTexte PrepareTexte: Vor replace KopfText: " + sapRechnungVO.kopfText);
           //LOG.debug("ReplaceTexte PrepareTexte: Values: " + mP2d2Formular.values());

          sapRechnungVO.kopfText = replace.replaceP2d2(sapRechnungVO.kopfText, mP2d2Formular);
           //LOG.debug("ReplaceTexte PrepareTexte: Nach replace KopfText: " + sapRechnungVO.kopfText);
        }
        catch (Exception ex)
        {
          buf = "Fehler beim Bearbeiten vom <kopfText> " +
                "für die Rechnung <" + sapRechnungVO.bfakt_id + "> : " + ex.getMessage();
          LOG.error("PrepareTexte - " + buf);
          throw new SESException(SESException.MASK,buf);
        }
      }
    }

    mP2d2Formular.clear();
    if(sapRechnungVO.var_text != "" || sapRechnungVO.var_text != null )
    {
      mP2d2Formular.putAll(replace.findP2d2(sapRechnungVO.var_text));
      if (mP2d2Formular.size() > 0)
      {
        mP2d2Formular = ReplaceTagsWithData(mP2d2Formular, null, sapRechnungVO, EreigDat);
        try
        {
          sapRechnungVO.var_text = replace.replaceP2d2(sapRechnungVO.var_text, mP2d2Formular);
           //LOG.debug("ReplaceTexte PrepareTexte: Platzhalter var_text ersetzt; BFAKT_ID: "+ sapRechnungVO.bfakt_id + " ");
        }
        catch (Exception ex)
        {
          buf = "Fehler beim Bearbeiten vom <var_text> " +
                "für die Rechnung <" + sapRechnungVO.bfakt_id + "> : " + ex.getMessage();
          LOG.error("PrepareTexte - " + buf);
          throw new SESException(SESException.MASK,buf);
        }
      }
    }
    mP2d2Formular.clear();
    if(sapRechnungVO.beilagenText != "" || sapRechnungVO.beilagenText != null)
    {
      mP2d2Formular.putAll(replace.findP2d2(sapRechnungVO.beilagenText));

      if (mP2d2Formular.size() > 0)
      {
        try
        {
          mP2d2Formular = ReplaceTagsWithData(mP2d2Formular, null, sapRechnungVO, EreigDat);
          sapRechnungVO.beilagenText = replace.replaceP2d2(sapRechnungVO.beilagenText, mP2d2Formular);
           //LOG.debug("ReplaceTexte PrepareTexte: Platzhalter beilagenText ersetzt; BFAKT_ID: "+ sapRechnungVO.bfakt_id + " ");
        }
        catch (Exception ex)
        {
          buf = "Fehler beim Bearbeiten vom <beilagenText> " +
                "für die Rechnung <" + sapRechnungVO.bfakt_id + "> : " + ex.getMessage();
          LOG.error("PrepareTexte - " + buf);
          throw new SESException(SESException.MASK,buf);
        }
      }
    }

     //LOG.debug("ReplaceTexte PrepareTexte: Nach beilagenText");
    mP2d2Formular.clear();
  }


  //---------------------------------------------------------------------------
  //  private methods
  //---------------------------------------------------------------------------

  /**
    * Methode schaut für alle möglichen %%$$Tags bei Positionen, ob sie in der
    * Map vorkommen und übergibt den dazugehörigen Wert.
    * Folgende Liste zeigt die Platzhalter und die entsprechenden Daten:
    *
    *   Platzhalter    Daten                  Bemerkungen
    *   %%$$Wert       BFAKTPOS.wert          Sollte nicht mehr verwendet werden
    *   %%$$Anzahl     BFAKTPOS.anzahl        Sollte nicht mehr verwendet werden
    *   %%$$Preis      BFAKTPOS.preis         Sollte nicht mehr verwendet werden
    *   %%$$RWert      BFAKTPOS.wert          Sollte nicht mehr verwendet werden
    *   %%$$RTotal     BFAKTUR.fakt_betrag    Wird durch SAP errechnet
    *   %%$$Anz        BFAKTPOS.anzahl        Sollte nicht mehr verwendet werden
    *   %%$$RAnz       BFAKTPOS.anzahl        Sollte nicht mehr verwendet werden
    *   %%$$Total      BFAKTUR.fakt_betrag    Wird durch SAP errechnet
    *   %%$$RPreis     BFAKTPOS.preis         Sollte nicht mehr verwendet werden
    *   %%$$PSchutit   BFAKTPOS.schutit_nr    Sollte nicht mehr verwendet werden
    *   %%$$Eintrag    BFAKTUR.schutit_nr     Sollte nicht mehr verwendet werden
    *   %%$$PTitel     SCHUTIT.beschreibung   Sollte nicht mehr verwendet werden
    *   %%$$Gesuch     SCHUTIT.gesuch_nr      Sollte nicht mehr verwendet werden
    *
    *   %%$$EreigDat   BFAKTPOS.dat_ereignis
    *   %%$$Verfall    BFAKTUR.verfall_dat
    *   %%$$Frist      BFAKTPOS.frist
    *   %%$$Schutit    BFAKTUR.schutit_nr
    *   %%$$Inhaber    BFAKTUR.inhaber
    *   %%$$Titel      SCHUTIT.beschreibung
    *   %%$$Text       BFAKTUR.var_text
    *   %%$$Untersch   BFAKTUR.unt_name
    *   %%$$Divers     BFAKTUR.txt_schluss
    *   %%$$Divers#    BFAKTUR.txt_schluss (ohne Blank)
    *   %%$$Schutzb    SCHUTIT.schutz_beginn
    *
    * @param  map       HashMap mit den %%$$Tags als Keys
    * @param  position  Aktuelle Rechnungsposition
    * @param  rechnung  SAP Rechnungs VO
    * @param  EreigDat  Ereignisdatum
    * @return HashMap   enthält nun auch die Werte zu den Keys
    */
    private HashMap ReplaceTagsWithData (HashMap map, SAPRechnungsPositionVO position,
                                         SAPRechnungVO rechnung, java.util.Date EreigDat)
    {
      IGEDate formDate = new IGEDate();

      //
      // Sprache setzen
      //
      if (rechnung.sprach_cd.equals("D"))
      {
        formDate.setLanguageD();
      }
      else if (rechnung.sprach_cd.equals("F"))
      {
        formDate.setLanguageF();
      }
      else
      {
        formDate.setLanguageI();
      }

      //
      // Folgende Platzhalter sollten nicht mehr verwendet werden,
      // es wird eine Standardmeldungen ausgeben
      //
      if(map.containsKey("%%$$Wert") || map.containsKey("%%$$RWert"))
      {
        if (map.containsKey("%%$$Wert"))
        {
           //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Wert gefunden");
          map.put("%%$$Wert", "Wert: " + MESSAGE);
        }
        else if (map.containsKey("%%$$RWert"))
        {
           //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$RWert gefunden");
          map.put("%%$$RWert", "RWert: " + MESSAGE);
        }
      }

      if ( map.containsKey("%%$$Anzahl") ||
           map.containsKey("%%$$Anz")    ||
           map.containsKey("%%$$RAnz")   )
      {
        if (map.containsKey("%%$$Anzahl"))
        {
           //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Anzahl gefunden");
          map.put("%%$$Anzahl", "Anzahl: " + MESSAGE);
        }
        else if (map.containsKey("%%$$Anz"))
        {
           //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Anz gefunden");
          map.put("%%$$Anz", "Anz: " + MESSAGE);
        }
        else if (map.containsKey("%%$$RAnz"))
        {
           //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$RAnz gefunden");
          map.put("%%$$RAnz", "RAnz: " + MESSAGE);
        }
      }

      if(map.containsKey("%%$$Preis"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Preis gefunden");
        map.put("%%$$Preis", "Preis: " + MESSAGE);
      }

      if(map.containsKey("%%$$RPreis"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$RPreis gefunden");
        map.put("%%$$RPreis", "RPreis: " + MESSAGE);
      }

      if(map.containsKey("%%$$Total"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Total gefunden");
        map.put("%%$$Total", "Total: " + MESSAGE);
      }

      if(map.containsKey("%%$$RTotal"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$RTotal gefunden");
        map.put("%%$$RTotal", "RTotal: " + MESSAGE);
      }

      if(map.containsKey("%%$$PSchutit"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$PSchutit gefunden");
        map.put("%%$$PSchutit", "PSchutit: " + MESSAGE);
      }

      if(map.containsKey("%%$$Eintrag"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Eintrag gefunden");
        map.put("%%$$Eintrag", "Eintrag: " + MESSAGE);
      }

      if(map.containsKey("%%$$PTitel"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$PTitel gefunden");
        map.put("%%$$PTitel", "PTitel: " + MESSAGE);
      }

      if(map.containsKey("%%$$RTotMwSt"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$RTotMwSt gefunden");
        map.put("%%$$RTotMwSt", "RTotMwSt: " + MESSAGE);
      }

      if(map.containsKey("%%$$Gesuch"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Gesuch gefunden");
        map.put("%%$$Gesuch", "Gesuch: " + MESSAGE);
      }

      if(map.containsKey("%%$$Untersch2"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Untersch2 gefunden");
        map.put("%%$$Untersch2", "Untersch2: " + MESSAGE);
      }


      //
      // Ersetzung der unterstützten Platzhalter
      //
      if(map.containsKey("%%$$EreigDat"))
      {
        if(EreigDat != null)
        {
           //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$EreigDat gefunden");
          map.put("%%$$EreigDat", formDate.setDateMMMM(EreigDat));
        }
        else
        {
           //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$EreigDat gefunden");
          map.put("%%$$EreigDat", "Ereignisdatum nicht vorhanden");
        }
      }

      if(map.containsKey("%%$$Frist"))
      {
        if (position == null)
        {
           //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Frist gefunden");
          map.put("%%$$Frist", formDate.setDateMMMM(rechnung.verfall_dat));
        }
        else
        {
           //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Frist gefunden");
          if(position.frist != null)
          {
            map.put("%%$$Frist", formDate.setDateMMMM(position.frist));
          }
          else
          {
            map.put("%%$$Frist", "");
          }
        }
      }

      if(map.containsKey("%%$$Schutit"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Schutit gefunden");
        map.put("%%$$Schutit", rechnung.schutit_nr);
      }

      if(map.containsKey("%%$$Verfall"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Verfall gefunden");
        if(rechnung.verfall_dat != null)
        {
          map.put("%%$$Verfall", formDate.setDateMMMM(rechnung.verfall_dat));
        }
      }

      if(map.containsKey("%%$$Inhaber"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Inhaber gefunden");
        map.put("%%$$Inhaber", rechnung.inhaber.toString());
      }

      if(map.containsKey("%%$$Titel"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Titel gefunden");
        map.put("%%$$Titel", rechnung.titel);
      }

      if(map.containsKey("%%$$Text"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Text gefunden");
        map.put("%%$$Text", rechnung.var_text);
      }

      if(map.containsKey("%%$$Untersch"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Untersch gefunden");
        map.put("%%$$Untersch", rechnung.unt_name);
      }

      if(map.containsKey("%%$$Divers"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Untersch gefunden");
        map.put("%%$$Divers", rechnung.diversText);
      }

      if(map.containsKey("%%$$Divers#"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Untersch gefunden");
        map.put("%%$$Divers#", rechnung.diversText);
      }

      if(map.containsKey("%%$$Schutzb"))
      {
         //LOG.debug("ReplaceTexte ReplaceTagsWithData: %%$$Schutzb gefunden");
        if(rechnung.schutz_beginn != null)
        {
          map.put("%%$$Schutzb", formDate.setDateMMMM(rechnung.schutz_beginn));
        }
      }


      //
      // Ende Verarbeitung
      //

      //LOG.debug("ReplaceTexte ReplaceTagsWithData: Ersetzte Platzhalter " + map.values() + "");
      return map;
    }

} // end class

