package ch.ige.ses.vo.ses;

import java.io.*;
import java.sql.Date;
import ch.ige.ses.vo.bagis.*;

/**
 * SES Value Object für die Tabelle ERROR
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 * @author    A. Rueegsegger, T. Iten
 * @version   $Id: ErrorVO.java,v 1.5 2006/12/13 14:58:39 iten Exp $
 */
public class ErrorVO implements Serializable
{
  //
  // const
  //
  public static final int ERROR_STAT_PENDENT          = 0;
  public static final int ERROR_STAT_ERLEDIGT         = 1;
  public static final int ERROR_STAT_ANNULIERT        = 2;

  public static final int EMPL_NOTIFY_KEINE           = 0;
  public static final int EMPL_NOTIFY_PENDENT         = 1;
  public static final int EMPL_NOTIFY_BENACHRICHTIGT  = 2;

  public static final int EMPL_MSG_TYPE_DEFAULT       = 0;
  public static final int EMPL_MSG_TYPE_MAX_EMPF      = 1;
  public static final int EMPL_MSG_TYPE_RECHNUNG      = 2;
  public static final int EMPL_MSG_TYPE_JGB           = 3;
  public static final int EMPL_MSG_TYPE_ADR           = 4;
  public static final int EMPL_MSG_TYPE_STAKO         = 5;

  public static final int DATA_TYPE_CD_NO_DATA                      = 0;
  public static final int DATA_TYPE_CD_STRING                       = 1;
  public static final int DATA_TYPE_CD_SAP_RECHNUNG_VO              = 10;
  public static final int DATA_TYPE_CD_SAP_RECHNUNG_ANNULATION_VO   = 11;
  public static final int DATA_TYPE_CD_SAP_RECHNUNG_STATUS_LIST_VO  = 12;
  public static final int DATA_TYPE_CD_SAP_JGB_JOB_VO               = 13;
  public static final int DATA_TYPE_CD_SAP_VERS_ADR_VO              = 14;
  public static final int DATA_TYPE_CD_SAP_ADR_STATUS_VO            = 15;
  public static final int DATA_TYPE_CD_STAKO_VO                     = 16;
  public static final int DATA_TYPE_CD_SAP_STAKO_JOB_VO             = 17;


  //
  // fields
  //
  public int      errorId         = 0;                     // not used for insert (autoincrement)
  public String   source          = null;
  public String   employeeId      = null;
  public int      employeeNotify  = EMPL_NOTIFY_KEINE;     // see EMPL_NOTIFY_*
  public int      employeeMsgType = EMPL_MSG_TYPE_DEFAULT; // see EMPL_MSG_TYPE_*
  public String   description     = null;
  public int      dataTypeCd      = DATA_TYPE_CD_NO_DATA;  // see DATA_TYPE_CD_*
  public Object   data            = null;
  public int      errorStat       = ERROR_STAT_PENDENT;    // see ERROR_STAT_*
  public Date     createdDat      = null;                  // not used for insert (default value)
  public Date     changedDat      = null;                  // not used for insert (default value)


  //----------------------------------------------------------------------------
  // constructors
  //----------------------------------------------------------------------------

  public ErrorVO()
  {
    // lazy bone
  }


  public ErrorVO ( String source, String description )
  {
    this.source      = source;
    this.description = description;
  }


  public ErrorVO ( String source, String description,
                   int dataTypeCd, Object data )
  {
    this(source,description);
    this.dataTypeCd  = dataTypeCd;
    this.data        = data;
  }


  public ErrorVO ( String source, String description,
                   int dataTypeCd, Object data,
                   String employeeId, int employeeNotify, int employeeMsgType )
  {
    this(source,description,dataTypeCd,data);

    this.employeeId      = employeeId;
    this.employeeNotify  = employeeNotify;
    this.employeeMsgType = employeeMsgType;
  }

  //----------------------------------------------------------------------------
  // methods
  //----------------------------------------------------------------------------

  public String toString()
  {
    StringBuffer buf = new StringBuffer();

    buf.append("  errorId          =  " + this.errorId              + "\n");
    buf.append("  source           =  " + this.source               + "\n");
    buf.append("  employeeId       =  " + this.employeeId           + "\n");
    buf.append("  employeeNotify   =  " + this.employeeNotify       + "\n");
    buf.append("  employeeMsgType  =  " + this.employeeMsgType      + "\n");
    buf.append("  description      :\n" + this.description          + "\n");
    buf.append("  dataTypeCd       =  " + this.dataTypeCd           + "\n");
    buf.append("  errorStat        =  " + this.errorStat            + "\n");
    buf.append("  createdDat       =  " + createdDat                + "\n");
    buf.append("  changedDat       =  " + changedDat                + "\n");
    buf.append("  data             :\n" + data                      + "\n");

    return buf.toString();
  }

} // end class

