package ch.ige.ses.vo.ses;

import java.io.Serializable;
import java.sql.Date;

/**
 * SAP Adressen Status Value Object um beim SapInbound Service
 * den Status der Adressen zu editieren. Löschvormerkungen und Buchungssperren
 * können gesetzt und zurückgenommen werden.
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: SAPAdrStatusVO.java,v 1.5 2006/12/13 14:58:39 iten Exp $
 */

public class SAPAdrStatusVO implements Serializable
{
  //
  // const
  //
  private static final String   LEGENDE_MODUS  = "(true=Löschvormerkung, false=Buchungssperre)";
  private static final String   LEGENDE_ACTION = "(X=setzen, BLANK=zurücknehmen)";

  //
  // fields
  //
  public int      adrNr         = 0;      //entspricht customer in SAP
  public String   compCode      = "1000";
  public String   salesOrg      = "1000";
  public String   distrChan     = "00";
  public String   division      = "00";
  public boolean  modus         = false;  // true = Löschvormerkung<br>
                                          // false = Buchungssperre
  public String   action        = " ";    // Abhängig von modus<br>
                                          // "X" = setzen
                                          // " " = zurücknehmen
  public String  employeeId     = "";

  //----------------------------------------------------------------------------
  // constructors
  //----------------------------------------------------------------------------

  public SAPAdrStatusVO()
  {
    // lazy bone
  }

  public SAPAdrStatusVO(int adrNr, boolean modus, String action)
  {
    this.adrNr  = adrNr;
    this.modus  = modus;
    this.action = action;
  }

  //----------------------------------------------------------------------------
  // methods
  //----------------------------------------------------------------------------
  public String toString()
  {
    StringBuffer buf = new StringBuffer();

    buf.append("  adrNr     =  " + this.adrNr       + "\n");
    buf.append("  compCode  =  " + this.compCode    + "\n");
    buf.append("  salesOrg  =  " + this.salesOrg    + "\n");
    buf.append("  distrChan =  " + this.distrChan   + "\n");
    buf.append("  division  =  " + this.division    + "\n");
    buf.append("  modus     =  " + this.modus       + " " + LEGENDE_MODUS + "\n");
    buf.append("  action    =  " + this.action      + " " + LEGENDE_ACTION + "\n");
    buf.append("  employeeId=  " + this.employeeId  + "\n");

    return buf.toString();
  }
} // end class

