package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.ObjectUtils;

import ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate;
import ch.ige.util.date.DateConverter;
import ch.ige.wdltool.vo.NegativeIpiTermVO;
import ch.ige.wdltool.vo.TermReplicationVO;

/**
 * Tests {@link ch.ige.wdltool.dao.NegativeIpiTermDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: NegativeIpiTermDAOTest.java,v 1.12 2008/04/29 16:38:11 heinzer Exp $
 */
public class NegativeIpiTermDAOTest extends BaseDAOWithCRUDTestTemplate<NegativeIpiTermDAO, NegativeIpiTermVO, Integer>
{
  /**
   * The only constructor
   * @param name The test's name
   */
  public NegativeIpiTermDAOTest( String name )
  {
    super( name );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getDAOClass()
  @Override
  protected Class<NegativeIpiTermDAO> getDAOClass()
  {
    return NegativeIpiTermDAO.class;
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NegativeIpiTermDAO#find(java.lang.Integer)}.
   * @throws Exception if the test fails
   */
  public void testFind() throws Exception
  {
    NegativeIpiTermVO negativeIpiTermVO = createAndInsert( true );
    NegativeIpiTermVO expectedVO = getSerializableVO();
    expectedVO.setIdentifier( negativeIpiTermVO.getIdentifier() );
    defaultTestRead( expectedVO, -1, 999999999, negativeIpiTermVO.getIdentifier() );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NegativeIpiTermDAO#findAll()}.
   * @throws Exception if the test fails
   */
  public void testFindAll() throws Exception
  {
    List<NegativeIpiTermVO> terms = getDAO().findAll();
    assertNotNull( "Checking List", terms );
    assertTrue( "Checking for content", !terms.isEmpty() );
    assertTrue( "Checking number of terms", terms.size() > 30 ); // This may change!
    for ( int i = 0; i < terms.size(); i++ )
    {
      // This may look stupid, but it didn't work...
      assertEquals( "Checking class", NegativeIpiTermVO.class, terms.get( i ).getClass() );
    }
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NegativeIpiTermDAO#insert(ch.ige.wdltool.vo.NegativeIpiTermVO)}.
   * @throws Exception if the test fails
   */
  public void testInsert() throws Exception
  {
    defaultTestCreate( true );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NegativeIpiTermDAO#update(ch.ige.wdltool.vo.NegativeIpiTermVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdate() throws Exception
  {
    defaultTestUpdate();
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NegativeIpiTermDAO#delete(java.lang.Integer)}.
   * @throws Exception if the test fails
   */
  public void testDelete() throws Exception
  {
    defaultTestDelete( -1, 999999999 );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.io.Serializable)
  @Override
  protected NegativeIpiTermVO read( Integer negativeIpiTermId ) throws SQLException
  {
    return getDAO().find( negativeIpiTermId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOs(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOs( NegativeIpiTermVO originalVO, NegativeIpiTermVO loadedVO )
  {
    assertEquals( "Checking ID", originalVO.getIdentifier(), loadedVO.getIdentifier() );
    assertEquals( "Checking class number", originalVO.getClassNumber(), loadedVO.getClassNumber() );
    assertEquals( "Checking language", originalVO.getLanguage(), loadedVO.getLanguage() );
    assertEquals( "Checking term", originalVO.getTerm(), loadedVO.getTerm() );
    assertEquals( "Checking private comment", originalVO.getPrivateComment(), loadedVO.getPrivateComment() );
    assertEquals( "Checking public comment", originalVO.getPublicComment(), loadedVO.getPublicComment() );
    assertEquals( "Checking change user", originalVO.getChangeUser(), loadedVO.getChangeUser() );
    assertEquals( "Checking creation user", originalVO.getCreationUser(), loadedVO.getCreationUser() );
    assertEquals( "Checking data source", originalVO.getDataSource(), loadedVO.getDataSource() );
    assertEquals( "Checking search term", originalVO.getSearchTerm(), loadedVO.getSearchTerm() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#validateVO(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void validateVO( NegativeIpiTermVO loadedVO )
  {
    assertTrue( "Checking change date", loadedVO.getChangeDate().after( GeneralTermDAOTest.getLatestMigration() ) );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  public boolean assertVOEqual( NegativeIpiTermVO originalVO, NegativeIpiTermVO serializedVO )
  {
    return super.assertVOEqual( originalVO, serializedVO ) && ObjectUtils.equals( originalVO.getChangeDate(), serializedVO.getChangeDate() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOsAndValidateLoaded(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  public void compareVOsAndValidateLoaded( NegativeIpiTermVO originalVO, NegativeIpiTermVO loadedVO )
  {
    super.compareVOsAndValidateLoaded( originalVO, loadedVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#insert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Integer insert( NegativeIpiTermVO negativeIpiTermVO )
  {
    return getDAO().insert( negativeIpiTermVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  public void modifyVOBeforeUpdate( NegativeIpiTermVO NegativeIpiTermVO )
  {
    NegativeIpiTermVO.setClassNumber( 33 );
    NegativeIpiTermVO.setLanguage( Locale.ITALIAN );
    NegativeIpiTermVO.setTerm( "use me" );
    NegativeIpiTermVO.setPrivateComment( "The private comment, modified" );
    NegativeIpiTermVO.setPublicComment( "The public comment, modified" );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#update(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void update( NegativeIpiTermVO negativeIpiTermVO )
  {
    getDAO().update( negativeIpiTermVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#delete(java.io.Serializable)
  @Override
  protected void delete( Integer negativeIpiTermId )
  {
    getDAO().delete( negativeIpiTermId );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NegativeIpiTermDAO#getReplicationTerms(String)}.
   * @throws Exception if the test fails
   */
  public void testGetReplicationTerms() throws Exception
  {
    TermReplicationVO negativeIpiTermsToReplicate = getDAO().getReplicationTerms( "wdldmz" );
    assertNotNull( "Checking terms to replicate", negativeIpiTermsToReplicate );
    assertNotNull( "Checking SQL statement", negativeIpiTermsToReplicate.getInsertStatement() );
    assertNotNull( "Checking terms", negativeIpiTermsToReplicate.getTermParameters() );
    assertTrue( "Checking number of terms", negativeIpiTermsToReplicate.getTermParameters().size() > 0 );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getSerializableVO()
  @Override
  public NegativeIpiTermVO getSerializableVO() throws SQLException
  {
    try
    {
      NegativeIpiTermVO negativeIpiTerm = new NegativeIpiTermVO();
      negativeIpiTerm.setIdentifier( 33 );
      negativeIpiTerm.setClassNumber( 42 );
      negativeIpiTerm.setLanguage( Locale.FRENCH );
      negativeIpiTerm.setTerm( "Dont use me" );
      negativeIpiTerm.setPrivateComment( "The private comment" );
      negativeIpiTerm.setPublicComment( "The public comment" );
      negativeIpiTerm.setChangeDate( DateConverter.stringToTimestamp( "03.02.2002 18:44:55.000" ) );
      negativeIpiTerm.setChangeUser( "billg" );
      negativeIpiTerm.setCreationUser( "steveb" );
      negativeIpiTerm.setPublic( false );
      negativeIpiTerm.setSearchTerm( "dont use me" );
      return negativeIpiTerm;
    }
    catch ( ParseException e )
    {
      throw new SQLException( e.getMessage() );
    }
  }
}
