package ch.ige.wdltool.ejb;

import java.util.Collection;
import java.util.List;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.wdltool.dao.IpiTermGroupDAO;
import ch.ige.wdltool.dao.IpiTermGroupDAOTest;
import ch.ige.wdltool.vo.IpiTermGroupVO;

/**
 * Tests {@link ch.ige.wdltool.ejb.WdlServices} for IPI term groups.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: WdlServicesForIpiTermGroupTest.java,v 1.7 2008/04/29 16:38:12 heinzer Exp $
 */
public class WdlServicesForIpiTermGroupTest extends BaseServiceTestTemplate<IpiTermGroupDAO, IpiTermGroupVO, Integer, WdlServices>
{
  /**
   * The unit under test
   */
  private WdlServices wdlServices;

  /**
   * Used for utility stuff
   */
  private IpiTermGroupDAOTest ipiTermGroupDAOTest;

  /**
   * The number of an existing IPI term group
   * This may change!
   */
  private static final int IPI_TERM_GROUP_NUMBER = 1000123;

  /**
   * The ID of {@link #IPI_TERM_GROUP_NUMBER}.
   * This may change!
   */
  private static final int IPI_TERM_GROUP_ID = 7928;

  /**
   * The only constructor.
   * @param name The test's name
   */
  public WdlServicesForIpiTermGroupTest( String name )
  {
    super( name );
    this.wdlServices = WdlServicesFactory.getStandaloneWdlServices();
    this.ipiTermGroupDAOTest = new IpiTermGroupDAOTest( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#findAllIpiTermGroups()}.
   * @throws Exception if the test fails
   */
  public void testFindAllIpiTermGroups() throws Exception
  {
    IpiTermGroupVO ipiTermGroup = createAndInsert( true );

    defaultTestReadCollection( ipiTermGroup, 1, (Object[]) null );

    // There's not much to do, really. Just check the sorting.
    List<IpiTermGroupVO> termGroups = (List<IpiTermGroupVO>) readWithService( (Object[]) null );
    assertTrue( "Check number of IPI term groups", termGroups.size() >= 1 );
    int previousTermGroupNumber = 0;
    for ( IpiTermGroupVO termGroup: termGroups )
    {
      assertTrue( "Check order", termGroup.getIpiTermNumber().intValue() > previousTermGroupNumber );
      previousTermGroupNumber = termGroup.getIpiTermNumber().intValue();
    }
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#findTermGroup(Integer)}.
   * @throws Exception if the test fails
   */
  public void testFindTermGroup() throws Exception
  {
    defaultTestRead( getExpectedVO(), -1, 808, IPI_TERM_GROUP_ID );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#readWithService(java.lang.Object[])
  @Override
  @SuppressWarnings( "unused" )
  public Collection<IpiTermGroupVO> readWithService( Object... searchParameters ) throws Exception
  {
    return getServices().findAllIpiTermGroups();
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#readWithService(java.io.Serializable)
  @Override
  public IpiTermGroupVO readWithService( Integer id ) throws Exception
  {
    return (IpiTermGroupVO) getServices().findTermGroup( id );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#deleteWithService(java.io.Serializable)
  @Override
  public void deleteWithService( Integer id ) throws Exception
  {
    // There's no service method for deleting
    DAOPool.get( IpiTermGroupDAO.class ).delete( id );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#insertWithService(ch.ige.dbutils.vo.AbstractVO)
  @Override
  public Integer insertWithService( IpiTermGroupVO ipiTermGroup ) throws Exception
  {
    return getServices().insertIpiTermGroup( ipiTermGroup ).getIdentifier();
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#getSerializableVO()
  @Override
  protected IpiTermGroupVO getSerializableVO()
  {
    return this.ipiTermGroupDAOTest.getSerializableVO();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getExpectedVO()
  @Override
  protected IpiTermGroupVO getExpectedVO()
  {
    IpiTermGroupVO ipiTermGroup = new IpiTermGroupVO();
    ipiTermGroup.setIdentifier( IPI_TERM_GROUP_ID );
    ipiTermGroup.setIpiTermNumber( IPI_TERM_GROUP_NUMBER );
    return ipiTermGroup;
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#compareVOsAndValidateLoaded(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOsAndValidateLoaded( IpiTermGroupVO originalVO, IpiTermGroupVO loadedVO )
  {
    this.ipiTermGroupDAOTest.compareVOsAndValidateLoaded( originalVO, loadedVO );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#getServices()
  @Override
  protected WdlServices getServices()
  {
    return this.wdlServices;
  }
}
