package ch.ige.wdltool.ejb.impl;

import java.io.Serializable;

import ch.ige.dbutils.dao.DAO;
import ch.ige.wdltool.exception.WdlValidationException;
import ch.ige.wdltool.util.TermUtils;
import ch.ige.wdltool.vo.AbstractTermVO;

/**
 * Contains special checks for terms.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * 
 * @param <D> The DAO type to use
 * @param <V> The VO type to use
 * @param <I> The ID type to use
 * @author   lewis_lee
 * @version  $Id: BaseTermServicesImpl.java,v 1.3 2008/06/05 12:50:32 lewis_lee Exp $
 */
abstract class BaseTermServicesImpl<D extends DAO, V extends AbstractTermVO, I extends Serializable> extends BaseServicesImpl<D, V, I>
{
  /**
   * Checks a value object, including special treatment for terms.
   * 
   * @param vo The value object to check
   * @param checkIdentifierExists If this is <code>true</code> and the value object is an
   *          {@link ch.ige.dbutils.vo.IdentifiableVO}, then its identifier must not be <code>null</code>
   * @throws WdlValidationException if the term already exists
   */
  protected void checkValueObject( V vo, boolean checkIdentifierExists ) throws WdlValidationException
  {
    TermUtils.setSearchTerm( vo );
    super.checkValueObject( vo, checkIdentifierExists );
    GeneralTermImpl.checkForExistingStopword( vo );
  }
}
