package ch.ige.wdltool.exception;

/**
 * Shows that a text contains an invalid character.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: InvalidCharacterException.java,v 1.1 2007/08/29 13:07:11 lewis_lee Exp $
 */
public class InvalidCharacterException extends Exception
{
  /**
   * The checked text
   */
  private String text;

  /**
   * The invalid character
   */
  private char invalidCharacter;

  /**
   * The position within the text where the invalid character was found
   */
  private int position;

  /**
   * Creates this with an error message.
   * @param message The error message
   * @param text The checked text
   * @param invalidCharacter The invalid character causing this Exception
   * @param position The position within the text where the invalid character was found
   */
  public InvalidCharacterException( String message, String text, char invalidCharacter, int position )
  {
    super( message );
    this.text = text;
    this.invalidCharacter = invalidCharacter;
    this.position = position;
  }

  /**
   * Gets the checked text.
   * @return The checked text
   */
  public String getText()
  {
    return this.text;
  }

  /**
   * Gets the invalid character.
   * @return The invalid character
   */
  public char getInvalidCharacter()
  {
    return this.invalidCharacter;
  }

  /**
   * Gets the position of the invalid character.
   * Note that this starts at 1, not at 0!
   * @return The position within the text where the invalid character was found, starting at 1
   */
  public int getPosition()
  {
    return this.position;
  }
}
