package ch.ige.wdltool.exception;

import javax.ejb.CreateException;

import ch.ige.dbutils.vo.AbstractVO;

/**
 * Is thrown when an object should be created, but the operation fails.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: WdlCreateException.java,v 1.2 2008/01/17 17:41:31 lewis_lee Exp $
 */
public class WdlCreateException extends CreateException
{
  /**
   * The value object that should have been created/inserted
   */
  private AbstractVO valueObject;

  /**
   * Creates this without message.
   * @param cause The causing Exception
   * @param valueObject The value object that should habe been created/inserted
   */
  public WdlCreateException( Throwable cause, AbstractVO valueObject )
  {
    this( null, cause, valueObject );
  }

  /**
   * Creates this with message.
   * @param message The error message
   * @param cause The causing Exception
   * @param valueObject The value object that should habe been created/inserted
   */
  public WdlCreateException( String message, Throwable cause, AbstractVO valueObject )
  {
    super( message );
    initCause( cause );
    this.valueObject = valueObject;
  }

  /**
   * Gets the value object that should have been created/inserted.
   * @return the value object that should have been created/inserted
   */
  public AbstractVO getValueObject()
  {
    return this.valueObject;
  }
}
