package ch.ige.wdltool.util;

import java.sql.Timestamp;

import org.apache.commons.lang.StringUtils;

import ch.ige.wdltool.dao.BaseTermGroupDAO;
import ch.ige.wdltool.vo.AbstractTermVO;
import ch.ige.wdltool.vo.IpiTermVO;
import ch.ige.wdltool.vo.NiceTermVO;
import ch.ige.wdlutil.text.SearchText;

/**
 * Utility methods for terms.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: TermUtils.java,v 1.4 2008/06/06 11:21:20 lewis_lee Exp $
 */
public class TermUtils
{
  /**
   * Gets a term's term group number.
   * @param term The term to check
   * @return The term group number or <code>null</code> if none exists
   */
  public static Integer getTermGroupNumber( AbstractTermVO term )
  {
    if ( term == null )
    {
      return null;
    }
    else
    {
      Integer termGroupNumber = null;
      if ( term instanceof NiceTermVO )
      {
        termGroupNumber = ( (NiceTermVO) term ).getNiceTermGroupNumber();
      }
      else if ( term instanceof IpiTermVO )
      {
        termGroupNumber = ( (IpiTermVO) term ).getIpiTermGroupNumber();
      }
      return termGroupNumber;
    }
  }

  /**
   * Sets the term group number of the term, if it has one.
   * @param term The term whose term group number is set
   */
  public static void setTermGroupNumber( AbstractTermVO term )
  {
    if ( term != null )
    {
      if ( term instanceof IpiTermVO )
      {
        IpiTermVO ipiTerm = (IpiTermVO) term;
        ipiTerm.setIpiTermGroupNumber( BaseTermGroupDAO.getTermGroupNumber( ipiTerm.getTermGroupId() ) );
      }
      else if ( term instanceof NiceTermVO )
      {
        NiceTermVO niceTerm = (NiceTermVO) term;
        niceTerm.setNiceTermGroupNumber( BaseTermGroupDAO.getTermGroupNumber( niceTerm.getTermGroupId() ) );
      }
    }
  }

  /**
   * Gets a term's term group ID.
   * @param term The term
   * @return The term group ID or <code>null</code> if none exists
   */
  public static Integer getTermGroupId( AbstractTermVO term )
  {
    if ( term == null )
    {
      return null;
    }
    else
    {
      Integer termGroupId = null;
      if ( term instanceof NiceTermVO )
      {
        termGroupId = ( (NiceTermVO) term ).getTermGroupId();
      }
      else if ( term instanceof IpiTermVO )
      {
        termGroupId = ( (IpiTermVO) term ).getTermGroupId();
      }
      return termGroupId;
    }
  }

  /**
   * Sets a term's term group ID.
   * @param term The term
   * @param termGroupId The term group ID to set
   */
  public static void setTermGroupId( AbstractTermVO term, Integer termGroupId )
  {
    if ( term != null )
    {
      if ( term instanceof NiceTermVO )
      {
        ( (NiceTermVO) term ).setTermGroupId( termGroupId );
      }
      else
      {
        ( (IpiTermVO) term ).setTermGroupId( termGroupId );
      }
    }
  }

  /**
   * Creates and sets the search term.
   * @param term The term where the search term is set
   */
  public static void setSearchTerm( AbstractTermVO term )
  {
    if ( term != null )
    {
      String termText = term.getTerm();
      if ( StringUtils.isNotEmpty( termText ) )
      {
        term.setSearchTerm( SearchText.toSearchText( termText, term.getLanguage() ) );
      }
    }
  }

  /**
   * Sets the change date/time to the current date/time.
   * @param term The term whose change date/time is set to now
   */
  public static void setChangeTimestamp( AbstractTermVO term )
  {
    if ( term != null )
    {
      Timestamp now = new Timestamp( System.currentTimeMillis() );
      term.setChangeDate( now );
    }
  }

  /**
   * Sets the renewal date/time to the current date/time.
   * @param term The term whose renewal date/time is set to now
   */
  public static void setRenewalTimestamp( IpiTermVO term )
  {
    if ( term != null )
    {
      Timestamp now = new Timestamp( System.currentTimeMillis() );
      term.setRenewal( now );
    }
  }
}
