package ch.ige.wdltool.vo;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;

import ch.ige.wdltool.vo.CheckedClassTextGroupVO.ModifiableInteger;

/**
 * Tests {@link ch.ige.wdltool.vo.CheckedClassTextGroupVO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author Christian Heinzer
 * @author Eric Lewis
 * @version  $Id: CheckedClassTextGroupVOTest.java,v 1.6 2007/07/10 11:41:31 lewis_lee Exp $
 */
public class CheckedClassTextGroupVOTest extends TestCase
{
  static
  {
    BasicConfigurator.configure();
  }

  /**
   * The first test text
   */
  private static final String TEST_TEXT1 = "blubber ";

  /**
   * The second test text
   */
  private static final String TEST_TEXT2 = " bla;";

  /**
   * The thir test text
   */
  private static final String TEST_TEXT3 = "der";

  /**
   * The only constructor.
   * @param name The test's name
   */
  public CheckedClassTextGroupVOTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.vo.CheckedClassTextGroupVO#CheckedClassTextGroupVO(String, Character)}.
   * @throws Exception if the test fails
   */
  public void testConstructor() throws Exception
  {
    CheckedClassTextGroupVO checkedClassTextGroup = new CheckedClassTextGroupVO( "whatever it means", '!' );
    assertEquals( "Checking number of parts", 2, checkedClassTextGroup.getClassTextParts().size() );
    assertEquals( "Checking first part", "whatever it means", checkedClassTextGroup.getClassTextParts().get( 0 ).getClassTextPart() );
    assertEquals( "Checking first part", "!", checkedClassTextGroup.getClassTextParts().get( 1 ).getClassTextPart() );
  }

  /**
   * Tests {@link ch.ige.wdltool.vo.CheckedClassTextGroupVO#removeClassTextPart(ClassTextPartVO)}.
   * @throws Exception if the test fails
   */
  public void testRemoveClassTextPart() throws Exception
  {
    CheckedClassTextGroupVO checkedClassTextGroup = createCheckedClassTextGroupVO();
    ClassTextPartVO secondPart = checkedClassTextGroup.getClassTextParts().get( 1 );

    assertTrue( "Checking for part", checkedClassTextGroup.getClassTextParts().contains( secondPart ) );
    assertEquals( "Checking number of parts", 3, checkedClassTextGroup.getClassTextParts().size() );

    checkedClassTextGroup.removeClassTextPart( secondPart );
    assertTrue( "Checking for part", !checkedClassTextGroup.getClassTextParts().contains( secondPart ) );
    assertEquals( "Checking number of parts", 2, checkedClassTextGroup.getClassTextParts().size() );
  }

  /**
   * Tests {@link ch.ige.wdltool.vo.CheckedClassTextGroupVO#addClassTextPart(ClassTextPartVO, Integer)}.
   * @throws Exception if the test fails
   */
  public void testAddClassTextPart() throws Exception
  {
    CheckedClassTextGroupVO checkedClassTextGroup = createCheckedClassTextGroupVO();
    assertEquals( "Checking number of parts", 3, checkedClassTextGroup.getClassTextParts().size() );

    ClassTextPartVO newPart = new ClassTextPartVO( "whatever", CheckedClassTextPart.INVALID );
    checkedClassTextGroup.addClassTextPart( newPart, null );
    assertEquals( "Checking number of parts", 4, checkedClassTextGroup.getClassTextParts().size() );
    assertEquals( "Checking index", 3, checkedClassTextGroup.getClassTextParts().indexOf( newPart ) );
    assertEquals( newPart, checkedClassTextGroup.getClassTextParts().get( 3 ) );

    // Insert at the beginning
    ClassTextPartVO newerPart = new ClassTextPartVO( "gets you", CheckedClassTextPart.UNKNOWN );
    checkedClassTextGroup.addClassTextPart( newerPart, 0 );
    assertEquals( "Checking number of parts", 5, checkedClassTextGroup.getClassTextParts().size() );
    assertEquals( "Checking index", 0, checkedClassTextGroup.getClassTextParts().indexOf( newerPart ) );
    assertEquals( newerPart, checkedClassTextGroup.getClassTextParts().get( 0 ) );

    // Insert at the end
    ClassTextPartVO newestPart = new ClassTextPartVO( "thru the night", CheckedClassTextPart.STOPWORD );
    checkedClassTextGroup.addClassTextPart( newestPart, 5 );
    assertEquals( "Checking number of parts", 6, checkedClassTextGroup.getClassTextParts().size() );
    assertEquals( "Checking index", 5, checkedClassTextGroup.getClassTextParts().indexOf( newestPart ) );
    assertEquals( newestPart, checkedClassTextGroup.getClassTextParts().get( 5 ) );

    // Out of bounds
    try
    {
      checkedClassTextGroup.addClassTextPart( newestPart, -1 );
      fail( "Too small index must fail" );
    }
    catch ( IndexOutOfBoundsException e )
    {
      // This is expected
    }

    try
    {
      checkedClassTextGroup.addClassTextPart( newestPart, 7 );
      fail( "Too big index must fail" );
    }
    catch ( IndexOutOfBoundsException e )
    {
      // This is expected
    }
  }

  /**
   * Tests {@link ch.ige.wdltool.vo.CheckedClassTextGroupVO#replaceClassTextPart(ClassTextPartVO, List)}.
   * @throws Exception if the test fails
   */
  public void testReplaceClassTextPart() throws Exception
  {
    CheckedClassTextGroupVO checkedClassTextGroup = createCheckedClassTextGroupVO();
    ClassTextPartVO unknownPart = new ClassTextPartVO( "you can't find me", CheckedClassTextPart.UNKNOWN );
    checkedClassTextGroup.replaceClassTextPart( unknownPart, checkedClassTextGroup.getClassTextParts() );
    assertEquals( checkedClassTextGroup.getClassTextParts(), createCheckedClassTextGroupVO().getClassTextParts() );

    ClassTextPartVO firstPart = checkedClassTextGroup.getClassTextParts().get( 0 );
    assertEquals( "Checking index", 0, checkedClassTextGroup.getClassTextParts().indexOf( firstPart ) );

    List<ClassTextPartVO> newParts = createCheckedClassTextGroupVO().getClassTextParts();
    Collections.reverse( newParts );
    checkedClassTextGroup.replaceClassTextPart( firstPart, newParts );
    assertEquals( "Checking number of parts", 5, checkedClassTextGroup.getClassTextParts().size() );
    assertEquals( "Checking index of replaced part", -1, checkedClassTextGroup.getClassTextParts().indexOf( firstPart ) );

    ClassTextPartVO newFirstPart = new ClassTextPartVO( TEST_TEXT3, CheckedClassTextPart.STOPWORD );
    assertEquals( newFirstPart, checkedClassTextGroup.getClassTextParts().get( 0 ) );

    // Replace at the end
    ClassTextPartVO lastPart = checkedClassTextGroup.getClassTextParts().get( 4 );
    List<ClassTextPartVO> newerParts = createCheckedClassTextGroupVO().getClassTextParts();
    Collections.reverse( newerParts );
    checkedClassTextGroup.replaceClassTextPart( lastPart, newerParts );
    assertEquals( "Checking number of parts", 7, checkedClassTextGroup.getClassTextParts().size() );
    assertEquals( "Checking index of replaced part", -1, checkedClassTextGroup.getClassTextParts().indexOf( lastPart ) );

    ClassTextPartVO newLastPart = new ClassTextPartVO( TEST_TEXT1, CheckedClassTextPart.INVALID );
    assertEquals( newLastPart, checkedClassTextGroup.getClassTextParts().get( 6 ) );
  }

  /**
   * Tests {@link ch.ige.wdltool.vo.CheckedClassTextGroupVO#getPartAt(ModifiableInteger)}.
   * @throws Exception if the test fails
   */
  public void testGetPartAt() throws Exception
  {
    CheckedClassTextGroupVO checkedClassTextGroupVO = createCheckedClassTextGroupVO();
    ClassTextPartVO classTextPart;

    try
    {
      ModifiableInteger modifiableInteger = checkedClassTextGroupVO.new ModifiableInteger( -1 );
      classTextPart = checkedClassTextGroupVO.getPartAt( modifiableInteger );
      fail( "Negative position must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    // Position 0
    ModifiableInteger modifiableInteger = checkedClassTextGroupVO.new ModifiableInteger( 0 );
    classTextPart = checkedClassTextGroupVO.getPartAt( modifiableInteger );
    assertEquals( "Checking position within class text part", 0, modifiableInteger.getValue() );
    assertEquals( "Checking class text part", TEST_TEXT1, classTextPart.getClassTextPart() );

    // At the end of a part
    modifiableInteger = checkedClassTextGroupVO.new ModifiableInteger( 7 );
    classTextPart = checkedClassTextGroupVO.getPartAt( modifiableInteger );
    assertEquals( "Checking position within class text part", 7, modifiableInteger.getValue() );
    assertEquals( "Checking class text part", TEST_TEXT1, classTextPart.getClassTextPart() );

    // At the beginning of next part
    modifiableInteger = checkedClassTextGroupVO.new ModifiableInteger( 8 );
    classTextPart = checkedClassTextGroupVO.getPartAt( modifiableInteger );
    assertEquals( "Checking position within class text part", 0, modifiableInteger.getValue() );
    assertEquals( "Checking class text part", TEST_TEXT2, classTextPart.getClassTextPart() );

    // Somewhere in-between
    modifiableInteger = checkedClassTextGroupVO.new ModifiableInteger( 9 );
    classTextPart = checkedClassTextGroupVO.getPartAt( modifiableInteger );
    assertEquals( "Checking position within class text part", 1, modifiableInteger.getValue() );
    assertEquals( "Checking class text part", TEST_TEXT2, classTextPart.getClassTextPart() );

    // At the end of next part
    modifiableInteger = checkedClassTextGroupVO.new ModifiableInteger( 12 );
    classTextPart = checkedClassTextGroupVO.getPartAt( modifiableInteger );
    assertEquals( "Checking position within class text part", 4, modifiableInteger.getValue() );
    assertEquals( "Checking class text part", TEST_TEXT2, classTextPart.getClassTextPart() );

    // This is out of bounds
    modifiableInteger.setValue( 300 );
    classTextPart = checkedClassTextGroupVO.getPartAt( modifiableInteger );
    assertNull( "Checking class text part", classTextPart );
    assertEquals( "Checking unknown index", 300, modifiableInteger.getValue() );

    // This is before the end
    modifiableInteger.setValue( TEST_TEXT1.length() + TEST_TEXT2.length() + TEST_TEXT3.length() - 2 );
    classTextPart = checkedClassTextGroupVO.getPartAt( modifiableInteger );
    assertEquals( "Checking position within class text part", 1, modifiableInteger.getValue() );
    assertEquals( "Checking class text part", TEST_TEXT3, classTextPart.getClassTextPart() );

    // This is the last character
    modifiableInteger.setValue( TEST_TEXT1.length() + TEST_TEXT2.length() + TEST_TEXT3.length() - 1 );
    classTextPart = checkedClassTextGroupVO.getPartAt( modifiableInteger );
    assertEquals( "Checking position within class text part", 2, modifiableInteger.getValue() );
    assertEquals( "Checking class text part", TEST_TEXT3, classTextPart.getClassTextPart() );

    // This is out of bounds
    int index = TEST_TEXT1.length() + TEST_TEXT2.length() + TEST_TEXT3.length();
    modifiableInteger.setValue( index );
    classTextPart = checkedClassTextGroupVO.getPartAt( modifiableInteger );
    assertEquals( "Checking position within class text part", index, modifiableInteger.getValue() );
    assertNull( "Checking class text part", classTextPart );
  }

  /**
   * Tests {@link ch.ige.wdltool.vo.CheckedClassTextGroupVO#splitClassTextParts(int, String, CheckedClassTextPart)}
   * with faulty data.
   * @throws Exception if the test fails
   */
  public void testSplitClassTextPartsWithFaults() throws Exception
  {
    CheckedClassTextGroupVO checkedClassTextGroupVO = createCheckedClassTextGroupVO();

    try
    {
      checkedClassTextGroupVO.splitClassTextParts( -1, "zzz", CheckedClassTextPart.STOPWORD );
      fail( "Negative index must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      checkedClassTextGroupVO.splitClassTextParts( (TEST_TEXT1 + TEST_TEXT2 + TEST_TEXT3).length(), "zzz", CheckedClassTextPart.STOPWORD );
      fail( "Too big index must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      checkedClassTextGroupVO.splitClassTextParts( 0, null, CheckedClassTextPart.STOPWORD );
      fail( "Null text must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      checkedClassTextGroupVO.splitClassTextParts( 0, "", CheckedClassTextPart.STOPWORD );
      fail( "Empty text must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }
    
    try
    {
      String newTestText1 = TEST_TEXT1 + "x";
      checkedClassTextGroupVO.splitClassTextParts( 0, newTestText1, CheckedClassTextPart.STOPWORD );
      fail( "Too long text must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      checkedClassTextGroupVO.splitClassTextParts( 0, "asdf", null );
      fail( "Null flag must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }
  }

  /**
   * Tests {@link ch.ige.wdltool.vo.CheckedClassTextGroupVO#splitClassTextParts(int, String, CheckedClassTextPart)}.
   * @throws Exception if the test fails
   */
  public void testSplitClassTextParts() throws Exception
  {
    CheckedClassTextGroupVO checkedClassTextGroupVO = createCheckedClassTextGroupVO();
    assertEquals( "Checking number of parts", 3, checkedClassTextGroupVO.getClassTextParts().size() );

    // At the beginning
    String restOfText1 = TEST_TEXT1.substring( 3 );
    checkedClassTextGroupVO.splitClassTextParts( 0, "zzz", CheckedClassTextPart.STOPWORD );
    assertEquals( "Checking number of parts", 4, checkedClassTextGroupVO.getClassTextParts().size() );
    ClassTextPartVO classTextPart1 = checkedClassTextGroupVO.getClassTextParts().get( 0 );
    assertEquals( new ClassTextPartVO( "zzz", CheckedClassTextPart.STOPWORD ), classTextPart1 );
    ClassTextPartVO classTextPart2 = checkedClassTextGroupVO.getClassTextParts().get( 1 );
    assertEquals( new ClassTextPartVO( restOfText1, CheckedClassTextPart.INVALID ), classTextPart2 );

    // Check the rest, too
    assertEquals( createCheckedClassTextGroupVO().getClassTextParts().get( 1 ), checkedClassTextGroupVO.getClassTextParts().get( 2 ) );
    assertEquals( createCheckedClassTextGroupVO().getClassTextParts().get( 2 ), checkedClassTextGroupVO.getClassTextParts().get( 3 ) );

    // In-between
    checkedClassTextGroupVO = createCheckedClassTextGroupVO();
    String headerOfText2 = TEST_TEXT2.substring( 0, 2 ); // <code> b</code>
    String trailerOfText2 = TEST_TEXT2.substring( 4 ); // <code>;</code>
    checkedClassTextGroupVO.splitClassTextParts( 10, "üä", CheckedClassTextPart.CORRECT ); // Replace <code>la</code>
    assertEquals( "Checking number of parts", 5, checkedClassTextGroupVO.getClassTextParts().size() );

    classTextPart2 = checkedClassTextGroupVO.getClassTextParts().get( 1 );
    assertEquals( new ClassTextPartVO( headerOfText2, CheckedClassTextPart.INVALID ), classTextPart2 );

    ClassTextPartVO classTextPart3 = checkedClassTextGroupVO.getClassTextParts().get( 2 );
    assertEquals( new ClassTextPartVO( "üä", CheckedClassTextPart.CORRECT ), classTextPart3 );

    ClassTextPartVO classTextPart4 = checkedClassTextGroupVO.getClassTextParts().get( 3 );
    assertEquals( new ClassTextPartVO( trailerOfText2, CheckedClassTextPart.INVALID ), classTextPart4 );

    // Check the rest, too
    assertEquals( createCheckedClassTextGroupVO().getClassTextParts().get( 0 ), checkedClassTextGroupVO.getClassTextParts().get( 0 ) );
    assertEquals( createCheckedClassTextGroupVO().getClassTextParts().get( 2 ), checkedClassTextGroupVO.getClassTextParts().get( 4 ) );

    // Last character
    checkedClassTextGroupVO = createCheckedClassTextGroupVO();
    String headerOfText3 = TEST_TEXT3.substring( 0, TEST_TEXT3.length() -1 ); // <code>de</code>
    checkedClassTextGroupVO.splitClassTextParts( 15, "R", CheckedClassTextPart.CORRECT ); // Replace <code>r</code>
    assertEquals( "Checking number of parts", 4, checkedClassTextGroupVO.getClassTextParts().size() );

    classTextPart3 = checkedClassTextGroupVO.getClassTextParts().get( 2 );
    assertEquals( new ClassTextPartVO( headerOfText3, CheckedClassTextPart.STOPWORD ), classTextPart3 );

    classTextPart4 = checkedClassTextGroupVO.getClassTextParts().get( 3 );
    assertEquals( new ClassTextPartVO( "R", CheckedClassTextPart.CORRECT ), classTextPart4 );

    // Check the rest, too
    assertEquals( createCheckedClassTextGroupVO().getClassTextParts().get( 0 ), checkedClassTextGroupVO.getClassTextParts().get( 0 ) );
    assertEquals( createCheckedClassTextGroupVO().getClassTextParts().get( 1 ), checkedClassTextGroupVO.getClassTextParts().get( 1 ) );
  }

  /**
   * Checks two class text part Lists for equality.
   * @param expectedClassTextParts The class text parts as expected
   * @param actualClassTextParts The class text parts as a result
   */
  private void assertEquals( List<ClassTextPartVO> expectedClassTextParts, List<ClassTextPartVO> actualClassTextParts )
  {
    assertEquals( "Checking number of class text parts", expectedClassTextParts.size(), actualClassTextParts.size() );
    for ( int i = 0; i < expectedClassTextParts.size(); i++ )
    {
      ClassTextPartVO expectedClassTextPart = expectedClassTextParts.get( i );
      ClassTextPartVO actualClassTextPart = actualClassTextParts.get( i );
      assertEquals( expectedClassTextPart, actualClassTextPart );
    }
  }

  /**
   * Checks two class text parts for equality.
   * @param expectedClassTextPart The class text part as expected
   * @param actualClassTextPart The class text part as a result
   */
  private void assertEquals( ClassTextPartVO expectedClassTextPart, ClassTextPartVO actualClassTextPart )
  {
    assertEquals( "Checking text", expectedClassTextPart.getClassTextPart(), actualClassTextPart.getClassTextPart() );
    assertEquals( "Checking check flag", expectedClassTextPart.getCheckFlag(), actualClassTextPart.getCheckFlag() );
  }

  /**
   * Creates a checked class text group for testing.
   * @return A checked class text group for testing
   */
  private static CheckedClassTextGroupVO createCheckedClassTextGroupVO()
  {
    CheckedClassTextGroupVO checkedClassTextGroupVO = new CheckedClassTextGroupVO( TEST_TEXT1 + TEST_TEXT2 + TEST_TEXT3, null );

    ClassTextPartVO classTextPart1 = new ClassTextPartVO( TEST_TEXT1, CheckedClassTextPart.INVALID );
    ClassTextPartVO classTextPart2 = new ClassTextPartVO( TEST_TEXT2, CheckedClassTextPart.INVALID );

    LinkedList<ClassTextPartVO> classTextParts = new LinkedList<ClassTextPartVO>();
    classTextParts.add( classTextPart1 );
    classTextParts.add( classTextPart2 );
    checkedClassTextGroupVO.setClassTextParts( classTextParts ); //initial

    ClassTextPartVO classTextPart = new ClassTextPartVO( TEST_TEXT3, CheckedClassTextPart.STOPWORD );
    checkedClassTextGroupVO.addClassTextPart( classTextPart, null ); //added one more

    return checkedClassTextGroupVO;
  }
}
