package ch.ige.wdltool.vo;

/**
 * Shows the result of a WDL validation for a class text.
 * The check start and stop can be anywhere in the class text, not just on word boundaries.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: CheckedClassTextPart.java,v 1.6 2008/01/18 15:39:04 lewis_lee Exp $
 */
public enum CheckedClassTextPart
{
  /**
   * This part is correct.
   */
  CORRECT,

  /**
   * This part contains unknown terms.
   */
  UNKNOWN,

  /**
   * This part is a stopword which can be ignored.
   */
  STOPWORD,

  /**
   * This part is invalid because it breaks the WDL syntax.
   */
  INVALID,

  /**
   * This part already exists, so it is a duplicate entry.
   */
  DUPLICATE,

  /**
   * This part is a bracket: <code>(</code> or <code>)</code> or <code>[</code> or <code>]</code>,
   * which is not {@link #CORRECT}.
   */
  BRACKET,

  /**
   * This is part of a term which is correct, but the character case is different
   * from the existing term.
   * Example: The existing term is <code>hello</code>, and the term is <code>hellO</code>,
   * then the <code>O</code> will have this status.
   */
  CORRECT_WRONG_CASE;
}
