package ch.ige.wdltool.vo;

import java.sql.Timestamp;

import org.hibernate.validator.Min;
import org.hibernate.validator.NotNull;
import org.hibernate.validator.Past;

import ch.ige.dbutils.vo.IdentifiableVO;
import ch.ige.wdltool.enums.DataSource;
import ch.ige.wdltool.enums.TermType;
import ch.ige.wdltool.validator.SearchTerm;
import ch.ige.wdltool.validator.Term;

/**
 * Contains a term which may become an IPI term, if it's approved.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: IpiTermCandidateVO.java,v 1.13 2008/04/29 16:38:11 heinzer Exp $
 */
public class IpiTermCandidateVO extends AbstractTermVO implements IdentifiableVO<Integer>
{
  /**
   * The empty constructor
   */
  public IpiTermCandidateVO()
  {
    super();
    this.isActive = false;
    this.usageCount = 1;
    // IPI term candidates are never public
    this.isPublic = false;
  }

  /**
   * Creates this from an abstract VO.
   * @param vo The VO whose values are copied
   */
  public IpiTermCandidateVO( AbstractTermVO vo )
  {
    super( vo );
    // IPI term candidates are never public
    this.isPublic = false;
    this.isActive = false;
    if ( !( vo instanceof IpiTermCandidateVO ) )
    {
      this.usageCount = 1;
    }
  }

  // @see ch.ige.wdltool.vo.IpiTermVO#initTermTypeAndDataSource()
  @Override
  protected void initTermTypeAndDataSource()
  {
    this.termType = TermType.IPI_TERM_CANDIDATE;
    this.dataSource = DataSource.IPI;
  }

  //*********************************************************
  // Same as IpiTermVO, but individual because of validation
  //*********************************************************

  // @see ch.ige.wdltool.vo.AbstractTermVO#getLatestOccurrence()
  @Override
  @Past
  public Timestamp getLatestOccurrence()
  {
    return super.getLatestOccurrence();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#setLatestOccurrence(java.sql.Timestamp)
  @Override
  public void setLatestOccurrence( Timestamp latestOccurrence )
  {
    super.setLatestOccurrence( latestOccurrence );
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#getUsageCount()
  @Override
  @Min( value = 1 )
  public Integer getUsageCount()
  {
    return super.getUsageCount();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#setUsageCount(java.lang.Integer)
  @Override
  public void setUsageCount( Integer usageCount )
  {
    super.setUsageCount( usageCount );
  }

  //*********************************************************
  // Same as NiceTermVO, but individual because of validation
  //*********************************************************

  // @see ch.ige.wdltool.vo.AbstractTermVO#getClassNumber()
  @Override
  @NotNull
  public Integer getClassNumber()
  {
    return super.getClassNumber();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#setClassNumber(java.lang.Integer)
  @Override
  public void setClassNumber( Integer classNumber )
  {
    super.setClassNumber( classNumber );
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#getTerm()
  @Override
  @Term
  public String getTerm()
  {
    return super.getTerm();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#getSearchTerm()
  @Override
  @SearchTerm
  public String getSearchTerm()
  {
    return super.getSearchTerm();
  }
}
