package ch.ige.wdltool.vo;

import java.sql.Date;
import java.sql.Timestamp;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.hibernate.validator.Min;
import org.hibernate.validator.NotNull;
import org.hibernate.validator.Past;

import ch.ige.dbutils.vo.IdentifiableVO;
import ch.ige.wdltool.enums.DataSource;
import ch.ige.wdltool.enums.TermType;
import ch.ige.wdltool.validator.SearchTerm;
import ch.ige.wdltool.validator.Term;

/**
 * A term which is only used within IPI. It must be renewed or deleted
 * after a configurable time.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: IpiTermVO.java,v 1.16 2008/12/09 16:08:27 bandschapp Exp $
 */
public class IpiTermVO extends AbstractTermVO implements IdentifiableVO<Integer>
{
  /**
   * The date when this expired.
   */
  private Date expirationDate;

  /**
   * The empty constructor
   */
  public IpiTermVO()
  {
    super();
  }

  /**
   * Creates this from an abstract VO.
   * @param vo The VO whose values are copied
   */
  public IpiTermVO( AbstractTermVO vo )
  {
    super( vo );
	  initTermTypeAndDataSource();
  }

  // @see ch.ige.wdltool.vo.NiceTermVO#initTermTypeAndDataSource()
  @Override
  protected void initTermTypeAndDataSource()
  {
    this.termType = TermType.IPI_TERM;
    this.dataSource = DataSource.IPI;
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#isActive()
  @Override
  public boolean isActive()
  {
    return super.isActive();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#setActive(boolean)
  @Override
  public void setActive( boolean isActive )
  {
    super.setActive( isActive );
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#getLatestOccurrence()
  @Override
  @Past
  public Timestamp getLatestOccurrence()
  {
    return super.getLatestOccurrence();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#setLatestOccurrence(java.sql.Timestamp)
  @Override
  public void setLatestOccurrence( Timestamp latestOccurrence )
  {
    super.setLatestOccurrence( latestOccurrence );
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#getRenewal()
  @Override
  @Past
  public Timestamp getRenewal()
  {
    return super.getRenewal();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#setRenewal(java.sql.Timestamp)
  @Override
  public void setRenewal( Timestamp renewal )
  {
    super.setRenewal( renewal );
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#getUsageCount()
  @Override
  @Min( value = 1 )
  public Integer getUsageCount()
  {
    return super.getUsageCount();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#setUsageCount(java.lang.Integer)
  @Override
  public void setUsageCount( Integer usageCount )
  {
    super.setUsageCount( usageCount );
  }

  //*********************************************************
  // Same as NiceTermVO, but individual because of validation
  //*********************************************************

  // @see ch.ige.wdltool.vo.AbstractTermVO#getClassNumber()
  @Override
  @NotNull
  public Integer getClassNumber()
  {
    return super.getClassNumber();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#setClassNumber(java.lang.Integer)
  @Override
  public void setClassNumber( Integer classNumber )
  {
    super.setClassNumber( classNumber );
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#getTermGroupId()
  @Override
  public Integer getTermGroupId()
  {
    return super.getTermGroupId();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#setTermGroupId(java.lang.Integer)
  @Override
  public void setTermGroupId( Integer termGroupId )
  {
    super.setTermGroupId( termGroupId );
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#getTerm()
  @Override
  @Term
  public String getTerm()
  {
    return super.getTerm();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#getSearchTerm()
  @Override
  @SearchTerm
  public String getSearchTerm()
  {
    return super.getSearchTerm();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#isPublic()
  @Override
  public boolean isPublic()
  {
    return super.isPublic();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#setPublic(boolean)
  @Override
  public void setPublic( boolean isPublic )
  {
    super.setPublic( isPublic );
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#getTermGroupTerms()
  @Override
  public Map<Locale, List<AbstractTermVO>> getTermGroupTerms()
  {
    return super.getTermGroupTerms();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#setTermGroupTerms(java.util.Map)
  @Override
  public void setTermGroupTerms( Map<Locale, List<AbstractTermVO>> termGroupTerms )
  {
    super.setTermGroupTerms( termGroupTerms );
  }

  /**
   * Gets the IPI term group's number.
   * <strong>Note: This is not always set, check the documentation of the method that creates this!</strong>
   * @return The IPI term group's number
   */
  public Integer getIpiTermGroupNumber()
  {
    return super.getTermGroupNumber();
  }

  /**
   * Sets the IPI term group's number
   * @param ipiTermGroupNumber The IPI term group's number
   */
  public void setIpiTermGroupNumber( Integer ipiTermGroupNumber )
  {
    super.setTermGroupNumber( ipiTermGroupNumber );
  }

  /**
   * Gets the expiration date.
   * <strong>Note: This is not always set, check the documentation of the method that creates this!</strong>
   * @return the expiration date
   */
  public Date getExpirationDate()
  {
    return this.expirationDate;
  }

  /**
   * Sets the expiration date.
   * @param expirationDate The expiration date to set
   */
  public void setExpirationDate( Date expirationDate )
  {
    this.expirationDate = expirationDate;
  }
}
