package ch.ige.wdltool.vo;

import java.io.Serializable;

/**
 * Contains the information an update or insert operation on an active term produces.
 * That is the new term's ID and a flag determining whether a candidate has been deleted because of the operation.
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Christian Heinzer
 * @version  $Id: TermIdCleanupCandidateStatusVO.java,v 1.3 2007/12/04 06:54:18 heinzer Exp $
 */
public class TermIdCleanupCandidateStatusVO implements Serializable
{
  /**
   * The new terms identifiers
   */
  private Integer newTermId;

  /**
   * If this is <code>true</code>, then a candidate has been deleted because of the new/updated term
   * if this is <code>false</code>, then no corresponding candidate existed 
   */
  private boolean candidateRemoved;

  /**
   * Public constructor.
   * @param newTermId The id of the new term
   * @param candidateRemoved The flag must be <code>true</code> when a candidate has been deleted because of the new/updated term and <code>false</code> otherwise
   */
  public TermIdCleanupCandidateStatusVO( Integer newTermId, boolean candidateRemoved )
  {
    this.newTermId = newTermId;
    this.candidateRemoved = candidateRemoved;
  }

  /**
   * Gets the new term.
   * @return the term
   */
  public Integer getNewTermId()
  {
    return this.newTermId;
  }

  /**
   * Gets the flag showing whether a candidate has been deleted because of the new/updated term
   * @return the flag 
   * If this is <code>true</code>, then a candidate has been deleted because of the new/updated term
   * if this is <code>false</code>, then no corresponding candidate existed 
   */
  public boolean isCandidateRemoved()
  {
    return this.candidateRemoved;
  }

}
